@@ -47,6 +47,7 @@
 #include "main.h"
 
 static char _buffer[32];
+static int _last_error;
 
 extern "C"{
   GB_INTERFACE GB EXPORT;
@@ -221,24 +222,31 @@ class FBResult{
       return st1->Fetch();
     };
 
-    bool Execute(char *query){
-      try{
-        st1->Execute(query);
-      }
-      catch (IBPP::Exception& e){
-        std::cout<<e.what()<<std::endl;
-        IBPP::SQLException* pe=dynamic_cast<IBPP::SQLException *>(&e);
-        if(pe && pe->EngineCode()!=335544321){
-          GB.Error("Error to Execute the Query !");
-          return FALSE;
-      }
-      else{
-        GB.Error("arithmetic exception, numeric overflow, or string truncation !");
-        return FALSE;
-      }
-    }
-      return TRUE;
-    };
+		bool Execute(char *query)
+		{
+			try
+			{
+				st1->Execute(query);
+			}
+			catch (IBPP::Exception& e)
+			{
+				std::cout<<e.what()<<std::endl;
+				IBPP::SQLException* pe=dynamic_cast<IBPP::SQLException *>(&e);
+				if (pe && pe->EngineCode()!=335544321)
+				{
+					_last_error = pe->SqlCode();
+					GB.Error("Query failed");
+					return FALSE;
+				}
+				else
+				{
+					_last_error = pe->EngineCode();
+					GB.Error("Arithmetic exception, numeric overflow, or string truncation !");
+					return FALSE;
+				}
+			}
+			return TRUE;
+		};
 
     void SetStatement(IBPP::Database db1,IBPP::Transaction tr1){
       st1 = IBPP::StatementFactory(db1, tr1);
@@ -327,6 +335,7 @@ class FBConnect{
       catch (IBPP::Exception& e){
         std::cout<<e.what()<<std::endl;
         IBPP::SQLException* pe=dynamic_cast<IBPP::SQLException *>(&e);
+        _last_error = pe->EngineCode();
         if(pe && pe->EngineCode()==335544472){
           GB.Error("Your user name and password not defined !");
           return FALSE;
@@ -357,6 +366,7 @@ class FBConnect{
       catch (IBPP::Exception& e){
         std::cout<<e.what()<<std::endl;
         IBPP::SQLException* pe=dynamic_cast<IBPP::SQLException *>(&e);
+        _last_error = pe->EngineCode();
         if(pe && pe->EngineCode()==335544472){
           GB.Error("Your user name and password not defined !");
           return FALSE;
@@ -646,8 +656,11 @@ int FBConnect::Execute(char *query, DB_RESULT *result, char *err){
   IBPP::Blob blb1=IBPP::BlobFactory(db1, tr1);
   FBResult *res;
   std::stringstream sstr;
-  if(query)
-    res=new FBResult;
+  
+  if (!query)
+  	return retour;
+  
+  res=new FBResult;
   res->SetnRecord();
   if(!tr1->Started()){
     tr1->Start();
@@ -787,70 +800,7 @@ int FBConnect::Execute(char *query, DB_RESULT *result, char *err){
 
 *****************************************************************************/
 
-static DB_DRIVER _driver =
-{
-  "firebird",
-  open_database,
-  close_database,
-  format_value,
-  format_blob,
-  exec_query,
-  begin_transaction,
-  commit_transaction,
-  rollback_transaction,
-  get_quote,
-  {
-    query_init,
-    query_fill,
-    blob_read,
-    query_release,
-    {
-      field_type,
-      field_name,
-      field_index,
-      field_length,
-    },
-  },
-  {
-    field_exist,
-    field_list,
-    field_info,
-  },
-  {
-    table_init,
-    table_index,
-    table_release,
-    table_exist,
-    table_list,
-    table_primary_key,
-    table_is_system,
-    table_type,
-    table_delete,
-    table_create,
-  },
-  {
-    index_exist,
-    index_list,
-    index_info,
-    index_delete,
-    index_create,
-  },
-  {
-    database_exist,
-    database_list,
-    database_is_system,
-    database_delete,
-    database_create,
-  },
-  {
-    user_exist,
-    user_list,
-    user_info,
-    user_delete,
-    user_create,
-    user_set_password
-  }
-};
+DECLARE_DRIVER(_driver, "firebird");
 
 /* internal function to quote a value stored as a string */
 
@@ -1043,10 +993,17 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db){
   db->version=db_version(con->GetVersion(hosttmp, usertmp, passwordtmp));
   if(!con->SetTransaction(IBPP::amWrite, IBPP::ilConcurrency, IBPP::lrWait))
     return TRUE;
-    db->flags.no_table_type = TRUE;
-    db->flags.no_serial = FALSE;
-    db->flags.no_blob = FALSE;
-    db->handle=con;
+    
+	db->flags.no_table_type = TRUE;
+	db->flags.no_serial = FALSE;
+	db->flags.no_blob = FALSE;
+	
+	// Apparently idbp is always case unsensitive
+	db->flags.no_case = TRUE;
+	db->ignore_case = TRUE;
+	
+	db->handle=con;
+	
 	// BM: Firebird LIMIT syntax
 	db->limit.keyword = "FIRST";
 	db->limit.pos = DB_LIMIT_AT_BEGIN;
@@ -1104,7 +1061,8 @@ static void format_blob(DB_BLOB *blob, DB_FORMAT_CALLBACK add)
 
 *****************************************************************************/
 
-static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add){
+static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add)
+{
   char *s;
   int l;
   int i;
@@ -1159,7 +1117,7 @@ static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add){
 
   Send a query to the server and gets the result.
 
-  <handle> is the database handle, as returned by open_database()
+  <db> is the database handle, as returned by open_database()
   <query> is the query string.
   <result> will receive the result handle of the query.
   <err> is an error message used when the query failed.
@@ -1168,16 +1126,20 @@ static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add){
 
 *****************************************************************************/
 
-static int exec_query(DB_DATABASE *db, char *query, DB_RESULT *result, char *err){
+static int exec_query(DB_DATABASE *db, char *query, DB_RESULT *result, char *err)
+{
   int retour=FALSE;
   FBConnect *con=(FBConnect *)db->handle;
   FBResult *res=NULL;
   retour=con->Execute((char*)query,(DB_RESULT *) &res,err);
   if(res)
     *result=(DB_RESULT *)res;
+  else
+  	db->error = _last_error;
   return retour;
 }
 
+
 /*****************************************************************************
 
   query_init()
@@ -1193,7 +1155,8 @@ static int exec_query(DB_DATABASE *db, char *query, DB_RESULT *result, char *err
 
 *****************************************************************************/
 
-static void query_init(DB_RESULT result, DB_INFO *info, long int *count){
+static void query_init(DB_RESULT result, DB_INFO *info, int *count)
+{
   FBResult *res = (FBResult *)result;
   if(res){
     *count = res->GetnRecord();
@@ -1216,7 +1179,8 @@ static void query_init(DB_RESULT result, DB_INFO *info, long int *count){
 
 *****************************************************************************/
 
-static void query_release(DB_RESULT result, DB_INFO *info){
+static void query_release(DB_RESULT result, DB_INFO *info)
+{
   FBResult *res = (FBResult *)result;
   delete res;
 }
@@ -1228,6 +1192,7 @@ static void query_release(DB_RESULT result, DB_INFO *info){
 
   Fill a result buffer with the value of each field of a record.
 
+  <db> is the database handle, as returned by open_database()
   <result> is the handle of the result.
   <pos> is the index of the record in the result.
   <buffer> points to an array having one element for each field in the
@@ -1238,7 +1203,8 @@ static void query_release(DB_RESULT result, DB_INFO *info){
 
 *****************************************************************************/
 
-static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next){
+static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next)
+{
   FBResult *res=(FBResult *)result;
   unsigned i;
   GB_VARIANT fantom;
@@ -1277,7 +1243,8 @@ static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int n
 
 *****************************************************************************/
 
-static void blob_read(DB_RESULT result, int pos, int field, DB_BLOB *blob){
+static void blob_read(DB_RESULT result, int pos, int field, DB_BLOB *blob)
+{
     //std::cout<<"read blob"<<std::endl;
   FBResult *res = (FBResult *)result;
   char *data;
@@ -1310,7 +1277,8 @@ static void blob_read(DB_RESULT result, int pos, int field, DB_BLOB *blob){
 
 *****************************************************************************/
 
-static char *field_name(DB_RESULT result, int field){
+static char *field_name(DB_RESULT result, int field)
+{
   FBResult *res=(FBResult *)result;
   // Attention position des colonnes a partir de 1 et pas 0
   return (char *) res->ColumnName(field+1);
@@ -1329,7 +1297,8 @@ static char *field_name(DB_RESULT result, int field){
 
 *****************************************************************************/
 
-static int field_index(DB_RESULT Result, char *name, DB_DATABASE *db){
+static int field_index(DB_RESULT Result, char *name, DB_DATABASE *db)
+{
   FBResult *res=(FBResult *)Result;
   // Attention position des Champs dans IBPP a partir de 1 et pas 0
   return res->ColumnNum(name)-1;
@@ -1536,7 +1505,7 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info){
   GB.Alloc((void **)&info->index, sizeof(int) * n);
   for (i = 0; i < n; i++){
     for (j = 0; j < info->nfield; j++){
-      if(strcasecmp(info->field[j].name, res->GetData(j,1).value._string.value) == 0){
+      if(strcmp(info->field[j].name, res->GetData(j,1).value._string.value) == 0){
         info->index[i] = j;
         break;
       }
@@ -1609,7 +1578,7 @@ static int table_exist(DB_DATABASE *db, char *table){
 
 *****************************************************************************/
 
-static long table_list(DB_DATABASE *db, char ***tables){
+static int table_list(DB_DATABASE *db, char ***tables){
   static char query[SQLMAXLEN];
   FBResult *res;
   unsigned i;
@@ -1740,7 +1709,9 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
   int i;
   DB.Query.Init();
   DB.Query.Add("CREATE TABLE ");
+  DB.Query.Add(QUOTE_STRING);
   DB.Query.Add(table);
+  DB.Query.Add(QUOTE_STRING);
   DB.Query.Add(" (");
   comma = FALSE;
   for (fp = fields; fp; fp = fp->next){
@@ -1748,7 +1719,11 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
       DB.Query.Add(", ");
     else
       comma = TRUE;
+	  
+	  DB.Query.Add(QUOTE_STRING);
     DB.Query.Add(fp->name);
+  	DB.Query.Add(QUOTE_STRING);
+    
     switch (fp->type){
       case GB_T_BOOLEAN:
       case GB_T_INTEGER: type = "INTEGER";
@@ -1771,7 +1746,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
           type = "VARCHAR(DEFAULT_VARCHAR_LEN)";
         }
         else{
-          sprintf(_buffer, "VARCHAR(%ld)", fp->length);
+          sprintf(_buffer, "VARCHAR(%d)", fp->length);
           type = _buffer;
         }
         break;
@@ -1859,7 +1834,7 @@ static int field_exist(DB_DATABASE *db, char *table, char *field){
 
 *****************************************************************************/
 
-static long field_list(DB_DATABASE *db, char *table, char ***fields){
+static int field_list(DB_DATABASE *db, char *table, char ***fields){
   static char query[SQLMAXLEN];
   FBResult *res;
   unsigned i;
@@ -1982,7 +1957,7 @@ static int index_exist(DB_DATABASE *db, char *table, char *index){
 
 *****************************************************************************/
 
-static long index_list(DB_DATABASE *db, char *table, char ***indexes){
+static int index_list(DB_DATABASE *db, char *table, char ***indexes){
   FBResult *res;
   static char query[SQLMAXLEN];
   snprintf(query,SQLMAXLEN-1,"select rdb$index_name from rdb$indices where rdb$indices.rdb$relation_name=upper('%s')",table);
@@ -2101,7 +2076,9 @@ static int index_create(DB_DATABASE *db, char *table, char *index, DB_INDEX *inf
   if (info->unique)
     DB.Query.Add("UNIQUE ");
   DB.Query.Add("INDEX ");
+  DB.Query.Add(QUOTE_STRING);
   DB.Query.Add(index);
+  DB.Query.Add(QUOTE_STRING);
   DB.Query.Add(" ON ");
   DB.Query.Add(table);
   DB.Query.Add(" ( ");
@@ -2150,7 +2127,7 @@ static int database_exist(DB_DATABASE *db, char *name){
 
 *****************************************************************************/
 
-static long database_list(DB_DATABASE *db, char ***databases){
+static int database_list(DB_DATABASE *db, char ***databases){
   return -1;
 }
 
@@ -2283,7 +2260,7 @@ static int user_exist(DB_DATABASE *db, char *name){
 
 *****************************************************************************/
 
-static long user_list(DB_DATABASE *db, char ***users){
+static int user_list(DB_DATABASE *db, char ***users){
   FBConnect *con = (FBConnect *)db->handle;
   UsersL ul=con->UserList();
   unsigned i;
@@ -2396,7 +2373,7 @@ static int user_set_password(DB_DATABASE *db, char *name, char *password){
   std::string nametmp=name,passwordtmp=password;
   std::transform(nametmp.begin(), nametmp.end(), nametmp.begin(), toupper);
   if(!con->ModifyUser(nametmp,passwordtmp)){
-    GB.Error("user &1 not Modified or not permission to modify this user !!!");
+    GB.Error("User &1 not modified or not permission to modify this user !!!");
     return TRUE;
   };
   return FALSE;
