@@ -32,6 +32,7 @@
 
 #include <mysql.h>
 
+#include "gb.db.proto.h"
 #include "main.h"
 
 typedef
@@ -53,7 +54,7 @@ static DB_DRIVER _driver;
 
 /* internal function to quote a value stored as a string */
 
-static void quote(char *data, long len, DB_FORMAT_CALLBACK add)
+static void quote_string(char *data, long len, DB_FORMAT_CALLBACK add)
 {
 	int i;
 	unsigned char c;
@@ -77,65 +78,47 @@ static void quote(char *data, long len, DB_FORMAT_CALLBACK add)
 
 /* internal function to quote a value stored as a blob */
 
-#define quote_blob quote
+#define quote_blob quote_string
 
-#if 0
-/* internal function to unquote a value stored as a string */
+/* Quote a string and returns the result as a temporary string */
 
-static int unquote(char *data, long len, DB_FORMAT_CALLBACK add)
+static char *get_quote_string(char *str, int len, char quote)
 {
-  int i;
-  char c;
-
-  if (!data || *data != '\'')
-    return TRUE;
-
-  for (i = 1;; i++)
-  {
-    c = data[i];
-    if (c == '\'')
-      break;
-    if (c == '\\')
-      i++;
-    (*add)(&data[i], 1);
-  }
-
-	return FALSE;
-}
-
-/* internal function to unquote a value stored as a blob */
-
-static int unquote_blob(char *data, long len, DB_FORMAT_CALLBACK add)
-{
-  int i;
-  char c;
-
-  //if (!data || *data != '\'')
-  //  return TRUE;
-
-  for (i = 0; i < len; i++)
-  {
-    c = data[i];
-    if (c == '\'')
-      break;
-    if (c == '\\')
-    {
-      i++;
-      c = data[i];
-      if (c >= '0' && c <= '9' && i < (len - 2))
-      {
-      	c = ((data[i] - '0') << 6) + ((data[i + 1] - '0') << 3) + (data[i + 2] - '0');
-      	i += 2;
-      	(*add)(&c, 1);
-      	continue;
-      }
+	char *res, *p, c;
+	int len_res;
+	int i;
+	
+	len_res = len;
+	for (i = 0; i < len; i++)
+	{
+		c = str[i];
+		if (c == quote || c == '\\' || c == 0)
+			len_res++;
+	}
+	
+	GB.TempString(&res, NULL, len_res);
+	
+	p = res;
+	for (i = 0; i < len; i++)
+	{
+		c = str[i];
+		if (c == '\\' || c == quote)
+		{
+			*p++ = c;
+			*p++ = c;
 		}
-    (*add)(&data[i], 1);
-  }
-
-	return FALSE;
+		else if (c == 0)
+		{
+			*p++ = '\\';
+			*p++ = '0';
+		}
+		else
+			*p++ = c;
+	}
+	*p = 0;
+	
+	return res;
 }
-#endif
 
 /* Internal function to convert a database type into a Gambas type */
 
@@ -172,7 +155,7 @@ static GB_TYPE conv_type(int type, int len)
 
     case FIELD_TYPE_BLOB: // TEXT
     	//fprintf(stderr, "FIELD_TYPE_BLOB: %d\n", len);
-    	if (len == 16777215 || len <= 0) // LONG BLOB
+    	if (len >= 0x1000000 || len <= 0) // LONG BLOB
     		return DB_T_BLOB;
 			else
 				return GB_T_STRING;
@@ -262,7 +245,7 @@ static int conv_string_type(const char *type, long *len)
 
 /* Internal function to convert a database value into a Gambas variant value */
 
-static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int type, int len)
+static void conv_data(int version, char *data, long data_length, GB_VARIANT_VALUE *val, int type, int len)
 {
   GB_VALUE conv;
   GB_DATE_SERIAL date;
@@ -283,7 +266,7 @@ static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int t
         GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);
 
         val->_integer.type = GB_T_INTEGER;
-        val->_integer.value = ((GB_INTEGER *)&conv)->value;
+        val->_integer.value = conv._integer.value;
       }
 
       break;
@@ -297,7 +280,7 @@ static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int t
       GB.NumberFromString(GB_NB_READ_INTEGER, data, strlen(data), &conv);
 
       val->type = GB_T_INTEGER;
-      val->_integer.value = ((GB_INTEGER *)&conv)->value;
+      val->_integer.value = conv._integer.value;
 
       break;
 
@@ -306,7 +289,7 @@ static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int t
       GB.NumberFromString(GB_NB_READ_LONG, data, strlen(data), &conv);
 
       val->type = GB_T_LONG;
-      val->_long.value = ((GB_LONG *)&conv)->value;
+      val->_long.value = conv._long.value;
 
       break;
 
@@ -317,7 +300,7 @@ static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int t
       GB.NumberFromString(GB_NB_READ_FLOAT, data, strlen(data), &conv);
 
       val->_float.type = GB_T_FLOAT;
-      val->_float.value = ((GB_FLOAT *)&conv)->value;
+      val->_float.value = conv._float.value;
 
       break;
 
@@ -326,6 +309,10 @@ static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int t
     case FIELD_TYPE_TIME:
     case FIELD_TYPE_TIMESTAMP:
 
+			// TIMESTAMP display format changed since MySQL 4.1!
+			if (type == FIELD_TYPE_TIMESTAMP && version >= 040100)
+				type = FIELD_TYPE_DATETIME;
+
       memset(&date, 0, sizeof(date));
 
       switch(type)
@@ -386,8 +373,8 @@ static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int t
       GB.MakeDate(&date, (GB_DATE *)&conv);
 
       val->_date.type = GB_T_DATE;
-      val->_date.date = ((GB_DATE *)&conv)->value.date;
-      val->_date.time = ((GB_DATE *)&conv)->value.time;
+      val->_date.date = conv._date.value.date;
+      val->_date.time = conv._date.value.time;
 
       break;
 
@@ -431,7 +418,7 @@ static void conv_data(char *data, long data_length, GB_VARIANT_VALUE *val, int t
 
 static char *query_param[3];
 
-static void query_get_param(int index, char **str, long *len)
+static void query_get_param(int index, char **str, int *len, char quote)
 {
   if (index > 3)
     return;
@@ -439,13 +426,20 @@ static void query_get_param(int index, char **str, long *len)
   index--;
   *str = query_param[index];
   *len = strlen(*str);
+  
+  if (quote == '\'' || quote == '`')
+  {
+  	*str = get_quote_string(*str, *len, quote);
+  	*len = GB.StringLength(*str);
+  }
 }
 
 /* Internal function to run a query */
 
-static int do_query(MYSQL *conn, const char *error, MYSQL_RES **pres,
+static int do_query(DB_DATABASE *db, const char *error, MYSQL_RES **pres,
                     const char *qtemp, int nsubst, ...)
 {
+	MYSQL *conn = (MYSQL *)db->handle;
   va_list args;
   int i;
   const char *query;
@@ -460,7 +454,7 @@ static int do_query(MYSQL *conn, const char *error, MYSQL_RES **pres,
     for (i = 0; i < nsubst; i++)
       query_param[i] = va_arg(args, char *);
 
-    query = GB.SubstString(qtemp, 0, query_get_param);
+    query = DB.SubstString(qtemp, 0, query_get_param);
   }
   else
     query = qtemp;
@@ -470,19 +464,25 @@ static int do_query(MYSQL *conn, const char *error, MYSQL_RES **pres,
 
   if(mysql_query(conn, query)){
     ret = TRUE;
-    GB.Error(error, mysql_error(conn));
+    if (error)
+    	GB.Error(error, mysql_error(conn));
   }
   else {
     res = mysql_store_result(conn);
     ret = FALSE;
     if (pres)
       *pres = res;
+    else
+      mysql_free_result(res);
   }
 
+	db->error = mysql_errno(conn);
   return ret;
 }
-/* Internal function to check database version number */
-long db_version(MYSQL *conn)
+
+/* Internal function to return database version number as a XXYYZZ integer number*/
+
+static int db_version(DB_DATABASE *db)
 {
   //Check db version
   const char *vquery = "select left(version(),6)";
@@ -490,7 +490,7 @@ long db_version(MYSQL *conn)
   MYSQL_RES *res;
   MYSQL_ROW row;
 
-  if (!do_query(conn, NULL, &res, vquery, 0))
+  if (!do_query(db, NULL, &res, vquery, 0))
   {
      unsigned int verMain, verMajor, verMinor;
      row = mysql_fetch_row(res);
@@ -501,6 +501,28 @@ long db_version(MYSQL *conn)
   return dbversion;
 }
 
+/* Search in the first column a result for a specific name */
+ 
+static bool search_result(MYSQL_RES *res, char *name, MYSQL_ROW *row)
+{
+	int i;
+	MYSQL_ROW r;
+	
+  for (i = 0; i < mysql_num_rows(res); i++ )
+  {
+    r = mysql_fetch_row(res);
+
+		if (!strcmp(r[0], name))
+		{
+			if (row) 
+				*row = r;
+			break;
+		}
+	}
+		
+	return (i >= mysql_num_rows(res));
+}
+
 /*****************************************************************************
 
   get_quote()
@@ -527,6 +549,43 @@ static char *get_quote(void)
 
 *****************************************************************************/
 
+static void set_character_set(DB_DATABASE *db)
+{
+	MYSQL_RES *res;
+	MYSQL_ROW row;
+
+//	sys_charset = GB.System.Charset();
+//	db_charset = NULL;
+
+// 	for (i = 0; i < strlen(sys_charset); i++)
+// 	{
+// 		c = sys_charset[i];
+// 		if (!c)
+// 			break;
+// 		c = tolower(c);
+// 		if (!((c >= 'a' && c <= 'z') || (c >= '0' && c <= '9')))
+// 			continue;
+// 		GB.AddString(&db_charset, (const char *)&c, 1);
+// 	}
+// 	
+// 	db_charset[i] = 0;
+
+	if (do_query(db, NULL, NULL, "set names 'utf8'", 0))
+		fprintf(stderr, "WARNING: Unable to set database charset to UTF-8\n");
+		
+//	GB.FreeString(&db_charset);
+   
+	if (do_query(db, "Unable to get database charset: &1", &res, "show variables like 'character_set_client'", 0))
+		return;
+	
+	if (search_result(res, "character_set_client", &row))
+		return;
+	
+  GB.NewString(&db->charset, row[1], 0);
+  //fprintf(stderr, "charset is '%s'\n", db->charset);
+  mysql_free_result(res);
+}
+
 static int open_database(DB_DESC *desc, DB_DATABASE *db)
 {
   MYSQL *conn;
@@ -556,15 +615,14 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 
   /* Character Set cannot be set at this point int time */
   //printf("Character set GB[%s] mysql[%s]\n", GB.System.Charset(), mysql_character_set_name(conn));
-  GB.NewString(&db->charset, mysql_character_set_name(conn), 0);
-
+  
   /* set dbversion */
-  db->version = db_version(conn);
-
-	/* flags: none at the moment */
-
 	db->handle = conn;
+  db->version = db_version(db);
 
+	set_character_set(db);
+	
+	/* flags: none at the moment */
   return FALSE;
 }
 
@@ -624,7 +682,7 @@ static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add)
     case GB_T_STRING:
     case GB_T_CSTRING:
 
-			quote(VALUE((GB_STRING *)arg).addr + VALUE((GB_STRING *)arg).start, VALUE((GB_STRING *)arg).len, add);
+			quote_string(VALUE((GB_STRING *)arg).addr + VALUE((GB_STRING *)arg).start, VALUE((GB_STRING *)arg).len, add);
       return TRUE;
 
     case GB_T_DATE:
@@ -679,7 +737,7 @@ static void format_blob(DB_BLOB *blob, DB_FORMAT_CALLBACK add)
 
   Send a query to the server and gets the result.
 
-  <handle> is the database handle, as returned by open_database()
+  <db> is the database handle, as returned by open_database()
   <query> is the query string.
   <result> will receive the result handle of the query.
   <err> is an error message used when the query failed.
@@ -690,7 +748,7 @@ static void format_blob(DB_BLOB *blob, DB_FORMAT_CALLBACK add)
 
 static int exec_query(DB_DATABASE *db, char *query, DB_RESULT *result, char *err)
 {
-  return do_query((MYSQL *)db->handle, err, (MYSQL_RES **)result, query, 0);
+	return do_query(db, err, (MYSQL_RES **)result, query, 0);
 }
 
 
@@ -749,6 +807,7 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 
   Fill a result buffer with the value of each field of a record.
 
+  <db> is the database handle, as returned by open_database()
   <result> is the handle of the result.
   <pos> is the index of the record in the result.
   <buffer> points to an array having one element for each field in the
@@ -760,7 +819,7 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 
 *****************************************************************************/
 
-static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next)
+static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next)
 {
   MYSQL_RES *res = (MYSQL_RES *)result;
   MYSQL_FIELD *field;
@@ -783,7 +842,7 @@ static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int n
     value.value.type = GB_T_NULL;
 
 		if (data)
-			conv_data(data, mysql_fetch_lengths(res)[i], &value.value, field->type, field->length);
+			conv_data(db->version, data, mysql_fetch_lengths(res)[i], &value.value, field->type, field->length);
 
 		GB.StoreVariant(&value, &buffer[i]);
   
@@ -886,40 +945,45 @@ static int field_index(DB_RESULT Result, char *name, DB_DATABASE *db)
   MYSQL_RES *result = (MYSQL_RES *)Result;
 
   fld = strchr(name, (int)FLD_SEP);
-  if (fld){ /* Field does includes table info */
-       GB.NewString(&table, name, fld - name);
-       fld = fld + 1;
-  }
-  else {
-          table = NULL;
-	  fld = name;
+  if (fld)
+	{ /* Field does includes table info */
+		GB.NewString(&table, name, fld - name);
+		fld = fld + 1;
+	}
+  else 
+  {
+		table = NULL;
+		fld = name;
   }
 
   num_fields = mysql_num_fields(result);
 
-  if (strcmp(name,fld)!=0){ /* table name included */
-     mysql_field_seek(result,0); /* start at beginning */
-     for (i = 0; i < num_fields; i++)
-     {
-         field = mysql_fetch_field(result);
-	 if ( strcasecmp( fld, field->name) == 0 && strcmp( table, field->table) == 0){
-	     GB.FreeString(&table); return i;
-	 }
-     }
-     fld = name;
+  if (strcmp(name,fld)!=0)
+  { /* table name included */
+		mysql_field_seek(result,0); /* start at beginning */
+		for (i = 0; i < num_fields; i++)
+		{
+			field = mysql_fetch_field(result);
+			if (strcmp( fld, field->name) == 0 && strcmp( table, field->table) == 0)
+			{
+				GB.FreeString(&table); 
+				return i;
+			}
+		}
+		fld = name;
   }
 
-  if (table)
-     GB.FreeString(&table);
+	if (table)
+		GB.FreeString(&table);
 
   /* Do not consider table name, also reached where table cannot be found. *
    * Mysql can include . in the fieldname!!                                */
-  mysql_field_seek(result,0); /* start at beginning */
+  mysql_field_seek(result, 0); /* start at beginning */
   for (i = 0; i < num_fields; i++)
   {
-      field = mysql_fetch_field(result);
-      if ( strcasecmp( fld, field->name) == 0)
-          return i;
+		field = mysql_fetch_field(result);
+		if (strcmp( fld, field->name) == 0)
+			return i;
   }
 
   return -1;
@@ -990,8 +1054,8 @@ static int begin_transaction(DB_DATABASE *db)
 {
   /* Autocommit is on by default. Lets set it off. */
   /* BM: why not doing that when we open the connection ? */
-  do_query((MYSQL *)db->handle, "Unable to set autocommit to 0: &1", NULL, "set autocommit=0", 0);
-  return do_query((MYSQL *)db->handle, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
+  do_query(db, "Unable to set autocommit to 0: &1", NULL, "set autocommit=0", 0);
+  return do_query(db, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
 }
 
 
@@ -1010,10 +1074,10 @@ static int begin_transaction(DB_DATABASE *db)
 
 static int commit_transaction(DB_DATABASE *db)
 {
-  bool ret = do_query((MYSQL *)db->handle, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
+  bool ret = do_query(db, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
   /* Autocommit needs to be set back on. */
   /* BM: and what happens if transactions are imbricated ? */
-  do_query((MYSQL *)db->handle, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
+  do_query(db, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
   return ret;
 }
 
@@ -1039,10 +1103,10 @@ static int commit_transaction(DB_DATABASE *db)
 
 static int rollback_transaction(DB_DATABASE *db)
 {
-  bool ret = do_query((MYSQL *)db->handle, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
+  bool ret = do_query(db, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
   /* Autocommit needs to be set back on. */
   /* BM: and what happens if transactions are imbricated ? */
-  do_query((MYSQL *)db->handle, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
+  do_query(db, "Unable to set autocommit to On: &1", NULL, "set autocommit=1", 0);
   return ret;
 }
 
@@ -1088,7 +1152,7 @@ static int table_init(DB_DATABASE *db, char *table, DB_INFO *info)
   if (n == 0)
     return TRUE;
 
-  GB.Alloc((void **)&info->field, sizeof(DB_FIELD) * n);
+  GB.Alloc((void **)POINTER(&info->field), sizeof(DB_FIELD) * n);
 
   i = 0;
 
@@ -1145,14 +1209,13 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
   char *qindex =
     "show index from `&1`";
 
-  MYSQL *conn = (MYSQL *)db->handle;
   MYSQL_RES *res;
   MYSQL_ROW row;
   int i, j, n;
 
   /* Index primaire */
 
-  if (do_query(conn, "Unable to get primary index: &1", &res, qindex, 1, table))
+  if (do_query(db, "Unable to get primary index: &1", &res, qindex, 1, table))
 	  return TRUE;
 
   for ( i = 0, n = 0;  i < mysql_num_rows(res); i++ )
@@ -1172,7 +1235,7 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
     return TRUE;
   }
 
-  GB.Alloc((void **)&info->index, sizeof(int) * n);
+  GB.Alloc((void **)POINTER(&info->index), sizeof(int) * n);
 
   for (i = 0; i < n; i++)
   {
@@ -1181,7 +1244,7 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
     {
         for (j = 0; j < info->nfield; j++)
         {
-            if (strcasecmp(info->field[j].name,  row[4]) == 0)
+            if (strcmp(info->field[j].name, row[4]) == 0)
             {
                 info->index[i] = j;
                 break;
@@ -1227,19 +1290,17 @@ static void table_release(DB_DATABASE *db, DB_INFO *info)
 
 static int table_exist(DB_DATABASE *db, char *table)
 {
-  MYSQL *conn = (MYSQL *)db->handle;
   MYSQL_RES *res;
   int exist;
 
   const char *query =
     "show tables like '&1'";
 
-  if (do_query(conn, "Unable to check table: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to check table: &1", &res, query, 1, table))
     return FALSE;
 
-  exist = mysql_num_rows(res);
+  exist = !search_result(res, table, NULL);
   mysql_free_result(res);
-  //return FALSE;
   return exist;
 }
 
@@ -1260,9 +1321,8 @@ static int table_exist(DB_DATABASE *db, char *table)
 
 *****************************************************************************/
 
-static long table_list(DB_DATABASE *db, char ***tables)
+static int table_list(DB_DATABASE *db, char ***tables)
 {
-  MYSQL *conn = (MYSQL *)db->handle;
   MYSQL_RES *res;
   MYSQL_ROW row;
   long i;
@@ -1271,7 +1331,7 @@ static long table_list(DB_DATABASE *db, char ***tables)
   const char *query =
     "show tables";
 
-  if (do_query(conn, "Unable to get tables", &res, query, 0))
+  if (do_query(db, "Unable to get tables", &res, query, 0))
     return -1;
 
   rows = mysql_num_rows(res);
@@ -1307,12 +1367,11 @@ static int table_primary_key(DB_DATABASE *db, char *table, char ***primary)
   const char *query =
     "show index from `&1`";
 
-  MYSQL *conn = (MYSQL *)db->handle;
   MYSQL_RES *res;
   MYSQL_ROW row;
   int i;
 
-  if (do_query(conn, "Unable to get primary key: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to get primary key: &1", &res, query, 1, table))
     return TRUE;
 
   GB.NewArray(primary, sizeof(char *), 0);
@@ -1351,12 +1410,11 @@ static int database_is_system(DB_DATABASE *db, char *name);
 
 static int table_is_system(DB_DATABASE *db, char *table)
 {
-  MYSQL *conn = (MYSQL *)db->handle;
   MYSQL_RES *res;
   MYSQL_ROW row;
   int system;
 
-  if (do_query(conn, "Unable to check database: &1", &res, "select database()", 0))
+  if (do_query(db, "Unable to check database: &1", &res, "select database()", 0))
     return FALSE;
 
   if (mysql_num_rows(res) != 1 )
@@ -1404,21 +1462,20 @@ static char *table_type(DB_DATABASE *db, char *table, char *settype)
   MYSQL_ROW row;
 
   if (settype){
-     if (do_query((MYSQL *)db->handle, "Cannot set table &1 to type &2", &res, update, 2, table, settype))
+     if (do_query(db, "Cannot set table &1 to type &2", &res, update, 2, table, settype))
         return NULL;
   }
 
-  if (do_query((MYSQL *)db->handle, "Invalid table: &1", &res, query, 1, table)){
+  if (do_query(db, "Invalid table: &1", &res, query, 1, table)){
     return NULL;
   }
 
-  if ( mysql_num_rows(res) != 1){
-     GB.Error("Unable to check table for: &1", table);
-     return NULL;
+  if (search_result(res, table, &row))
+  {
+		GB.Error("Unable to check table for: &1", table);
+		return NULL;
   }
 
-  row = mysql_fetch_row(res);
-  /*GB.NewZeroString(&type, row[1]);*/
   strcpy(buffer, row[1]);
   mysql_free_result(res);
   return buffer;
@@ -1441,9 +1498,7 @@ static char *table_type(DB_DATABASE *db, char *table, char *settype)
 
 static int table_delete(DB_DATABASE *db, char *table)
 {
-  return
-    do_query((MYSQL *)db->handle, "Unable to delete table: &1", NULL,
-      "drop table `&1`", 1, table);
+  return do_query(db, "Unable to delete table: &1", NULL, "drop table `&1`", 1, table);
 }
 
 /*****************************************************************************
@@ -1469,7 +1524,6 @@ static int table_delete(DB_DATABASE *db, char *table)
 
 static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **primary, char *tabletype)
 {
-  MYSQL *conn = (MYSQL *)db->handle;
   DB_FIELD *fp;
   char *type = NULL;
   int comma;
@@ -1490,9 +1544,9 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
     else
       comma = TRUE;
 
-    /*DB.Query.Add("`");*/
-    DB.Query.Add(fp->name);
-    /*DB.Query.Add("`");*/
+		DB.Query.Add(QUOTE_STRING);
+		DB.Query.Add(fp->name);
+		DB.Query.Add(QUOTE_STRING);
 
 		if (fp->type == DB_T_SERIAL)
 			DB.Query.Add(" BIGINT UNSIGNED NOT NULL AUTO_INCREMENT ");
@@ -1513,7 +1567,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
 						type = "TEXT";
 					else
 					{
-						sprintf(_buffer, "VARCHAR(%ld)", fp->length);
+						sprintf(_buffer, "VARCHAR(%d)", fp->length);
 						type = _buffer;
 					}
 
@@ -1552,7 +1606,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
 
       for (fp = fields; fp; fp = fp->next) //Check type of primary field
       {
-              if(strcasecmp(fp->name, primary[i]) == 0){
+              if(strcmp(fp->name, primary[i]) == 0){
                 if (fp->length <= 0 || fp->length > 255){
                         if (fp->type == GB_T_STRING)
                            DB.Query.Add("(255)");
@@ -1573,7 +1627,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
   }
 
   /* printf("table_create syntax: %s\n", DB.Query.Get());*/
-  return do_query(conn, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
 }
 
 /*****************************************************************************
@@ -1590,38 +1644,6 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
 
 *****************************************************************************/
 
-#if 0
-static int field_exist(DB_DATABASE *db, char *table, char *field)
-{
-  MYSQL *conn = (MYSQL *)db->handle;
-  MYSQL_RES *res;
-  int exist;
-  MYSQL_FIELD *f;
-
-  /* For some reason under mysql 4.0 some fields are not found by
-   * mysql_list_fields( conn, table, field )E.g. Create_tmp_table_priv
-   * in host */
-
-  res = mysql_list_fields( conn, table, field);
-  if (!res){
-     GB.Error("Unable to check field: &1", mysql_error(conn));
-     return FALSE;
-  }
-
-  /* (BM) need more test because field can contain wildcard ! */
-
-  exist = FALSE;
-  if (mysql_num_fields(res) == 1)
-  {
-    f = mysql_fetch_field(res);
-    exist = strcmp(field, f->name) == 0;
-  }
-
-  mysql_free_result(res);
-  return exist;
-}
-#endif
-
 static int field_exist(DB_DATABASE *db, char *table, char *field)
 {
   const char *query =
@@ -1630,10 +1652,10 @@ static int field_exist(DB_DATABASE *db, char *table, char *field)
   MYSQL_RES *res;
   int exist;
 
-  if (do_query((MYSQL *)db->handle, "Unable to check field: &1", &res, query, 2, table, field))
+  if (do_query(db, "Unable to check field: &1", &res, query, 2, table, field))
     return FALSE;
 
-  exist = mysql_num_rows(res) == 1;
+	exist = !search_result(res, field, NULL);
   mysql_free_result(res);
 
   return exist;
@@ -1658,19 +1680,17 @@ static int field_exist(DB_DATABASE *db, char *table, char *field)
 
 *****************************************************************************/
 
-static long field_list(DB_DATABASE *db, char *table, char ***fields)
+static int field_list(DB_DATABASE *db, char *table, char ***fields)
 {
   const char *query =
     "show columns from `&1`";
 
   long i, n;
-  MYSQL *conn = (MYSQL *)db->handle;
   MYSQL_RES *res;
   MYSQL_ROW row;
 
-  if (do_query(conn, "Unable to get fields: &1", &res, query, 1, table)){
+  if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
      return -1;
-  }
 
   n = mysql_num_rows(res);
 
@@ -1717,22 +1737,14 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
   int type;
   long len = 0;
 
-  if (do_query((MYSQL *)db->handle, "Unable to get field info: &1", &res, query, 2, table, field))
+  if (do_query(db, "Unable to get field info: &1", &res, query, 2, table, field))
     return TRUE;
 
-  if (mysql_num_rows(res) != 1)
-  {
-    GB.Error("Unable to find field &1.&2", table, field);
-    return TRUE;
-  }
-
-  row = mysql_fetch_row(res);
-
-  if (strcmp(row[0], field))
-  {
-    GB.Error("Unable to find field &1.&2 !", table, field);
-    return TRUE;
-  }
+	if (search_result(res, field, &row))
+	{
+		GB.Error("Unable to find field &2 in table &1", table, field);
+		return TRUE;
+	}
 
   info->name = NULL;
   type = conv_string_type(row[1], &len);
@@ -1750,7 +1762,7 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
 
   info->def._object.type = GB_T_NULL;
 
-	if (info->type == GB_T_LONG && strstr(row[5], "auto_increment"))
+	if ((info->type == GB_T_INTEGER || info->type == GB_T_LONG) && strstr(row[5], "auto_increment"))
 		info->type = DB_T_SERIAL;
 	else
 	{
@@ -1767,7 +1779,7 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
 
 			if (val && *val)
 			{
-				conv_data(val, 0, &def.value, type, len);
+				conv_data(db->version, val, 0, &def.value, type, len);
 				GB.StoreVariant(&def, &info->def);
 			}
 		}
@@ -1800,7 +1812,7 @@ static int index_exist(DB_DATABASE *db, char *table, char *index)
   MYSQL_ROW row;
   int i, n;
 
-  if (do_query((MYSQL *)db->handle, "Unable to check index: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to check index: &1", &res, query, 1, table))
     return FALSE;
 
   for ( i = 0, n = 0; i < mysql_num_rows(res); i++ )
@@ -1835,7 +1847,7 @@ static int index_exist(DB_DATABASE *db, char *table, char *index)
 
 *****************************************************************************/
 
-static long index_list(DB_DATABASE *db, char *table, char ***indexes)
+static int index_list(DB_DATABASE *db, char *table, char ***indexes)
 {
   const char *query = "show index from `&1`";
 
@@ -1843,7 +1855,7 @@ static long index_list(DB_DATABASE *db, char *table, char ***indexes)
   MYSQL_ROW row;
   long i, n, no_indexes;
 
-  if (do_query((MYSQL *)db->handle, "Unable to get indexes: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to get indexes: &1", &res, query, 1, table))
     return -1;
 
   for ( i = 0, no_indexes = 0; i < mysql_num_rows(res); i++ )
@@ -1892,10 +1904,10 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
     "show index from `&1`";
 
   MYSQL_RES *res;
-  MYSQL_ROW row;
+  MYSQL_ROW row = 0;
   int i, n;
 
-  if (do_query((MYSQL *)db->handle, "Unable to get index info: &1", &res, query, 2, table, index))
+  if (do_query(db, "Unable to get index info: &1", &res, query, 2, table, index))
     return TRUE;
 
   n = mysql_num_rows(res);
@@ -1911,7 +1923,7 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
 
 	if (n != 1)
   {
-    GB.Error("Unable to find index &1.&2", table, index);
+    GB.Error("Unable to find index &2 in table &1", table, index);
     return TRUE;
   }
 
@@ -1957,7 +1969,7 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
 static int index_delete(DB_DATABASE *db, char *table, char *index)
 {
   return
-    do_query((MYSQL *)db->handle, "Unable to delete index: &1", NULL,
+    do_query(db, "Unable to delete index: &1", NULL,
       "drop index `&1`", 1, index);
 }
 
@@ -1992,7 +2004,7 @@ static int index_create(DB_DATABASE *db, char *table, char *index, DB_INDEX *inf
   DB.Query.Add(info->fields);
   DB.Query.Add(" )");
 
-  return do_query((MYSQL *)db->handle, "Cannot create index: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create index: &1", NULL, DB.Query.Get(), 0);
 }
 
 /*****************************************************************************
@@ -2014,10 +2026,12 @@ static int database_exist(DB_DATABASE *db, char *name)
   MYSQL_RES *res;
   int exist;
 
-  res = mysql_list_dbs( conn, name);
-  if (!res){
-     GB.Error("Unable to check database: &1", mysql_error(conn));
-     return FALSE;
+  res = mysql_list_dbs(conn, name);
+  if (!res)
+  {
+		db->error = mysql_errno(conn);	
+		GB.Error("Unable to check database: &1", mysql_error(conn));
+		return FALSE;
   }
 
   exist = mysql_num_rows(res);
@@ -2041,15 +2055,16 @@ static int database_exist(DB_DATABASE *db, char *name)
  *
  ******************************************************************************/
 
-static long database_list(DB_DATABASE *db, char ***databases)
+static int database_list(DB_DATABASE *db, char ***databases)
 {
   long i, rows;
   MYSQL *conn = (MYSQL *)db->handle;
   MYSQL_RES *res;
   MYSQL_ROW row;
 
-  res = mysql_list_dbs( conn, 0);
+  res = mysql_list_dbs(conn, 0);
   if (!res){
+  	db->error = mysql_errno(conn);
      GB.Error("Unable to get databases: &1", mysql_error(conn));
      return -1;
   }
@@ -2111,7 +2126,7 @@ static int database_delete(DB_DATABASE *db, char *name)
 	}
 
   return
-     do_query((MYSQL *)db->handle, "Unable to delete database: &1", NULL,
+     do_query(db, "Unable to delete database: &1", NULL,
      "drop database `&1`", 1, name);
 }
 
@@ -2132,7 +2147,7 @@ static int database_delete(DB_DATABASE *db, char *name)
 static int database_create(DB_DATABASE *db, char *name)
 {
   return
-     do_query((MYSQL *)db->handle, "Unable to create database: &1", NULL,
+     do_query(db, "Unable to create database: &1", NULL,
      "create database `&1`", 1, name);
 }
 
@@ -2172,10 +2187,10 @@ static int user_exist(DB_DATABASE *db, char *name)
 
   _host = strrchr(_name,'@') + 1;
   _token = _host - 1;
-  _token[0] = (char) NULL;
+  _token[0] = 0;
 
 
- if (do_query((MYSQL *)db->handle, "Unable to check user: &1@&2", &res, query, 2, _name, _host)){
+ if (do_query(db, "Unable to check user: &1@&2", &res, query, 2, _name, _host)){
     free(_name);
     //free(_host);
     return FALSE;
@@ -2206,7 +2221,7 @@ static int user_exist(DB_DATABASE *db, char *name)
  *
  ******************************************************************************/
 
-static long user_list(DB_DATABASE *db, char ***users)
+static int user_list(DB_DATABASE *db, char ***users)
 {
   const char *query = "select user, host from mysql.user";
 
@@ -2216,7 +2231,7 @@ static long user_list(DB_DATABASE *db, char ***users)
   long i, count, length;
   char *_username;
 
-  if (do_query((MYSQL *)db->handle, "Unable to get users: &1", &res, query, 0))
+  if (do_query(db, "Unable to get users: &1", &res, query, 0))
     return -1;
 
   count = mysql_num_rows(res);
@@ -2300,9 +2315,9 @@ static int user_info(DB_DATABASE *db, char *name, DB_USER *info )
 
   _host = strrchr(_name,'@') + 1;
   _token = _host - 1;
-  _token[0] = (char) NULL;
+  _token[0] = 0;
 
- if (do_query((MYSQL *)db->handle, "Unable to check user info: &1@&2", &res, query, 2, _name, _host)){
+ if (do_query(db, "Unable to check user info: &1@&2", &res, query, 2, _name, _host)){
     free(_name);
     return TRUE;
  }
@@ -2367,12 +2382,12 @@ static int user_delete(DB_DATABASE *db, char *name)
 
   _host = strrchr(_name,'@') + 1;
   _token = _host - 1;
-  _token[0] = (char) NULL;
+  _token[0] = 0;
 
  //Still need to look at the removal of privileges
- // _ret =  do_query((MYSQL *)db->handle, "Unable to delete user: &1", NULL,
+ // _ret =  do_query(db, "Unable to delete user: &1", NULL,
  //	           "revoke all on *.* from &1@&2", 2, _name, _host);
-  _ret =  do_query((MYSQL *)db->handle, "Unable to delete user: &1", NULL,
+  _ret =  do_query(db, "Unable to delete user: &1", NULL,
 	          _delete, 2, _name, _host);
   free(_name);
   return _ret;
@@ -2429,7 +2444,7 @@ static int user_create(DB_DATABASE *db, char *name, DB_USER *info)
 
   free(_name);
 
-  return do_query((MYSQL *)db->handle, "Cannot create user: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create user: &1", NULL, DB.Query.Get(), 0);
 }
 
 /*****************************************************************************
@@ -2470,7 +2485,7 @@ static int user_set_password(DB_DATABASE *db, char *name, char *password)
   free(_name);
 
   return
-      do_query((MYSQL *)db->handle, "Cannot change user password: &1",
+      do_query(db, "Cannot change user password: &1",
         NULL, DB.Query.Get(), 0);
 }
 
@@ -2481,71 +2496,7 @@ static int user_set_password(DB_DATABASE *db, char *name, char *password)
 
 *****************************************************************************/
 
-static DB_DRIVER _driver =
-{
-  "mysql",
-  (void *)open_database,
-  (void *)close_database,
-  (void *)format_value,
-  (void *)format_blob,
-  (void *)exec_query,
-  (void *)begin_transaction,
-  (void *)commit_transaction,
-  (void *)rollback_transaction,
-  (void *)get_quote,
-  {
-    (void *)query_init,
-    (void *)query_fill,
-    (void *)blob_read,
-    (void *)query_release,
-    {
-      (void *)field_type,
-      (void *)field_name,
-      (void *)field_index,
-      (void *)field_length,
-    },
-  },
-  {
-    (void *)field_exist,
-    (void *)field_list,
-    (void *)field_info,
-  },
-  {
-    (void *)table_init,
-    (void *)table_index,
-    (void *)table_release,
-    (void *)table_exist,
-    (void *)table_list,
-    (void *)table_primary_key,
-    (void *)table_is_system,
-    (void *)table_type,
-    (void *)table_delete,
-    (void *)table_create,
-  },
-  {
-    (void *)index_exist,
-    (void *)index_list,
-    (void *)index_info,
-    (void *)index_delete,
-    (void *)index_create,
-  },
-  {
-    (void *)database_exist,
-    (void *)database_list,
-    (void *)database_is_system,
-    (void *)database_delete,
-    (void *)database_create,
-  },
-  {
-    (void *)user_exist,
-    (void *)user_list,
-    (void *)user_info,
-    (void *)user_delete,
-    (void *)user_create,
-    (void *)user_set_password
-  }
-};
-
+DECLARE_DRIVER(_driver, "mysql");
 
 /*****************************************************************************
 
