@@ -33,6 +33,15 @@
 #include <postgres.h>
 #include <pg_type.h>
 
+#ifdef PACKAGE_NAME
+	#undef PACKAGE_NAME
+	#undef PACKAGE_BUGREPORT
+	#undef PACKAGE_STRING
+	#undef PACKAGE_TARNAME
+	#undef PACKAGE_VERSION
+#endif
+
+#include "gb.db.proto.h"
 #include "main.h"
 
 
@@ -42,19 +51,23 @@ DB_INTERFACE DB EXPORT;
 
 static char _buffer[32];
 static DB_DRIVER _driver;
+static int _last_error;
 /*static int _print_query = FALSE;*/
 
 /* Internal function to check the result of a query */
 
 static int check_result(PGresult *res, const char *err)
 {
+	_last_error = 0;
+	
   if (!res)
   {
     GB.Error("Out of memory");
     return TRUE;
   }
 
-  switch (PQresultStatus(res))
+	_last_error = PQresultStatus(res);
+  switch (_last_error)
   {
     case PGRES_COMMAND_OK:
     case PGRES_TUPLES_OK:
@@ -71,7 +84,7 @@ static int check_result(PGresult *res, const char *err)
 
 /* internal function to quote a value stored as a string */
 
-static void quote(char *data, long len, DB_FORMAT_CALLBACK add)
+static void quote_string(char *data, long len, DB_FORMAT_CALLBACK add)
 {
 	int i;
 	unsigned char c;
@@ -96,6 +109,46 @@ static void quote(char *data, long len, DB_FORMAT_CALLBACK add)
 	(*add)("'", 1);
 }
 
+/* Quote a string and returns the result as a temporary string */
+
+static char *get_quote_string(char *str, int len, char quote)
+{
+	char *res, *p, c;
+	int len_res;
+	int i;
+	
+	len_res = len;
+	for (i = 0; i < len; i++)
+	{
+		c = str[i];
+		if (c == quote || c == '\\' || c == 0)
+			len_res++;
+	}
+	
+	GB.TempString(&res, NULL, len_res);
+	
+	p = res;
+	for (i = 0; i < len; i++)
+	{
+		c = str[i];
+		if (c == '\\' || c == quote)
+		{
+			*p++ = c;
+			*p++ = c;
+		}
+		/*else if (c == 0)
+		{
+			*p++ = '\\';
+			*p++ = '0';
+		}*/
+		else
+			*p++ = c;
+	}
+	*p = 0;
+	
+	return res;
+}
+
 /* internal function to quote a value stored as a blob */
 
 static void quote_blob(char *data, long len, DB_FORMAT_CALLBACK add)
@@ -128,7 +181,7 @@ static void quote_blob(char *data, long len, DB_FORMAT_CALLBACK add)
 
 /* internal function to unquote a value stored as a string */
 
-static int unquote(char *data, long len, DB_FORMAT_CALLBACK add)
+static int unquote_string(char *data, long len, DB_FORMAT_CALLBACK add)
 {
   int i;
   char c;
@@ -372,7 +425,7 @@ static void conv_data(char *data, long len, GB_VARIANT_VALUE *val, Oid type)
 
 static char *query_param[3];
 
-static void query_get_param(int index, char **str, long *len)
+static void query_get_param(int index, char **str, int *len, char quote)
 {
   if (index > 3)
     return;
@@ -380,13 +433,20 @@ static void query_get_param(int index, char **str, long *len)
   index--;
   *str = query_param[index];
   *len = strlen(*str);
+  
+  if (quote == '\'')
+  {
+  	*str = get_quote_string(*str, *len, quote);
+  	*len = GB.StringLength(*str);
+  }
 }
 
 /* Internal function to run a query */
 
-static int do_query(PGconn *conn, const char *error, PGresult **pres,
+static int do_query(DB_DATABASE *db, const char *error, PGresult **pres,
                     const char *qtemp, int nsubst, ...)
 {
+	PGconn *conn = (PGconn *)db->handle;
   va_list args;
   int i;
   const char *query;
@@ -401,7 +461,7 @@ static int do_query(PGconn *conn, const char *error, PGresult **pres,
     for (i = 0; i < nsubst; i++)
       query_param[i] = va_arg(args, char *);
 
-    query = GB.SubstString(qtemp, 0, query_get_param);
+    query = DB.SubstString(qtemp, 0, query_get_param);
   }
   else
     query = qtemp;
@@ -422,12 +482,13 @@ static int do_query(PGconn *conn, const char *error, PGresult **pres,
       PQclear(res);
   }
 
+	db->error = _last_error;
   return ret;
 }
 
 /* Internal function to check database version number */
 
-static long db_version(PGconn *conn)
+static int db_version(DB_DATABASE *db)
 {
   //Check db version
   const char *vquery =
@@ -435,7 +496,7 @@ static long db_version(PGconn *conn)
   long dbversion =0;
   PGresult *res;
 
-  if (!do_query(conn, NULL, &res, vquery, 0))
+  if (!do_query(db, NULL, &res, vquery, 0))
   {
     unsigned int verMain, verMajor, verMinor;
     sscanf(PQgetvalue(res, 0, 0),"%2u.%2u.%2u", &verMain, &verMajor, &verMinor);
@@ -515,12 +576,22 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
     return TRUE;
   }
 
+  /* get version */
+
+  db->handle = conn;
+  db->version = db_version(db);
+
+	/* flags */
+
+	db->flags.no_table_type = TRUE;
+	db->flags.no_nest = TRUE;
+
   /* encoding */
 
   if (PQsetClientEncoding(conn, GB.System.Charset()))
     fprintf(stderr, "gb.db.postgresql: cannot set encoding to %s\n", GB.System.Charset());
 
-  if (!do_query(conn, NULL, &res, query, 0))
+  if (!do_query(db, NULL, &res, query, 0))
   {
     GB.NewString(&db->charset, PQgetvalue(res, 0, 0), 0);
     PQclear(res);
@@ -528,16 +599,6 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
   else
     db->charset = NULL;
 
-  /* get version */
-
-  db->version = db_version(conn);
-
-	/* flags */
-
-	db->flags.no_table_type = TRUE;
-	db->flags.no_nest = TRUE;
-
-  db->handle = conn;
   return FALSE;
 }
 
@@ -597,7 +658,7 @@ static int format_value(GB_VALUE *arg, DB_FORMAT_CALLBACK add)
     case GB_T_STRING:
     case GB_T_CSTRING:
 
-			quote(VALUE((GB_STRING *)arg).addr + VALUE((GB_STRING *)arg).start, VALUE((GB_STRING *)arg).len, add);
+			quote_string(VALUE((GB_STRING *)arg).addr + VALUE((GB_STRING *)arg).start, VALUE((GB_STRING *)arg).len, add);
       return TRUE;
 
     case GB_T_DATE:
@@ -666,7 +727,7 @@ static void format_blob(DB_BLOB *blob, DB_FORMAT_CALLBACK add)
 
 static int exec_query(DB_DATABASE *db, char *query, DB_RESULT *result, char *err)
 {
-  return do_query((PGconn *)db->handle, err, (PGresult **)result, query, 0);
+  return do_query(db, err, (PGresult **)result, query, 0);
 }
 
 
@@ -720,6 +781,7 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 
   Fill a result buffer with the value of each field of a record.
 
+  <db> is the database handle, as returned by open_database()
   <result> is the handle of the result.
   <pos> is the index of the record in the result.
   <buffer> points to an array having one element for each field in the
@@ -730,7 +792,7 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 
 *****************************************************************************/
 
-static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next)
+static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next)
 {
   PGresult *res = (PGresult *)result;
   int i;
@@ -825,14 +887,13 @@ static char *field_name(DB_RESULT result, int field)
 static int field_index(DB_RESULT Result, char *name, DB_DATABASE *db)
 {
   PGresult *result = (PGresult *)Result;
-  PGconn *conn = (PGconn *)db->handle;
 
   char *fld;
   int index;
   char *table;
   int numfields, oid;
   char *qfield =
-    "select oid from pg_class where relname = lower('&1') "
+    "select oid from pg_class where relname = '&1' "
     "and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
   fld = strchr(name, (int)FLD_SEP);
@@ -850,7 +911,7 @@ static int field_index(DB_RESULT Result, char *name, DB_DATABASE *db)
         /* Need to find the OID for the table */
         PGresult *oidres;
 
-        if (do_query(conn, "Unable to get OID for table &1", &oidres, qfield, 1, table)){
+        if (do_query(db, "Unable to get OID for table &1", &oidres, qfield, 1, table)){
 	    GB.FreeString(&table);
 	    return -1;
         }
@@ -871,7 +932,7 @@ static int field_index(DB_RESULT Result, char *name, DB_DATABASE *db)
 	if (PQftable((PGresult *)result, index) != oid){
 	   numfields = PQnfields((PGresult *)result);
 	   while ( ++index < numfields){
-	       if (strcasecmp(PQfname((PGresult *)result, index),
+	       if (strcmp(PQfname((PGresult *)result, index),
 				       fld) == 0){ //Check Fieldname
 	          if (PQftable((PGresult *)result, index) == oid){ //check oid
 		     break; // is the required table oid
@@ -965,7 +1026,7 @@ static int field_length(DB_RESULT result, int field)
 
 static int begin_transaction(DB_DATABASE *db)
 {
-  return do_query((PGconn *)db->handle, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
+  return do_query(db, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
 }
 
 
@@ -984,7 +1045,7 @@ static int begin_transaction(DB_DATABASE *db)
 
 static int commit_transaction(DB_DATABASE *db)
 {
-  return do_query((PGconn *)db->handle, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
+  return do_query(db, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
 }
 
 
@@ -1003,7 +1064,7 @@ static int commit_transaction(DB_DATABASE *db)
 
 static int rollback_transaction(DB_DATABASE *db)
 {
-  return do_query((PGconn *)db->handle, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
+  return do_query(db, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
 }
 
 
@@ -1033,12 +1094,11 @@ static int table_init(DB_DATABASE *db, char *table, DB_INFO *info)
   char *qfield =
     "select pg_attribute.attname, pg_attribute.atttypid::int,pg_attribute.atttypmod "
       "from pg_class, pg_attribute "
-      "where pg_class.relname = lower('&1') "
+      "where pg_class.relname = '&1' "
         "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
         "and pg_attribute.attnum > 0 "
         "and pg_attribute.attrelid = pg_class.oid ";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int i, n;
   DB_FIELD *f;
@@ -1049,7 +1109,7 @@ static int table_init(DB_DATABASE *db, char *table, DB_INFO *info)
 
   /* Liste des champs */
 
-  if (do_query(conn, "Unable to get table fields: &1", &res, qfield, 1, table))
+  if (do_query(db, "Unable to get table fields: &1", &res, qfield, 1, table))
     return TRUE;
 
   info->nfield = n = PQntuples(res);
@@ -1059,7 +1119,7 @@ static int table_init(DB_DATABASE *db, char *table, DB_INFO *info)
     return TRUE;
   }
 
-  GB.Alloc((void **)&info->field, sizeof(DB_FIELD) * n);
+  GB.Alloc(POINTER(&info->field), sizeof(DB_FIELD) * n);
 
   for (i = 0; i < n; i++)
   {
@@ -1117,20 +1177,19 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
   const char *qindex =
     "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
       "from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
-        "where pg_cl.relname = lower('&1_pkey') AND pg_cl.oid = pg_ind.indexrelid "
+        "where pg_cl.relname = '&1_pkey' AND pg_cl.oid = pg_ind.indexrelid "
           "and (pg_cl.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
           "and pg_att2.attrelid = pg_ind.indexrelid "
           "and pg_att1.attrelid = pg_ind.indrelid "
           "and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
         "order by pg_att2.attnum";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int i, j, n;
 
   /* Index primaire */
 
-  if (do_query(conn, "Unable to get primary index: &1", &res, qindex, 1, table))
+  if (do_query(db, "Unable to get primary index: &1", &res, qindex, 1, table))
     return TRUE;
 
   n = info->nindex = PQntuples(res);
@@ -1142,13 +1201,13 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
     return TRUE;
   }
 
-  GB.Alloc((void **)&info->index, sizeof(int) * n);
+  GB.Alloc(POINTER(&info->index), sizeof(int) * n);
 
   for (i = 0; i < n; i++)
   {
     for (j = 0; j < info->nfield; j++)
     {
-      if (strcasecmp(info->field[j].name, PQgetvalue(res, i, 0)) == 0)
+      if (strcmp(info->field[j].name, PQgetvalue(res, i, 0)) == 0)
       {
         info->index[i] = j;
         /*printf("index[%d] = %d\n", i, j);*/
@@ -1197,7 +1256,7 @@ static int table_exist(DB_DATABASE *db, char *table)
 {
   const char *query73 =
     "select relname from pg_class where (relkind = 'r' or relkind = 'v') "
-    "and (relname = lower('&1')) "
+    "and (relname = '&1') "
     "and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
   /*const char *query74_1 =
@@ -1206,13 +1265,12 @@ static int table_exist(DB_DATABASE *db, char *table)
   const char *query74_2 =
     "select viewname from pg_views where schemaname <> 'information_schema' and viewname = lower('&1')";*/
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int exist;
 
   //if (1) //version < 70400)
   //{
-    if (do_query(conn, "Unable to check table: &1", &res, query73, 1, table))
+    if (do_query(db, "Unable to check table: &1", &res, query73, 1, table))
       return FALSE;
   /*}
   else
@@ -1253,18 +1311,17 @@ static int table_exist(DB_DATABASE *db, char *table)
 
 *****************************************************************************/
 
-static long table_list_73(DB_DATABASE *db, char ***tables)
+static int table_list_73(DB_DATABASE *db, char ***tables)
 {
   const char *query =
     "select relname from pg_class where (relkind = 'r' or relkind = 'v') "
     "and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int i;
   long count;
 
-  if (do_query(conn, "Unable to get tables: &1", &res, query, 0))
+  if (do_query(db, "Unable to get tables: &1", &res, query, 0))
     return -1;
 
   if (tables)
@@ -1283,7 +1340,7 @@ static long table_list_73(DB_DATABASE *db, char ***tables)
 }
 
 #if 0
-static long table_list_74(DB_DATABASE handle, char ***tables)
+static int table_list_74(DB_DATABASE handle, char ***tables)
 {
   const char *query1 =
     "select tablename from pg_tables where schemaname <> 'information_schema'";
@@ -1324,7 +1381,7 @@ static long table_list_74(DB_DATABASE handle, char ***tables)
 }
 #endif
 
-static long table_list(DB_DATABASE *db, char ***tables)
+static int table_list(DB_DATABASE *db, char ***tables)
 {
   //if (version < 70400)
     return table_list_73(db, tables);
@@ -1354,18 +1411,17 @@ static int table_primary_key(DB_DATABASE *db, char *table, char ***primary)
   const char *query =
     "select pg_att1.attname, pg_att1.atttypid::int, pg_cl.relname "
       "from pg_attribute pg_att1, pg_attribute pg_att2, pg_class pg_cl, pg_index pg_ind "
-        "where pg_cl.relname = lower('&1_pkey') AND pg_cl.oid = pg_ind.indexrelid "
+        "where pg_cl.relname = '&1_pkey' AND pg_cl.oid = pg_ind.indexrelid "
           "and (pg_cl.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
           "and pg_att2.attrelid = pg_ind.indexrelid "
           "and pg_att1.attrelid = pg_ind.indrelid "
           "and pg_att1.attnum = pg_ind.indkey[pg_att2.attnum-1] "
         "order by pg_att2.attnum";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int i;
 
-  if (do_query(conn, "Unable to get primary key: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to get primary key: &1", &res, query, 1, table))
     return TRUE;
 
   GB.NewArray(primary, sizeof(char *), PQntuples(res));
@@ -1397,16 +1453,15 @@ static int table_is_system(DB_DATABASE *db, char *table)
 {
   const char *query =
     "select 1 from pg_class where (relkind = 'r' or relkind = 'v') "
-    "and (relname = lower('&1')) "
+    "and (relname = '&1') "
     "and (relnamespace not in (select oid from pg_namespace where nspname <> 'public'))";
 
   //const char *query1 =
   //  "select 1 from pg_tables where tablename = lower('&1') and schemaname = 'public'";
 
   const char *query2 =
-    "select 1 from pg_views where viewname = lower('&1') and schemaname = 'public'";
+    "select 1 from pg_views where viewname = '&1' and schemaname = 'public'";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int exist;
 
@@ -1418,7 +1473,7 @@ static int table_is_system(DB_DATABASE *db, char *table)
         return FALSE;
   }*/
 
-  if (do_query(conn, "Unable to check table: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to check table: &1", &res, query, 1, table))
     return FALSE;
 
   exist = PQntuples(res) == 1;
@@ -1427,7 +1482,7 @@ static int table_is_system(DB_DATABASE *db, char *table)
   if (exist)
     return FALSE;
 
-  if (do_query(conn, "Unable to check table: &1", &res, query2, 1, table))
+  if (do_query(db, "Unable to check table: &1", &res, query2, 1, table))
     return FALSE;
 
   exist = PQntuples(res) == 1;
@@ -1457,7 +1512,7 @@ static int table_is_system(DB_DATABASE *db, char *table)
 static int table_delete(DB_DATABASE *db, char *table)
 {
   return
-    do_query((PGconn *)db->handle, "Unable to delete table: &1", NULL,
+    do_query(db, "Unable to delete table: &1", NULL,
       "drop table \"&1\"", 1, table);
 }
 
@@ -1480,7 +1535,6 @@ static int table_delete(DB_DATABASE *db, char *table)
 
 static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **primary, char *not_used)
 {
-  PGconn *conn = (PGconn *)db->handle;
   DB_FIELD *fp;
   int comma;
   char *type;
@@ -1500,9 +1554,9 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
     else
       comma = TRUE;
 
-    /*DB.Query.Add(QUOTE_STRING);*/
+    DB.Query.Add(QUOTE_STRING);
     DB.Query.Add(fp->name);
-    /*DB.Query.Add(QUOTE_STRING);*/
+    DB.Query.Add(QUOTE_STRING);
 
 		if (fp->type == DB_T_SERIAL)
 		{
@@ -1527,7 +1581,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
 						type = "TEXT";
 					else
 					{
-						sprintf(_buffer, "VARCHAR(%ld)", fp->length);
+						sprintf(_buffer, "VARCHAR(%d)", fp->length);
 						type = _buffer;
 					}
 
@@ -1568,7 +1622,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
 
   DB.Query.Add(" )");
 
-  return do_query(conn, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
 }
 
 
@@ -1589,16 +1643,16 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
 static int field_exist(DB_DATABASE *db, char *table, char *field)
 {
   const char *query = "select pg_attribute.attname from pg_class, pg_attribute "
-    "where pg_class.relname = lower('&1') "
+    "where pg_class.relname = '&1' "
     "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
-    "and pg_attribute.attname = lower('&2') "
+    "and pg_attribute.attname = '&2' "
     "and pg_attribute.attnum > 0 "
     "and pg_attribute.attrelid = pg_class.oid ";
 
   PGresult *res;
   int exist;
 
-  if (do_query((PGconn *)db->handle, "Unable to check field: &1", &res, query, 2, table, field))
+  if (do_query(db, "Unable to check field: &1", &res, query, 2, table, field))
     return FALSE;
 
   exist = PQntuples(res) == 1;
@@ -1626,10 +1680,10 @@ static int field_exist(DB_DATABASE *db, char *table, char *field)
 
 *****************************************************************************/
 
-static long field_list(DB_DATABASE *db, char *table, char ***fields)
+static int field_list(DB_DATABASE *db, char *table, char ***fields)
 {
   const char *query = "select pg_attribute.attname from pg_class, pg_attribute "
-    "where pg_class.relname = lower('&1') "
+    "where pg_class.relname = '&1' "
     "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_attribute.attnum > 0 "
     "and pg_attribute.attrelid = pg_class.oid";
@@ -1638,7 +1692,7 @@ static long field_list(DB_DATABASE *db, char *table, char ***fields)
   int i;
   long count;
 
-  if (do_query((PGconn *)db->handle, "Unable to get fields: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
     return -1;
 
   if (fields)
@@ -1679,9 +1733,9 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
     "pg_attribute.atttypmod, pg_attribute.attnotnull, pg_attrdef.adsrc "
     "from pg_class, pg_attribute "
     "left join pg_attrdef on (pg_attrdef.adrelid = pg_attribute.attrelid and pg_attrdef.adnum = pg_attribute.attnum) "
-    "where pg_class.relname = lower('&1') "
+    "where pg_class.relname = '&1' "
     "and (pg_class.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
-    "and pg_attribute.attname = lower('&2') "
+    "and pg_attribute.attname = '&2' "
     "and pg_attribute.attnum > 0 "
     "and pg_attribute.attrelid = pg_class.oid";
 
@@ -1690,7 +1744,7 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
   GB_VARIANT def;
   char *val;
 
-  if (do_query((PGconn *)db->handle, "Unable to get field info: &1", &res, query, 2, table, field))
+  if (do_query(db, "Unable to get field info: &1", &res, query, 2, table, field))
     return TRUE;
 
   if (PQntuples(res) != 1)
@@ -1741,7 +1795,7 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
 					default:
 
 					  DB.Query.Init();
-						if (!unquote(val, PQgetlength(res, 0, 4), DB.Query.AddLength))
+						if (!unquote_string(val, PQgetlength(res, 0, 4), DB.Query.AddLength))
 							val = DB.Query.Get();
 
 						conv_data(val, -1, &def.value, type);
@@ -1776,11 +1830,11 @@ static int index_exist(DB_DATABASE *db, char *table, char *index)
 {
   const char *query =
     "select pg_class.relname from pg_class, pg_index, pg_class pg_class2 "
-    "where pg_class2.relname = lower('&1') "
+    "where pg_class2.relname = '&1' "
     "and (pg_class2.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_index.indrelid = pg_class2.oid "
     "and pg_index.indexrelid = pg_class.oid "
-    "and pg_class.relname = lower('&2')";
+    "and pg_class.relname = '&2'";
 
   /*const char *query = "select relname from pg_class, pg_index "
     "where pg_class.relname = '&1' "
@@ -1789,7 +1843,7 @@ static int index_exist(DB_DATABASE *db, char *table, char *index)
   PGresult *res;
   int exist;
 
-  if (do_query((PGconn *)db->handle, "Unable to check index: &1", &res, query, 2, table, index))
+  if (do_query(db, "Unable to check index: &1", &res, query, 2, table, index))
     return FALSE;
 
   exist = PQntuples(res) == 1;
@@ -1817,10 +1871,10 @@ static int index_exist(DB_DATABASE *db, char *table, char *index)
 
 *****************************************************************************/
 
-static long index_list(DB_DATABASE *db, char *table, char ***indexes)
+static int index_list(DB_DATABASE *db, char *table, char ***indexes)
 {
   const char *query = "select pg_class.relname from pg_class, pg_index, pg_class pg_class2 "
-    "where pg_class2.relname = lower('&1') "
+    "where pg_class2.relname = '&1' "
     "and (pg_class2.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_index.indrelid = pg_class2.oid "
     "and pg_index.indexrelid = pg_class.oid ";
@@ -1829,7 +1883,7 @@ static long index_list(DB_DATABASE *db, char *table, char ***indexes)
   int i;
   long count;
 
-  if (do_query((PGconn *)db->handle, "Unable to get indexes: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to get indexes: &1", &res, query, 1, table))
     return TRUE;
 
   if (indexes)
@@ -1867,11 +1921,11 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
 {
   const char *query =
     "select indisunique, indisprimary, indexrelid from pg_class, pg_index, pg_class pg_class2 "
-    "where pg_class2.relname = lower('&1') "
+    "where pg_class2.relname = '&1' "
     "and (pg_class2.relnamespace not in (select oid from pg_namespace where nspname = 'information_schema')) "
     "and pg_index.indrelid = pg_class2.oid "
     "and pg_index.indexrelid = pg_class.oid "
-    "and pg_class.relname = lower('&2')";
+    "and pg_class.relname = '&2'";
 
   const char *query_field =
     "select pg_att1.attname "
@@ -1886,7 +1940,7 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
   char indexrelid[16];
   int i;
 
-  if (do_query((PGconn *)db->handle, "Unable to get index info: &1", &res, query, 2, table, index))
+  if (do_query(db, "Unable to get index info: &1", &res, query, 2, table, index))
     return TRUE;
 
   if (PQntuples(res) != 1)
@@ -1903,7 +1957,7 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
 
   PQclear(res);
 
-  if (do_query((PGconn *)db->handle, "Unable to get index info: &1", &res, query_field, 1, indexrelid))
+  if (do_query(db, "Unable to get index info: &1", &res, query_field, 1, indexrelid))
     return TRUE;
 
   DB.Query.Init();
@@ -1941,7 +1995,7 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
 static int index_delete(DB_DATABASE *db, char *table, char *index)
 {
   return
-    do_query((PGconn *)db->handle, "Unable to delete index: &1", NULL,
+    do_query(db, "Unable to delete index: &1", NULL,
       "drop index \"&1\"", 1, index);
 }
 
@@ -1977,7 +2031,7 @@ static int index_create(DB_DATABASE *db, char *table, char *index, DB_INDEX *inf
   DB.Query.Add(info->fields);
   DB.Query.Add(" )");
 
-  return do_query((PGconn *)db->handle, "Cannot create index: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create index: &1", NULL, DB.Query.Get(), 0);
 }
 
 
@@ -1998,13 +2052,12 @@ static int database_exist(DB_DATABASE *db, char *name)
 {
   const char *query =
     "select datname from pg_database where (datallowconn = 't') "
-    "and (datname = lower('&1'))";
+    "and (datname = '&1')";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int exist;
 
-  if (do_query(conn, "Unable to check database: &1", &res, query, 1, name))
+  if (do_query(db, "Unable to check database: &1", &res, query, 1, name))
     return FALSE;
 
   exist = PQntuples(res) == 1;
@@ -2032,17 +2085,16 @@ static int database_exist(DB_DATABASE *db, char *name)
 
 *****************************************************************************/
 
-static long database_list(DB_DATABASE *db, char ***databases)
+static int database_list(DB_DATABASE *db, char ***databases)
 {
   const char *query =
     "select datname from pg_database where datallowconn and datname <> 'template1'";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int i;
   long count;
 
-  if (do_query(conn, "Unable to get databases: &1", &res, query, 0))
+  if (do_query(db, "Unable to get databases: &1", &res, query, 0))
     return -1;
 
   if (databases)
@@ -2079,13 +2131,13 @@ static int database_is_system(DB_DATABASE *db, char *name)
 {
   const char *query =
     "select datname from pg_database where datallowconn "
-    "and (datname = lower('&1')) and datistemplate";
+    "and (datname = '&1')) and datistemplate";
+    //"and (datname = lower('&1')) and datistemplate";
 
-  PGconn *conn = (PGconn *)db->handle;
   PGresult *res;
   int system;
 
-  if (do_query(conn, "Unable to check database: &1", &res, query, 1, name))
+  if (do_query(db, "Unable to check database: &1", &res, query, 1, name))
     return TRUE;
 
   system = PQntuples(res) == 1;
@@ -2136,7 +2188,7 @@ static char *table_type(DB_DATABASE *db, char *table, char *newtype)
 static int database_delete(DB_DATABASE *db, char *name)
 {
   return
-    do_query((PGconn *)db->handle, "Unable to delete database: &1", NULL,
+    do_query(db, "Unable to delete database: &1", NULL,
       "drop database \"&1\"", 1, name);
 }
 
@@ -2158,7 +2210,7 @@ static int database_delete(DB_DATABASE *db, char *name)
 static int database_create(DB_DATABASE *db, char *name)
 {
   return
-    do_query((PGconn *)db->handle, "Unable to create database: &1", NULL,
+    do_query(db, "Unable to create database: &1", NULL,
       "create database \"&1\"", 1, name);
 }
 
@@ -2184,7 +2236,7 @@ static int user_exist(DB_DATABASE *db, char *name)
   PGresult *res;
   int exist;
 
-  if (do_query((PGconn *)db->handle, "Unable to check user: &1", &res, query, 1, name))
+  if (do_query(db, "Unable to check user: &1", &res, query, 1, name))
     return FALSE;
 
   exist = PQntuples(res) == 1;
@@ -2211,7 +2263,7 @@ static int user_exist(DB_DATABASE *db, char *name)
 
 *****************************************************************************/
 
-static long user_list(DB_DATABASE *db, char ***users)
+static int user_list(DB_DATABASE *db, char ***users)
 {
   const char *query = "select usename from pg_user ";
 
@@ -2219,7 +2271,7 @@ static long user_list(DB_DATABASE *db, char ***users)
   int i;
   long count;
 
-  if (do_query((PGconn *)db->handle, "Unable to get users: &1", &res, query, 0))
+  if (do_query(db, "Unable to get users: &1", &res, query, 0))
     return -1;
 
   if (users)
@@ -2264,7 +2316,7 @@ static int user_info(DB_DATABASE *db, char *name, DB_USER *info)
 
   PGresult *res;
 
-  if (do_query((PGconn *)db->handle, "Unable to get user info: &1", &res, query, 1, name))
+  if (do_query(db, "Unable to get user info: &1", &res, query, 1, name))
     return TRUE;
 
   if (PQntuples(res) != 1)
@@ -2279,7 +2331,7 @@ static int user_info(DB_DATABASE *db, char *name, DB_USER *info)
 
   PQclear(res);
 
-  if (!do_query((PGconn *)db->handle, NULL, &res, query_passwd, 1, name))
+  if (!do_query(db, NULL, &res, query_passwd, 1, name))
   {
     if (*PQgetvalue(res, 0, 0))
       GB.NewString(&info->password, "***", 0);
@@ -2306,7 +2358,7 @@ static int user_info(DB_DATABASE *db, char *name, DB_USER *info)
 static int user_delete(DB_DATABASE *db, char *name)
 {
   return
-    do_query((PGconn *)db->handle, "Unable to delete user: &1", NULL,
+    do_query(db, "Unable to delete user: &1", NULL,
       "drop user \"&1\"", 1, name);
 }
 
@@ -2348,7 +2400,7 @@ static int user_create(DB_DATABASE *db, char *name, DB_USER *info)
   }
 
   /*_print_query = TRUE;*/
-  return do_query((PGconn *)db->handle, "Cannot create user: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create user: &1", NULL, DB.Query.Get(), 0);
 }
 
 
@@ -2378,7 +2430,7 @@ static int user_set_password(DB_DATABASE *db, char *name, char *password)
   DB.Query.Add("'");
 
   return
-    do_query((PGconn *)db->handle, "Cannot change user password: &1",
+    do_query(db, "Cannot change user password: &1",
       NULL, DB.Query.Get(), 0);
 }
 
@@ -2388,71 +2440,7 @@ static int user_set_password(DB_DATABASE *db, char *name, char *password)
 
 *****************************************************************************/
 
-static DB_DRIVER _driver =
-{
-  "postgresql",
-  (void *)open_database,
-  (void *)close_database,
-  (void *)format_value,
-  (void *)format_blob,
-  (void *)exec_query,
-  (void *)begin_transaction,
-  (void *)commit_transaction,
-  (void *)rollback_transaction,
-  (void *)get_quote,
-  {
-    (void *)query_init,
-    (void *)query_fill,
-   	(void *)blob_read,
-    (void *)query_release,
-    {
-      (void *)field_type,
-      (void *)field_name,
-      (void *)field_index,
-      (void *)field_length,
-    },
-  },
-  {
-    (void *)field_exist,
-    (void *)field_list,
-    (void *)field_info,
-  },
-  {
-    (void *)table_init,
-    (void *)table_index,
-    (void *)table_release,
-    (void *)table_exist,
-    (void *)table_list,
-    (void *)table_primary_key,
-    (void *)table_is_system,
-    (void *)table_type,
-    (void *)table_delete,
-    (void *)table_create,
-  },
-  {
-    (void *)index_exist,
-    (void *)index_list,
-    (void *)index_info,
-    (void *)index_delete,
-    (void *)index_create,
-  },
-  {
-    (void *)database_exist,
-    (void *)database_list,
-    (void *)database_is_system,
-    (void *)database_delete,
-    (void *)database_create,
-  },
-  {
-    (void *)user_exist,
-    (void *)user_list,
-    (void *)user_info,
-    (void *)user_delete,
-    (void *)user_create,
-    (void *)user_set_password
-  }
-};
-
+DECLARE_DRIVER(_driver, "postgresql");
 
 /*****************************************************************************
 
