@@ -57,70 +57,7 @@ static int _print_query = FALSE;
 
 *****************************************************************************/
 
-static DB_DRIVER _driver =
-{
-  "sqlite2",
-  open_database,
-  close_database,
-  format_value,
-  format_blob,
-  exec_query,
-  begin_transaction,
-  commit_transaction,
-  rollback_transaction,
-  get_quote,
-  {
-    query_init,
-    query_fill,
-    blob_read,
-    query_release,
-    {
-      field_type,
-      field_name,
-      field_index,
-      field_length,
-    },
-  },
-  {
-    field_exist,
-    field_list,
-    field_info,
-  },
-  {
-    table_init,
-    table_index,
-    table_release,
-    table_exist,
-    table_list,
-    table_primary_key,
-    table_is_system,
-    table_type,
-    table_delete,
-    table_create,
-  },
-  {
-    index_exist,
-    index_list,
-    index_info,
-    index_delete,
-    index_create,
-  },
-  {
-    database_exist,
-    database_list,
-    database_is_system,
-    database_delete,
-    database_create,
-  },
-  {
-    user_exist,
-    user_list,
-    user_info,
-    user_delete,
-    user_create,
-    user_set_password
-  }
-};
+DECLARE_DRIVER(_driver, "sqlite2");
 
 
 /* Internal function to convert a database type into a Gambas type */
@@ -213,71 +150,79 @@ static void conv_data(char *data, GB_VARIANT_VALUE *val, fType type)
 
       break;
 
-    case ft_Date:
+		case ft_Date:
 
-      memset(&date, 0, sizeof(date));
+			memset(&date, 0, sizeof(date));
 
-          switch(strlen(data))
-          {
-            case 14:
-              sscanf(data, "%4hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
-              date.sec = (short)sec;
-              date.msec = (short)(sec * 1000 + 0.5);
-              break;
-            case 12:
-              sscanf(data, "%2hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
-              date.sec = (short)sec;
-              date.msec = (short)(sec * 1000 + 0.5);
-              break;
-            case 10:
-	      if (sscanf(data, "%4hu-%2hu-%2hu", &date.year, &date.month,
-				  &date.day) != 3){
-	      	if (sscanf(data, "%4hu/%2hu/%2hu", &date.year, &date.month,
-				  &date.day) != 3){
-                   if (sscanf(data, "%4hu:%2hu:%lf", &date.hour, &date.min,
-			       &sec) == 3){
-                       date.sec = (short)sec;
-                       date.msec = (short)(sec * 1000 + 0.5);
-		   }
-		   else {
-	              sscanf(data, "%2hu%2hu%2hu%2hu%2hu", &date.year,
-	              &date.month, &date.day, &date.hour, &date.min );
-		   }
-	      	}
-	      }
+			switch (strlen(data))
+			{
+				case 14:
+					sscanf(data, "%4hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month,
+								 &date.day, &date.hour, &date.min, &sec);
+					date.sec = (short) sec;
+					date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
+					break;
+				case 12:
+					sscanf(data, "%2hu%2hu%2hu%2hu%2hu%lf", &date.year, &date.month,
+								 &date.day, &date.hour, &date.min, &sec);
+					date.sec = (short) sec;
+					date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
+					break;
+				case 10:
+					if (sscanf(data, "%4hu-%2hu-%2hu", &date.year, &date.month,
+										 &date.day) != 3)
+					{
+						if (sscanf(data, "%4hu/%2hu/%2hu", &date.year, &date.month,
+											 &date.day) != 3)
+						{
+							if (sscanf(data, "%4hu:%2hu:%lf", &date.hour, &date.min,
+												 &sec) == 3)
+							{
+								date.sec = (short) sec;
+								date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
+							}
+							else
+							{
+								sscanf(data, "%2hu%2hu%2hu%2hu%2hu", &date.year,
+											 &date.month, &date.day, &date.hour, &date.min);
+							}
+						}
+					}
 
-              break;
-            case 8:
-              if (sscanf(data, "%4hu%2hu%2hu", &date.year, &date.month,
-				      &date.day) != 3){
-                  sscanf(data, "%2hu/%2hu/%2hu", &date.year, &date.month,
-				 &date.day);
-	      }
-              break;
-            case 6:
-              sscanf(data, "%2hu%2hu%2hu", &date.year, &date.month, &date.day);
-              break;
-            case 4:
-              sscanf(data, "%2hu%2hu", &date.year, &date.month);
-              break;
-            case 2:
-              sscanf(data, "%2hu", &date.year);
-              break;
-	    default:
-              sscanf(data, "%4hu-%2hu-%2hu %2hu:%2hu:%lf", &date.year, &date.month, &date.day, &date.hour, &date.min, &sec);
-               date.sec = (short)sec;
-               date.msec = (short)(sec * 1000 + 0.5);
-          }
-          if (date.year < 100)
-              date.year += 1900;
+					break;
+				case 8:
+					if (sscanf(data, "%4hu%2hu%2hu", &date.year, &date.month,
+										 &date.day) != 3)
+					{
+						sscanf(data, "%2hu/%2hu/%2hu", &date.year, &date.month,
+									 &date.day);
+					}
+					break;
+				case 6:
+					sscanf(data, "%2hu%2hu%2hu", &date.year, &date.month, &date.day);
+					break;
+				case 4:
+					sscanf(data, "%2hu%2hu", &date.year, &date.month);
+					break;
+				case 2:
+					sscanf(data, "%2hu", &date.year);
+					break;
+				default:
+					sscanf(data, "%4hu-%2hu-%2hu %2hu:%2hu:%lf", &date.year,
+								 &date.month, &date.day, &date.hour, &date.min, &sec);
+					date.sec = (short) sec;
+					date.msec = (short) ((sec - date.sec) * 1000 + 0.5);
+			}
+			if (date.year < 100)
+				date.year += 1900;
 
-      GB.MakeDate(&date, (GB_DATE *)&conv);
+			GB.MakeDate(&date, (GB_DATE *) & conv);
 
-      val->_date.type = GB_T_DATE;
-      val->_date.date = ((GB_DATE *)&conv)->value.date;
-      val->_date.time = ((GB_DATE *)&conv)->value.time;
+			val->_date.type = GB_T_DATE;
+			val->_date.date = ((GB_DATE *) & conv)->value.date;
+			val->_date.time = ((GB_DATE *) & conv)->value.time;
 
-      break;
+			break;
 
     case ft_String:
     case ft_WideString:
@@ -297,21 +242,28 @@ static void conv_data(char *data, GB_VARIANT_VALUE *val, fType type)
 
 static char *query_param[3];
 
-static void query_get_param(int index, char **str, long *len)
+static void query_get_param(int index, char **str, int *len, char quote)
 {
-  if (index > 3)
-    return;
+	if (index > 3)
+		return;
 
-  index--;
-  *str = query_param[index];
-  *len = strlen(*str);
+	index--;
+	*str = query_param[index];
+	*len = strlen(*str);
+
+	if (quote == '\'')
+	{
+		*str = DB.QuoteString(*str, *len, quote);
+		*len = GB.StringLength(*str);
+	}
 }
 
 /* Internal function to run a query */
 
-static int do_query(SqliteDatabase *conn, const char *error, Dataset **pres,
+static int do_query(DB_DATABASE *db, const char *error, Dataset **pres,
                     const char *qtemp, int nsubst, ...)
 {
+	SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   va_list args;
   int i;
   const char *query;
@@ -326,7 +278,7 @@ static int do_query(SqliteDatabase *conn, const char *error, Dataset **pres,
     for (i = 0; i < nsubst; i++)
       query_param[i] = va_arg(args, char *);
 
-    query = GB.SubstString(qtemp, 0, query_get_param);
+    query = DB.SubstString(qtemp, 0, query_get_param);
   }
   else
     query = qtemp;
@@ -337,7 +289,7 @@ static int do_query(SqliteDatabase *conn, const char *error, Dataset **pres,
   }
 
   if (DB.IsDebug())
-    fprintf(stderr, "sqlite: %p: %s\n", conn, query);
+    fprintf(stderr, "sqlite2: %p: %s\n", conn, query);
 
   if (strncasecmp("select",query,6) == 0){
 
@@ -365,6 +317,14 @@ static int do_query(SqliteDatabase *conn, const char *error, Dataset **pres,
         }
   }
 
+	if (!pres)
+		res->close();
+  
+  if (ret)
+  	db->error = conn->lastError();
+  else
+  	db->error = 0;
+  
   return ret;
 }
 
@@ -460,7 +420,8 @@ char *GetDatabaseHome()
 {
   char *env = NULL;
   char *dbhome = NULL;
-  GB.Alloc((void **)&dbhome, MAX_PATH);
+  
+  GB.Alloc(POINTER(&dbhome), MAX_PATH);
 
   /* Check for Environment variable */
 
@@ -550,10 +511,10 @@ int WalkDirectory( char *dir, char ***databases ){
 }
 
 /* Internal function to check database version number */
-long db_version()
+int db_version()
 {
   //Check db version
-  long dbversion =0;
+  int dbversion =0;
   unsigned int verMain, verMajor, verMinor;
   sscanf(sqlite_version,"%2u.%2u.%2u", &verMain, &verMajor, &verMinor);
   dbversion = ((verMain * 10000) + (verMajor * 100) + verMinor);
@@ -623,6 +584,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
   else
   {
     GB.Error("Unable to locate database: &1", name);
+    delete conn;
     return TRUE;
   }
 
@@ -633,6 +595,7 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
   {
     GB.Error("Cannot open database: &1", conn->getErrorMsg());
     conn->disconnect();
+    delete conn;
     return TRUE;
   }
 
@@ -669,7 +632,10 @@ static void close_database(DB_DATABASE *db)
   SqliteDatabase *conn = (SqliteDatabase *)db->handle;
 
   if (conn)
+  {
     conn->disconnect();
+    delete conn;
+   }
 }
 
 
@@ -770,7 +736,7 @@ static void format_blob(DB_BLOB *blob, DB_FORMAT_CALLBACK add)
 
 static int exec_query(DB_DATABASE *db, char *query, DB_RESULT *result, char *err)
 {
-  return do_query((SqliteDatabase *)db->handle, err, (Dataset **)result, query, 0);
+  return do_query(db, err, (Dataset **)result, query, 0);
 }
 
 
@@ -789,7 +755,7 @@ static int exec_query(DB_DATABASE *db, char *query, DB_RESULT *result, char *err
 
 *****************************************************************************/
 
-static void query_init(DB_RESULT result, DB_INFO *info, long *count)
+static void query_init(DB_RESULT result, DB_INFO *info, int *count)
 {
   Dataset *res = (Dataset *)result;
 
@@ -828,6 +794,7 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 
   Fill a result buffer with the value of each field of a record.
 
+  <db> is the database handle, as returned by open_database()
   <result> is the handle of the result.
   <pos> is the index of the record in the result.
   <buffer> points to an array having one element for each field in the
@@ -839,7 +806,8 @@ static void query_release(DB_RESULT result, DB_INFO *info)
 
 *****************************************************************************/
 
-static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next)
+#if 0
+static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int next)
 {
   Dataset *res = (Dataset *)result;
   int i;
@@ -869,6 +837,43 @@ static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE *buffer, int n
 
   return FALSE;
 }
+#endif
+
+static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VALUE * buffer, int next)
+{
+	Dataset *res = (Dataset *) result;
+	int i;
+	char *data;
+	GB_VARIANT value;
+
+	if (!next)
+		res->seek(pos);
+	else
+		res->next();
+
+	for (i = 0; i < res->fieldCount(); i++)
+	{
+		//GB.NewString( &data, res->fv(res->fieldName(i)).get_asString().data(),0);
+		if (res->fv(i).get_isNull())
+			data = NULL;
+		else
+			data = (char *) res->fv(i).get_asString().data();
+			
+		//fprintf(stderr, "query_fill: %d.%d %s\n", pos, i, data);
+
+		value.type = GB_T_VARIANT;
+		value.value._object.type = GB_T_NULL;
+
+		//if (field->type != FIELD_TYPE_NULL)
+		if (data)
+			conv_data(data, &value.value, (fType) res->fieldType(i));
+
+		//GB.FreeString(&data);
+		GB.StoreVariant(&value, &buffer[i]);
+	}
+
+	return FALSE;
+}
 
 
 /*****************************************************************************
@@ -995,7 +1000,7 @@ static int field_length(DB_RESULT result, int field)
 
 static int begin_transaction(DB_DATABASE *db)
 {
-  return do_query((SqliteDatabase *)db->handle, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
+  return do_query(db, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
 }
 
 
@@ -1014,7 +1019,7 @@ static int begin_transaction(DB_DATABASE *db)
 
 static int commit_transaction(DB_DATABASE *db)
 {
-  return do_query((SqliteDatabase *)db->handle, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
+  return do_query(db, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
 }
 
 
@@ -1033,7 +1038,7 @@ static int commit_transaction(DB_DATABASE *db)
 
 static int rollback_transaction(DB_DATABASE *db)
 {
-  return do_query((SqliteDatabase *)db->handle, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
+  return do_query(db, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
 }
 
 /*****************************************************************************
@@ -1061,7 +1066,6 @@ static int table_init(DB_DATABASE *db, char *table, DB_INFO *info)
 {
   char *qfield = "PRAGMA table_info('&1')";
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
   int i, n;
   DB_FIELD *f;
@@ -1073,7 +1077,7 @@ static int table_init(DB_DATABASE *db, char *table, DB_INFO *info)
 
   /* Liste des champs */
 
-  if (do_query(conn, "Unable to get table fields: &1", &res, qfield, 1, table))
+  if (do_query(db, "Unable to get table fields: &1", &res, qfield, 1, table))
 	                    return TRUE;
 
   result_set* r = (result_set*) res->getExecRes();
@@ -1084,7 +1088,7 @@ static int table_init(DB_DATABASE *db, char *table, DB_INFO *info)
     return TRUE;
   }
 
-  GB.Alloc((void **)&info->field, sizeof(DB_FIELD) * n);
+  GB.Alloc(POINTER(&info->field), sizeof(DB_FIELD) * n);
 
 
   for (i = 0; i < n; i++)
@@ -1135,14 +1139,13 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
   char *qindex1 = "PRAGMA index_list('&1')";
   char *qindex2 = "PRAGMA index_info('&1')";
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
   char *sql = NULL;
   int n = 0;
 
   /* Index primaire */
 
-  if (do_query(conn, "Unable to get primary index: &1", &res, qindex1, 1, table))
+  if (do_query(db, "Unable to get primary index: &1", &res, qindex1, 1, table))
 	  return TRUE;
 
   result_set* r = (result_set*) res->getExecRes();
@@ -1161,7 +1164,7 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
        GB.NewString( &sql, r->records[i][1].get_asString().data(), 0);
        res->close();
 
-       if (do_query(conn, "Unable to get information on primary index: &1", &res, qindex2, 1, sql)){
+       if (do_query(db, "Unable to get information on primary index: &1", &res, qindex2, 1, sql)){
            res->close();
            GB.FreeString(&sql);
            return TRUE;
@@ -1170,7 +1173,7 @@ static int table_index(DB_DATABASE *db, char *table, DB_INFO *info)
 
        r = (result_set*) res->getExecRes();
        info->nindex = r->records.size();
-       GB.Alloc((void **)&info->index, sizeof(int) * info->nindex);
+       GB.Alloc(POINTER(&info->index), sizeof(int) * info->nindex);
 
        for (i = 0; i < info->nindex; i++){
            info->index[i] = r->records[i][1].get_asShort();
@@ -1226,11 +1229,10 @@ static int table_exist(DB_DATABASE *db, char *table)
 	  return TRUE;
   }
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
   int exist;
 
-  if (do_query(conn, "Unable to check table: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to check table: &1", &res, query, 1, table))
 	         return FALSE;
 
   exist = res->num_rows();
@@ -1254,19 +1256,18 @@ static int table_exist(DB_DATABASE *db, char *table)
 
 *****************************************************************************/
 
-static long table_list(DB_DATABASE *db, char ***tables)
+static int table_list(DB_DATABASE *db, char ***tables)
 {
 	  const char *query =
 	   "select tbl_name from "
 	   "( select tbl_name from sqlite_master where type = 'table' union "
 	   "   select tbl_name from sqlite_temp_master where type = 'table')";
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
-  long rows;
+  int rows;
   int i = 0;
 
-  if (do_query(conn, "Unable to get tables: &1", &res, query, 0))
+  if (do_query(db, "Unable to get tables: &1", &res, query, 0))
         return -1;
 
   rows = res->num_rows();
@@ -1307,12 +1308,11 @@ static int table_primary_key(DB_DATABASE *db, char *table, char ***primary)
   char *qindex1 = "PRAGMA index_list('&1')";
   char *qindex2 = "PRAGMA index_info('&1')";
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
   int i, n;
   char *sql;
 
-  if (do_query(conn, "Unable to get primary key: &1", &res, qindex1, 1, table))
+  if (do_query(db, "Unable to get primary key: &1", &res, qindex1, 1, table))
     return TRUE;
 
   GB.NewArray(primary, sizeof(char *), 0);
@@ -1331,7 +1331,7 @@ static int table_primary_key(DB_DATABASE *db, char *table, char ***primary)
        GB.NewString( &sql, r->records[i][1].get_asString().data(), 0);
        res->close();
 
-       if (do_query(conn, "Unable to get primary key: &1", &res, qindex2, 1, sql)){
+       if (do_query(db, "Unable to get primary key: &1", &res, qindex2, 1, sql)){
            res->close();
            GB.FreeString(&sql);
            return TRUE;
@@ -1420,7 +1420,7 @@ static char *table_type(DB_DATABASE *db, char *table, char *type)
 static int table_delete(DB_DATABASE *db, char *table)
 {
   return
-    do_query((SqliteDatabase *)db->handle, "Unable to delete table: &1", NULL,
+    do_query(db, "Unable to delete table: &1", NULL,
       "drop table '&1'", 1, table);
 }
 
@@ -1442,7 +1442,6 @@ static int table_delete(DB_DATABASE *db, char *table)
 
 static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **primary, char *not_used)
 {
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   DB_FIELD *fp;
   int comma;
   char *type;
@@ -1464,7 +1463,9 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
     else
       comma = TRUE;
 
+	  DB.Query.Add(QUOTE_STRING);
     DB.Query.Add(fp->name);
+  	DB.Query.Add(QUOTE_STRING);
 
     switch (fp->type)
     {
@@ -1479,7 +1480,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
           type = "TEXT";
         else
         {
-          sprintf(_buffer, "VARCHAR(%ld)", fp->length);
+          sprintf(_buffer, "VARCHAR(%d)", fp->length);
           type = _buffer;
         }
 
@@ -1519,7 +1520,7 @@ static int table_create(DB_DATABASE *db, char *table, DB_FIELD *fields, char **p
 
   DB.Query.Add(" )");
 
-  return do_query(conn, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
 }
 
 /*****************************************************************************
@@ -1540,12 +1541,11 @@ static int field_exist(DB_DATABASE *db, char *table, char *field)
 {
   const char *query = "PRAGMA table_info('&1')";
 
-  long i, n;
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
+  int i, n;
   Dataset *res;
   int exist = 0;
 
-  if (do_query(conn, "Unable to find field: &1.&2", &res, query, 2, table, field)){
+  if (do_query(db, "Unable to find field: &1.&2", &res, query, 2, table, field)){
     return FALSE;
   }
 
@@ -1579,15 +1579,14 @@ static int field_exist(DB_DATABASE *db, char *table, char *field)
 
 *****************************************************************************/
 
-static long field_list(DB_DATABASE *db, char *table, char ***fields)
+static int field_list(DB_DATABASE *db, char *table, char ***fields)
 {
   const char *query = "PRAGMA table_info('&1')";
 
-  long i, n;
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
+  int i, n;
   Dataset *res;
 
-  if (do_query(conn, "Unable to get fields: &1", &res, query, 1, table)){
+  if (do_query(db, "Unable to get fields: &1", &res, query, 1, table)){
        return -1;
   }
 
@@ -1632,14 +1631,13 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
   Dataset *res;
   GB_VARIANT def;
   char *val;
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   char *_fieldName = NULL;
   char *_fieldType = NULL;
   char *_defaultValue = NULL;
   bool _fieldNotNull = FALSE;
   int i, n;
 
-  if (do_query( conn, "Unable to get fields: &1", &res, query,1, table))
+  if (do_query(db, "Unable to get fields: &1", &res, query,1, table))
   {
      return TRUE;
   }
@@ -1662,7 +1660,7 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
        }
   }
 
-  if (strcmp(_fieldName, field))
+  if (i >= n)
   {
       GB.Error("Unable to find field &1.&2", table, field);
       return TRUE;
@@ -1674,7 +1672,7 @@ static int field_info(DB_DATABASE *db, char *table, char *field, DB_FIELD *info)
 
   info->def._object.type = GB_T_NULL;
 
-  if ( _fieldNotNull == TRUE )
+  if ( _fieldNotNull)
   {
     def.type = GB_T_VARIANT;
     def.value._object.type = GB_T_NULL;
@@ -1715,11 +1713,10 @@ static int index_exist(DB_DATABASE *db, char *table, char *index)
 	  " name = '&2' ) "
 	  "where tbl_name = '&1'";
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
   int exist;
 
-  if (do_query(conn, "Unable to check table: &1", &res, query, 2, table, index))
+  if (do_query(db, "Unable to check table: &1", &res, query, 2, table, index))
 	         return FALSE;
 
   exist = res->num_rows();
@@ -1744,7 +1741,7 @@ static int index_exist(DB_DATABASE *db, char *table, char *index)
 
 *****************************************************************************/
 
-static long index_list(DB_DATABASE *db, char *table, char ***indexes)
+static int index_list(DB_DATABASE *db, char *table, char ***indexes)
 {
 
   const char *query =
@@ -1753,11 +1750,10 @@ static long index_list(DB_DATABASE *db, char *table, char ***indexes)
    " union select name from sqlite_temp_master where type = 'index' and "
    " tbl_name = '&1')";
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
   int no_indexes, i = 0;
 
-  if (do_query(conn, "Unable to get tables: &1", &res, query, 1, table))
+  if (do_query(db, "Unable to get tables: &1", &res, query, 1, table))
         return -1;
 
   no_indexes = res->num_rows();
@@ -1798,11 +1794,10 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
   char *qindex1 = "PRAGMA index_list('&1')";
   char *qindex2 = "PRAGMA index_info('&1')";
 
-  SqliteDatabase *conn = (SqliteDatabase *)db->handle;
   Dataset *res;
   int i, j, n;
 
-  if (do_query(conn, "Unable to get index info for table: &1", &res, qindex1, 1, table))
+  if (do_query(db, "Unable to get index info for table: &1", &res, qindex1, 1, table))
     return TRUE;
 
   result_set* r = (result_set*) res->getExecRes();
@@ -1834,7 +1829,7 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
 
   DB.Query.Init();
 
-  if (do_query(conn, "Unable to get index info for : &1", &res, qindex2, 1, index)){
+  if (do_query(db, "Unable to get index info for : &1", &res, qindex2, 1, index)){
     res->close();
     return TRUE;
   }
@@ -1877,7 +1872,7 @@ static int index_info(DB_DATABASE *db, char *table, char *index, DB_INDEX *info)
 static int index_delete(DB_DATABASE *db, char *table, char *index)
 {
   return
-    do_query((SqliteDatabase *)db->handle, "Unable to delete index: &1", NULL,
+    do_query(db, "Unable to delete index: &1", NULL,
       "drop index '&1'", 1, index);
 }
 
@@ -1904,15 +1899,15 @@ static int index_create(DB_DATABASE *db, char *table, char *index, DB_INDEX *inf
   DB.Query.Add("CREATE ");
   if (info->unique)
     DB.Query.Add("UNIQUE ");
-  DB.Query.Add("INDEX ");
+  DB.Query.Add("INDEX '");
   DB.Query.Add(index);
-  DB.Query.Add(" ON ");
+  DB.Query.Add("' ON ");
   DB.Query.Add(table);
   DB.Query.Add(" ( ");
   DB.Query.Add(info->fields);
   DB.Query.Add(" )");
 
-  return do_query((SqliteDatabase *)db->handle, "Cannot create index: &1", NULL, DB.Query.Get(), 0);
+  return do_query(db, "Cannot create index: &1", NULL, DB.Query.Get(), 0);
 }
 
 /*****************************************************************************
@@ -1964,7 +1959,7 @@ static int database_exist(DB_DATABASE *db, char *name)
  *   areas are walked through.
  ******************************************************************************/
 
-static long database_list(DB_DATABASE *db, char ***databases)
+static int database_list(DB_DATABASE *db, char ***databases)
 {
   char *dbhome;
 
@@ -1981,7 +1976,7 @@ static long database_list(DB_DATABASE *db, char ***databases)
   if (dbhome){
       //GB.Error("Unable to get databases: &1", "Can't find current directory");
       WalkDirectory( dbhome, databases );
-      GB.Free((void **)&dbhome);
+      GB.Free(POINTER(&dbhome));
   }
 
   /*if (getcwd(cwd, MAX_PATH) != NULL){
@@ -2096,7 +2091,7 @@ static int database_create(DB_DATABASE *db, char *name)
     dir = GetDatabaseHome();
     mkdir(dir, S_IRWXU);
     GB.NewString(&fullpath, dir, 0);
-    GB.Free((void **)&dir);
+    GB.Free(POINTER(&dir));
   }
 
   if (fullpath[strlen(fullpath) - 1] != '/')
@@ -2116,10 +2111,13 @@ _CREATE_DATABASE:
   }
 
   //Create and remove a table to initialize database
-  if (!do_query(&conn2, "Unable to initialise database", NULL, "CREATE TABLE GAMBAS (FIELD1 TEXT)", 0))
-    do_query(&conn2, NULL, NULL, "DROP TABLE GAMBAS",0);
+  db->handle = &conn2;
+  if (!do_query(db, "Unable to initialise database", NULL, "CREATE TABLE GAMBAS (FIELD1 TEXT)", 0))
+    do_query(db, NULL, NULL, "DROP TABLE GAMBAS",0);
 
   conn2.disconnect();
+  db->handle = conn;
+  
   return FALSE;
 }
 
@@ -2244,7 +2242,7 @@ static int user_exist(DB_DATABASE *db, char *name)
  ******************************************************************************/
 
 
-static long user_list(DB_DATABASE *db, char ***users)
+static int user_list(DB_DATABASE *db, char ***users)
 {
 	//Should we use GB.HashTable.New etc. to ensure
 	//duplicates are not reported back, then transfer
