@@ -57,70 +57,7 @@ static int _print_query = FALSE;
 
 *****************************************************************************/
 
-static DB_DRIVER _driver = {
-	"sqlite3",
-	open_database,
-	close_database,
-	format_value,
-	format_blob,
-	exec_query,
-	begin_transaction,
-	commit_transaction,
-	rollback_transaction,
-	get_quote,
-	{
-	 query_init,
-	 query_fill,
-	 blob_read,
-	 query_release,
-	 {
-		field_type,
-		field_name,
-		field_index,
-		field_length,
-		},
-	 },
-	{
-	 field_exist,
-	 field_list,
-	 field_info,
-	 },
-	{
-	 table_init,
-	 table_index,
-	 table_release,
-	 table_exist,
-	 table_list,
-	 table_primary_key,
-	 table_is_system,
-	 table_type,
-	 table_delete,
-	 table_create,
-	 },
-	{
-	 index_exist,
-	 index_list,
-	 index_info,
-	 index_delete,
-	 index_create,
-	 },
-	{
-	 database_exist,
-	 database_list,
-	 database_is_system,
-	 database_delete,
-	 database_create,
-	 },
-	{
-	 user_exist,
-	 user_list,
-	 user_info,
-	 user_delete,
-	 user_create,
-	 user_set_password}
-};
-
-
+DECLARE_DRIVER(_driver, "sqlite3");
 
 /* Internal function to convert a database type into a Gambas type */
 
@@ -312,7 +249,7 @@ static void conv_data(char *data, GB_VARIANT_VALUE * val, fType type)
 
 /* internal function to quote a value stored as a blob */
 
-static void quote_blob(char *data, long len, DB_FORMAT_CALLBACK add)
+static void quote_blob(char *data, int len, DB_FORMAT_CALLBACK add)
 {
 	int i;
 	unsigned char c;
@@ -342,7 +279,7 @@ static void quote_blob(char *data, long len, DB_FORMAT_CALLBACK add)
 
 static char *query_param[3];
 
-static void query_get_param(int index, char **str, long *len)
+static void query_get_param(int index, char **str, int *len, char quote)
 {
 	if (index > 3)
 		return;
@@ -350,13 +287,20 @@ static void query_get_param(int index, char **str, long *len)
 	index--;
 	*str = query_param[index];
 	*len = strlen(*str);
+
+	if (quote == '\'')
+	{
+		*str = DB.QuoteString(*str, *len, quote);
+		*len = GB.StringLength(*str);
+	}
 }
 
 /* Internal function to run a query */
 
-static int do_query(SqliteDatabase * conn, const char *error, Dataset ** pres,
-										const char *qtemp, int nsubst, ...)
+static int do_query(DB_DATABASE *db, const char *error, Dataset **pres,
+                    const char *qtemp, int nsubst, ...)
 {
+	SqliteDatabase *conn = (SqliteDatabase *)db->handle;
 	va_list args;
 	int i;
 	const char *query;
@@ -371,7 +315,7 @@ static int do_query(SqliteDatabase * conn, const char *error, Dataset ** pres,
 		for (i = 0; i < nsubst; i++)
 			query_param[i] = va_arg(args, char *);
 
-		query = GB.SubstString(qtemp, 0, query_get_param);
+		query = DB.SubstString(qtemp, 0, query_get_param);
 	}
 	else
 		query = qtemp;
@@ -382,11 +326,10 @@ static int do_query(SqliteDatabase * conn, const char *error, Dataset ** pres,
 	}
 
 	if (DB.IsDebug())
-		fprintf(stderr, "sqlite: %p: %s\n", conn, query);
+		fprintf(stderr, "sqlite3: %p: %s\n", conn, query);
 
 	if (strncasecmp("select", query, 6) == 0)
 	{
-
 		if (res->query(query))
 		{
 			ret = FALSE;
@@ -417,7 +360,15 @@ static int do_query(SqliteDatabase * conn, const char *error, Dataset ** pres,
 			GB.Error(error, conn->getErrorMsg());
 		}
 	}
+	
+	if (!pres)
+		res->close();
 
+  if (ret)
+  	db->error = conn->lastError();
+  else
+  	db->error = 0;
+  
 	return ret;
 }
 
@@ -551,7 +502,7 @@ char *GetDatabaseHome()
 	char *env = NULL;
 	char *dbhome = NULL;
 
-	GB.Alloc((void **) &dbhome, MAX_PATH);
+	GB.Alloc(POINTER(&dbhome), MAX_PATH);
 
 	/* Check for Environment variable */
 
@@ -649,10 +600,10 @@ int WalkDirectory(char *dir, char ***databases)
 }
 
 /* Internal function to check database version number */
-long db_version()
+int db_version()
 {
 	//Check db version
-	long dbversion = 0;
+	int dbversion = 0;
 	unsigned int verMain, verMajor, verMinor;
 
 	sscanf(sqlite3_libversion(), "%2u.%2u.%2u", &verMain, &verMajor, &verMinor);
@@ -780,9 +731,10 @@ static void close_database(DB_DATABASE * db)
 	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 
 	if (conn)
+	{
 		conn->disconnect();
-
-  delete conn;
+		delete conn;
+	}
 }
 
 
@@ -885,7 +837,7 @@ static void format_blob(DB_BLOB * blob, DB_FORMAT_CALLBACK add)
 static int exec_query(DB_DATABASE * db, char *query, DB_RESULT * result,
 											char *err)
 {
-	return do_query((SqliteDatabase *) db->handle, err, (Dataset **) result,
+	return do_query(db, err, (Dataset **) result,
 									query, 0);
 }
 
@@ -905,7 +857,7 @@ static int exec_query(DB_DATABASE * db, char *query, DB_RESULT * result,
 
 *****************************************************************************/
 
-static void query_init(DB_RESULT result, DB_INFO * info, long *count)
+static void query_init(DB_RESULT result, DB_INFO * info, int *count)
 {
 	Dataset *res = (Dataset *) result;
 
@@ -945,6 +897,7 @@ static void query_release(DB_RESULT result, DB_INFO * info)
 
   Fill a result buffer with the value of each field of a record.
 
+  <db> is the database handle, as returned by open_database()
   <result> is the handle of the result.
   <pos> is the index of the record in the result.
   <buffer> points to an array having one element for each field in the
@@ -956,8 +909,7 @@ static void query_release(DB_RESULT result, DB_INFO * info)
 
 *****************************************************************************/
 
-static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE * buffer,
-											int next)
+static int query_fill(DB_DATABASE *db, DB_RESULT result, int pos, GB_VARIANT_VALUE * buffer, int next)
 {
 	Dataset *res = (Dataset *) result;
 	int i;
@@ -976,6 +928,8 @@ static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE * buffer,
 			data = NULL;
 		else
 			data = (char *) res->fv(i).get_asString().data();
+			
+		//fprintf(stderr, "query_fill: %d.%d %s\n", pos, i, data);
 
 		value.type = GB_T_VARIANT;
 		value.value._object.type = GB_T_NULL;
@@ -984,8 +938,6 @@ static int query_fill(DB_RESULT result, int pos, GB_VARIANT_VALUE * buffer,
 		if (data)
 			conv_data(data, &value.value, (fType) res->fieldType(i));
 
-		//fprintf(stderr, "query_fill: %d: %d -> %ld: %s\n", i, res->fieldType(i), value.value.type, data);
-
 		//GB.FreeString(&data);
 		GB.StoreVariant(&value, &buffer[i]);
 	}
@@ -1133,8 +1085,7 @@ static int field_length(DB_RESULT result, int field)
 
 static int begin_transaction(DB_DATABASE * db)
 {
-	return do_query((SqliteDatabase *) db->handle,
-									"Unable to begin transaction: &1", NULL, "BEGIN", 0);
+	return do_query(db, "Unable to begin transaction: &1", NULL, "BEGIN", 0);
 }
 
 
@@ -1153,8 +1104,7 @@ static int begin_transaction(DB_DATABASE * db)
 
 static int commit_transaction(DB_DATABASE * db)
 {
-	return do_query((SqliteDatabase *) db->handle,
-									"Unable to commit transaction: &1", NULL, "COMMIT", 0);
+	return do_query(db, "Unable to commit transaction: &1", NULL, "COMMIT", 0);
 }
 
 
@@ -1173,8 +1123,7 @@ static int commit_transaction(DB_DATABASE * db)
 
 static int rollback_transaction(DB_DATABASE * db)
 {
-	return do_query((SqliteDatabase *) db->handle,
-									"Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
+	return do_query(db, "Unable to rollback transaction: &1", NULL, "ROLLBACK", 0);
 }
 
 /*****************************************************************************
@@ -1203,7 +1152,6 @@ static int table_init(DB_DATABASE * db, char *table, DB_INFO * info)
 {
 	char *qfield = "PRAGMA table_info('&1')";
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
 	int i, n;
 	DB_FIELD *f;
@@ -1215,8 +1163,7 @@ static int table_init(DB_DATABASE * db, char *table, DB_INFO * info)
 
 	/* Liste des champs */
 
-	if (do_query
-			(conn, "Unable to get table fields: &1", &res, qfield, 1, table))
+	if (do_query(db, "Unable to get table fields: &1", &res, qfield, 1, table))
 		return TRUE;
 
 	result_set *r = (result_set *) res->getExecRes();
@@ -1228,8 +1175,7 @@ static int table_init(DB_DATABASE * db, char *table, DB_INFO * info)
 		return TRUE;
 	}
 
-	GB.Alloc((void **) &info->field, sizeof(DB_FIELD) * n);
-
+	GB.Alloc(POINTER(&info->field), sizeof(DB_FIELD) * n);
 
 	for (i = 0; i < n; i++)
 	{
@@ -1280,7 +1226,6 @@ static int table_index(DB_DATABASE * db, char *table, DB_INFO * info)
 	char *qindex1 = "PRAGMA index_list('&1')";
 	char *qindex2 = "PRAGMA index_info('&1')";
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
 	char *sql = NULL;
 	int n = 0;
@@ -1288,8 +1233,7 @@ static int table_index(DB_DATABASE * db, char *table, DB_INFO * info)
 
 	/* Index primaire */
 
-	if (do_query
-			(conn, "Unable to get primary index: &1", &res, qindex1, 1, table))
+	if (do_query(db, "Unable to get primary index: &1", &res, qindex1, 1, table))
 		return TRUE;
 
 	result_set *r = (result_set *) res->getExecRes();
@@ -1303,14 +1247,14 @@ static int table_index(DB_DATABASE * db, char *table, DB_INFO * info)
 		res->close();
 
 		if (do_query
-				(conn, "Unable to get primary index: &1", &res,
+				(db, "Unable to get primary index: &1", &res,
 				 "PRAGMA table_info('&1')", 1, table))
 			return TRUE;
 
 		r = (result_set *) res->getExecRes();
 
 		info->nindex = 1;
-		GB.Alloc((void **) &info->index, sizeof(int));
+		GB.Alloc(POINTER(&info->index), sizeof(int));
 
 		for (i = 0; i < (int) r->records.size(); i++)
 		{
@@ -1333,7 +1277,7 @@ static int table_index(DB_DATABASE * db, char *table, DB_INFO * info)
 			res->close();
 
 			if (do_query
-					(conn, "Unable to get information on primary index: &1", &res,
+					(db, "Unable to get information on primary index: &1", &res,
 					 qindex2, 1, sql))
 			{
 				res->close();
@@ -1344,7 +1288,7 @@ static int table_index(DB_DATABASE * db, char *table, DB_INFO * info)
 
 			r = (result_set *) res->getExecRes();
 			info->nindex = r->records.size();
-			GB.Alloc((void **) &info->index, sizeof(int) * info->nindex);
+			GB.Alloc(POINTER(&info->index), sizeof(int) * info->nindex);
 
 			for (i = 0; i < info->nindex; i++)
 			{
@@ -1403,11 +1347,10 @@ static int table_exist(DB_DATABASE * db, char *table)
 		return TRUE;
 	}
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
 	int exist;
 
-	if (do_query(conn, "Unable to check table: &1", &res, query, 1, table))
+	if (do_query(db, "Unable to check table: &1", &res, query, 1, table))
 		return FALSE;
 
 	exist = res->num_rows();
@@ -1431,19 +1374,18 @@ static int table_exist(DB_DATABASE * db, char *table)
 
 *****************************************************************************/
 
-static long table_list(DB_DATABASE * db, char ***tables)
+static int table_list(DB_DATABASE * db, char ***tables)
 {
 	const char *query =
 		"select tbl_name from "
 		"( select tbl_name from sqlite_master where type = 'table' union "
 		"   select tbl_name from sqlite_temp_master where type = 'table')";
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
-	long rows;
+	int rows;
 	int i = 0;
 
-	if (do_query(conn, "Unable to get tables: &1", &res, query, 0))
+	if (do_query(db, "Unable to get tables: &1", &res, query, 0))
 		return -1;
 
 	rows = res->num_rows();
@@ -1485,14 +1427,13 @@ static int table_primary_key(DB_DATABASE * db, char *table, char ***primary)
 	char *qindex1 = "PRAGMA index_list('&1')";
 	char *qindex2 = "PRAGMA index_info('&1')";
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
 	int i, n;
 	char *sql;
 	result_set *r;
 
 	if (do_query
-			(conn, "Unable to get primary key: &1", &res, qindex1, 1, table))
+			(db, "Unable to get primary key: &1", &res, qindex1, 1, table))
 		return TRUE;
 
 	GB.NewArray(primary, sizeof(char *), 0);
@@ -1507,7 +1448,7 @@ static int table_primary_key(DB_DATABASE * db, char *table, char ***primary)
 			res->close();
 
 			if (do_query
-					(conn, "Unable to get primary key: &1", &res, qindex2, 1, sql))
+					(db, "Unable to get primary key: &1", &res, qindex2, 1, sql))
 			{
 				res->close();
 				GB.FreeString(&sql);
@@ -1540,7 +1481,7 @@ static int table_primary_key(DB_DATABASE * db, char *table, char ***primary)
 	if (GB.Count(*primary) == 0)
 	{
 		if (do_query
-				(conn, "Unable to get primary key: &1", &res,
+				(db, "Unable to get primary key: &1", &res,
 				 "PRAGMA table_info('&1')", 1, table))
 			return TRUE;
 
@@ -1616,7 +1557,7 @@ static char *table_type(DB_DATABASE * db, char *table, char *type)
 static int table_delete(DB_DATABASE * db, char *table)
 {
 	return
-		do_query((SqliteDatabase *) db->handle, "Unable to delete table: &1",
+		do_query(db, "Unable to delete table: &1",
 						 NULL, "drop table '&1'", 1, table);
 }
 
@@ -1639,7 +1580,6 @@ static int table_delete(DB_DATABASE * db, char *table)
 static int table_create(DB_DATABASE * db, char *table, DB_FIELD * fields,
 												char **primary, char *not_used)
 {
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	DB_FIELD *fp;
 	int comma;
 	char *type;
@@ -1662,7 +1602,9 @@ static int table_create(DB_DATABASE * db, char *table, DB_FIELD * fields,
 		else
 			comma = TRUE;
 
+		DB.Query.Add(QUOTE_STRING);
 		DB.Query.Add(fp->name);
+		DB.Query.Add(QUOTE_STRING);
 
 		if (fp->type == DB_T_SERIAL)
 		{
@@ -1698,7 +1640,7 @@ static int table_create(DB_DATABASE * db, char *table, DB_FIELD * fields,
 						type = "TEXT";
 					else
 					{
-						sprintf(_buffer, "VARCHAR(%ld)", fp->length);
+						sprintf(_buffer, "VARCHAR(%d)", fp->length);
 						type = _buffer;
 					}
 
@@ -1741,7 +1683,7 @@ static int table_create(DB_DATABASE * db, char *table, DB_FIELD * fields,
 
 	DB.Query.Add(" )");
 
-	return do_query(conn, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
+	return do_query(db, "Cannot create table: &1", NULL, DB.Query.Get(), 0);
 }
 
 /*****************************************************************************
@@ -1762,13 +1704,12 @@ static int field_exist(DB_DATABASE * db, char *table, char *field)
 {
 	const char *query = "PRAGMA table_info('&1')";
 
-	long i, n;
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
+	int i, n;
 	Dataset *res;
 	int exist = 0;
 
 	if (do_query
-			(conn, "Unable to find field: &1.&2", &res, query, 2, table, field))
+			(db, "Unable to find field: &1.&2", &res, query, 2, table, field))
 	{
 		return FALSE;
 	}
@@ -1804,15 +1745,14 @@ static int field_exist(DB_DATABASE * db, char *table, char *field)
 
 *****************************************************************************/
 
-static long field_list(DB_DATABASE * db, char *table, char ***fields)
+static int field_list(DB_DATABASE * db, char *table, char ***fields)
 {
 	const char *query = "PRAGMA table_info('&1')";
 
-	long i, n;
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
+	int i, n;
 	Dataset *res;
 
-	if (do_query(conn, "Unable to get fields: &1", &res, query, 1, table))
+	if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
 	{
 		return -1;
 	}
@@ -1860,14 +1800,13 @@ static int field_info(DB_DATABASE * db, char *table, char *field,
 	Dataset *res;
 	GB_VARIANT def;
 	char *val;
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	char *_fieldName = NULL;
 	char *_fieldType = NULL;
 	char *_defaultValue = NULL;
 	bool _fieldNotNull = FALSE;
 	int i, n;
 
-	if (do_query(conn, "Unable to get fields: &1", &res, query, 1, table))
+	if (do_query(db, "Unable to get fields: &1", &res, query, 1, table))
 	{
 		return TRUE;
 	}
@@ -1956,11 +1895,10 @@ static int index_exist(DB_DATABASE * db, char *table, char *index)
 		"select tbl_name from sqlite_temp_master where type = 'index' and "
 		" name = '&2' ) " "where tbl_name = '&1'";
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
 	int exist;
 
-	if (do_query(conn, "Unable to check table: &1", &res, query, 2, table, index))
+	if (do_query(db, "Unable to check table: &1", &res, query, 2, table, index))
 		return FALSE;
 
 	exist = res->num_rows();
@@ -1985,7 +1923,7 @@ static int index_exist(DB_DATABASE * db, char *table, char *index)
 
 *****************************************************************************/
 
-static long index_list(DB_DATABASE * db, char *table, char ***indexes)
+static int index_list(DB_DATABASE * db, char *table, char ***indexes)
 {
 
 	const char *query =
@@ -1994,11 +1932,10 @@ static long index_list(DB_DATABASE * db, char *table, char ***indexes)
 		" union select name from sqlite_temp_master where type = 'index' and "
 		" tbl_name = '&1')";
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
 	int no_indexes, i = 0;
 
-	if (do_query(conn, "Unable to get tables: &1", &res, query, 1, table))
+	if (do_query(db, "Unable to get tables: &1", &res, query, 1, table))
 		return -1;
 
 	no_indexes = res->num_rows();
@@ -2042,12 +1979,11 @@ static int index_info(DB_DATABASE * db, char *table, char *index,
 	char *qindex1 = "PRAGMA index_list('&1')";
 	char *qindex2 = "PRAGMA index_info('&1')";
 
-	SqliteDatabase *conn = (SqliteDatabase *) db->handle;
 	Dataset *res;
 	int i, j, n;
 
 	if (do_query
-			(conn, "Unable to get index info for table: &1", &res, qindex1, 1,
+			(db, "Unable to get index info for table: &1", &res, qindex1, 1,
 			 table))
 		return TRUE;
 
@@ -2087,7 +2023,7 @@ static int index_info(DB_DATABASE * db, char *table, char *index,
 	DB.Query.Init();
 
 	if (do_query
-			(conn, "Unable to get index info for : &1", &res, qindex2, 1, index))
+			(db, "Unable to get index info for : &1", &res, qindex2, 1, index))
 	{
 		res->close();
 		return TRUE;
@@ -2131,8 +2067,8 @@ static int index_info(DB_DATABASE * db, char *table, char *index,
 static int index_delete(DB_DATABASE * db, char *table, char *index)
 {
 	return
-		do_query((SqliteDatabase *) db->handle, "Unable to delete index: &1",
-						 NULL, "drop index '&1'", 1, index);
+		do_query(db, "Unable to delete index: &1",
+						 NULL, "drop index &1", 1, index);
 }
 
 /*****************************************************************************
@@ -2159,15 +2095,15 @@ static int index_create(DB_DATABASE * db, char *table, char *index,
 	DB.Query.Add("CREATE ");
 	if (info->unique)
 		DB.Query.Add("UNIQUE ");
-	DB.Query.Add("INDEX ");
+	DB.Query.Add("INDEX '");
 	DB.Query.Add(index);
-	DB.Query.Add(" ON ");
+	DB.Query.Add("' ON ");
 	DB.Query.Add(table);
 	DB.Query.Add(" ( ");
 	DB.Query.Add(info->fields);
 	DB.Query.Add(" )");
 
-	return do_query((SqliteDatabase *) db->handle, "Cannot create index: &1",
+	return do_query(db, "Cannot create index: &1",
 									NULL, DB.Query.Get(), 0);
 }
 
@@ -2224,7 +2160,7 @@ static int database_exist(DB_DATABASE * db, char *name)
 
  ******************************************************************************/
 
-static long database_list(DB_DATABASE * db, char ***databases)
+static int database_list(DB_DATABASE * db, char ***databases)
 {
 	char *dbhome;
 
@@ -2242,7 +2178,7 @@ static long database_list(DB_DATABASE * db, char ***databases)
 	{
 		//GB.Error("Unable to get databases: &1", "Can't find current directory");
 		WalkDirectory(dbhome, databases);
-		GB.Free((void **) &dbhome);
+		GB.Free(POINTER(&dbhome));
 	}
 
 	/*if (getcwd(cwd, MAX_PATH) != NULL){
@@ -2359,7 +2295,7 @@ static int database_create(DB_DATABASE * db, char *name)
 		dir = GetDatabaseHome();
 		mkdir(dir, S_IRWXU);
 		GB.NewString(&fullpath, dir, 0);
-		GB.Free((void **) &dir);
+		GB.Free(POINTER(&dir));
 	}
 
 	if (fullpath[strlen(fullpath) - 1] != '/')
@@ -2380,12 +2316,13 @@ _CREATE_DATABASE:
 	}
 
 	//Create and remove a table to initialize database
-	if (!do_query
-			(&conn2, "Unable to initialise database", NULL,
-			 "CREATE TABLE GAMBAS (FIELD1 TEXT)", 0))
-		do_query(&conn2, NULL, NULL, "DROP TABLE GAMBAS", 0);
+	db->handle = &conn2;
+	if (!do_query(db, "Unable to initialise database", NULL, "CREATE TABLE GAMBAS (FIELD1 TEXT)", 0))
+		do_query(db, NULL, NULL, "DROP TABLE GAMBAS", 0);
 
 	conn2.disconnect();
+	db->handle = conn;
+	
 	return FALSE;
 }
 
@@ -2524,7 +2461,7 @@ static int user_exist(DB_DATABASE * db, char *name)
  ******************************************************************************/
 
 
-static long user_list(DB_DATABASE * db, char ***users)
+static int user_list(DB_DATABASE * db, char ***users)
 {
 	//Should we use GB.HashTable.New etc. to ensure
 	//duplicates are not reported back, then transfer
