@@ -62,7 +62,7 @@ static int my_sqlite3_exec(
   char **azCols = 0;
 
   int nRetry = 0;
-  int nChange = 0;
+  //int nChange = 0;
   int nCallback;
 
   if( zSql==0 ) return SQLITE_OK;
@@ -90,7 +90,7 @@ static int my_sqlite3_exec(
     	goto exec_out;
 
     //azCols = sqliteMalloc(2*nCol*sizeof(const char *));
-  	GB.Alloc((void **)&azCols, 2*nCol*sizeof(const char *));
+  	GB.Alloc(POINTER(&azCols), 2*nCol*sizeof(const char *));
 
     if( nCol && !azCols ){
       rc = SQLITE_NOMEM;
@@ -138,27 +138,28 @@ static int my_sqlite3_exec(
       }
     }
 
-    GB.Free((void **)&azCols);
+    GB.Free(POINTER(&azCols));
     azCols = 0;
   }
 
 exec_out:
   if( pStmt ) sqlite3_finalize(pStmt);
   if( azCols )
-    GB.Free((void **)&azCols);
+    GB.Free(POINTER(&azCols));
 
   //if( sqlite3_malloc_failed ){
   //  rc = SQLITE_NOMEM;
   //}
-  if( rc!=SQLITE_OK && rc==sqlite3_errcode(db) && pzErrMsg ){
-    //*pzErrMsg = malloc(1+strlen(sqlite3_errmsg(db)));
-  	GB.Alloc((void **)pzErrMsg, 1+strlen(sqlite3_errmsg(db)));
-    if( *pzErrMsg ){
-      strcpy(*pzErrMsg, sqlite3_errmsg(db));
-    }
-  }else if( pzErrMsg ){
-    *pzErrMsg = 0;
-  }
+  
+//   GB.Free(POINTER(pzErrMsg));
+//   
+//   if( rc!=SQLITE_OK && rc==sqlite3_errcode(db) && pzErrMsg ){
+//     //*pzErrMsg = malloc(1+strlen(sqlite3_errmsg(db)));
+//   	GB.Alloc(POINTER(pzErrMsg), 1+strlen(sqlite3_errmsg(db)));
+//     if( *pzErrMsg ){
+//       strcpy(*pzErrMsg, sqlite3_errmsg(db));
+//     }
+//   }
 
   return rc;
 }
@@ -253,7 +254,7 @@ static int callback(void *res_ptr, int ncol, char **reslt, char **cols, sqlite3_
 	{
 		for (int i = 0; i < ncol; i++)
 		{
-			//fprintf(stderr, "callback: %d: %d\n", i, sqlite3_column_bytes(stmt, i));
+			//fprintf(stderr, "callback: [%d] %s: %s\n", i, cols[i], reslt[i]);
 			if (reslt[i] == NULL)
 			{
 				r->records[sz][i].set_isNull(r->record_header[i].type);
@@ -308,9 +309,7 @@ static int callback(void *res_ptr, int ncol, char **reslt, char **cols, sqlite3_
 						r->records[sz][i].set_asString(reslt[i]);
 				}
 			}
-			//rec[i] = v;
 		}
-		//r->records[sz] = rec;
 	}
 	//printf("Fsz: [%i]\n",r->record_header.size());
 	//printf("Recs: [%i]\n",r->records.size());
@@ -328,7 +327,6 @@ static int old_callback(void *res_ptr, int ncol, char **reslt, char **cols)
 
 SqliteDatabase::SqliteDatabase()
 {
-
 	active = false;
 	_in_transaction = false;			// for transaction
 
@@ -346,12 +344,12 @@ SqliteDatabase::~SqliteDatabase()
 }
 
 
-		 Dataset *SqliteDatabase::CreateDataset() const
-		 {
-			 return new SqliteDataset((SqliteDatabase *) this);
-		 }
+Dataset *SqliteDatabase::CreateDataset() const
+{
+	return new SqliteDataset((SqliteDatabase *) this);
+}
 
-		 int SqliteDatabase::status(void)
+int SqliteDatabase::status(void)
 {
 	if (active == false)
 		return DB_CONNECTION_NONE;
@@ -360,7 +358,8 @@ SqliteDatabase::~SqliteDatabase()
 
 int SqliteDatabase::setErr(int err_code)
 {
-
+	last_err = err_code;
+	
 	switch (err_code)
 	{
 		case SQLITE_OK:
@@ -445,12 +444,12 @@ int SqliteDatabase::connect()
 	if (sqlite3_open(db.c_str(), &conn) == SQLITE_OK)
 	{
 		//cout << "Connected!\n";
-		char *err = NULL;
+		//char *err = NULL;
 
 		if (setErr
 				(sqlite3_exec
 				 (getHandle(), "PRAGMA empty_result_callbacks=ON", NULL, NULL,
-					&err)) != SQLITE_OK)
+					NULL)) != SQLITE_OK)
 		{
 			GB.Error(getErrorMsg());
 		}
@@ -460,14 +459,14 @@ int SqliteDatabase::connect()
 		if (setErr
 				(sqlite3_exec
 				 (getHandle(), "PRAGMA short_column_names=OFF", NULL, NULL,
-					&err)) != SQLITE_OK)
+					NULL)) != SQLITE_OK)
 		{														//NG
 			GB.Error(getErrorMsg());	//NG
 		}
 		if (setErr
 				(sqlite3_exec
 				 (getHandle(), "PRAGMA full_column_names=ON", NULL, NULL,
-					&err)) != SQLITE_OK)
+					NULL)) != SQLITE_OK)
 		{														//NG
 			GB.Error(getErrorMsg());	//NG
 		}
@@ -547,7 +546,7 @@ void SqliteDatabase::start_transaction()
 {
 	if (active)
 	{
-		sqlite3_exec(conn, "begin", NULL, NULL, NULL);
+		last_err = sqlite3_exec(conn, "begin", NULL, NULL, NULL);
 		_in_transaction = true;
 	}
 }
@@ -556,7 +555,7 @@ void SqliteDatabase::commit_transaction()
 {
 	if (active)
 	{
-		sqlite3_exec(conn, "commit", NULL, NULL, NULL);
+		last_err = sqlite3_exec(conn, "commit", NULL, NULL, NULL);
 		_in_transaction = false;
 	}
 }
@@ -565,7 +564,7 @@ void SqliteDatabase::rollback_transaction()
 {
 	if (active)
 	{
-		sqlite3_exec(conn, "rollback", NULL, NULL, NULL);
+		last_err = sqlite3_exec(conn, "rollback", NULL, NULL, NULL);
 		_in_transaction = false;
 	}
 }
@@ -578,6 +577,7 @@ SqliteDataset::SqliteDataset():Dataset()
 {
 	haveError = false;
 	db = NULL;
+// 	errmsg = NULL;
 }
 
 
@@ -585,12 +585,13 @@ SqliteDataset::SqliteDataset(SqliteDatabase * newDb):Dataset(newDb)
 {
 	haveError = false;
 	db = newDb;
+// 	errmsg = NULL;
 }
 
 SqliteDataset::~SqliteDataset()
 {
-	if (errmsg)
-		sqlite3_free_table(&errmsg);
+/*	if (errmsg)
+		sqlite3_free_table(&errmsg);*/
 }
 
 
@@ -729,7 +730,7 @@ int SqliteDataset::exec(const string & sql)
 
 	for (retry = 1; retry <= 2; retry++)
 	{
-		res = sqlite3_exec(handle(), sql.c_str(), old_callback, &exec_res, &errmsg);
+		res = sqlite3_exec(handle(), sql.c_str(), old_callback, &exec_res, NULL);
 		if (res != SQLITE_SCHEMA)
 			break;
 	}
@@ -770,14 +771,14 @@ bool SqliteDataset::query(const char *query)
 		)
 		GB.Error("MUST be select SQL or PRAGMA table or index!");
 
-	close();
+	//close();
 
 	//cout <<  "Curr size "<<num_rows()<<"\n\n";
 	result.conn = handle();				//NG
 
 	for (retry = 1; retry <= 2; retry++)
 	{
-		res = my_sqlite3_exec(handle(), query, &callback, &result, &errmsg);
+		res = my_sqlite3_exec(handle(), query, &callback, &result, NULL);
 		if (res != SQLITE_SCHEMA)
 			break;
 	}
@@ -830,6 +831,7 @@ void SqliteDataset::close()
 	fields_object->clear();
 	ds_state = dsInactive;
 	active = false;
+	delete this;
 }
 
 
@@ -964,7 +966,7 @@ fType GetFieldType(char *Type, unsigned int *length)
 	char *upper;
 	char *_left, *_right;
 	fType rType;									/* For return */
-	unsigned int rTypeLen;
+	unsigned int rTypeLen = 0;
 
 	upper = Type;
 
@@ -1037,7 +1039,7 @@ fType GetFieldType(char *Type, unsigned int *length)
 		/* Length is for when value is used as a string */
 		rTypeLen = ft_Date_Length;
 	}
-	else if (strstr(Type, "DOUBLE"))
+	else if (strstr(Type, "DOUBLE") || strstr(Type, "FLOAT8"))
 	{
 		rType = ft_Double;
 		/* Length is for when value is used as a string */
@@ -1047,7 +1049,7 @@ fType GetFieldType(char *Type, unsigned int *length)
 																   MySQL "real" which is a
 																   synonym of "double" */
 					 || strstr(Type, "FLOAT")
-					 || strstr(Type, "FLOAT8") || strstr(Type, "FLOAT4"))
+					 || strstr(Type, "FLOAT4"))
 	{
 		rType = ft_Float;
 		/* Length is for when value is used as a string */
