@@ -29,10 +29,6 @@
 
 #define __CBUTTON_CPP
 
-
-
-#include "widgets.h"
-#include "gambas.h"
 #include "CButton.h"
 #include "CContainer.h"
 #include "CPicture.h"
@@ -45,6 +41,7 @@ void gb_raise_button_Click(gControl *sender)
 	
 	if (!_ob) return;
 	GB.Raise((void*)_ob,EVENT_Click,0);
+	CACTION_raise(_ob);
 }
 
 /***************************************************************
@@ -52,69 +49,40 @@ void gb_raise_button_Click(gControl *sender)
 CONSTRUCTORS
 
 ****************************************************************/
+
 BEGIN_METHOD(CBUTTON_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	THIS->widget=new gButton(Parent->widget,0);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
+	InitControl(new gButton(CONTAINER(VARG(parent)), gButton::Button), (CWIDGET*)THIS);
 	BUTTON->onClick=gb_raise_button_Click;
 
 END_METHOD
 
-BEGIN_METHOD_VOID(CBUTTON_free)
-
-	if (THIS->picture) 
-	{
-		GB.Unref((void**)&THIS->picture);
-		THIS->picture=NULL;
-	}
-
-END_METHOD
-
 
 BEGIN_METHOD(CTOGGLEBUTTON_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-	
-	THIS->widget=new gButton(Parent->widget,1);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
+	InitControl(new gButton(CONTAINER(VARG(parent)), gButton::Toggle), (CWIDGET*)THIS);
 	BUTTON->onClick=gb_raise_button_Click;
 
 END_METHOD
 
 BEGIN_METHOD(CCHECKBOX_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
+	InitControl(new gButton(CONTAINER(VARG(parent)), gButton::Check), (CWIDGET*)THIS);
 	THIS->f_value=0;
-	THIS->widget=new gButton(Parent->widget,2);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
 	BUTTON->onClick=gb_raise_button_Click;
 
 END_METHOD
 
 BEGIN_METHOD(CRADIOBUTTON_new, GB_OBJECT parent)
-
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	THIS->widget=new gButton(Parent->widget,3);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
+	
+	InitControl(new gButton(CONTAINER(VARG(parent)), gButton::Radio), (CWIDGET*)THIS);
 	BUTTON->onClick=gb_raise_button_Click;
 
 END_METHOD
 
 BEGIN_METHOD(CTOOLBUTTON_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	THIS->widget=new gButton(Parent->widget,4);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
+	InitControl(new gButton(CONTAINER(VARG(parent)), gButton::Tool), (CWIDGET*)THIS);
 	BUTTON->onClick=gb_raise_button_Click;
 
 END_METHOD
@@ -130,24 +98,20 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CBUTTON_picture)
 
-	CPICTURE *pic=NULL;
-	gPicture *buf;
-
-	if (READ_PROPERTY) { GB.ReturnObject((void*)THIS->picture); return; }
-
-	pic=(CPICTURE*)VPROP(GB_OBJECT);
-
-	if (pic) GB.Ref((void*)pic);
-	if (THIS->picture) GB.Unref((void**)&THIS->picture);
-	THIS->picture=pic;
-
-	if (!pic ) BUTTON->setPicture(NULL);  
-	else       BUTTON->setPicture(pic->picture); 
+	if (READ_PROPERTY)
+	{
+		gPicture *pic = BUTTON->picture();
+		GB.ReturnObject(pic ? pic->getTagValue() : 0);
+	}
+	else
+	{
+		CPICTURE *pic = (CPICTURE *)VPROP(GB_OBJECT);
+		BUTTON->setPicture(pic ? pic->picture : 0);
+	}
 
 END_PROPERTY
 
 
-
 BEGIN_PROPERTY(CBUTTON_border)
 
 	if (READ_PROPERTY) { GB.ReturnBoolean(BUTTON->getBorder()); return; }
@@ -181,8 +145,10 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CTOOLBUTTON_toggle)
 
-	if (READ_PROPERTY){ GB.ReturnBoolean(BUTTON->getToggle()); return; }
-	BUTTON->setToggle(VPROP(GB_INTEGER));	
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(BUTTON->isToggle());
+	else
+		BUTTON->setToggle(VPROP(GB_INTEGER));	
 
 END_PROPERTY
 
@@ -203,7 +169,7 @@ BEGIN_PROPERTY(CCHECKBOX_value)
 	if (READ_PROPERTY) 
 	{ 
 		if (THIS->f_value && BUTTON->inconsistent()) { GB.ReturnInteger(1); return; }
-		GB.ReturnInteger( BUTTON->value() );
+		GB.ReturnInteger( BUTTON->value() ? -1 : 0 );
 		return;
 		
 	}
@@ -214,13 +180,20 @@ BEGIN_PROPERTY(CCHECKBOX_value)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(CBUTTON_radio)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(BUTTON->isRadio());
+	else
+		BUTTON->setRadio(VPROP(GB_BOOLEAN));
+
+END_PROPERTY
 
 GB_DESC CButtonDesc[] =
 {
   GB_DECLARE("Button", sizeof(CBUTTON)), GB_INHERITS("Control"),
   
   GB_METHOD("_new", NULL, CBUTTON_new, "(Parent)Container;"),
-  GB_METHOD("_free",NULL,CBUTTON_free,NULL),
   
   GB_PROPERTY("Text", "s", CBUTTON_text),
   GB_PROPERTY("Caption", "s", CBUTTON_text),
@@ -231,11 +204,10 @@ GB_DESC CButtonDesc[] =
   GB_PROPERTY("Cancel", "b", CBUTTON_cancel),
   GB_PROPERTY("Value", "b", CBUTTON_value),
 
-  GB_CONSTANT("_Properties", "s", CBUTTON_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
-
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
+	BUTTON_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
@@ -244,19 +216,18 @@ GB_DESC CToggleButtonDesc[] =
   GB_DECLARE("ToggleButton", sizeof(CBUTTON)), GB_INHERITS("Control"),
 
   GB_METHOD("_new", NULL, CTOGGLEBUTTON_new, "(Parent)Container;"),
-  GB_METHOD("_free",NULL,CBUTTON_free,NULL),
 
   GB_PROPERTY("Text", "s", CBUTTON_text),
   GB_PROPERTY("Caption", "s", CBUTTON_text),
   GB_PROPERTY("Picture", "Picture", CBUTTON_picture),
   GB_PROPERTY("Value", "b", CBUTTON_value),
   GB_PROPERTY("Border", "b", CBUTTON_border),
-
-  GB_CONSTANT("_Properties", "s", CTOGGLEBUTTON_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
+  GB_PROPERTY("Radio", "b", CBUTTON_radio),
 
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
+	TOGGLEBUTTON_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
@@ -271,11 +242,10 @@ GB_DESC CRadioButtonDesc[] =
 
   GB_PROPERTY("Value", "b", CBUTTON_value),
 
-  GB_CONSTANT("_Properties", "s", CRADIOBUTTON_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
-
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
+	RADIOBUTTON_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
@@ -293,13 +263,12 @@ GB_DESC CCheckBoxDesc[] =
   GB_PROPERTY("Caption", "s", CBUTTON_text),
   GB_PROPERTY("Tristate", "b", CCHECKBOX_tristate),
 
-  GB_PROPERTY("Value", "i<CheckBox,False,True,None>", CCHECKBOX_value),
-
-  GB_CONSTANT("_Properties", "s", CCHECKBOX_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
+  GB_PROPERTY("Value", "i", CCHECKBOX_value),
 
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
+	CHECKBOX_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
@@ -309,7 +278,6 @@ GB_DESC CToolButtonDesc[] =
   GB_DECLARE("ToolButton", sizeof(CBUTTON)), GB_INHERITS("Control"),
 
   GB_METHOD("_new", NULL, CTOOLBUTTON_new, "(Parent)Container;"),
-  GB_METHOD("_free",NULL,CBUTTON_free,NULL),
 
   GB_PROPERTY("Text", "s", CBUTTON_text),
   GB_PROPERTY("Caption", "s", CBUTTON_text),
@@ -317,12 +285,12 @@ GB_DESC CToolButtonDesc[] =
   GB_PROPERTY("Value", "b", CBUTTON_value),
   GB_PROPERTY("Toggle", "b", CTOOLBUTTON_toggle),
   GB_PROPERTY("Border", "b", CBUTTON_border),
-
-  GB_CONSTANT("_Properties", "s", CTOOLBUTTON_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
+  GB_PROPERTY("Radio", "b", CBUTTON_radio),
 
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
+	TOOLBUTTON_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
