@@ -29,10 +29,9 @@
 #ifndef __CDRAWINGAREA_H
 #define __CDRAWINGAREA_H
 
-#include "gambas.h"
-#include "widgets.h"
+#include "main.h"
 #include "CWidget.h"
-
+#include "gdrawingarea.h"
 
 #ifndef __CDRAWINGAREA_CPP
 extern GB_DESC CDrawingAreaDesc[];
@@ -41,10 +40,6 @@ extern GB_DESC CDrawingAreaDesc[];
 #define THIS    ((CDRAWINGAREA *)_object)
 #define DRAWING ((gDrawingArea*)((CWIDGET *)_object)->widget)
 
-
-#define CDRAWINGAREA_PROPERTIES CWIDGET_PROPERTIES \
-  ",Cached,Tracking,Focus,Merge,Border"
-
 #endif
 
 typedef  struct 
