@@ -28,11 +28,8 @@
 #ifndef __CFONT_H
 #define __CFONT_H
 
-
 #include "main.h"
-#include "gambas.h"
-#include "widgets.h"
-#include "CWidget.h"
+#include "gfont.h"
 
 #ifndef __CFONT_CPP
 extern GB_DESC CFontDesc[];
@@ -43,15 +40,21 @@ extern GB_DESC CFontsDesc[];
 #define THIS ((CFONT *)_object)
 #define FONT (THIS->font)
 
-
 #endif
 
+typedef
+  void (*FONT_FUNC)(gFont *, void *);
+
 typedef struct 
-{
+  {
     GB_BASE ob;
     gFont *font;
+    FONT_FUNC func;
+    void *object;
+    enum { Name, Size, Grade, Bold, Italic, Underline, StrikeOut };
+  }  
+  CFONT;
 
-}  CFONT;
-
+CFONT *CFONT_create(gFont *font, FONT_FUNC func = 0, void *object = 0);
 
 #endif
