@@ -29,94 +29,86 @@
 
 #define __CGRIDVIEW_CPP
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <gtk/gtk.h>
-
-#include "gambas.h"
-#include "main.h"
-#include "widgets.h"
-#include "tablerender.h"
 #include "CWidget.h"
 #include "CContainer.h"
 #include "CGridView.h"
 #include "CPicture.h"
+#include "CFont.h"
 
-DECLARE_EVENT(EVENT_Click);     /* simple click */
-DECLARE_EVENT(EVENT_RowClick);
-DECLARE_EVENT(EVENT_ColumnClick);
-DECLARE_EVENT(EVENT_FooterClick);
-DECLARE_EVENT(EVENT_Activate);  /* double click */
+DECLARE_EVENT(EVENT_Click);
+DECLARE_EVENT(EVENT_Activate);
 DECLARE_EVENT(EVENT_Scroll);
 DECLARE_EVENT(EVENT_Data);
+DECLARE_EVENT(EVENT_ColumnClick);
+DECLARE_EVENT(EVENT_RowClick);
+DECLARE_EVENT(EVENT_ColumnResize);
+DECLARE_EVENT(EVENT_RowResize);
+DECLARE_EVENT(EVENT_Change);
+DECLARE_EVENT(EVENT_Select);
+DECLARE_EVENT(EVENT_FooterClick);
 
-void grid_scroll(gGridView *sender)
+static void raise_event(gGridView *sender, int event)
 {
-	CWIDGET *_ob=GetObject(sender);
-
-	if (!_ob) return;
-	if (GB.CanRaise(_ob,EVENT_Scroll)) 
-		GB.Raise((void*)_ob,EVENT_Scroll,0);
+	CWIDGET *_object = GetObject(sender);
+	GB.Raise(THIS, event, 0);
 }
 
-void grid_col_click(gGridView *sender,long col)
+static void raise_event_int(gGridView *sender, int event, int arg)
 {
-	CWIDGET *_ob=GetObject(sender);
-
-	if (!_ob) return;
-	if (GB.CanRaise(_ob,EVENT_ColumnClick)) 
-		GB.Raise((void*)_ob,EVENT_ColumnClick,1,GB_T_INTEGER,col);
+	CWIDGET *_object = GetObject(sender);
+	GB.Raise(THIS, event, 1, GB_T_INTEGER, arg);
 }
 
-void grid_foot_click(gGridView *sender,long col)
+static void raise_col_click(gGridView *sender, int col)
 {
-	CWIDGET *_ob=GetObject(sender);
-	
-	if (!_ob) return;
-	if (GB.CanRaise(_ob,EVENT_FooterClick)) 
-		GB.Raise((void*)_ob,EVENT_FooterClick,1,GB_T_INTEGER,col);
+	raise_event_int(sender, EVENT_ColumnClick, col);
 }
 
-void grid_row_click(gGridView *sender,long row)
+static void raise_row_click(gGridView *sender,int row)
 {
-	CWIDGET *_ob=GetObject(sender);
-	
-	if (!_ob) return;
-	if (GB.CanRaise(_ob,EVENT_RowClick)) 
-		GB.Raise((void*)_ob,EVENT_RowClick,1,GB_T_INTEGER,row);
+	raise_event_int(sender, EVENT_RowClick, row);
 }
 
-void grid_click(gGridView *sender,long row,long col)
+static void raise_col_resize(gGridView *sender, int col)
 {
-	CWIDGET *_ob=GetObject(sender);
-	
-	if (!_ob) return;
-	if (GB.CanRaise(_ob,EVENT_Click)) 
-	{
-		((CGRIDVIEW*)_ob)->curr_row=row;
-		((CGRIDVIEW*)_ob)->curr_col=col;
-		GB.Raise((void*)_ob,EVENT_Click,0);
-	}
+	raise_event_int(sender, EVENT_ColumnResize, col);
 }
 
-void grid_activate(gGridView *sender,long row,long col)
+static void raise_row_resize(gGridView *sender,int row)
 {
-	CWIDGET *_ob=GetObject(sender);
-	
-	if (!_ob) return;
-	if (GB.CanRaise(_ob,EVENT_Activate)) 
-	{
-		((CGRIDVIEW*)_ob)->curr_row=row;
-		((CGRIDVIEW*)_ob)->curr_col=col;
-		GB.Raise((void*)_ob,EVENT_Activate,0);
-	}
+	raise_event_int(sender, EVENT_RowResize, row);
 }
 
-void grid_data_func(gTableData **fill,long row, long col,void *_object)
+static void raise_foot_click(gGridView *sender, int col)
+{
+	raise_event_int(sender, EVENT_FooterClick, col);
+}
+
+static void raise_click(gGridView *sender, int row, int col)
+{
+	raise_event(sender, EVENT_Click);
+}
+
+static void raise_activate(gGridView *sender, int row, int col)
+{
+	raise_event(sender, EVENT_Activate);
+}
+
+static void raise_change(gGridView *sender)
+{
+	raise_event(sender, EVENT_Change);
+}
+
+static void raise_scroll(gGridView *sender)
+{
+	raise_event(sender, EVENT_Scroll);
+}
+
+static void raise_data(gTableData *fill,int row, int col,void *_object)
 {
 	if (GB.CanRaise(THIS,EVENT_Data))
 	{
-		THIS->data=(*fill);
+		THIS->data = fill;
 		GB.Raise(_object,EVENT_Data,2,GB_T_INTEGER,row,GB_T_INTEGER,col);
 		THIS->data=NULL;
 	}
@@ -137,27 +129,54 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDVIEWDATA_picture)
 
-	stub("CGRIDVIEWDATA_picture");
-	if (READ_PROPERTY) { GB.ReturnNull(); }
+	if (READ_PROPERTY)
+		GB.ReturnObject(THIS->data->picture ? THIS->data->picture->getTagValue() : 0);
+	else
+	{
+		CPICTURE *pict = (CPICTURE *)VPROP(GB_OBJECT);
+		THIS->data->setPicture(pict ? pict->picture : 0);
+	}
 
 END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDVIEWDATA_alignment)
 
-	stub("CGRIDVIEWDATA_alignment");
-	if (READ_PROPERTY) { GB.ReturnInteger(0); }
+	if (READ_PROPERTY)
+		GB.ReturnInteger(THIS->data->alignment);
+	else
+		THIS->data->alignment = VPROP(GB_INTEGER);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(CGRIDVIEWDATA_font)
+
+	CFONT *font;
+
+	if (READ_PROPERTY)
+		GB.ReturnObject(CFONT_create(THIS->data->font));
+	else
+	{
+		font = (CFONT *)VPROP(GB_OBJECT);	
+		THIS->data->setFont(font ? font->font : 0);
+	}
 
 END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDVIEWDATA_text)
 
 	if (READ_PROPERTY)
-	{
 		GB.ReturnNewString(THIS->data->text,0);
-		return;
-	}
+	else
+		THIS->data->setText(GB.ToZeroString(PROP(GB_STRING)));
 
-	THIS->data->setText(PROP(GB_STRING)->value.addr);
+END_PROPERTY
+
+BEGIN_PROPERTY(CGRIDVIEWDATA_rich_text)
+
+	if (READ_PROPERTY)
+		GB.ReturnNewString(THIS->data->richText,0);
+	else
+		THIS->data->setRichText(GB.ToZeroString(PROP(GB_STRING)));
 
 END_PROPERTY
 
@@ -186,43 +205,54 @@ BEGIN_PROPERTY(CGRIDVIEWDATA_fg)
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CGRIDVIEWDATA_xpad)
+BEGIN_PROPERTY(CGRIDVIEWDATA_padding)
 
 	if (READ_PROPERTY)
+		GB.ReturnInteger(THIS->data->padding);
+	else
 	{
-		GB.ReturnInteger(THIS->data->xpad);
-		return;
+		THIS->data->padding = VPROP(GB_INTEGER);
+		if (THIS->data->padding < 0) 
+			THIS->data->padding = 0;
 	}
 
-	THIS->data->xpad=VPROP(GB_INTEGER);
-	if (THIS->data->xpad<0) THIS->data->xpad=0;
-
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEWDATA_ypad)
-
-	if (READ_PROPERTY)
-	{
-		GB.ReturnInteger(THIS->data->ypad);
-		return;
-	}
-
-	THIS->data->xpad=VPROP(GB_INTEGER);
-	if (THIS->data->xpad<0) THIS->data->ypad=0;
-
-END_PROPERTY
-
-
-
 
 BEGIN_PROPERTY(CGRIDVIEWITEM_picture)
 
-	stub("CGRIDVIEWITEM_picture");
-	if (READ_PROPERTY) { GB.ReturnNull(); }
+	if (READ_PROPERTY)
+		GB.ReturnObject(GRIDVIEW->itemPicture(THIS->row, THIS->col));
+	else
+	{
+		CPICTURE *pict = (CPICTURE *)VPROP(GB_OBJECT);
+		GRIDVIEW->setItemPicture(THIS->row, THIS->col, pict ? pict->picture : 0);
+	}
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEWITEM_key)
+BEGIN_PROPERTY(CGRIDVIEWITEM_font)
+
+	if (READ_PROPERTY)
+		GB.ReturnObject(GRIDVIEW->itemFont(THIS->row, THIS->col));
+	else
+	{
+		CFONT *font = (CFONT *)VPROP(GB_OBJECT);
+		GRIDVIEW->setItemFont(THIS->row, THIS->col, font ? font->font : 0);
+	}
+
+END_PROPERTY
+
+BEGIN_PROPERTY(CGRIDVIEWITEM_alignment)
+
+	if (READ_PROPERTY) 
+		GB.ReturnInteger(GRIDVIEW->itemAlignment(THIS->row,THIS->col));
+	else
+		GRIDVIEW->setItemAlignment( THIS->row, THIS->col,VPROP(GB_INTEGER) );
+
+END_PROPERTY
+
+/*BEGIN_PROPERTY(CGRIDVIEWITEM_key)
 
 	GB.ReturnInteger(THIS->row);
 
@@ -232,7 +262,7 @@ BEGIN_PROPERTY(CGRIDVIEWITEM_column)
 
 	GB.ReturnInteger(THIS->col);
 
-END_PROPERTY
+END_PROPERTY*/
 
 BEGIN_PROPERTY(CGRIDVIEWITEM_x)
 
@@ -256,23 +286,23 @@ BEGIN_PROPERTY(CGRIDVIEWITEM_h)
 
 	GB.ReturnInteger(GRIDVIEW->itemH(THIS->row));
 
-END_PROPERTY
-
-BEGIN_PROPERTY(CGRIDVIEWITEM_alignment)
-
-
-
 END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDVIEWITEM_text)
 
 	if (READ_PROPERTY) 
-	{ 
-		GB.ReturnNewString(GRIDVIEW->itemText(THIS->row,THIS->col),0); 
-		return; 
-	}
+		GB.ReturnNewZeroString(GRIDVIEW->itemText(THIS->row,THIS->col)); 
+	else
+		GRIDVIEW->setItemText(THIS->row, THIS->col, GB.ToZeroString(PROP(GB_STRING)));
 
-	GRIDVIEW->setItemText( THIS->row, THIS->col,PROP(GB_STRING)->value.addr );
+END_PROPERTY
+
+BEGIN_PROPERTY(CGRIDVIEWITEM_rich_text)
+
+	if (READ_PROPERTY) 
+		GB.ReturnNewZeroString(GRIDVIEW->itemRichText(THIS->row,THIS->col)); 
+	else
+		GRIDVIEW->setItemRichText(THIS->row, THIS->col, GB.ToZeroString(PROP(GB_STRING)));
 
 END_PROPERTY
 
@@ -300,31 +330,16 @@ BEGIN_PROPERTY(CGRIDVIEWITEM_fg)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEWITEM_xpad)
+BEGIN_PROPERTY(CGRIDVIEWITEM_padding)
 
 	if (READ_PROPERTY) 
-	{ 
-		GB.ReturnInteger(GRIDVIEW->itemXPad(THIS->row,THIS->col)); 
-		return; 
-	}
-
-	GRIDVIEW->setItemXPad( THIS->row, THIS->col,VPROP(GB_INTEGER) );
+		GB.ReturnInteger(GRIDVIEW->itemPadding(THIS->row,THIS->col));
+	else
+		GRIDVIEW->setItemPadding( THIS->row, THIS->col,VPROP(GB_INTEGER) );
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEWITEM_ypad)
-
-	if (READ_PROPERTY) 
-	{ 
-		GB.ReturnInteger(GRIDVIEW->itemYPad(THIS->row,THIS->col)); 
-		return; 
-	}
-
-	GRIDVIEW->setItemYPad( THIS->row, THIS->col,VPROP(GB_INTEGER) );
-
-END_PROPERTY
-
-BEGIN_PROPERTY(CGRIDVIEWITEM_selected)
+/*BEGIN_PROPERTY(CGRIDVIEWITEM_selected)
 
 	if (READ_PROPERTY) 
 	{ 
@@ -334,7 +349,7 @@ BEGIN_PROPERTY(CGRIDVIEWITEM_selected)
 
 	GRIDVIEW->setItemSelected( THIS->row, THIS->col,VPROP(GB_BOOLEAN) );
 
-END_PROPERTY
+END_PROPERTY*/
 
 BEGIN_METHOD_VOID(CGRIDVIEWITEM_clear)
 
@@ -353,6 +368,7 @@ BEGIN_METHOD_VOID(CGRIDVIEWITEM_ensure_visible)
 	GRIDVIEW->ensureVisible(THIS->row,THIS->col);
 
 END_METHOD
+
 /*************************************************
 
  GridViewColumns
@@ -375,7 +391,7 @@ END_METHOD
 
 BEGIN_PROPERTY(CGRIDVIEW_columns_resizable)
 
-	long bc;
+	int bc;
 
 	if (READ_PROPERTY) 
 	{ 
@@ -414,7 +430,7 @@ BEGIN_PROPERTY(CGRIDVIEW_column_width)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEW_columns_width)
+BEGIN_PROPERTY(CGRIDCOLS_width)
 
 	int bc;
 
@@ -437,11 +453,25 @@ BEGIN_PROPERTY(CGRIDVIEW_column_resizable)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(CGRIDCOLS_height)
+
+	GB.ReturnInteger(WIDGET->headerHeight());
+
+END_PROPERTY
+
+BEGIN_METHOD_VOID(CGRIDCOL_refresh)
+
+	GRIDVIEW->queryUpdate(-1, THIS->col);
+
+END_METHOD
+
+
 /*************************************************
 
  GridViewRows
  
 **************************************************/
+
 BEGIN_PROPERTY(CGRIDVIEW_count)
 
 	if (READ_PROPERTY) { GB.ReturnInteger(GRIDVIEW->rowCount()); return; }
@@ -460,7 +490,7 @@ END_PROPERTY
 
 BEGIN_METHOD(CGRIDVIEW_rows_get,GB_INTEGER Row;)
 
-	if ( (VARG(Row)<0) || (VARG(Row)>=GRIDVIEW->columnCount() ) )
+	if ( (VARG(Row)<0) || (VARG(Row)>=GRIDVIEW->rowCount() ) )
 	{
 		GB.Error("Bad row index");
 		GB.ReturnNull();
@@ -472,7 +502,7 @@ BEGIN_METHOD(CGRIDVIEW_rows_get,GB_INTEGER Row;)
 
 END_METHOD
 
-BEGIN_PROPERTY(CGRIDVIEW_rows_height)
+BEGIN_PROPERTY(CGRIDROWS_height)
 
 	int bc;
 
@@ -482,6 +512,75 @@ BEGIN_PROPERTY(CGRIDVIEW_rows_height)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(CGRIDROWS_width)
+
+	GB.ReturnInteger(WIDGET->rowWidth());
+
+END_PROPERTY
+
+BEGIN_METHOD(CGRIDROWS_select, GB_INTEGER start; GB_INTEGER length)
+
+	int start, length;
+
+	start = VARGOPT(start, 0);
+	if (start < 0)
+		start = 0;
+
+	length = VARGOPT(length, WIDGET->rowCount() - start);
+	if (length < 0)
+		length = 0;
+
+	WIDGET->clearSelection();
+	WIDGET->selectRows(start, length);
+
+END_PROPERTY
+
+
+BEGIN_METHOD(CGRIDROWS_select_all, GB_BOOLEAN sel)
+
+	WIDGET->clearSelection();
+	if (VARGOPT(sel, TRUE))
+		WIDGET->selectRows(0, WIDGET->rowCount());
+
+END_PROPERTY
+
+BEGIN_METHOD_VOID(CGRIDROWS_unselect)
+
+	GRIDVIEW->clearSelection();
+
+END_METHOD
+
+BEGIN_METHOD(CGRIDROWS_remove, GB_INTEGER start; GB_INTEGER length)
+
+	int start = VARG(start);
+	int length = VARGOPT(length, 1);
+
+	if (start < 0 || start >= GRIDVIEW->rowCount() || length <= 0 || (start + length) > GRIDVIEW->rowCount())
+	{
+		GB.Error(GB_ERR_ARG);
+		return;
+	}
+	
+	GRIDVIEW->removeRows(start, length);
+	
+END_METHOD
+
+
+BEGIN_METHOD(CGRIDROWS_insert, GB_INTEGER start; GB_INTEGER length)
+
+	int start = VARG(start);
+	int length = VARGOPT(length, 1);
+
+	if (start < 0 || length <= 0 || start > GRIDVIEW->rowCount())
+	{
+		GB.Error(GB_ERR_ARG);
+		return;
+	}
+	
+	GRIDVIEW->insertRows(start, length);
+
+END_METHOD
+
 BEGIN_PROPERTY(CGRIDVIEW_rows_resizable)
 
 	int bc;
@@ -508,17 +607,25 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDVIEW_row_selected)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(GRIDVIEW->rowSelected(THIS->row)); return; }
-	GRIDVIEW->setRowSelected(THIS->row,VPROP(GB_BOOLEAN));
+	if (READ_PROPERTY) 
+		GB.ReturnBoolean(GRIDVIEW->rowSelected(THIS->row));
+	else
+		GRIDVIEW->setRowSelected(THIS->row,VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
+BEGIN_METHOD_VOID(CGRIDROW_refresh)
+
+	GRIDVIEW->queryUpdate(THIS->row, -1);
+
+END_METHOD
 
 /******************************************************
 
  GridView
  
 *******************************************************/
+
 BEGIN_PROPERTY(CGRIDVIEW_scrollbar)
 
 	if (READ_PROPERTY) { GB.ReturnInteger(GRIDVIEW->scrollBar()); return; }
@@ -573,40 +680,38 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDVIEW_current)
 	
-	long row,col;
-
-	GRIDVIEW->getGridCursor(&row,&col);
-	if ( (col==-1) || (row==-1) ) { GB.ReturnNull(); return; }
-	THIS->row=row;
-	THIS->col=col;
-	RETURN_SELF();
+	GRIDVIEW->getCursor(&THIS->row, &THIS->col);
+	
+	if (THIS->row < 0 || THIS->col < 0)
+		GB.ReturnNull(); 
+	else
+		RETURN_SELF();
 	
 END_PROPERTY
 
-BEGIN_METHOD(CGRIDVIEW_new, GB_OBJECT Parent;)
+BEGIN_METHOD(CGRIDVIEW_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
+	InitControl(new gGridView(CONTAINER(VARG(parent))),(CWIDGET*)THIS);
 
 	THIS->row=-1;
 	THIS->col=-1;
 
-	THIS->widget=new gGridView(Parent->widget);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
-
-	GRIDVIEW->setDataFunc((void*)grid_data_func,_object);
-	GRIDVIEW->onActivate=grid_activate;
-	GRIDVIEW->onClick=grid_click;
-	GRIDVIEW->onRowClick=grid_row_click;
-	GRIDVIEW->onColumnClick=grid_col_click;
-	GRIDVIEW->onFooterClick=grid_foot_click;
-	GRIDVIEW->onScroll=grid_scroll;
+	GRIDVIEW->setDataFunc((void*)raise_data, _object);
+	GRIDVIEW->onActivate = raise_activate;
+	GRIDVIEW->onClick = raise_click;
+	GRIDVIEW->onChange = raise_change;
+	GRIDVIEW->onRowClick = raise_row_click;
+	GRIDVIEW->onColumnClick = raise_col_click;
+	GRIDVIEW->onFooterClick = raise_foot_click;
+	GRIDVIEW->onScroll = raise_scroll;
+	GRIDVIEW->onRowResize = raise_row_resize;
+	GRIDVIEW->onColumnResize = raise_col_resize;
 
 END_METHOD
 
 BEGIN_METHOD(CGRIDVIEW_find, GB_INTEGER X; GB_INTEGER Y;)
 
-	long px,py;
+	int px,py;
 
 	px=GRIDVIEW->rowAt(VARG(Y));
 	py=GRIDVIEW->columnAt(VARG(X));
@@ -622,20 +727,14 @@ END_METHOD
 
 BEGIN_METHOD(CGRIDVIEW_rowat, GB_INTEGER X;)
 
-	long px;
-
 	GB.ReturnInteger(GRIDVIEW->rowAt(VARG(X)));
 
-
 END_METHOD
 
 BEGIN_METHOD(CGRIDVIEW_colat, GB_INTEGER X;)
 
-	long px;
-
 	GB.ReturnInteger(GRIDVIEW->columnAt(VARG(X)));
 
-
 END_METHOD
 
 BEGIN_METHOD_VOID(CGRIDVIEW_clear)
@@ -670,38 +769,39 @@ BEGIN_METHOD(CGRIDVIEW_get, GB_INTEGER Key; GB_INTEGER Column;)
 
 END_METHOD
 
+
 BEGIN_PROPERTY(CGRIDVIEW_row)
 
-	int row;
-
-	if (READ_PROPERTY) { GB.ReturnInteger(THIS->row); return; }
-
-	if ( (!GRIDVIEW->rowCount()) || (!GRIDVIEW->columnCount()) ) return;
-	
-	row=VPROP(GB_INTEGER);
-	if (row<0) row=-1;
-	if (row>=GRIDVIEW->rowCount()) row=GRIDVIEW->rowCount()-1;
-	THIS->row=row;
+	int row, col;
 	
+	WIDGET->getCursor(&row, &col);
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(row);
+	else
+		WIDGET->setCursor(VPROP(GB_INTEGER), col);
 
 END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDVIEW_column)
 
-	int col;
-
-	if (READ_PROPERTY) { GB.ReturnInteger(THIS->col); return; }
-
-	if ( (!GRIDVIEW->rowCount()) || (!GRIDVIEW->columnCount()) ) return;
-	
-	col=VPROP(GB_INTEGER);
-	if (col<0) col=-1;
-	if (col>=GRIDVIEW->columnCount()) col=GRIDVIEW->columnCount()-1;
-	THIS->col=col;
+	int row, col;
 	
+	WIDGET->getCursor(&row, &col);
+
+	if (READ_PROPERTY)
+		GB.ReturnInteger(col);
+	else
+		WIDGET->setCursor(row, VPROP(GB_INTEGER));
 
 END_PROPERTY
 
+BEGIN_METHOD(CGRIDVIEW_move_to, GB_INTEGER row; GB_INTEGER col)
+
+  WIDGET->setCursor(VARG(row), VARG(col));
+
+END_METHOD
+
 BEGIN_PROPERTY(CGRIDVIEW_border)
 
 	if (READ_PROPERTY) { GB.ReturnBoolean(GRIDVIEW->getBorder()); return; }
@@ -722,13 +822,6 @@ BEGIN_PROPERTY(CGRIDVIEW_mode)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEW_item)
-
-	THIS->row=THIS->curr_row;
-	THIS->col=THIS->curr_col;
-	RETURN_SELF();
-
-END_PROPERTY
 
 /***************************************************************************
 
@@ -737,25 +830,18 @@ Headers, Footers, and Row Separators
 ****************************************************************************/
 BEGIN_PROPERTY(CGRIDVIEWHEADER_visible)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(GRIDVIEW->headersVisible()); return; }
-	GRIDVIEW->setHeadersVisible(VPROP(GB_BOOLEAN));
+	if (READ_PROPERTY) { GB.ReturnInteger(GRIDVIEW->headersVisible()); return; }
+	GRIDVIEW->setHeadersVisible(VPROP(GB_INTEGER));
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEWFOOTER_visible)
+BEGIN_PROPERTY(CGRIDVIEW_footer)
 
 	if (READ_PROPERTY) { GB.ReturnBoolean(GRIDVIEW->footersVisible()); return; }
 	GRIDVIEW->setFootersVisible(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEWROWSEP_visible)
-
-	if (READ_PROPERTY) { GB.ReturnBoolean(GRIDVIEW->rowSeparatorVisible()); return; }
-	GRIDVIEW->setRowSeparatorVisible(VPROP(GB_BOOLEAN));
-
-END_PROPERTY
-
 BEGIN_PROPERTY(CGRIDVIEW_column_headertext)
 
 	if (READ_PROPERTY)
@@ -767,7 +853,7 @@ BEGIN_PROPERTY(CGRIDVIEW_column_headertext)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(CGRIDVIEW_column_footertext)
+BEGIN_PROPERTY(CGRIDVIEW_column_footer_text)
 
 	if (READ_PROPERTY)
 	{
@@ -788,17 +874,19 @@ BEGIN_PROPERTY(CGRIDVIEW_row_text)
 	GRIDVIEW->setRowText(THIS->row,PROP(GB_STRING)->value.addr);
 
 END_PROPERTY
+
 /***************************************************************************
 
   Gambas Interfaces
 
 ***************************************************************************/
+
 GB_DESC CGridViewItemDesc[] =
 {
-  GB_DECLARE(".GridCell", 0), GB_VIRTUAL_CLASS(),
+  GB_DECLARE(".GridViewCell", 0), GB_VIRTUAL_CLASS(),
   
-  GB_PROPERTY_READ("Row", "i", CGRIDVIEWITEM_key),
-  GB_PROPERTY_READ("Column", "i", CGRIDVIEWITEM_column),  
+  //GB_PROPERTY_READ("Row", "i", CGRIDVIEWITEM_key),
+  //GB_PROPERTY_READ("Column", "i", CGRIDVIEWITEM_column),  
   GB_PROPERTY_READ("X", "i", CGRIDVIEWITEM_x),
   GB_PROPERTY_READ("Y", "i", CGRIDVIEWITEM_y),
   GB_PROPERTY_READ("Left", "i", CGRIDVIEWITEM_x),
@@ -808,16 +896,17 @@ GB_DESC CGridViewItemDesc[] =
   GB_PROPERTY_READ("W", "i", CGRIDVIEWITEM_w),
   GB_PROPERTY_READ("H", "i", CGRIDVIEWITEM_h),
   
-  GB_PROPERTY("Picture", "Picture", CGRIDVIEWDATA_picture),
+  GB_PROPERTY("Picture", "Picture", CGRIDVIEWITEM_picture),
   GB_PROPERTY("Text", "s", CGRIDVIEWITEM_text),
+  GB_PROPERTY("RichText", "s", CGRIDVIEWITEM_rich_text),
   GB_PROPERTY("Background", "i", CGRIDVIEWITEM_bg),
   GB_PROPERTY("BackColor", "i", CGRIDVIEWITEM_bg),
   GB_PROPERTY("Foreground", "i", CGRIDVIEWITEM_fg),
   GB_PROPERTY("ForeColor", "i", CGRIDVIEWITEM_fg),
-  GB_PROPERTY("XPad", "i", CGRIDVIEWITEM_xpad),
-  GB_PROPERTY("YPad", "i", CGRIDVIEWITEM_ypad),
+  GB_PROPERTY("Padding", "i", CGRIDVIEWITEM_padding),
   GB_PROPERTY("Alignment", "i", CGRIDVIEWITEM_alignment),
-  GB_PROPERTY("Selected","b",CGRIDVIEWITEM_selected),
+  GB_PROPERTY("Font", "Font", CGRIDVIEWITEM_font),
+  //GB_PROPERTY("Selected","b",CGRIDVIEWITEM_selected),
 
   GB_METHOD("Clear", NULL, CGRIDVIEWITEM_clear, NULL),
   GB_METHOD("Refresh",NULL,CGRIDVIEWITEM_refresh,NULL),
@@ -827,17 +916,18 @@ GB_DESC CGridViewItemDesc[] =
 
 GB_DESC CGridViewDataDesc[] =
 {
-  GB_DECLARE(".GridData", 0), GB_VIRTUAL_CLASS(),
+  GB_DECLARE(".GridViewData", 0), GB_VIRTUAL_CLASS(),
 
-  GB_PROPERTY("Picture", "Picture", CGRIDVIEWITEM_picture),
+  GB_PROPERTY("Picture", "Picture", CGRIDVIEWDATA_picture),
   GB_PROPERTY("Text", "s", CGRIDVIEWDATA_text),
+  GB_PROPERTY("RichText", "s", CGRIDVIEWDATA_rich_text),
   GB_PROPERTY("Background", "i", CGRIDVIEWDATA_bg),
   GB_PROPERTY("BackColor", "i", CGRIDVIEWDATA_bg),
   GB_PROPERTY("Foreground", "i", CGRIDVIEWDATA_fg),
   GB_PROPERTY("ForeColor", "i", CGRIDVIEWDATA_fg),
-  GB_PROPERTY("XPad", "i", CGRIDVIEWDATA_xpad),
-  GB_PROPERTY("YPad", "i", CGRIDVIEWDATA_ypad),
+  GB_PROPERTY("Padding", "i", CGRIDVIEWDATA_padding),
   GB_PROPERTY("Alignment", "i", CGRIDVIEWDATA_alignment),
+  GB_PROPERTY("Font", "Font", CGRIDVIEWDATA_font),
 
   GB_END_DECLARE
 };
@@ -846,41 +936,61 @@ GB_DESC CGridViewColumnDesc[] =
 {
   GB_DECLARE(".GridViewColumn", 0), GB_VIRTUAL_CLASS(),
   GB_PROPERTY("Resizable","b",CGRIDVIEW_column_resizable),
+  GB_PROPERTY_READ("X", "i", CGRIDVIEWITEM_x),
+  GB_PROPERTY_READ("Left", "i", CGRIDVIEWITEM_x),
   GB_PROPERTY("Width", "i", CGRIDVIEW_column_width),
   GB_PROPERTY("W", "i", CGRIDVIEW_column_width),
-  GB_PROPERTY("HeaderText","s",CGRIDVIEW_column_headertext),
-  GB_PROPERTY("FooterText","s",CGRIDVIEW_column_footertext),
+  GB_PROPERTY("Text","s",CGRIDVIEW_column_headertext),
+  GB_PROPERTY("Title","s",CGRIDVIEW_column_headertext),
+  //GB_PROPERTY("HeaderText","s",CGRIDVIEW_column_headertext),
+  //GB_PROPERTY("FooterText","s",CGRIDVIEW_column_footertext),
+  GB_METHOD("Refresh", NULL, CGRIDCOL_refresh, NULL),
   GB_END_DECLARE
 };
 
 GB_DESC CGridViewColumnsDesc[] =
 {
-  GB_DECLARE(".GridColumns", 0), GB_VIRTUAL_CLASS(),
+  GB_DECLARE(".GridViewColumns", 0), GB_VIRTUAL_CLASS(),
   GB_METHOD("_get", ".GridViewColumn", CGRIDVIEW_columns_get, "(Column)i"),
   GB_PROPERTY("Resizable","b",CGRIDVIEW_columns_resizable),
   GB_PROPERTY("Count", "i", CGRIDVIEW_columns_count),
-  GB_PROPERTY("Width", "i", CGRIDVIEW_columns_width),
+  GB_PROPERTY("Width", "i", CGRIDCOLS_width),
+  GB_PROPERTY("W", "i", CGRIDCOLS_width),
+  GB_PROPERTY_READ("Height", "i", CGRIDCOLS_height),
+  GB_PROPERTY_READ("H", "i", CGRIDCOLS_height),
   GB_END_DECLARE
 };
 
 GB_DESC CGridViewRowDesc[] =
 {
   GB_DECLARE(".GridViewRow", 0), GB_VIRTUAL_CLASS(),
+  GB_PROPERTY_READ("Y", "i", CGRIDVIEWITEM_y),
+  GB_PROPERTY_READ("Top", "i", CGRIDVIEWITEM_y),
   GB_PROPERTY("Height", "i", CGRIDVIEW_row_height),
   GB_PROPERTY("H", "i", CGRIDVIEW_row_height),
   GB_PROPERTY("Resizable","b",CGRIDVIEW_row_resizable),
   GB_PROPERTY("Text","s",CGRIDVIEW_row_text),
+  GB_PROPERTY("Title","s",CGRIDVIEW_row_text),
   GB_PROPERTY("Selected","b",CGRIDVIEW_row_selected),
+  GB_METHOD("Refresh", NULL, CGRIDROW_refresh, NULL),
   GB_END_DECLARE
 };
 
 GB_DESC CGridViewRowsDesc[] =
 {
-  GB_DECLARE(".GridRows", 0), GB_VIRTUAL_CLASS(),
+  GB_DECLARE(".GridViewRows", 0), GB_VIRTUAL_CLASS(),
   GB_METHOD("_get", ".GridViewRow", CGRIDVIEW_rows_get, "(Row)i"),
   GB_PROPERTY("Count", "i", CGRIDVIEW_count),
-  GB_PROPERTY("Height", "i", CGRIDVIEW_rows_height),
   GB_PROPERTY("Resizable","b",CGRIDVIEW_rows_resizable),
+  GB_PROPERTY("Height", "i", CGRIDROWS_height),
+  GB_PROPERTY("H", "i", CGRIDROWS_height),
+  GB_PROPERTY_READ("Width", "i", CGRIDROWS_width),
+  GB_PROPERTY_READ("W", "i", CGRIDROWS_width),
+  GB_METHOD("Select", NULL, CGRIDROWS_select, "[(Start)i(Length)i]"),
+  GB_METHOD("SelectAll", NULL, CGRIDROWS_select_all, "[(Selected)b]"),
+  GB_METHOD("Unselect", NULL, CGRIDROWS_unselect, NULL),
+  GB_METHOD("Remove", NULL, CGRIDROWS_remove, "(Start)i[(Length)i]"),
+  GB_METHOD("Insert", NULL, CGRIDROWS_insert, "(Start)i[(Length)i]"),
   GB_END_DECLARE
 };
 
@@ -888,11 +998,13 @@ GB_DESC CGridViewDesc[] =
 {
   GB_DECLARE("GridView", sizeof(CGRIDVIEW)), GB_INHERITS("Control"),
 
-  GB_CONSTANT("_Properties", "s", CGRIDVIEW_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
+  GB_CONSTANT("None", "i", 0),
+  GB_CONSTANT("Horizontal", "i", 1),
+  GB_CONSTANT("Vertical", "i", 2),
+  GB_CONSTANT("Both", "i", 3),
 
   GB_METHOD("_new", NULL, CGRIDVIEW_new, "(Parent)Container;"),
-  GB_METHOD("_get", ".GridCell", CGRIDVIEW_get, "(Row)i(Column)i"),
+  GB_METHOD("_get", ".GridViewCell", CGRIDVIEW_get, "(Row)i(Column)i"),
   GB_METHOD("Clear", NULL, CGRIDVIEW_clear, NULL),
   GB_METHOD("Find", "b", CGRIDVIEW_find, "(X)i(Y)i"),
   GB_METHOD("RowAt","i",CGRIDVIEW_rowat,"(Y)i"),
@@ -900,20 +1012,24 @@ GB_DESC CGridViewDesc[] =
 
   GB_PROPERTY("Border", "b", CGRIDVIEW_border),
   GB_PROPERTY("Mode","i",CGRIDVIEW_mode),
-  GB_PROPERTY("ScrollBar", "i<Scroll>", CGRIDVIEW_scrollbar),
+  GB_PROPERTY("ScrollBar", "i", CGRIDVIEW_scrollbar),
+  GB_PROPERTY("Grid", "b", CGRIDVIEW_grid),
+  GB_PROPERTY("Resizable","b",CGRIDVIEW_columns_resizable),
+  GB_PROPERTY("Header", "i", CGRIDVIEWHEADER_visible),
+  //GB_PROPERTY("Footer", "b", CGRIDVIEWFOOTER_visible),
+    
   GB_PROPERTY("ScrollX", "i", CGRIDVIEW_scrollX),
   GB_PROPERTY("ScrollY", "i", CGRIDVIEW_scrollY),
-  GB_PROPERTY("Grid", "b", CGRIDVIEW_grid),
+  
   GB_PROPERTY("Row", "i", CGRIDVIEW_row),
   GB_PROPERTY("Column", "i", CGRIDVIEW_column),
-  GB_PROPERTY("Header", "b", CGRIDVIEWHEADER_visible),
-  GB_PROPERTY("Footer", "b", CGRIDVIEWFOOTER_visible),
-  GB_PROPERTY("RowSeparator", "b", CGRIDVIEWROWSEP_visible),
-  GB_PROPERTY("Data", ".GridData",CGRIDVIEW_data),
-  GB_PROPERTY_SELF("Columns", ".GridColumns"),
-  GB_PROPERTY_SELF("Rows", ".GridRows"),
-  GB_PROPERTY_READ("Current", ".GridCell", CGRIDVIEW_current),
-  GB_PROPERTY_READ("Item", ".GridCell", CGRIDVIEW_item),
+  GB_METHOD("MoveTo", NULL, CGRIDVIEW_move_to, "(Row)i(Column)i"),
+  
+  GB_PROPERTY_READ("Data", ".GridViewData",CGRIDVIEW_data),
+  
+  GB_PROPERTY_SELF("Columns", ".GridViewColumns"),
+  GB_PROPERTY_SELF("Rows", ".GridViewRows"),
+  GB_PROPERTY_READ("Current", ".GridViewCell", CGRIDVIEW_current),
   GB_PROPERTY_READ("ClientX", "i", CGRIDVIEW_client_x),
   GB_PROPERTY_READ("ClientY", "i",  CGRIDVIEW_client_y),
   GB_PROPERTY_READ("ClientWidth", "i", CGRIDVIEW_client_width),
@@ -921,13 +1037,18 @@ GB_DESC CGridViewDesc[] =
   GB_PROPERTY_READ("ClientHeight", "i", CGRIDVIEW_client_height),
   GB_PROPERTY_READ("ClientH", "i", CGRIDVIEW_client_height),
 
+  GB_EVENT("Change", NULL, NULL, &EVENT_Change),
+  GB_EVENT("Select", NULL, NULL, &EVENT_Select),
   GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
-  GB_EVENT("RowClick",NULL,"(Row)i",&EVENT_RowClick),
-  GB_EVENT("ColumnClick",NULL,"(Column)i",&EVENT_ColumnClick),
-  GB_EVENT("FooterClick",NULL,"(Column)i",&EVENT_FooterClick),
-  GB_EVENT("Scroll",NULL,NULL,&EVENT_Scroll),
-  GB_EVENT("Data",NULL,"(Row)i(Column)i",&EVENT_Data),
+  GB_EVENT("Scroll", NULL, NULL, &EVENT_Scroll),
+  GB_EVENT("Data", NULL, "(Row)i(Column)i", &EVENT_Data),
+  GB_EVENT("ColumnClick", NULL, "(Column)i", &EVENT_ColumnClick),
+  GB_EVENT("RowClick", NULL, "(Row)i", &EVENT_RowClick),
+  GB_EVENT("ColumnResize", NULL, "(Column)i", &EVENT_ColumnResize),
+  GB_EVENT("RowResize", NULL, "(Row)i", &EVENT_RowResize),
 
+	GRIDVIEW_DESCRIPTION,
+	
   GB_END_DECLARE
 };
