@@ -29,8 +29,8 @@
 #ifndef __CLISTBOX_H
 #define __CLISTBOX_H
 
-#include "gambas.h"
-#include "widgets.h"
+#include "main.h"
+#include "glistbox.h"
 
 #ifndef __CLISTBOX_CPP
 extern GB_DESC CListBoxDesc[];
@@ -41,9 +41,6 @@ extern GB_DESC CListBoxItemDesc[];
 #define THIS   ((CLISTBOX *)_object)
 #define LISTBOX ((gListBox *)THIS->widget)
 
-
-#define CLISTBOX_PROPERTIES CWIDGET_PROPERTIES ",List,Mode{Select.*},Sorted"
-
 #endif
 
 
