@@ -29,7 +29,8 @@
 #ifndef __CMESSAGE_H
 #define __CMESSAGE_H
 
-#include "gambas.h"
+#include "main.h"
+#include "gmessage.h"
 
 #ifndef __CMESSAGE_CPP
 extern GB_DESC CMessageDesc[];
