@@ -29,10 +29,9 @@
 #ifndef __CPROGRESS_H
 #define __CPROGRESS_H
 
-#include "gambas.h"
-#include "widgets.h"
-
+#include "main.h"
 #include "CWidget.h"
+#include "gprogressbar.h"
 
 #ifndef __CPROGRESS_CPP
 extern GB_DESC CProgressDesc[];
