@@ -28,14 +28,13 @@
 
 #define __CSCREEN_CPP
 
-#include <stdio.h>
-#include "gambas.h"
-#include "main.h"
-#include "widgets.h"
-#include "CScreen.h"
 #include "CWindow.h"
 #include "CPicture.h"
 #include "CFont.h"
+#include "CScreen.h"
+
+#include "gapplication.h"
+#include "gmainwindow.h"
 
 extern long CWINDOW_Embedder;
 extern bool CWINDOW_Embedded;
@@ -44,6 +43,8 @@ extern int MAIN_scale;
 
 char *CAPPLICATION_Theme = 0;
 
+static int _busy = 0;
+
 BEGIN_PROPERTY(CSCREEN_width)
 
 	GB.ReturnInteger(gDesktop::width());
@@ -83,6 +84,23 @@ BEGIN_METHOD_VOID(CSCREEN_grab)
 
 END_METHOD
 
+
+static void set_font(gFont *font, void *object = 0)
+{
+  gDesktop::setFont(font);
+	MAIN_scale = gDesktop::scale();
+}
+
+BEGIN_PROPERTY(CAPP_font)
+
+  if (READ_PROPERTY)
+    GB.ReturnObject(CFONT_create(gDesktop::font(), set_font));
+  else if (VPROP(GB_OBJECT))
+    set_font(((CFONT*)VPROP(GB_OBJECT))->font);
+
+END_PROPERTY
+
+#if 0
 BEGIN_PROPERTY(CSCREEN_font)
 
   	CFONT *Font;
@@ -102,17 +120,13 @@ BEGIN_PROPERTY(CSCREEN_font)
 	if (!Font) return;
 	if (!Font->font) return;
 	gDesktop::setFont(Font->font);
-	MAIN_scale=gDesktop::scale();
 
 END_PROPERTY
-
+#endif
 
 BEGIN_PROPERTY(CSCREEN_active_window)
 
-  gMainWindow *win=gDesktop::activeWindow();
-  
-  if (!win) { GB.ReturnNull(); return; }
-  GB.ReturnObject(GetObject(win));
+	GB.ReturnObject(CWINDOW_Active);
 
 END_PROPERTY
 
@@ -125,14 +139,23 @@ BEGIN_PROPERTY(CSCREEN_active_control)
 
 END_PROPERTY
 
-int CSCREEN_busy_count=0;
-
 BEGIN_PROPERTY(CSCREEN_busy)
 
-  stub ("CSCREEN_busy");
-  
-  if (READ_PROPERTY) { GB.ReturnInteger(CSCREEN_busy_count); return; }
-  CSCREEN_busy_count=VPROP(GB_INTEGER);
+  int busy;
+
+  if (READ_PROPERTY)
+    GB.ReturnInteger(_busy);
+  else
+  {
+    busy = VPROP(GB_INTEGER);
+
+    if (_busy == 0 && busy != 0)
+      gApplication::setBusy(true);
+    else if (_busy > 0 && busy == 0)
+      gApplication::setBusy(false);
+
+    _busy = busy;
+  }
 
 END_PROPERTY
 
@@ -160,11 +183,7 @@ BEGIN_PROPERTY(CAPP_tooltip_font)
 	
 	if (READ_PROPERTY)
 	{
-		GB.New((void **)&Font, GB.FindClass("Font"), 0, 0);
-		Font->font->unref();
-		Font->font=gApplication::toolTipsFont();
-		Font->font->ref();
-		GB.ReturnObject(Font);
+		GB.ReturnObject(CFONT_create(gApplication::toolTipsFont()));
 		return;
 	}
 	
@@ -172,7 +191,6 @@ BEGIN_PROPERTY(CAPP_tooltip_font)
 	if (!Font) return;
 	if (!Font->font) return;
 	gApplication::setToolTipsFont(Font->font);
-
   
 END_PROPERTY
 
@@ -249,10 +267,10 @@ GB_DESC CApplicationDesc[] =
 
   GB_STATIC_METHOD("_exit", NULL, CAPP_exit, NULL),
 
-  GB_STATIC_PROPERTY("Font", "Font", CSCREEN_font),
-  GB_STATIC_PROPERTY_READ("ActiveControl","o",CSCREEN_active_control),
-  GB_STATIC_PROPERTY_READ("ActiveWindow", "o", CSCREEN_active_window),
-  GB_STATIC_PROPERTY_READ("MainWindow", "o", CAPP_main_window),
+  GB_STATIC_PROPERTY("Font", "Font", CAPP_font),
+  GB_STATIC_PROPERTY_READ("ActiveControl","Control",CSCREEN_active_control),
+  GB_STATIC_PROPERTY_READ("ActiveWindow", "Window", CSCREEN_active_window),
+  GB_STATIC_PROPERTY_READ("MainWindow", "Window", CAPP_main_window),
   GB_STATIC_PROPERTY("Busy", "i", CSCREEN_busy),
   GB_STATIC_PROPERTY_SELF("ToolTip", ".ApplicationTooltip"),
   
