@@ -29,9 +29,8 @@
 #ifndef __CSCROLLVIEW_H
 #define __CSCROLLVIEW_H
 
-#include "gambas.h"
-#include "widgets.h"
-
+#include "main.h"
+#include "gscrollview.h"
 #include "CWidget.h"
 #include "CContainer.h"
 
@@ -42,9 +41,6 @@ extern GB_DESC CScrollViewDesc[];
 #define THIS    ((CSCROLLVIEW *)_object)
 #define SCROLLVIEW  ((gScrollView *)THIS->widget)
 
-#define CSCROLLVIEW_PROPERTIES CWIDGET_PROPERTIES \
-  "," "Arrangement,Spacing,Padding" ",Border,ScrollBar"
-
 #endif
 
 typedef  struct
