@@ -29,10 +29,9 @@
 #ifndef __CSEPARATOR_H
 #define __CSEPARATOR_H
 
-#include "widgets.h"
-#include "gambas.h"
+#include "main.h"
 #include "CWidget.h"
-
+#include "gseparator.h"
 
 #ifndef __CSEPARATOR_CPP
 extern GB_DESC CSeparatorDesc[];
