@@ -29,8 +29,8 @@
 #ifndef __CSPINBOX_H
 #define __CSPINBOX_H
 
-#include "gambas.h"
-#include "widgets.h"
+#include "main.h"
+#include "gspinbox.h"
 
 #ifndef __CSPINBOX_CPP
 
@@ -41,9 +41,6 @@ extern GB_DESC CSpinBoxDesc[];
 #define THIS    ((CSPINBOX *)_object)
 #define SPINBOX  ((gSpinBox*)THIS->widget)
 
-#define CSPINBOX_PROPERTIES CWIDGET_PROPERTIES \
-  ",Value,MinValue,MaxValue,Step,Prefix,Suffix,Wrap"
-
 #endif
 
 typedef  struct
