@@ -29,8 +29,8 @@
 #ifndef __CSPLITTER_H
 #define __CSPLITTER_H
 
-#include "gambas.h"
-#include "widgets.h"
+#include "main.h"
+#include "gsplitter.h"
 #include "CWidget.h"
 #include "CContainer.h"
 
@@ -43,10 +43,6 @@ extern GB_DESC CVSplitDesc[];
 
 #define WIDGET ((gSplitter *)((CWIDGET *)_object)->widget)
 
-#define CSPLITTER_PROPERTIES CWIDGET_PROPERTIES
-
-
-
 #endif
 
 typedef  struct
