@@ -40,31 +40,31 @@
 
 
 DECLARE_EVENT(EVENT_Change);
-DECLARE_EVENT(EVENT_Activate); /*TODO in ComboBox*/
-DECLARE_EVENT(EVENT_Click);    /*TODO in ComboBox*/
+DECLARE_EVENT(EVENT_Activate);
+DECLARE_EVENT(EVENT_Click);
 
-void tBox_raise_Change(gTextBox *sender)
+/*static void txt_post_change(void *_object)
 {
-	CWIDGET *_ob=GetObject((gControl*)sender);
-	
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Change,0);
+	GB.Raise(THIS, EVENT_Change, 0);
+	GB.Unref(POINTER(&_object));
+}*/
+
+static void txt_raise_change(gTextBox *sender)
+{
+	CWIDGET *_object = GetObject((gControl*)sender);
+	GB.Raise(THIS, EVENT_Change, 0);
 }
 
-void tBox_raise_Activate(gTextBox *sender)
+static void txt_raise_activate(gTextBox *sender)
 {
-	CWIDGET *_ob=GetObject((gControl*)sender);
-	
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Activate,0);
+	CWIDGET *_object = GetObject((gControl*)sender);
+	GB.Raise(THIS, EVENT_Activate, 0);
 }
 
-void cBox_raise_Change(gComboBox *sender)
+static void cmb_raise_click(gComboBox *sender)
 {
-	CWIDGET *_ob=GetObject((gControl*)sender);
-	
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Click,0);
+	CWIDGET *_object = GetObject((gControl*)sender);
+	GB.Raise(THIS, EVENT_Click, 0);
 }
 
 /***************************************************************************
@@ -75,20 +75,16 @@ void cBox_raise_Change(gComboBox *sender)
 
 BEGIN_METHOD(CTEXTBOX_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	THIS->widget=new gTextBox(Parent->widget);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
-	TEXTBOX->onChange=tBox_raise_Change;
-	TEXTBOX->onActivate=tBox_raise_Activate;
+	InitControl(new gTextBox(CONTAINER(VARG(parent))), (CWIDGET*)THIS);
+	TEXTBOX->onChange = txt_raise_change;
+	TEXTBOX->onActivate = txt_raise_activate;
 	
 END_METHOD
 
 
 BEGIN_METHOD_VOID(CTEXTBOX_clear)
 
-	TEXTBOX->setText("");
+	TEXTBOX->clear();
 
 END_METHOD
 
@@ -102,13 +98,10 @@ END_METHOD
 
 BEGIN_PROPERTY(CTEXTBOX_text)
 
-	char *vl="";
-
-	if (READ_PROPERTY) { GB.ReturnNewString(TEXTBOX->text(),0); return; }
-	
-	if ( PROP(GB_STRING)->value.addr ) vl=PROP(GB_STRING)->value.addr;
-	TEXTBOX->setText(vl);
-
+	if (READ_PROPERTY)
+		GB.ReturnNewZeroString(TEXTBOX->text());
+	else
+		TEXTBOX->setText(GB.ToZeroString(PROP(GB_STRING)));
 
 END_PROPERTY
 
@@ -138,7 +131,7 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CTEXTBOX_read_only)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(TEXTBOX->readOnly()); return; }
+	if (READ_PROPERTY) { GB.ReturnBoolean(TEXTBOX->isReadOnly()); return; }
 	TEXTBOX->setReadOnly(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
@@ -174,6 +167,13 @@ BEGIN_PROPERTY(CTEXTBOX_selection)
 
 END_PROPERTY
 
+
+BEGIN_METHOD_VOID(CTEXTBOX_selected)
+
+  GB.ReturnBoolean(TEXTBOX->isSelected());
+
+END_METHOD
+
 /***************************************************************************
 
   .TextBox.Selection
@@ -244,59 +244,30 @@ END_METHOD
 
 BEGIN_METHOD(CCOMBOBOX_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	THIS->widget=new gComboBox(Parent->widget);
-	InitControl(THIS->widget,(CWIDGET*)THIS);
-	COMBOBOX->onChange=cBox_raise_Change;
-
-END_METHOD
-
-BEGIN_METHOD_VOID(CCOMBOBOX_popup)
-
-	COMBOBOX->popup();
-
-END_METHOD
-
-
-BEGIN_METHOD_VOID(CCOMBOBOX_clear)
-
-	COMBOBOX->clear();
+	InitControl(new gComboBox(CONTAINER(VARG(parent))), (CWIDGET*)THIS);
+	
+	COMBOBOX->onClick = cmb_raise_click;
+	COMBOBOX->onChange = txt_raise_change;
+	COMBOBOX->onActivate = txt_raise_activate;
 
 END_METHOD
 
 
 BEGIN_PROPERTY(CCOMBOBOX_text)
 
-	char *buf;
-
 	if (READ_PROPERTY)
-	{
-		buf=COMBOBOX->text();
-		GB.ReturnNewString(buf,0);
-		if (buf) g_free(buf);
-		return;
-	}
-	COMBOBOX->setText(GB.ToZeroString(PROP(GB_STRING)));
+		GB.ReturnNewZeroString(COMBOBOX->text());
+	else
+		COMBOBOX->setText(GB.ToZeroString(PROP(GB_STRING)));
 
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CCOMBOBOX_length)
+BEGIN_METHOD_VOID(CCOMBOBOX_popup)
 
-	GB.ReturnInteger(COMBOBOX->length());
+	COMBOBOX->popup();
 
-END_PROPERTY
-
-
-
-BEGIN_PROPERTY(CCOMBOBOX_read_only)
-
-	if (READ_PROPERTY) { GB.ReturnBoolean(COMBOBOX->readOnly()); return; }
-	COMBOBOX->setReadOnly(VPROP(GB_BOOLEAN));
-
-END_PROPERTY
+END_METHOD
 
 
 BEGIN_METHOD(CCOMBOBOX_get, GB_INTEGER index)
@@ -317,17 +288,10 @@ END_METHOD
 
 BEGIN_PROPERTY(CCOMBOBOX_item_text)
 
-	char *buf;
-
 	if (READ_PROPERTY)
-	{
-		buf=COMBOBOX->itemText(THIS->index);
-		GB.ReturnNewString(buf,0);
-		if (buf) g_free(buf);
-		return;
-	}
-	
-	COMBOBOX->setItemText(THIS->index,GB.ToZeroString(PROP(GB_STRING)));
+		GB.ReturnNewZeroString(COMBOBOX->itemText(THIS->index));
+	else	
+		COMBOBOX->setItemText(THIS->index,GB.ToZeroString(PROP(GB_STRING)));
 
 
 END_PROPERTY
@@ -393,60 +357,35 @@ END_METHOD
 
 BEGIN_PROPERTY(CCOMBOBOX_list)
 
-	GB_ARRAY Array=NULL;
-	char **buf=NULL;
-	char *ctmp;
-	long b=0,ct=0;
+	GB_ARRAY array;
+	int i;
+	char *text;
 	
 	if (READ_PROPERTY)
 	{
-		buf=COMBOBOX->list();
-		if (!buf) 
+		GB.Array.New(&array, GB_T_STRING, COMBOBOX->count());
+		for (i = 0; i < COMBOBOX->count(); i++)
 		{
-			GB.Array.New(&Array,GB_T_STRING,0);
-			GB.ReturnObject(Array);
-			return;
+			GB.NewString(&text, COMBOBOX->itemText(i), 0);
+			*((char **)GB.Array.Get(array, i)) = text;
 		}
-
-		while (buf[b++]) ct++;
-		GB.Array.New(&Array,GB_T_STRING,ct);
-		b=0;
-		while (buf[b])
-		{
-			ctmp=NULL;
-			GB.NewString(&ctmp,buf[b],strlen(buf[b]));
-			*((char **)GB.Array.Get(Array,b++)) = ctmp;
-		} 
-		b=0;
-		while(buf[b]) g_free(buf[b++]);
-		g_free(buf);
-		GB.ReturnObject(Array);
-		return;
+		
+		GB.ReturnObject(array);
 	}
-
-	
-	
-	Array=VPROP(GB_OBJECT);
-	ct=GB.Array.Count(Array);
-	if (!ct) COMBOBOX->setList(NULL);
-	buf=(char**)g_malloc(sizeof(char*)*(ct+1));
-	buf[ct]=NULL;
-	if (buf)
+	else
 	{
-		for (b=0;b<ct;b++)
-		{	
-			buf[b]=*((char **)GB.Array.Get(Array,b)); 
-			if (!buf[b]) buf[b]="";
+		array = VPROP(GB_OBJECT);
+		COMBOBOX->clear();
+		if (array)
+		{
+			for (i = 0; i < GB.Array.Count(array); i++)
+				COMBOBOX->add(*((char **)GB.Array.Get(array, i)));
 		}
 	}
-	COMBOBOX->setList(buf);
-	if (buf) g_free(buf);
-
 
 END_PROPERTY
 
 
-
 /***************************************************************************
 
   Descriptions
@@ -462,17 +401,11 @@ GB_DESC CTextBoxSelectionDesc[] =
   GB_PROPERTY_READ("Start", "i", CTEXTBOX_sel_start),
   GB_PROPERTY_READ("Pos", "i", CTEXTBOX_sel_start),
 
-  GB_METHOD("Clear", NULL, CTEXTBOX_sel_clear, NULL),
   GB_METHOD("Hide", NULL, CTEXTBOX_sel_clear, NULL),
-  GB_METHOD("All", NULL, CTEXTBOX_sel_all, NULL),
-  GB_METHOD("_call", NULL, CTEXTBOX_sel_select, "[(Start)i(Length)i]"),
 
   GB_END_DECLARE
 };
 
-
-
-
 GB_DESC CTextBoxDesc[] =
 {
   GB_DECLARE("TextBox", sizeof(CTEXTBOX)), GB_INHERITS("Control"),
@@ -480,7 +413,7 @@ GB_DESC CTextBoxDesc[] =
   GB_METHOD("_new", NULL, CTEXTBOX_new, "(Parent)Container;"),
 
   GB_PROPERTY("Text", "s", CTEXTBOX_text),
-  GB_PROPERTY("Alignment", "i<Align,Normal,Left,Center,Right>", CTEXTBOX_alignment),
+  GB_PROPERTY("Alignment", "i", CTEXTBOX_alignment),
   GB_PROPERTY_READ("Length", "i", CTEXTBOX_length),
   GB_PROPERTY("Pos", "i", CTEXTBOX_pos),
   GB_PROPERTY("ReadOnly", "b", CTEXTBOX_read_only),
@@ -489,16 +422,18 @@ GB_DESC CTextBoxDesc[] =
   GB_PROPERTY("MaxLength", "i", CTEXTBOX_max_length),
 
   GB_PROPERTY_SELF("Selection", ".TextBoxSelection"),
-  GB_PROPERTY_SELF("Select", ".TextBoxSelection"),
-
-  GB_CONSTANT("_Properties", "s", CTEXTBOX_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "KeyPress"),
+  GB_METHOD("Select", NULL, CTEXTBOX_sel_select, "[(Start)i(Length)i]"),
+  GB_METHOD("SelectAll", NULL, CTEXTBOX_sel_all, NULL),
+  GB_METHOD("Unselect", NULL, CTEXTBOX_sel_clear, NULL),
+  GB_PROPERTY_READ("Selected", "b", CTEXTBOX_selected),
 
   GB_METHOD("Clear", NULL, CTEXTBOX_clear, NULL),
   GB_METHOD("Insert", NULL, CTEXTBOX_insert, "(Text)s"),
 
   GB_EVENT("Change", NULL, NULL, &EVENT_Change),
   GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
+  
+  TEXTBOX_DESCRIPTION,
 
   GB_END_DECLARE
 };
@@ -519,26 +454,28 @@ GB_DESC CComboBoxDesc[] =
 {
   GB_DECLARE("ComboBox", sizeof(CCOMBOBOX)), GB_INHERITS("Control"),
 
-  GB_CONSTANT("_Properties", "s", CCOMBOBOX_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
-
   GB_METHOD("_new", NULL, CCOMBOBOX_new, "(Parent)Container;"),
   GB_METHOD("_get", ".ComboBoxItem", CCOMBOBOX_get, "(Index)i"),
   GB_METHOD("Popup", NULL, CCOMBOBOX_popup, NULL),
-  GB_METHOD("Clear", NULL, CCOMBOBOX_clear, NULL),
+  GB_METHOD("Clear", NULL, CTEXTBOX_clear, NULL),
   GB_METHOD("Insert", NULL, CTEXTBOX_insert, "(Text)s"),
   GB_METHOD("Add", NULL, CCOMBOBOX_add, "(Item)s[(Index)i]"),
   GB_METHOD("Remove", NULL, CCOMBOBOX_remove, "(Index)i"),
   GB_METHOD("Find", "i", CCOMBOBOX_find, "(Item)s"),
 
   GB_PROPERTY("Text", "s", CCOMBOBOX_text),
-  GB_PROPERTY_READ("Length", "i", CCOMBOBOX_length),
+  GB_PROPERTY_READ("Length", "i", CTEXTBOX_length),
   GB_PROPERTY("Pos", "i", CTEXTBOX_pos),
-  GB_PROPERTY("ReadOnly", "b", CCOMBOBOX_read_only),
+  GB_PROPERTY("ReadOnly", "b", CTEXTBOX_read_only),
   GB_PROPERTY("Password", "b", CTEXTBOX_password),
   GB_PROPERTY("MaxLength", "i", CTEXTBOX_max_length),
+  
   GB_PROPERTY_SELF("Selection", ".TextBoxSelection"),
-  GB_PROPERTY_SELF("Select", ".TextBoxSelection"),
+  GB_METHOD("Select", NULL, CTEXTBOX_sel_select, "[(Start)i(Length)i]"),
+  GB_METHOD("SelectAll", NULL, CTEXTBOX_sel_all, NULL),
+  GB_METHOD("Unselect", NULL, CTEXTBOX_sel_clear, NULL),
+  GB_PROPERTY_READ("Selected", "b", CTEXTBOX_selected),
+
   GB_PROPERTY("Sorted", "b", CCOMBOBOX_sorted),
   GB_PROPERTY("List", "String[]", CCOMBOBOX_list),
   GB_PROPERTY_READ("Count", "i", CCOMBOBOX_count),
@@ -549,6 +486,8 @@ GB_DESC CComboBoxDesc[] =
   GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
+	COMBOBOX_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
