@@ -28,7 +28,8 @@
 #ifndef __CTRAYICON_H
 #define __CTRAYICON_H
 
-#include "gambas.h"
+#include "main.h"
+#include "gtrayicon.h"
 #include "CPicture.h"
 
 
