@@ -28,25 +28,18 @@
 
 #define __CWIDGET_CPP
 
-#include "gambas.h"
-#include "widgets.h"
 #include "CWidget.h"
 #include "CWindow.h"
+#include "CMenu.h"
 #include "CFont.h"
 #include "CMouse.h"
 #include "CPicture.h"
-#include "CImage.h"
 #include "CContainer.h"
+#include "CClipboard.h"
 #include "CFrame.h"
 
-#include <stdlib.h>
-
-#include <stdio.h>
-
 extern int MAIN_scale;
 
-static void *CLASS_Image=NULL;
-
 DECLARE_EVENT(EVENT_Enter);
 DECLARE_EVENT(EVENT_GotFocus);
 DECLARE_EVENT(EVENT_LostFocus);
@@ -55,6 +48,7 @@ DECLARE_EVENT(EVENT_KeyRelease);
 DECLARE_EVENT(EVENT_Leave);
 DECLARE_EVENT(EVENT_MouseDown);
 DECLARE_EVENT(EVENT_MouseMove);
+DECLARE_EVENT(EVENT_MouseDrag);
 DECLARE_EVENT(EVENT_MouseUp);
 DECLARE_EVENT(EVENT_MouseWheel);
 DECLARE_EVENT(EVENT_DblClick);
@@ -68,11 +62,35 @@ DECLARE_EVENT(EVENT_Plugged);
 DECLARE_EVENT(EVENT_Unplugged);
 DECLARE_EVENT(EVENT_PlugError);
 
-long allWidgetsCount=0;
-void **allWidgets=NULL;
+//static void *CLASS_Image = NULL;
+static void *CLASS_UserContainer = NULL;
+static void *CLASS_UserControl = NULL;
 
-void *CLASS_UserContainer=NULL;
-void *CLASS_UserControl=NULL;
+/** Action *****************************************************************/
+
+static bool has_action(void *control)
+{
+	if (GB.Is(control, GB.FindClass("Menu")))
+		return ((CMENU *)(control))->widget->action();
+	else
+		return ((CWIDGET *)(control))->widget->action();
+}
+
+static void set_action(void *control, bool v)
+{
+	if (GB.Is(control, GB.FindClass("Menu")))
+		((CMENU *)(control))->widget->setAction(v);
+	else
+		((CWIDGET *)(control))->widget->setAction(v);
+}
+
+#define HAS_ACTION(_control) has_action(_control)
+#define SET_ACTION(_control, _flag) set_action(_control, _flag)
+
+#include "gb.form.action.h"
+
+
+/** Control ****************************************************************/
 
 void gb_plug_raise_plugged(gControl *sender)
 {
@@ -90,44 +108,60 @@ void gb_plug_raise_unplugged(gControl *sender)
 	GB.Raise((void*)_ob,EVENT_Unplugged,0);
 }
 
-
+// static void raise_menu(void *_object)
+// {
+// 	GB.Raise(THIS, EVENT_Menu, 0);
+// 	GB.Unref(POINTER(&_object));
+// }
 
 bool gb_raise_MouseEvent(gControl *sender,long type)
 {
-	CWIDGET *_ob=GetObject(sender);
+	void *ob = GetObject(sender);
 	
-	if (!_ob) return false;
+	if (!ob) return false; // possible, for MouseDrag for example
 	
 	switch(type)
 	{
 		case gEvent_MousePress:
-			GB.Raise((void*)_ob,EVENT_MouseDown,0);
+			GB.Raise(ob, EVENT_MouseDown, 0);
 			break;
 			
 		case gEvent_MouseRelease:
-			GB.Raise((void*)_ob,EVENT_MouseUp,0);
+			GB.Raise(ob, EVENT_MouseUp, 0);
 			break;
 			
 		case gEvent_MouseMove:
-			GB.Raise((void*)_ob,EVENT_MouseMove,0);
+			GB.Raise(ob, EVENT_MouseMove, 0);
+			break;
+			
+		case gEvent_MouseDrag:
+			if (gMouse::button() && GB.CanRaise(ob, EVENT_MouseDrag))
+				GB.Raise(ob, EVENT_MouseDrag, 0);
 			break;
 			
 		case gEvent_MouseWheel:
-			GB.Raise((void*)_ob,EVENT_MouseWheel,0);
+			GB.Raise(ob, EVENT_MouseWheel, 0);
 			break;
 			
 		case gEvent_MouseMenu:
-			GB.Raise((void*)_ob,EVENT_Menu,0);
+			if (GB.CanRaise(ob, EVENT_Menu))
+			{
+				GB.Raise(ob, EVENT_Menu, 0);
+				return true;
+				//GB.Ref((void *)_ob);
+				//GB.Post((GB_POST_FUNC)raise_menu, (long)_ob);
+				//return true;
+			}
 			break;
 			
 		case gEvent_MouseDblClick:
-			GB.Raise((void*)_ob,EVENT_DblClick,0);
+			//fprintf(stderr, "dblclick: %s (%p)\n", sender->name(), _ob);
+			GB.Raise(ob, EVENT_DblClick, 0);
 			break;
 		
 	}
 	
 	return false;
-		
 }
 
 bool gb_raise_KeyEvent(gControl *sender,long type)
@@ -179,12 +213,12 @@ void gb_raise_FocusEvent(gControl *sender,long type)
 }
 
 
-void gb_raise_Drag(gControl *sender)
+bool gb_raise_Drag(gControl *sender)
 {
 	CWIDGET *_ob=GetObject(sender);
 	
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Drag,0);
+	if (!_ob) return false;
+	return GB.Raise((void*)_ob,EVENT_Drag,0);
 }
 
 bool gb_raise_DragMove(gControl *sender)
@@ -205,52 +239,38 @@ void gb_raise_Drop(gControl *sender)
 
 void DeleteControl(gControl *control)
 {
-	CWIDGET *widget=NULL;
-	long bucle,b2;
+	CWIDGET *widget = (CWIDGET*)control->hFree;
 	
-	widget=(CWIDGET*)control->hFree;
+	if (widget) 
+	{
+		GB.StoreVariant(NULL, (void *)&widget->tag);
+		CACTION_register(widget, NULL);
+	}
 	
-	if (widget) GB.StoreVariant(NULL, (void *)&widget->tag);
-	
-	for(bucle=0;bucle<allWidgetsCount;bucle++)
-		if (allWidgets[bucle]==control)
-		{
-			
-			for (b2=bucle;b2<(allWidgetsCount-1);b2++)
-				allWidgets[b2]=allWidgets[b2+1];
-			
-			allWidgetsCount--;
-			if (!allWidgetsCount)
-				GB.Free((void**)&allWidgets);
-			else
-				GB.Realloc((void**)&allWidgets,sizeof(void*)*allWidgetsCount);
-		}
-	
-		
 	WINDOW_kill((CWINDOW*)widget);
 	
 	if (widget) 
 	{ 
 		widget->widget=NULL;
-		GB.Unref((void**)&widget);
-		
-	}
-	
+		GB.Unref(POINTER(&widget));
+	}	
 }
 
 
-void InitControl(gControl *control,CWIDGET *widget)
+void InitControl(gControl *control, CWIDGET *widget)
 {
+	char *name;
+	
 	GB.Ref((void*)widget);
 	
+	widget->widget = control;
 	control->hFree=(void*)widget;
 	
-	if (!allWidgets)
-		GB.Alloc((void**)&allWidgets,sizeof(void*));
-	else
-		GB.Realloc((void**)&allWidgets,(allWidgetsCount+1)*sizeof(void*));
-	
-	allWidgets[allWidgetsCount++]=control;
+	name = GB.GetLastEventName();
+	if (!name)
+		name = GB.GetClassName((void *)widget);
+		
+	control->setName(name);
 	
 	control->onFinish=DeleteControl;
 	control->onMouseEvent=gb_raise_MouseEvent;
@@ -260,20 +280,12 @@ void InitControl(gControl *control,CWIDGET *widget)
 	control->onDragMove=gb_raise_DragMove;
 	control->onDrop=gb_raise_Drop;
 	control->onEnterLeave=gb_raise_EnterLeave;
-
-
-}
-
-long ChildrenCount(gControl *control)
-{
-	long bucle;
-	long nnum=0;
 	
-	for (bucle=0;bucle<allWidgetsCount;bucle++)
-		if ( ((gControl*)allWidgets[bucle])->pr==control) nnum++;
-
+	if (control->isContainer())
+		((gContainer *)control)->onArrange = CCONTAINER_cb_arrange;
 	
-	return nnum;
+	if (control->parent())
+		CCONTAINER_raise_insert((CCONTAINER *)control->parent()->hFree, widget);
 }
 
 CWIDGET *GetContainer(CWIDGET *control)
@@ -284,52 +296,11 @@ CWIDGET *GetContainer(CWIDGET *control)
 	if (!CLASS_UserControl) CLASS_UserControl=GB.FindClass("UserControl");
 	
 	if ( GB.Is (control,CLASS_UserContainer) || GB.Is (control,CLASS_UserControl) )
-	{
-		if (  ((CUSERCONTROL*)control)->ct )
-			return (CWIDGET*)((CUSERCONTROL*)control)->ct;
-		else
-			return control;
-	}
+		return (CWIDGET*)((CUSERCONTROL*)control)->container;
 	
 	return control;
 }
 
-CWIDGET *GetObject(gControl *control)
-{
-	long bucle;
-	
-	for (bucle=0;bucle<allWidgetsCount;bucle++)
-		if (allWidgets[bucle]==control) return (CWIDGET*)control->hFree;
-	
-	return NULL;
-}
-
-CWIDGET *GetChild(gControl *control,long index)
-{
-	CWIDGET *buf=NULL;
-	long bucle;
-	long nnum=0;
-	
-	for (bucle=0;bucle<allWidgetsCount;bucle++)
-		if ( ((gControl*)allWidgets[bucle])->pr==control)
-		{
-			if (nnum==index)
-			{
-				buf=(CWIDGET*)((gControl*)allWidgets[bucle])->hFree;
-				break;
-			}
-			else
-				nnum++;
-		}
-
-	
-	return buf;
-	
-	
-}
-
-
-
 int CWIDGET_check(void *_object)
 {
 	if (!CONTROL) return true;
@@ -341,13 +312,10 @@ int CWIDGET_check(void *_object)
 Embedder
 
 **************************************************************************************/
+
 BEGIN_METHOD(CPLUGIN_new, GB_OBJECT parent)
 
-	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);
-	Parent=(CCONTAINER*)GetContainer ((CWIDGET*)Parent);
-
-	((CPLUGIN*)_object)->widget=new gPlugin(Parent->widget);
-	InitControl( ((CPLUGIN*)_object)->widget,(CWIDGET*)_object);
+	InitControl(new gPlugin(CONTAINER(VARG(parent))), (CWIDGET*)_object);
 	((gPlugin*)((CPLUGIN*)_object)->widget)->onPlug=gb_plug_raise_plugged;
 	((gPlugin*)((CPLUGIN*)_object)->widget)->onUnplug=gb_plug_raise_unplugged;
 
@@ -359,13 +327,9 @@ BEGIN_PROPERTY(CPLUGIN_client)
 
 END_PROPERTY
 
-BEGIN_METHOD(CPLUGIN_embed,GB_INTEGER id;GB_BOOLEAN Prepared)
+BEGIN_METHOD(CPLUGIN_embed, GB_INTEGER id; GB_BOOLEAN prepared)
 
-	bool Prepared=true;
-	
-	if (!MISSING(Prepared)) Prepared=VARG(Prepared);
-	
-	((gPlugin*)((CPLUGIN*)_object)->widget)->plug(VARG(id),Prepared);
+	((gPlugin*)((CPLUGIN*)_object)->widget)->plug(VARG(id), VARGOPT(prepared, TRUE));
 
 END_METHOD
 
@@ -375,17 +339,17 @@ BEGIN_METHOD_VOID(CPLUGIN_discard)
 
 END_METHOD
 
-BEGIN_PROPERTY(CPLUGIN_border)
-
-  	if (READ_PROPERTY) 
-	{
-  		GB.ReturnInteger(((gPlugin*)((CPLUGIN*)_object)->widget)->getBorder());
-		return;
-   	}
-	
-	((gPlugin*)((CPLUGIN*)_object)->widget)->setBorder(VPROP(GB_INTEGER));
-
-END_PROPERTY
+// BEGIN_PROPERTY(CPLUGIN_border)
+// 
+//   	if (READ_PROPERTY) 
+// 	{
+//   		GB.ReturnInteger(((gPlugin*)((CPLUGIN*)_object)->widget)->getBorder());
+// 		return;
+//    	}
+// 	
+// 	((gPlugin*)((CPLUGIN*)_object)->widget)->setBorder(VPROP(GB_INTEGER));
+// 
+// END_PROPERTY
 
 
 /**************************************************************************************
@@ -441,21 +405,19 @@ END_PROPERTY
 
 
 
-BEGIN_PROPERTY(CWIDGET_font)
+BEGIN_PROPERTY(CCONTROL_font)
 
-	CFONT *Font;
-	
-	if (READ_PROPERTY)
-	{
-		GB.New((void **)&Font, GB.FindClass("Font"), 0, 0);
-		delete Font->font;
-		Font->font=CONTROL->font();
-		GB.ReturnObject(Font);
-		return;
-	}
-	
-	Font=(CFONT*)VPROP(GB_OBJECT);
-	if (Font) CONTROL->setFont(Font->font);
+  CFONT *font;
+
+  if (READ_PROPERTY)
+  {
+    GB.ReturnObject(CFONT_create(CONTROL->font(), 0, THIS));
+  }
+  else
+  {
+    font = (CFONT *)VPROP(GB_OBJECT);
+    CONTROL->setFont(font ? font->font : 0);
+  }
 
 END_PROPERTY
 
@@ -470,8 +432,10 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CWIDGET_visible)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(CONTROL->visible()); return; }
-	CONTROL->setVisible(VPROP(GB_BOOLEAN));
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(CONTROL->isVisible());
+	else
+		CONTROL->setVisible(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
@@ -543,7 +507,8 @@ END_METHOD
 
 BEGIN_METHOD_VOID(CWIDGET_delete)
 	
-	if (CONTROL) { CONTROL->destroy(); THIS->widget=NULL; }
+	if (CONTROL)
+		CONTROL->destroy();
 		
 END_METHOD
 
@@ -561,16 +526,26 @@ BEGIN_METHOD_VOID(CWIDGET_hide)
 
 END_METHOD
 
-BEGIN_METHOD(CWIDGET_reparent,GB_OBJECT Parent;)
+BEGIN_METHOD(CWIDGET_reparent, GB_OBJECT parent; GB_INTEGER x; GB_INTEGER y)
 
-	CCONTAINER *Parent=(CCONTAINER*)VARG(Parent);
+	CCONTAINER *parent=(CCONTAINER*)VARG(parent);
+	int x, y;
 	
-	if (!Parent) { GB.Error("Null parent"); return; }
-	if (!Parent->widget) { GB.Error("Invalid parent"); return; }
-	if (!CONTROL->parent()) { GB.Error("Unable to reparent a top level window"); return; }
+	if (GB.CheckObject(parent))
+		return;
 	
-	CONTROL->reparent((gContainer*)Parent->widget);
+	x = CONTROL->left();
+	y = CONTROL->top();
 	
+	if (!MISSING(x) && !MISSING(y))
+	{
+    x = VARG(x);
+    y = VARG(y);
+	}
+	
+	//if (!CONTROL->parent()) { GB.Error("Unable to reparent a top level window"); return; }
+	
+	CONTROL->reparent((gContainer*)parent->widget, x, y);
 
 END_METHOD
 
@@ -591,14 +566,26 @@ END_METHOD
 
 BEGIN_PROPERTY(CWIDGET_next)
 
-  
+	if (READ_PROPERTY)
+		GB.ReturnObject(GetObject(CONTROL->next()));
+	else
+	{
+		CWIDGET *next = (CWIDGET *)VPROP(GB_OBJECT);
+		CONTROL->setNext(next ? next->widget : NULL);
+	}
   
 END_PROPERTY
 
 
 BEGIN_PROPERTY(CWIDGET_previous)
 
-
+	if (READ_PROPERTY)
+		GB.ReturnObject(GetObject(CONTROL->previous()));
+	else
+	{
+		CWIDGET *previous = (CWIDGET *)VPROP(GB_OBJECT);
+		CONTROL->setPrevious(previous ? previous->widget : NULL);
+	}
   
 END_PROPERTY
 
@@ -658,7 +645,7 @@ BEGIN_PROPERTY(CWIDGET_cursor)
 	{
 		cur=CONTROL->cursor();
 		if (!cur) return;
-		GB.New((void **)&Cur, GB.FindClass("Cursor"), 0, 0);
+		GB.New(POINTER(&Cur), GB.FindClass("Cursor"), 0, 0);
 		Cur->cur=cur;
 		GB.ReturnObject((void*)Cur);
 		return;
@@ -674,16 +661,20 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CWIDGET_background)
 
-	if (READ_PROPERTY)	{ GB.ReturnInteger(CONTROL->backGround()); return; }
-	CONTROL->setBackGround(VPROP(GB_INTEGER));
+	if (READ_PROPERTY)
+		GB.ReturnInteger(CONTROL->background());
+	else
+		CONTROL->setBackground(VPROP(GB_INTEGER));
 
 END_PROPERTY
 
 
 BEGIN_PROPERTY(CWIDGET_foreground)
 
-	if (READ_PROPERTY)	{ GB.ReturnInteger(CONTROL->foreGround()); return; }
-	CONTROL->setForeGround(VPROP(GB_INTEGER));
+	if (READ_PROPERTY)
+		GB.ReturnInteger(CONTROL->foreground());
+	else
+		CONTROL->setForeground(VPROP(GB_INTEGER));
 
 END_PROPERTY
 
@@ -726,7 +717,7 @@ BEGIN_METHOD_VOID(CWIDGET_grab)
 
 	CPICTURE *img;
 
-	GB.New((void **)&img, GB.FindClass("Picture"), 0, 0);
+	GB.New(POINTER(&img), GB.FindClass("Picture"), 0, 0);
 	if (img->picture) delete img->picture;
 	img->picture=CONTROL->grab();
 	GB.ReturnObject((void*)img);
@@ -736,41 +727,37 @@ END_METHOD
 
 BEGIN_METHOD(CWIDGET_drag, GB_VARIANT data; GB_STRING format)
 
-	CPICTURE *pic;
-	CIMAGE *img;
+	CDRAG_drag(THIS, &VARG(data), MISSING(format) ? NULL : GB.ToZeroString(ARG(format)));
 	
-	if (VARG(data).type == GB_T_STRING)
-	{
-		CONTROL->dragText(VARG(data)._string.value);
-		return;
-	}
-	
-	if (!CLASS_Image) CLASS_Image = GB.FindClass("Image");
-	
-	if (VARG(data).type >= GB_T_OBJECT && GB.Is(VARG(data)._object.value, CLASS_Image))
-	{
-		img = (CIMAGE*)VARG(data)._object.value;
-		CONTROL->dragImage(img->image);
-		return;
-	}
-
 END_METHOD
 
 
 BEGIN_PROPERTY(CWIDGET_drop)
 
-	if (READ_PROPERTY) { GB.ReturnBoolean(CONTROL->acceptDrops()); return; }
-	CONTROL->setAcceptDrops(VPROP(GB_BOOLEAN));
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(CONTROL->acceptDrops());
+	else
+		CONTROL->setAcceptDrops(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
 
-// BM: stub for the Name property
-
 BEGIN_PROPERTY(CCONTROL_name)
 
 	if (READ_PROPERTY)
-		GB.ReturnNull();
+		GB.ReturnNewZeroString(CONTROL->name());
+	else
+		CONTROL->setName(GB.ToZeroString(PROP(GB_STRING)));
+
+END_PROPERTY
+
+
+BEGIN_PROPERTY(CCONTROL_action)
+
+	if (READ_PROPERTY)
+		CACTION_get(THIS);
+	else
+		CACTION_register(THIS, GB.ToZeroString(PROP(GB_STRING)));
 
 END_PROPERTY
 
@@ -791,13 +778,13 @@ GB_DESC CWidgetDesc[] =
   GB_METHOD("Delete", NULL, CWIDGET_delete, NULL),
   GB_METHOD("Show", NULL, CWIDGET_show, NULL),
   GB_METHOD("Hide", NULL, CWIDGET_hide, NULL),
-  GB_METHOD("Reparent",NULL,CWIDGET_reparent,"(Parent)Container"),
+  GB_METHOD("Reparent",NULL,CWIDGET_reparent,"(Parent)Container;[(X)i(Y)i]"),
   
   GB_METHOD("Raise", NULL, CWIDGET_raise, NULL),
   GB_METHOD("Lower", NULL, CWIDGET_lower, NULL),
 
-  GB_PROPERTY_READ("Next", "Control", CWIDGET_next),
-  GB_PROPERTY_READ("Previous", "Control", CWIDGET_previous),
+  GB_PROPERTY("Next", "Control", CWIDGET_next),
+  GB_PROPERTY("Previous", "Control", CWIDGET_previous),
   
   GB_METHOD("SetFocus", NULL, CWIDGET_set_focus, NULL),
   GB_METHOD("Refresh", NULL, CWIDGET_refresh, "[(X)i(Y)i(Width)i(Height)i]"),
@@ -820,7 +807,7 @@ GB_DESC CWidgetDesc[] =
   GB_PROPERTY("Expand", "b", CWIDGET_expand),
   GB_PROPERTY("Ignore", "b", CWIDGET_ignore),
 
-  GB_PROPERTY("Font", "Font", CWIDGET_font),
+  GB_PROPERTY("Font", "Font", CCONTROL_font),
   GB_PROPERTY("Background", "i", CWIDGET_background),
   GB_PROPERTY("BackColor", "i", CWIDGET_background),
   GB_PROPERTY("Foreground", "i", CWIDGET_foreground),
@@ -829,12 +816,13 @@ GB_DESC CWidgetDesc[] =
   GB_PROPERTY("Design", "b", CWIDGET_design),
   GB_PROPERTY("Name", "s", CCONTROL_name),
   GB_PROPERTY("Tag", "v", CWIDGET_tag),
-  GB_PROPERTY("Mouse", "i" MOUSE_CONSTANTS, CWIDGET_mouse), 
+  GB_PROPERTY("Mouse", "i", CWIDGET_mouse), 
   GB_PROPERTY("Cursor", "Cursor", CWIDGET_cursor),
   GB_PROPERTY("ToolTip", "s", CWIDGET_tooltip),
   GB_PROPERTY("Drop", "b", CWIDGET_drop),
+  GB_PROPERTY("Action", "s", CCONTROL_action),
 
-  GB_PROPERTY_READ("Parent", "Control", CWIDGET_parent),
+  GB_PROPERTY_READ("Parent", "Container", CWIDGET_parent),
   GB_PROPERTY_READ("Window", "Window", CWIDGET_window),
   GB_PROPERTY_READ("Id", "i", CWIDGET_id),
   GB_PROPERTY_READ("Handle", "i", CWIDGET_id),
@@ -847,6 +835,7 @@ GB_DESC CWidgetDesc[] =
   GB_EVENT("Leave", NULL, NULL, &EVENT_Leave),
   GB_EVENT("MouseDown", NULL, NULL, &EVENT_MouseDown),
   GB_EVENT("MouseMove", NULL, NULL, &EVENT_MouseMove),
+  GB_EVENT("MouseDrag", NULL, NULL, &EVENT_MouseDrag),
   GB_EVENT("MouseUp", NULL, NULL, &EVENT_MouseUp),
   GB_EVENT("MouseWheel", NULL, NULL, &EVENT_MouseWheel),
   GB_EVENT("DblClick", NULL, NULL, &EVENT_DblClick),
@@ -855,8 +844,7 @@ GB_DESC CWidgetDesc[] =
   GB_EVENT("DragMove", NULL, NULL, &EVENT_DragMove),
   GB_EVENT("Drop", NULL, NULL, &EVENT_Drop),
 
-  GB_CONSTANT("_DefaultEvent", "s", "MouseDown"),
-  GB_CONSTANT("_Properties", "s", CWIDGET_PROPERTIES), 
+	CONTROL_DESCRIPTION,
 
   GB_END_DECLARE
 };
@@ -870,13 +858,13 @@ GB_DESC CPluginDesc[] =
   GB_METHOD("Discard", NULL, CPLUGIN_discard, NULL),
   
   GB_PROPERTY_READ("Client", "i", CPLUGIN_client),
-  GB_PROPERTY("Border", "i<Border>", CPLUGIN_border),
+  //GB_PROPERTY("Border", "i<Border>", CPLUGIN_border),
   
   GB_EVENT("Embed", NULL, NULL, &EVENT_Plugged),
   GB_EVENT("Close", NULL, NULL, &EVENT_Unplugged),
   GB_EVENT("Error", NULL, NULL, &EVENT_PlugError), /* TODO */
   
-  GB_CONSTANT("_Properties", "s", CPLUGIN_PROPERTIES),
+  EMBEDDER_DESCRIPTION,
   
   GB_END_DECLARE
 };
