@@ -28,23 +28,22 @@
 
 #define __CWINDOW_CPP
 
-#include <stdio.h>
-
-#include "gambas.h"
 #include "main.h"
-#include "widgets.h"
 #include "CWindow.h"
 #include "CWidget.h"
 #include "CContainer.h"
 #include "CPicture.h"
 #include "CMenu.h"
+#include "CDraw.h"
+#include "gapplication.h"
 
 long CWINDOW_Embedder = 0;
 bool CWINDOW_Embedded = false;
 
+CWINDOW *CWINDOW_Active = NULL;
+
 static CWINDOW *MAIN_Window=NULL;
 static long MODAL_windows=0;
-static long WINDOW_return=0;
 
 DECLARE_EVENT(EVENT_Open);
 DECLARE_EVENT(EVENT_Show);
@@ -54,6 +53,9 @@ DECLARE_EVENT(EVENT_Activate);
 DECLARE_EVENT(EVENT_Deactivate);
 DECLARE_EVENT(EVENT_Move);
 DECLARE_EVENT(EVENT_Resize);
+DECLARE_EVENT(EVENT_Title);
+DECLARE_EVENT(EVENT_Icon);
+
 
 CWINDOW* WINDOW_get_main()
 {
@@ -62,7 +64,11 @@ CWINDOW* WINDOW_get_main()
 
 void WINDOW_kill(CWINDOW *win)
 {
-	if (MAIN_Window==win) MAIN_Window=NULL;
+	if (MAIN_Window==win) 
+	{
+		//fprintf(stderr, "MAIN_Window = NULL\n");
+		MAIN_Window=NULL;
+	}
 }
 
 
@@ -76,7 +82,7 @@ void gb_raise_window_Open(gMainWindow *sender)
 
 void gb_post_window_Open(gMainWindow *sender)
 {
-	GB.Post( (void (*)())gb_raise_window_Open,(long)sender);
+	GB.Post((GB_POST_FUNC)gb_raise_window_Open, (long)sender);
 }
 
 void gb_raise_window_Show(gMainWindow *sender)
@@ -85,6 +91,8 @@ void gb_raise_window_Show(gMainWindow *sender)
 
 	if (!_ob) return;
 	GB.Raise((void*)_ob,EVENT_Show,0);
+	if (!sender->spontaneous())
+		CACTION_raise(_ob);
 }
 
 void gb_post_window_Show(gMainWindow *sender)
@@ -98,6 +106,8 @@ void gb_raise_window_Hide(gMainWindow *sender)
 
 	if (!_ob) return;
 	GB.Raise((void*)_ob,EVENT_Hide,0);
+	if (!sender->spontaneous())
+		CACTION_raise(_ob);
 }
 
 void gb_raise_window_Move(gMainWindow *sender)
@@ -116,6 +126,56 @@ void gb_raise_window_Resize(gMainWindow *sender)
 	GB.Raise((void*)_ob,EVENT_Resize,0);
 }
 
+static bool close_window(CWINDOW *_object, int ret = 0)
+{
+	THIS->ret = ret;
+
+  return WINDOW->close();
+}
+
+static bool closeAll()
+{
+  int i;
+  gMainWindow *win;
+  CWINDOW *window;
+  
+  for(i = 0; i < gMainWindow::count(); i++)
+  {
+    win = gMainWindow::get(i);
+    if (!win)
+      break;
+    //fprintf(stderr, "closeAll: try %p\n", win);
+    window = (CWINDOW *)GetObject(win);
+    if (window == MAIN_Window)
+      continue;
+    if (close_window(window))
+      return true;
+  }
+  
+  return false;
+}
+
+static bool deleteAll()
+{
+  int i;
+  gMainWindow *win;
+  CWINDOW *window;
+  
+  for(i = 0; i < gMainWindow::count(); i++)
+  {
+    win = gMainWindow::get(i);
+    if (!win)
+      break;
+    //fprintf(stderr, "closeAll: try %p\n", win);
+    window = (CWINDOW *)GetObject(win);
+    if (window == MAIN_Window)
+      continue;
+    win->destroy();
+  }
+  
+  return false;
+}
+
 bool gb_raise_window_Close(gMainWindow *sender)
 {
 	CWINDOW *_ob=(CWINDOW*)GetObject(sender);
@@ -123,51 +183,98 @@ bool gb_raise_window_Close(gMainWindow *sender)
 	if (!_ob) return false;
 	if (GB.Raise((void*)_ob,EVENT_Close,0)) return true;
 
-	if (_ob->persistent)
+  if (MAIN_Window && sender == MAIN_Window->widget)
+  {
+    if (closeAll())
+      return true;
+    if (!sender->isPersistent())
+    	deleteAll();
+  }
+
+	// BM: isn't it already done?
+	/*if (sender->isPersistent()) 
 	{
 		sender->hide();
 		return true;
-	}
+	}*/
 
 	if (_ob->embed)
 	{
 		CWINDOW_Embedder = 0;
 		CWINDOW_Embedded = false;
 	}
-	_ob->modal=0;
+	
 	return false;
-
 }
 
+static void activate_window(gMainWindow *window)
+{
+	CWINDOW *active;
+
+	if (window)
+	{
+		for(;;)
+		{
+			active = (CWINDOW *)GetObject(window);
+			if (window->isTopLevel())
+				break;
+			if (GB.CanRaise(active, EVENT_Activate))
+				break;
+			window = window->parent()->window();
+		}
+	}
+	else
+		active = NULL;
+
+	if (active == CWINDOW_Active)
+		return;
+
+	if (CWINDOW_Active)
+	{
+	  GB.Raise(CWINDOW_Active, EVENT_Deactivate, 0);
+		CWINDOW_Active = 0;
+	}
+
+	if (active)
+	{
+	  GB.Raise(active, EVENT_Activate, 0);
+	}
+
+  CWINDOW_Active = active;
+}
+
+static void cb_activate(gMainWindow *sender)
+{
+	activate_window(sender);
+}
+
+static void cb_deactivate(gMainWindow *sender)
+{
+	activate_window(NULL);
+}
 
 /***************************************************************************
 
   Window
 
 ***************************************************************************/
+
 BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent;)
 
 	GB_CLASS CLASS_container;
 	CWIDGET *parent=NULL;
-	long plug=0;
+	int plug = 0;
 
-	THIS->persistent=0;
-	if (!MISSING(parent))
+	if (!MISSING(parent) && VARG(parent))
 	{
-		if (VARG(parent))
-		{
-			CLASS_container = GB.FindClass("Container");
-			if ( !GB.Is(VARG(parent), CLASS_container) )
-			{
-				GB.Error("The parent of a Window must be a Container or a Workspace");
-				return;
-			}
-			parent=(CWIDGET*)VARG(parent);
-			parent=GetContainer ((CWIDGET*)parent);
-		}
+    CLASS_container = GB.FindClass("Container");
+  	if (GB.Conv((GB_VALUE *)(void *)ARG(parent), (GB_TYPE)CLASS_container))
+  		return;
+	
+    parent=(CWIDGET*)VARG(parent);
+    parent=GetContainer ((CWIDGET*)parent);
 	}
 
-
 	if ( CWINDOW_Embedder && (!CWINDOW_Embedded) && (!parent) )
 	{
 		plug=CWINDOW_Embedder;
@@ -175,34 +282,36 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent;)
 	}
 
 	if (!parent) THIS->widget=new gMainWindow(plug);
-	else         THIS->widget=new gMainWindow(parent->widget);
+	else         THIS->widget=new gMainWindow((gContainer *)parent->widget);
 
 	InitControl(THIS->widget,(CWIDGET*)THIS);
-	WINDOW->onOpen=gb_post_window_Open;
-	WINDOW->onShow=gb_post_window_Show;
-	WINDOW->onHide=gb_raise_window_Hide;
-	WINDOW->onMove=gb_raise_window_Move;
-	WINDOW->onResize=gb_raise_window_Resize;
-	WINDOW->onClose=gb_raise_window_Close;
+	
+	WINDOW->onOpen = gb_raise_window_Open;
+	WINDOW->onShow = gb_raise_window_Show;
+	WINDOW->onHide = gb_raise_window_Hide;
+	WINDOW->onMove = gb_raise_window_Move;
+	WINDOW->onResize = gb_raise_window_Resize;
+	WINDOW->onClose = gb_raise_window_Close;
+	WINDOW->onActivate = cb_activate;
+	WINDOW->onDeactivate = cb_deactivate;
+	WINDOW->resize(200,150);
 
-	if ( (!MAIN_Window) && (!parent)) MAIN_Window=THIS;
+	if ( (!MAIN_Window) && (!parent)) 
+	{
+		MAIN_Window=THIS;
+		//fprintf(stderr, "MAIN_Window = %p\n", MAIN_Window);
+	}
 
-	if (parent)
+	/*if (parent)
 	{
 		gb_post_window_Open(WINDOW);
 		gb_post_window_Show(WINDOW);
-	}
-
+	}*/
 
 END_METHOD
 
 BEGIN_METHOD_VOID(CWINDOW_free)
 
-	if (THIS->icon) GB.Unref((void**)&THIS->icon);
-	if (THIS->picture) GB.Unref((void**)&THIS->picture);
-	THIS->icon=NULL;
-	THIS->picture=NULL;
-
 END_METHOD
 
 BEGIN_METHOD_VOID(CFORM_new)
@@ -240,17 +349,16 @@ END_METHOD
 
 BEGIN_METHOD_VOID(CWINDOW_next)
 
-	long *vl;
+	int *vl;
 
-	vl=(long*)GB.GetEnum();
-	if (gApplication::winCount()<=vl[0])
-	{
+	vl = (int *)GB.GetEnum();
+	
+	if (gMainWindow::count() <= *vl)
 		GB.StopEnum();
-	}
 	else
 	{
-		GB.ReturnObject ( gApplication::winItem(vl[0])->hFree );
-		vl[0]++;
+		GB.ReturnObject(gMainWindow::get(*vl)->hFree);
+		(*vl)++;
 	}
 
 END_METHOD
@@ -258,31 +366,23 @@ END_METHOD
 
 BEGIN_PROPERTY(CWINDOW_count)
 
-	GB.ReturnInteger( gApplication::winCount());
+	GB.ReturnInteger(gMainWindow::count());
 
 END_PROPERTY
 
 
-BEGIN_METHOD(CWINDOW_get, GB_INTEGER id)
+BEGIN_METHOD(CWINDOW_get_from_id, GB_INTEGER index)
 
-	gControl *ob=gApplication::winItem(VARG(id));
+	gMainWindow *ob = gMainWindow::get(VARG(index));
 
-	if (ob) GB.ReturnObject(ob->hFree);
+	GB.ReturnObject(ob ? ob->hFree : NULL);
 
 END_METHOD
 
+
 BEGIN_METHOD(CWINDOW_close, GB_INTEGER ret)
 
-	if (THIS->modal)
-	{
-		WINDOW_return=VARG(ret);
-		THIS->modal=0;
-	}
-
-	if (THIS->persistent)
-		WINDOW->hide();
-	else
-		WINDOW->destroy();
+  GB.ReturnBoolean(close_window(THIS, VARGOPT(ret, 0)));
 
 END_METHOD
 
@@ -296,28 +396,21 @@ END_METHOD
 
 BEGIN_METHOD_VOID(CWINDOW_show_modal)
 
-	WINDOW_return=0;
-	THIS->modal=1;
+	THIS->ret = 0;
 	MODAL_windows++;
 	WINDOW->showModal();
-	while (THIS->modal)
-	{
-		do_iteration();
-		gControl::cleanRemovedControls();
-		if (!MAIN_Window) break;
-	}
 	MODAL_windows--;
-	GB.ReturnInteger(WINDOW_return);
+	GB.ReturnInteger(THIS->ret);
 
 END_METHOD
 
 
 BEGIN_METHOD_VOID(CWINDOW_show)
 
-	if (MODAL_windows)
-		CWINDOW_show_modal(THIS, NULL);
-	else
-		WINDOW->show();
+	//if (MODAL_windows)
+	//	CWINDOW_show_modal(THIS, NULL);
+	//else
+	WINDOW->show();
 
 END_METHOD
 
@@ -328,23 +421,21 @@ BEGIN_PROPERTY(CWINDOW_modal)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(CWINDOW_top_level)
 
+	GB.ReturnBoolean(WINDOW->isTopLevel());
+
+END_PROPERTY
+
+// 
 
 
 BEGIN_PROPERTY(CWINDOW_persistent)
 
 	if (READ_PROPERTY)
-	{
-		GB.ReturnBoolean (THIS->persistent);
-		return;
-	}
-
-	if (WINDOW->pr) return;
-
-	if (VPROP(GB_BOOLEAN))
-		THIS->persistent=1;
+		GB.ReturnBoolean (WINDOW->isPersistent());
 	else
-		THIS->persistent=0;
+	 WINDOW->setPersistent(VPROP(GB_BOOLEAN));
 
 END_PROPERTY
 
@@ -353,15 +444,14 @@ BEGIN_PROPERTY(CWINDOW_text)
 
 	if (READ_PROPERTY) { GB.ReturnNewString(WINDOW->text(),0); return; }
 	WINDOW->setText((const char*)GB.ToZeroString(PROP(GB_STRING)));
+	GB.Raise(THIS, EVENT_Title, 0);
 
 END_PROPERTY
 
 
-
-
 BEGIN_PROPERTY(CWINDOW_border)
 
-	if (READ_PROPERTY) { GB.ReturnInteger(WINDOW->_border()); return; }
+	if (READ_PROPERTY) { GB.ReturnInteger(WINDOW->getBorder()); return; }
 	WINDOW->setBorder(VPROP(GB_INTEGER));
 
 END_PROPERTY
@@ -369,22 +459,17 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CWINDOW_icon)
 
-	CPICTURE *pic=NULL;
-	gPicture *hPic=NULL;
-
 	if (READ_PROPERTY)
 	{
-		GB.ReturnObject(THIS->icon);
-		return;
+		gPicture *pic = WINDOW->icon();
+		GB.ReturnObject(pic ? pic->getTagValue() : 0);
+	}
+	else
+	{
+		CPICTURE *pic = (CPICTURE *)VPROP(GB_OBJECT);
+		WINDOW->setIcon(pic ? pic->picture : 0);
+		GB.Raise(THIS, EVENT_Icon, 0);
 	}
-
-	pic=(CPICTURE*)VPROP(GB_OBJECT);
-	if (pic) GB.Ref((void*)pic);
-	if (THIS->icon) GB.Unref((void**)&THIS->icon);
-	THIS->icon=pic;
-	if (!pic) WINDOW->setIcon(NULL);
-	else      WINDOW->setIcon(pic->picture);
-
 
 END_PROPERTY
 
@@ -404,6 +489,15 @@ BEGIN_PROPERTY(CWINDOW_skip_taskbar)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(CWINDOW_toolbox)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(WINDOW->isToolBox());
+	else
+		WINDOW->setToolBox(VPROP(GB_BOOLEAN));
+
+END_PROPERTY
+
 BEGIN_PROPERTY(CWINDOW_mask)
 
 	if (READ_PROPERTY){  GB.ReturnBoolean(WINDOW->mask()); return; }
@@ -413,22 +507,17 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CWINDOW_picture)
 
-	CPICTURE *pic;
-
 	if (READ_PROPERTY)
 	{
-		GB.ReturnObject(THIS->picture);
-		return;
+		gPicture *pic = WINDOW->picture();
+		GB.ReturnObject(pic ? pic->getTagValue() : 0);
 	}
-
-	pic=(CPICTURE*)VPROP(GB_OBJECT);
-	if (pic) GB.Ref((void*)pic);
-	if (THIS->picture) GB.Unref((void**)&THIS->picture);
-	THIS->picture=pic;
-	if (!pic) WINDOW->setPicture(NULL);
-	else      WINDOW->setPicture(pic->picture);
-
-
+	else
+	{
+		CPICTURE *pic = (CPICTURE *)VPROP(GB_OBJECT);
+		WINDOW->setPicture(pic ? pic->picture : 0);
+	}
+	
 END_PROPERTY
 
 
@@ -480,6 +569,7 @@ BEGIN_PROPERTY(CWINDOW_sticky)
 
 END_PROPERTY
 
+
 BEGIN_METHOD_VOID(CWINDOW_menu_next)
 
 	CMENU *Mn;
@@ -497,6 +587,65 @@ BEGIN_METHOD_VOID(CWINDOW_menu_next)
 END_PROPERTY
 
 
+BEGIN_METHOD(CWINDOW_menu_get, GB_INTEGER index)
+
+	int index = VARG(index);
+	
+	if (index < 0 || index >= gMenu::winChildCount(WINDOW))
+	{
+		GB.Error(GB_ERR_BOUND);
+		return;
+	}
+	
+	GB.ReturnObject(gMenu::winChildMenu(WINDOW, index));
+
+END_METHOD
+
+BEGIN_PROPERTY(CWINDOW_control_count)
+
+	GB.ReturnInteger(WINDOW->controlCount());
+
+END_PROPERTY
+
+
+BEGIN_METHOD_VOID(CWINDOW_control_next)
+
+	int index;
+	gControl *control;
+
+	index = ENUM(int);
+	
+	control = WINDOW->getControl(index);
+	
+	if (!control)
+	{
+		GB.StopEnum();
+		return;
+	}
+	
+	ENUM(int) = index + 1;
+	GB.ReturnObject(GetObject(control));
+
+END_PROPERTY
+
+
+BEGIN_METHOD(CWINDOW_get, GB_STRING name)
+
+	gControl *control = WINDOW->getControl(GB.ToZeroString(ARG(name)));
+
+	if (!control)
+		GB.ReturnNull();
+	else
+		GB.ReturnObject(GetObject(control));
+
+END_METHOD
+
+BEGIN_PROPERTY(CWINDOW_closed)
+
+	GB.ReturnBoolean(WINDOW->isClosed());
+
+END_PROPERTY
+
 
 
 /***************************************************************************
@@ -505,17 +654,27 @@ END_PROPERTY
 
 ***************************************************************************/
 
-
 GB_DESC CWindowMenusDesc[] =
 {
   GB_DECLARE(".WindowMenus", 0), GB_VIRTUAL_CLASS(),
 
   GB_METHOD("_next", "Menu", CWINDOW_menu_next, NULL),
+  GB_METHOD("_get", "Menu", CWINDOW_menu_get, "(Index)i"),
   GB_PROPERTY_READ("Count", "i", CWINDOW_menu_count),
 
   GB_END_DECLARE
 };
 
+GB_DESC CWindowControlsDesc[] =
+{
+  GB_DECLARE(".WindowControls", 0), GB_VIRTUAL_CLASS(),
+
+  GB_METHOD("_next", "Control", CWINDOW_control_next, NULL),
+  GB_METHOD("_get", "Control", CWINDOW_get, "(Name)s"),
+  GB_PROPERTY_READ("Count", "i", CWINDOW_control_count),
+
+  GB_END_DECLARE
+};
 
 GB_DESC CWindowDesc[] =
 {
@@ -531,14 +690,19 @@ GB_DESC CWindowDesc[] =
 
   GB_METHOD("_new", NULL, CWINDOW_new, "[(Parent)Control;]"),
   GB_METHOD("_free",NULL,CWINDOW_free,NULL),
+  GB_METHOD("_get", "Control", CWINDOW_get, "(Name)s"),
 
   GB_METHOD("Close", "b", CWINDOW_close, "[(Return)i]"),
   GB_METHOD("Raise", NULL, CWINDOW_raise, NULL),
   GB_METHOD("Show", NULL, CWINDOW_show, NULL),
+  //GB_METHOD("Hide", NULL, CWINDOW_hide, NULL),
+  //GB_PROPERTY("Visible", "b", CWINDOW_visible),
   GB_METHOD("ShowModal", "i", CWINDOW_show_modal, NULL),
   GB_METHOD("ShowDialog", "i", CWINDOW_show_modal, NULL),
   GB_METHOD("Center", NULL, CWINDOW_center, NULL),
   GB_PROPERTY_READ("Modal", "b", CWINDOW_modal),
+  GB_PROPERTY_READ("TopLevel", "b", CWINDOW_top_level),
+  GB_PROPERTY_READ("Closed", "b", CWINDOW_closed),
 
   GB_PROPERTY("Stacking","i",CWINDOW_stacking),
   GB_PROPERTY("Sticky","b",CWINDOW_sticky),
@@ -547,7 +711,7 @@ GB_DESC CWindowDesc[] =
   GB_PROPERTY("Title", "s", CWINDOW_text),
   GB_PROPERTY("Caption", "s", CWINDOW_text),
   GB_PROPERTY("Icon", "Picture", CWINDOW_icon),
-  GB_PROPERTY("Border", "i<Window,None,Fixed,Resizable>", CWINDOW_border),
+  GB_PROPERTY("Border", "i", CWINDOW_border),
 
   GB_PROPERTY("Minimized", "b", CWINDOW_minimized),
   GB_PROPERTY("Maximized", "b", CWINDOW_maximized),
@@ -555,15 +719,18 @@ GB_DESC CWindowDesc[] =
 
   GB_PROPERTY("TopOnly", "b", CWINDOW_top_only),
   GB_PROPERTY("SkipTaskbar", "b", CWINDOW_skip_taskbar),
-  GB_PROPERTY("Arrangement", "i<Arrange>", CCONTAINER_arrangement),
+  GB_PROPERTY("ToolBox", "b", CWINDOW_toolbox),
+  GB_PROPERTY("Arrangement", "i", CCONTAINER_arrangement),
 
   GB_PROPERTY("Padding", "i", CCONTAINER_padding),
   GB_PROPERTY("Spacing", "i", CCONTAINER_spacing),
+  GB_PROPERTY("AutoResize", "b", CCONTAINER_auto_resize),
 
   GB_PROPERTY("Mask","b",CWINDOW_mask),
   GB_PROPERTY("Picture", "Picture", CWINDOW_picture),
 
   GB_PROPERTY_SELF("Menus", ".WindowMenus"),
+  GB_PROPERTY_SELF("Controls", ".WindowControls"),
 
   GB_CONSTANT("_Properties", "s", CWINDOW_PROPERTIES),
   GB_CONSTANT("_DefaultEvent", "s", "Open"),
@@ -577,6 +744,10 @@ GB_DESC CWindowDesc[] =
   GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),
   GB_EVENT("Show", NULL, NULL, &EVENT_Show),
   GB_EVENT("Hide", NULL, NULL, &EVENT_Hide),
+  GB_EVENT("Title", NULL, NULL, &EVENT_Title),
+  GB_EVENT("Icon", NULL, NULL, &EVENT_Icon),
+
+  GB_INTERFACE("Draw", &DRAW_Interface),
 
   GB_END_DECLARE
 };
@@ -587,7 +758,7 @@ GB_DESC CWindowsDesc[] =
   GB_DECLARE("Windows", 0), GB_VIRTUAL_CLASS(),
 
   GB_STATIC_METHOD("_next", "Window", CWINDOW_next, NULL),
-  GB_STATIC_METHOD("_get", "Window", CWINDOW_get, "(Id)i"),
+  GB_STATIC_METHOD("_get", "Window", CWINDOW_get_from_id, "(Id)i"),
   GB_STATIC_PROPERTY_READ("Count", "i", CWINDOW_count),
 
   GB_END_DECLARE
