@@ -28,11 +28,62 @@
 
 #include "widgets.h"
 #include "font-parser.h"
-#include <gtk/gtk.h>
-#include <stdlib.h>
-#include <string.h>
+#include "gdesktop.h"
+#include "gb.form.font.h"
+
+#include <math.h>
+
+static void set_font_from_string(gFont *font, const char *str)
+{
+  gchar **tokens, **p;
+  gchar *elt;
+  int grade;
+  double size;
+  
+  font->setBold(false);
+  font->setItalic(false);
+  font->setUnderline(false);
+  font->setStrikeOut(false);
+  font->setName(gDesktop::font()->name());
+  font->setSize(gDesktop::font()->size());
+  
+  if (!str || !*str)
+  	return;
+  	
+	tokens = g_strsplit(str, ",", 0);
+  
+  p = tokens;
+  for(p = tokens; *p; p++)
+  {
+  	elt = *p;
+  	
+  	if (!strcasecmp(elt, "bold"))
+  		font->setBold(true);
+		else if (!strcasecmp(elt, "italic"))
+  		font->setItalic(true);
+		else if (!strcasecmp(elt, "underline"))
+  		font->setUnderline(true);
+		else if (!strcasecmp(elt, "strikeout"))
+  		font->setStrikeOut(true);
+		else if (elt[0] == '+' || elt[0] == '-' || elt[0] == '0')
+		{
+			grade = atoi(elt);
+			if (grade || elt[0] == '0')
+				font->setGrade(grade);
+		}
+		else
+		{
+			size = atof(elt);
+			if (size != 0.0)
+				font->setSize(size);
+			else
+				font->setName(elt);
+		}
+  }
+  
+  g_strfreev(tokens);
+}
 
-#include <stdio.h>
 
 /********************************************************************************
 
@@ -42,6 +93,8 @@ gFont
 static int FONT_n_families;
 GList *FONT_families = NULL;
 
+static int _nfont = 0;
+
 void gFont::init()
 {
   PangoFontFamily **_families;
@@ -85,6 +138,9 @@ void gFont::exit()
 	}
 	
 	if (FONT_families) g_list_free(FONT_families);
+	
+	//if (_nfont)
+  //  fprintf(stderr, "WARNING: %d gFont objects were not freed.\n", _nfont);
 }
 
 long gFont::count()
@@ -101,9 +157,9 @@ const char *gFont::familyItem(long pos)
 	return (const char*)g_list_nth (FONT_families,pos)->data;
 }
 
+#if 0
 void gFont::updateWidget()
 {
-
 	if (!wid) return;
 
 	PangoFontDescription *desc=pango_context_get_font_description(ct);
@@ -131,16 +187,23 @@ void gFont::updateWidget()
 	}
 
 }
+#endif
 
-gFont::gFont(GtkWidget *wg)
+void gFont::realize()
+{
+  gShare::gShare();
+	strike=false;
+	uline=false;
+	_nfont++;  
+	//fprintf(stderr, "new gFont() %p\n", this);
+}
+
+gFont::gFont(GtkWidget *wid)
 {
 	PangoAttrList *lst;
 	PangoAttrIterator* iter;
 	
-	nref=1;
-	strike=false;
-	uline=false;
-	wid=wg;
+	realize();
 	ct=gtk_widget_create_pango_context(wid);
 	g_object_ref(ct);
 	
@@ -163,57 +226,52 @@ gFont::gFont()
 {
 	GtkStyle *sty=gtk_widget_get_default_style();
 	
-	nref=1;
-	strike=false;
-	uline=false;
-	wid=NULL;
+  realize();
 	ct=gdk_pango_context_get();
 	pango_context_set_font_description(ct,sty->font_desc);
 }
 
-gFont::gFont(char *name)
+gFont::gFont(PangoFontDescription *fd)
+{
+	//GtkStyle *sty=gtk_widget_get_default_style();
+	
+	realize();
+	ct=gdk_pango_context_get();
+	pango_context_set_font_description(ct, fd);
+}
+
+gFont::gFont(const char *name)
 {
 	GtkStyle *sty=gtk_widget_get_default_style();
 	
-	nref=1;
-	strike=false;
-	uline=false;
-	wid=NULL;
+	realize();
 	ct=gdk_pango_context_get();
 	pango_context_set_font_description(ct,sty->font_desc);
 
-	if (name) {
-		gb_fontparser_parse(name);
-		if ( gb_fontparser_underline() ) { this->setUnderline(true); } else { this->setUnderline(false); }
-		if ( gb_fontparser_strikeout() ) { this->setStrikeOut(true); } else { this->setStrikeOut(false); }
-		if ( gb_fontparser_bold() )      { this->setBold(true); } else { this->setBold(false); }
-		if ( gb_fontparser_italic() )    { this->setItalic(true); } else { this->setItalic(false); } 
-		if ( gb_fontparser_name() ) this->setName( gb_fontparser_name() );
-		if ( gb_fontparser_size() )
-		{
-			if ( gb_fontparser_relative() )
-				this->setSize(this->size() +(double)gb_fontparser_size()); 
-			else                            
-				this->setSize((double)gb_fontparser_size());
-		}
-	}
+	set_font_from_string(this, name);
+}
+
+gFont *gFont::copy()
+{
+	gFont *f = new gFont();
+	
+	f->setName(name());
+	f->setSize(size());
+	f->setBold(bold());
+	f->setItalic(italic());
+	f->setUnderline(underline());
+	f->setStrikeOut(strikeOut());
+	
+	return f;
 }
 
 gFont::~gFont()
 {
 	g_object_unref(ct);
+	_nfont--;
+  //fprintf(stderr, "delete gFont() %p\n", this);
 }
 
-void gFont::ref()
-{
-	nref++;
-}
-
-void gFont::unref()
-{
-	nref--;
-	if (nref<=0) delete this;
-}
 
 long gFont::ascent()
 {
@@ -256,7 +314,7 @@ void gFont::setBold(bool vl)
 	else
 		pango_font_description_set_weight(desc,PANGO_WEIGHT_NORMAL);
 		
-	updateWidget();
+	//updateWidget();
 }
 
 bool gFont::italic()
@@ -278,14 +336,14 @@ void gFont::setItalic(bool vl)
 	else
 		pango_font_description_set_style(desc,PANGO_STYLE_NORMAL);
 		
-	updateWidget();
+	//updateWidget();
 }
 
-const char* gFont::name()
+char* gFont::name()
 {
 	PangoFontDescription *desc=pango_context_get_font_description(ct);
 
-	return pango_font_description_get_family (desc);
+	return (char *)pango_font_description_get_family (desc);
 	
 }
 
@@ -295,7 +353,7 @@ void gFont::setName(char *nm)
 	
 	pango_font_description_set_family (desc,nm);
 	
-	updateWidget();
+	//updateWidget();
 }
 
 double gFont::size()
@@ -305,25 +363,58 @@ double gFont::size()
 	PangoFontDescription *desc=pango_context_get_font_description(ct);
 
 	size=pango_font_description_get_size(desc);
-	return size/(double)PANGO_SCALE;
+	return size / (double)PANGO_SCALE;
+}
+
+int gFont::grade()
+{
+	double desktop = gDesktop::font()->size();
+	return SIZE_TO_GRADE(size(), desktop);
 }
 
 void gFont::setSize(double sz)
 {
 	PangoFontDescription *desc=pango_context_get_font_description(ct);
 	
-	pango_font_description_set_size (desc,sz*PANGO_SCALE);
+	pango_font_description_set_size (desc,(int)(sz*PANGO_SCALE));
 	
-	updateWidget();
+	//updateWidget();
 }
 
-const char* gFont::toString()
+void gFont::setGrade(int grade)
 {
-	PangoFontDescription *desc=pango_context_get_font_description(ct);
-	return pango_font_description_to_string(desc);
+	double desktop = gDesktop::font()->size();
+	
+	if (grade < -4 || grade > 16)
+		return;
+	
+	setSize(GRADE_TO_SIZE(grade, desktop));
 }
 
-long gFont::width(const char *text)
+const char *gFont::toString()
+{
+	//PangoFontDescription *desc=pango_context_get_font_description(ct);
+	//return pango_font_description_to_string(desc);
+	
+	GString *desc = g_string_new(name());
+	char *ret;
+	
+	g_string_append_printf(desc, ",%g", (double)((int)(size() * 10 + 0.5)) / 10);
+	if (bold())
+		g_string_append(desc, ",Bold");
+	if (italic())
+		g_string_append(desc, ",Italic");
+	if (underline())
+		g_string_append(desc, ",Underline");
+	if (strikeOut())
+		g_string_append(desc, ",StrikeOut");
+
+	ret = g_string_free(desc, false);
+	gt_free_later(ret);
+	return ret;
+}
+
+int gFont::width(const char *text, int len)
 {
 	PangoLayout *ly;
 	int w;
@@ -332,35 +423,39 @@ long gFont::width(const char *text)
 	if (!strlen(text)) return 0;
 	
 	ly=pango_layout_new(ct);
-	pango_layout_set_text(ly,text,-1);
+	pango_layout_set_text(ly,text,len);
 	pango_layout_get_size(ly,&w,NULL);
 	g_object_unref(G_OBJECT(ly));
 	
 	return w/PANGO_SCALE;
 }
 
-long gFont::height(const char *text)
+int gFont::height(const char *text, int len)
 {
 	PangoLayout *ly;
 	int h;
 	
-	if (!text) return 0;
-	if (!strlen(text)) return 0;
+	if (!text || !*text) return 0;
 	
 	ly=pango_layout_new(ct);
-	pango_layout_set_text(ly,text,-1);
+	pango_layout_set_text(ly,text,len);
 	pango_layout_get_size(ly,NULL,&h);
 	g_object_unref(G_OBJECT(ly));
 	
 	return h/PANGO_SCALE;
 }
 
+int gFont::height()
+{
+	return height(" ", 1);
+}
+
 bool gFont::scalable()
 {
 	bool ret=false;
 	
 	PangoFontDescription *desc=pango_context_get_font_description(ct);
-	PangoFontDescription *tmp;
+	//PangoFontDescription *tmp;
 	const char* name=pango_font_description_get_family(desc);
 	PangoFontFamily **families;
 	PangoFontFace **faces;
@@ -368,7 +463,7 @@ bool gFont::scalable()
 	int n_families;
 	int n_faces;
 	int n_sizes;
-	int b2;
+	//int b2;
 	const char *buf;
 	
 	if (!name) return false;
@@ -452,13 +547,11 @@ long gFont::resolution()
 
 bool gFont::strikeOut()
 {
-	stub("strikeOut(): partially working");
 	return strike;
 }
 
 bool gFont::underline()
 {
-	stub("underline(): partially working");
 	return uline;
 }
 
@@ -469,16 +562,16 @@ void gFont::setResolution(long vl)
 
 void gFont::setStrikeOut(bool vl)
 {
-	stub("setStrikeOut(): partially working");
+	//stub("setStrikeOut(): partially working");
 	strike=vl;
-	updateWidget();
+	//updateWidget();
 }
 
 void gFont::setUnderline(bool vl)
 {
-	stub("setUnderline(): partially working");
+	//stub("setUnderline(): partially working");
 	uline=vl;
-	updateWidget();
+	//updateWidget();
 }
 
 
