@@ -25,14 +25,11 @@
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ***************************************************************************/
-#include "widgets.h"
-#include <gtk/gtk.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <errno.h>
 
-static char * stock_gambas_xpm[] = {
+#include "widgets.h"
+#include "gstock.h"
+
+static const char *stock_gambas_xpm[] = {
 "16 16 133 2",
 "  	c None",
 ". 	c #C4C4C4",
@@ -184,7 +181,7 @@ static char * stock_gambas_xpm[] = {
 "        3.4.5.6.^.7.8.9.^.      ",
 "            0.a.b.^.7.          "};
 
-static char * stock_linux_xpm[] = {
+static const char * stock_linux_xpm[] = {
 "16 16 109 2",
 "  	c None",
 ". 	c #818181",
@@ -313,7 +310,7 @@ static char * stock_linux_xpm[] = {
 "        '.).        !.!.        "};
 
 
-static char * stock_zoom_xpm[] = {
+static const char * stock_zoom_xpm[] = {
 "24 24 127 2",
 "  	c None",
 ". 	c #343434",
@@ -470,11 +467,12 @@ static char * stock_zoom_xpm[] = {
 
 
 
-gImage *gStock_missing(char *vl)
+static gPicture *gStock_missing(char *vl)
 {
-	gImage *img=NULL,*img2;
-	char **tmp=NULL;
-	bool do_flip=false;
+	gPicture *img = NULL;
+	const char **tmp = NULL;
+	bool do_flip = false;
+	GdkPixbuf *buf, *buf2;
 	
 	if       (!strcasecmp(vl,"gambas"))        tmp=stock_gambas_xpm;          
 	else if  (!strcasecmp(vl,"linux"))         tmp=stock_linux_xpm;   
@@ -483,10 +481,10 @@ gImage *gStock_missing(char *vl)
 	
 	if (tmp)
 	{
-		img=new gImage(0,0);
-		img->image=gdk_pixbuf_new_from_xpm_data((const char**)tmp); 
+		buf = gdk_pixbuf_new_from_xpm_data((const char**)tmp);
+		img = new gPicture(buf);
 		
-		if (do_flip)
+		/*if (do_flip)
 		{
 			if (gtk_widget_get_default_direction ()==GTK_TEXT_DIR_RTL)
 			{
@@ -494,118 +492,135 @@ gImage *gStock_missing(char *vl)
 				delete img;
 				img=img2;
 			}
-		}
+		}*/
 	}   
 	
 	return img;
 }
 
 
-void gStock_parse(char *vl,char **resul)
+static void gStock_parse(char *vl, const char **result)
 {
+	static const char *parse[] = {
+	"device/cdrom",       GTK_STOCK_CDROM,
+	"device/floppy",      GTK_STOCK_FLOPPY,
+	"device/harddisk",    GTK_STOCK_HARDDISK,
+	"device/printer",     GTK_STOCK_PRINT,
+	"device/network",     GTK_STOCK_NETWORK, // (*)
 	
-	*resul=NULL;
+	"dialog/auth",        GTK_STOCK_DIALOG_AUTHENTICATION,
+	"dialog/error",       GTK_STOCK_DIALOG_ERROR,
+	"dialog/info",        GTK_STOCK_DIALOG_INFO,
+	"dialog/question",    GTK_STOCK_DIALOG_QUESTION,
+	"dialog/warning",     GTK_STOCK_DIALOG_WARNING,
+
+	"dnd/simple",         GTK_STOCK_DND, // (*)
+	"dnd/multiple",       GTK_STOCK_DND_MULTIPLE, // (*)
 	
-	if       (!strcasecmp(vl,"device/cdrom"))       *resul=GTK_STOCK_CDROM;
-	else if  (!strcasecmp(vl,"device/floppy"))      *resul=GTK_STOCK_FLOPPY;
-	else if  (!strcasecmp(vl,"device/harddisk"))    *resul=GTK_STOCK_HARDDISK;
-	else if  (!strcasecmp(vl,"device/printer"))     *resul=GTK_STOCK_PRINT;
-	else if  (!strcasecmp(vl,"device/network"))     *resul=GTK_STOCK_NETWORK; // (*)
+	"go/bottom",          GTK_STOCK_GOTO_BOTTOM,
+	"go/down",            GTK_STOCK_GO_DOWN,
+	"go/first",           GTK_STOCK_GOTO_FIRST,
+	"go/last",            GTK_STOCK_GOTO_LAST,
+	"go/left",            GTK_STOCK_GO_BACK,
+	"go/right",           GTK_STOCK_GO_FORWARD,
+	"go/top",             GTK_STOCK_GOTO_TOP,
+	"go/up",              GTK_STOCK_GO_UP,
+	"go/home",            GTK_STOCK_HOME,
+
+	"sort/ascending",     GTK_STOCK_SORT_ASCENDING, // (*)
+	"sort/descending",    GTK_STOCK_SORT_DESCENDING, // (*)
 	
-	else if  (!strcasecmp(vl,"dialog/auth"))        *resul=GTK_STOCK_DIALOG_AUTHENTICATION;
-	else if  (!strcasecmp(vl,"dialog/error"))       *resul=GTK_STOCK_DIALOG_ERROR;
-	else if  (!strcasecmp(vl,"dialog/info"))        *resul=GTK_STOCK_DIALOG_INFO;
-	else if  (!strcasecmp(vl,"dialog/question"))    *resul=GTK_STOCK_DIALOG_QUESTION;
-	else if  (!strcasecmp(vl,"dialog/warning"))     *resul=GTK_STOCK_DIALOG_WARNING;
-
-	else if  (!strcasecmp(vl,"dnd/simple"))         *resul=GTK_STOCK_DND; // (*)
-	else if  (!strcasecmp(vl,"dnd/multiple"))       *resul=GTK_STOCK_DND_MULTIPLE; // (*)
+	"text/bold",          GTK_STOCK_BOLD,
+	"text/center",        GTK_STOCK_JUSTIFY_CENTER,
+	"text/font",          GTK_STOCK_SELECT_FONT,
+	"text/italic",        GTK_STOCK_ITALIC,
+	"text/justify",       GTK_STOCK_JUSTIFY_FILL,
+	"text/left",          GTK_STOCK_JUSTIFY_LEFT,
+	"text/right",         GTK_STOCK_JUSTIFY_RIGHT,
+	"text/strikethrough", GTK_STOCK_STRIKETHROUGH,
+	"text/underline",     GTK_STOCK_UNDERLINE,
+	"text/indent",        GTK_STOCK_INDENT, // (*)
+	"text/unindent",      GTK_STOCK_UNINDENT, // (*)
 	
-	else if  (!strcasecmp(vl,"go/bottom"))          *resul=GTK_STOCK_GOTO_BOTTOM;
-	else if  (!strcasecmp(vl,"go/down"))            *resul=GTK_STOCK_GO_DOWN;
-	else if  (!strcasecmp(vl,"go/first"))           *resul=GTK_STOCK_GOTO_FIRST;
-	else if  (!strcasecmp(vl,"go/last"))            *resul=GTK_STOCK_GOTO_LAST;
-	else if  (!strcasecmp(vl,"go/left"))            *resul=GTK_STOCK_GO_BACK;
-	else if  (!strcasecmp(vl,"go/right"))           *resul=GTK_STOCK_GO_FORWARD;
-	else if  (!strcasecmp(vl,"go/top"))             *resul=GTK_STOCK_GOTO_TOP;
-	else if  (!strcasecmp(vl,"go/up"))              *resul=GTK_STOCK_GO_UP;
-	else if  (!strcasecmp(vl,"go/home"))            *resul=GTK_STOCK_HOME;
+	"media/forward",      GTK_STOCK_MEDIA_FORWARD, 
+	"media/next",         GTK_STOCK_MEDIA_NEXT, 
+	"media/pause",        GTK_STOCK_MEDIA_PAUSE, 
+	"media/play",         GTK_STOCK_MEDIA_PLAY, 
+	"media/previous",     GTK_STOCK_MEDIA_PREVIOUS, 
+	"media/record",       GTK_STOCK_MEDIA_RECORD, 
+	"media/rewind",       GTK_STOCK_MEDIA_REWIND, 
+	"media/stop",         GTK_STOCK_MEDIA_STOP, 
 
-	else if  (!strcasecmp(vl,"sort/ascending"))     *resul=GTK_STOCK_SORT_ASCENDING; // (*)
-	else if  (!strcasecmp(vl,"sort/descending"))    *resul=GTK_STOCK_SORT_DESCENDING; // (*)
+	"zoom/zoomfit",       GTK_STOCK_ZOOM_FIT,
+	"zoom/zoomin",        GTK_STOCK_ZOOM_IN,
+	"zoom/zoomnormal",    GTK_STOCK_ZOOM_100,
+	"zoom/zoomout",       GTK_STOCK_ZOOM_OUT,
+
+	"add",                GTK_STOCK_ADD, // (*)
+	"apply",              GTK_STOCK_APPLY,
+	"cancel",             GTK_STOCK_CANCEL,
+	"clear",              GTK_STOCK_CLEAR, // (*)
+	"close",              GTK_STOCK_CLOSE,
+	"color",              GTK_STOCK_SELECT_COLOR,
+	"colorpicker",        GTK_STOCK_COLOR_PICKER,
+	"convert",            GTK_STOCK_CONVERT, // (*)
+
+	"connect",            GTK_STOCK_CONNECT, 
+	"copy",               GTK_STOCK_COPY,
+	"cut",                GTK_STOCK_CUT,
+	"delete",             GTK_STOCK_DELETE,
+	"directory",          GTK_STOCK_DIRECTORY, 
+	"disconnect",         GTK_STOCK_DISCONNECT, 
+	"edit",               GTK_STOCK_EDIT, 
+	"file",               GTK_STOCK_FILE, 
+	"execute",            GTK_STOCK_EXECUTE, // (*)
+	"find",               GTK_STOCK_FIND,
+	"findandreplace",     GTK_STOCK_FIND_AND_REPLACE, // (*)
+	"help",               GTK_STOCK_HELP,
+	"index",              GTK_STOCK_INDEX, // (*)
+	"jumpto",             GTK_STOCK_JUMP_TO, // (*)
+	"missingimage",       GTK_STOCK_MISSING_IMAGE, // (*)
+	"new",                GTK_STOCK_NEW,
+	"no",                 GTK_STOCK_NO, // (*)
+	"ok",                 GTK_STOCK_OK,
+	"open",               GTK_STOCK_OPEN,
+	"paste",              GTK_STOCK_PASTE,
+	"preferences",        GTK_STOCK_PREFERENCES,
+	"print",              GTK_STOCK_PRINT,
+	"printpreview",       GTK_STOCK_PRINT_PREVIEW, // (*)
+	"properties",         GTK_STOCK_PROPERTIES,
+	"quit",               GTK_STOCK_QUIT,
+	"redo",               GTK_STOCK_REDO,
+	"refresh",            GTK_STOCK_REFRESH,
+	"remove",             GTK_STOCK_REMOVE, // (*)
+	"revert",             GTK_STOCK_REVERT_TO_SAVED, // (*)
+	"save",               GTK_STOCK_SAVE,
+	"saveas",             GTK_STOCK_SAVE_AS,
+	"spellcheck",         GTK_STOCK_SPELL_CHECK,
+	"stop",               GTK_STOCK_STOP,
+	"undelete",           GTK_STOCK_UNDELETE, // (*)
+	"undo",               GTK_STOCK_UNDO,
+	"yes",                GTK_STOCK_YES,// (*)
+	NULL, NULL
+	};
 	
-	else if  (!strcasecmp(vl,"text/bold"))          *resul=GTK_STOCK_BOLD;
-	else if  (!strcasecmp(vl,"text/center"))        *resul=GTK_STOCK_JUSTIFY_CENTER;
-	else if  (!strcasecmp(vl,"text/font"))          *resul=GTK_STOCK_SELECT_FONT;
-	else if  (!strcasecmp(vl,"text/italic"))        *resul=GTK_STOCK_ITALIC;
-	else if  (!strcasecmp(vl,"text/justify"))       *resul=GTK_STOCK_JUSTIFY_FILL;
-	else if  (!strcasecmp(vl,"text/left"))          *resul=GTK_STOCK_JUSTIFY_LEFT;
-	else if  (!strcasecmp(vl,"text/right"))         *resul=GTK_STOCK_JUSTIFY_RIGHT;
-	else if  (!strcasecmp(vl,"text/strikethrough")) *resul=GTK_STOCK_STRIKETHROUGH;
-	else if  (!strcasecmp(vl,"text/underline"))     *resul=GTK_STOCK_UNDERLINE;
-	else if  (!strcasecmp(vl,"text/indent"))        *resul=GTK_STOCK_INDENT; // (*)
-	else if  (!strcasecmp(vl,"text/unindent"))      *resul=GTK_STOCK_UNINDENT; // (*)
+	const char **p = parse;
 	
-	else if  (!strcasecmp(vl,"media/forward"))      *resul=GTK_STOCK_MEDIA_FORWARD; 
-	else if  (!strcasecmp(vl,"media/next"))         *resul=GTK_STOCK_MEDIA_NEXT; 
-	else if  (!strcasecmp(vl,"media/pause"))        *resul=GTK_STOCK_MEDIA_PAUSE; 
-	else if  (!strcasecmp(vl,"media/play"))         *resul=GTK_STOCK_MEDIA_PLAY; 
-	else if  (!strcasecmp(vl,"media/previous"))     *resul=GTK_STOCK_MEDIA_PREVIOUS; 
-	else if  (!strcasecmp(vl,"media/record"))       *resul=GTK_STOCK_MEDIA_RECORD; 
-	else if  (!strcasecmp(vl,"media/rewind"))       *resul=GTK_STOCK_MEDIA_REWIND; 
-	else if  (!strcasecmp(vl,"media/stop"))         *resul=GTK_STOCK_MEDIA_STOP; 
-
-	else if  (!strcasecmp(vl,"zoom/zoomfit"))       *resul=GTK_STOCK_ZOOM_FIT;
-	else if  (!strcasecmp(vl,"zoom/zoomin"))        *resul=GTK_STOCK_ZOOM_IN;
-	else if  (!strcasecmp(vl,"zoom/zoomnormal"))    *resul=GTK_STOCK_ZOOM_100;
-	else if  (!strcasecmp(vl,"zoom/zoomout"))       *resul=GTK_STOCK_ZOOM_OUT;
-
-	else if  (!strcasecmp(vl,"add"))                *resul=GTK_STOCK_ADD; // (*)
-	else if  (!strcasecmp(vl,"apply"))              *resul=GTK_STOCK_APPLY;
-	else if  (!strcasecmp(vl,"cancel"))             *resul=GTK_STOCK_CANCEL;
-	else if  (!strcasecmp(vl,"clear"))              *resul=GTK_STOCK_CLEAR; // (*)
-	else if  (!strcasecmp(vl,"close"))              *resul=GTK_STOCK_CLOSE;
-	else if  (!strcasecmp(vl,"color"))              *resul=GTK_STOCK_SELECT_COLOR;
-	else if  (!strcasecmp(vl,"colorpicker"))        *resul=GTK_STOCK_COLOR_PICKER;
-	else if  (!strcasecmp(vl,"convert"))            *resul=GTK_STOCK_CONVERT; // (*)
-
-	else if  (!strcasecmp(vl,"connect"))            *resul=GTK_STOCK_CONNECT; 
-	else if  (!strcasecmp(vl,"copy"))               *resul=GTK_STOCK_COPY;
-	else if  (!strcasecmp(vl,"cut"))                *resul=GTK_STOCK_CUT;
-	else if  (!strcasecmp(vl,"delete"))             *resul=GTK_STOCK_DELETE;
-	else if  (!strcasecmp(vl,"directory"))          *resul=GTK_STOCK_DIRECTORY; 
-	else if  (!strcasecmp(vl,"disconnect"))         *resul=GTK_STOCK_DISCONNECT; 
-	else if  (!strcasecmp(vl,"edit"))               *resul=GTK_STOCK_EDIT; 
-	else if  (!strcasecmp(vl,"file"))               *resul=GTK_STOCK_FILE; 
-	else if  (!strcasecmp(vl,"execute"))            *resul=GTK_STOCK_EXECUTE; // (*)
-	else if  (!strcasecmp(vl,"find"))               *resul=GTK_STOCK_FIND;
-	else if  (!strcasecmp(vl,"findandreplace"))     *resul=GTK_STOCK_FIND_AND_REPLACE; // (*)
-	else if  (!strcasecmp(vl,"help"))               *resul=GTK_STOCK_HELP;
-	else if  (!strcasecmp(vl,"index"))              *resul=GTK_STOCK_INDEX; // (*)
-	else if  (!strcasecmp(vl,"jumpto"))             *resul=GTK_STOCK_JUMP_TO; // (*)
-	else if  (!strcasecmp(vl,"missingimage"))       *resul=GTK_STOCK_MISSING_IMAGE; // (*)
-	else if  (!strcasecmp(vl,"new"))                *resul=GTK_STOCK_NEW;
-	else if  (!strcasecmp(vl,"no"))                 *resul=GTK_STOCK_NO; // (*)
-	else if  (!strcasecmp(vl,"ok"))                 *resul=GTK_STOCK_OK;
-	else if  (!strcasecmp(vl,"open"))               *resul=GTK_STOCK_OPEN;
-	else if  (!strcasecmp(vl,"paste"))              *resul=GTK_STOCK_PASTE;
-	else if  (!strcasecmp(vl,"preferences"))        *resul=GTK_STOCK_PREFERENCES;
-	else if  (!strcasecmp(vl,"print"))              *resul=GTK_STOCK_PRINT;
-	else if  (!strcasecmp(vl,"printpreview"))       *resul=GTK_STOCK_PRINT_PREVIEW; // (*)
-	else if  (!strcasecmp(vl,"properties"))         *resul=GTK_STOCK_PROPERTIES;
-	else if  (!strcasecmp(vl,"quit"))               *resul=GTK_STOCK_QUIT;
-	else if  (!strcasecmp(vl,"redo"))               *resul=GTK_STOCK_REDO;
-	else if  (!strcasecmp(vl,"refresh"))            *resul=GTK_STOCK_REFRESH;
-	else if  (!strcasecmp(vl,"remove"))             *resul=GTK_STOCK_REMOVE; // (*)
-	else if  (!strcasecmp(vl,"revert"))             *resul=GTK_STOCK_REVERT_TO_SAVED; // (*)
-	else if  (!strcasecmp(vl,"save"))               *resul=GTK_STOCK_SAVE;
-	else if  (!strcasecmp(vl,"saveas"))             *resul=GTK_STOCK_SAVE_AS;
-	else if  (!strcasecmp(vl,"spellcheck"))         *resul=GTK_STOCK_SPELL_CHECK;
-	else if  (!strcasecmp(vl,"stop"))               *resul=GTK_STOCK_STOP;
-	else if  (!strcasecmp(vl,"undelete"))           *resul=GTK_STOCK_UNDELETE; // (*)
-	else if  (!strcasecmp(vl,"undo"))               *resul=GTK_STOCK_UNDO;
-	else if  (!strcasecmp(vl,"yes"))                *resul=GTK_STOCK_YES;// (*)
-
+	for(;;)
+	{
+		if (!*p)
+		{
+			*result = NULL;
+			return;
+		}
+		if (!strcasecmp(vl, *p))
+		{
+			*result = p[1];
+			return;
+		}
+		
+		p += 2;
+	}	
 }
 
 	
@@ -613,13 +628,13 @@ gPicture* gStock::get(char *vl)
 {
 	GtkIconSet *item;
 	GtkStyle *style;
-	gImage *tmp=NULL,*tmp2=NULL;
-	gPicture *ret=NULL;
+	gPicture *tmp=NULL,*tmp2=NULL;
 	gchar **buf;
-	long size;
-	long r_type;
+	int size = -1;
+	int r_type;
 	bool adapt=false;
-	char *stock_icon;
+	const char *stock_icon;
+	GdkPixbuf *img;
 	
 	if (!vl) return NULL;
 	if (!strlen(vl)) return NULL;
@@ -629,6 +644,7 @@ gPicture* gStock::get(char *vl)
 	if (!buf[0]) { g_strfreev(buf); return NULL; }
 	if (!buf[1]) { g_strfreev(buf); return NULL; }
 	
+	errno = 0;
 	size=strtol(buf[0],NULL,10);
 	if (errno)
 	{
@@ -637,7 +653,7 @@ gPicture* gStock::get(char *vl)
 	  else if(!strcasecmp(buf[0],"largetoolbar")) { size=24; r_type=GTK_ICON_SIZE_LARGE_TOOLBAR;}
 	  else if(!strcasecmp(buf[0],"button"))       { size=20; r_type=GTK_ICON_SIZE_BUTTON;}
 	  else if(!strcasecmp(buf[0],"dnd"))          { size=32; r_type=GTK_ICON_SIZE_DND;}
-	  else if(!strcasecmp(buf[0],"dialog"))       { size=48; r_type=GTK_ICON_SIZE_DIALOG;}
+	  else /*if(!strcasecmp(buf[0],"dialog"))*/   { size=48; r_type=GTK_ICON_SIZE_DIALOG;}
 	}
 	else
 	{
@@ -656,8 +672,7 @@ gPicture* gStock::get(char *vl)
 	gStock_parse(buf[1],&stock_icon);
 	if (stock_icon)
 	{
-		GtkSettings *set=gtk_settings_get_default();
-	    style=gtk_rc_get_style_by_paths(set,NULL,"GtkButton",GTK_TYPE_BUTTON);
+    style = gt_get_style("GtkButton", GTK_TYPE_BUTTON);
 		item=gtk_style_lookup_icon_set(style,stock_icon);
 		
 		if (!item)
@@ -665,11 +680,11 @@ gPicture* gStock::get(char *vl)
 			g_strfreev(buf);
 			return NULL;
 		}	
-		tmp=new gImage(0,0);
 		
-	    tmp->image=gtk_icon_set_render_icon(item,style, \
-		                                    gtk_widget_get_default_direction(), \
-		                                    GTK_STATE_NORMAL,(GtkIconSize)r_type,NULL,NULL);
+		img = gtk_icon_set_render_icon(item,style,
+                                   gtk_widget_get_default_direction(),
+                                   GTK_STATE_NORMAL,(GtkIconSize)r_type,NULL,NULL);
+		tmp = new gPicture(img);
 	}
 	else
 	{
@@ -683,14 +698,11 @@ gPicture* gStock::get(char *vl)
 	if (adapt)
 	{
 		tmp2=tmp->stretch(size,size,true);
-		delete tmp;
+		tmp->unref();
 		tmp=tmp2;
 	}
 	
-	ret=tmp->getPicture();
-	delete tmp;
-	
-	return ret;
+	return tmp;
 }
 
 
