@@ -25,15 +25,24 @@
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ***************************************************************************/
+
 #include "widgets.h"
-#include <gtk/gtk.h>
+
 #ifdef GDK_WINDOWING_X11
+#ifndef GAMBAS_DIRECTFB
+#define GOT_TRAYICON
+#endif
+#endif
+
+
+#ifdef GOT_TRAYICON
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
+#endif
+#include "gapplication.h"
+#include "gmouse.h"
+#include "gtrayicon.h"
 
 #define SYSTEM_TRAY_REQUEST_DOCK    0
 #define SYSTEM_TRAY_BEGIN_MESSAGE   1
@@ -48,7 +57,7 @@ Default picture
 
 ******************************************************************************/
 /* XPM */
-static char *_default_trayicon[] = {
+static const char *_default_trayicon[] = {
 /* columns rows colors chars-per-pixel */
 "24 24 200 2",
 "   c black",
@@ -283,7 +292,7 @@ static char *_default_trayicon[] = {
 Low level stuff
 
 ******************************************************************************/
-
+#ifdef GOT_TRAYICON
 void XTray_RequestDock(Display *xdisplay,Window icon)
 {
 	Window xmanager=None;
@@ -343,7 +352,7 @@ void XTray_getSize(Display *xdisplay,Window icon,unsigned int *w,unsigned int *h
 void XTray_getPosition(Display *xdisplay,Window icon,unsigned int *x,unsigned int *y)
 {
 	Window rwin;
-	Window pwin,pwin2;
+	Window pwin;
 	Window *cwin;
 	unsigned int count;
 	XWindowAttributes Attr;
@@ -366,7 +375,7 @@ void XTray_getPosition(Display *xdisplay,Window icon,unsigned int *x,unsigned in
 	} while (pwin);
 }
 
-
+#endif
 
 
 /*************************************************************************
@@ -374,7 +383,8 @@ void XTray_getPosition(Display *xdisplay,Window icon,unsigned int *x,unsigned in
 gTrayIcon
 
 **************************************************************************/
-bool  tray_enterleave(GtkWidget *widget, GdkEventCrossing *e,gTrayIcon *data)
+
+static bool  tray_enterleave(GtkWidget *widget, GdkEventCrossing *e,gTrayIcon *data)
 {
 	if (e->type==GDK_ENTER_NOTIFY)
 	{
@@ -387,14 +397,15 @@ bool  tray_enterleave(GtkWidget *widget, GdkEventCrossing *e,gTrayIcon *data)
 	return false;
 }
 
-void tray_end (GtkWidget *object,gTrayIcon *data)
+static void tray_destroy (GtkWidget *object,gTrayIcon *data)
 {
-	if (data->onHide) { data->onHide=false; return; }
-	if (data->onDestroy) data->onDestroy(data);
-	delete data;
+	//if (data->onHide) { data->onHide=false; return; }
+	//if (data->onDestroy) data->onDestroy(data);
+	data->plug = NULL;
+	//delete data;
 }
 
-gboolean tray_event(GtkWidget *widget, GdkEvent *event,gTrayIcon *data)
+static gboolean tray_event(GtkWidget *widget, GdkEvent *event,gTrayIcon *data)
 {
 	if (!gApplication::userEvents()) return false;
 
@@ -409,16 +420,17 @@ gboolean tray_event(GtkWidget *widget, GdkEvent *event,gTrayIcon *data)
 }
 
 
-gboolean tray_down (GtkWidget *widget,GdkEventButton *event,gTrayIcon *data)
+static gboolean tray_down (GtkWidget *widget,GdkEventButton *event,gTrayIcon *data)
 {
 
 	if (!gApplication::userEvents()) return false;
 
 	if (data->onMousePress)
 	{
-		gMouse::setValid(true,(long)event->x,(long)event->y,event->button,event->state,0,0);
+		gMouse::validate();
+		gMouse::setMouse((int)event->x, (int)event->y, event->button, event->state);
 		data->onMousePress(data);
-		gMouse::setValid(false,0,0,0,0,0,0);
+		gMouse::invalidate();
 	}
 
 	if (event->button==3)
@@ -428,21 +440,31 @@ gboolean tray_down (GtkWidget *widget,GdkEventButton *event,gTrayIcon *data)
 	return false;
 }
 
-gboolean tray_up (GtkWidget *widget,GdkEventButton *event,gTrayIcon *data)
+static gboolean tray_up (GtkWidget *widget,GdkEventButton *event,gTrayIcon *data)
 {
 	if (!gApplication::userEvents()) return false;
 
 	if (data->onMouseRelease)
 	{
-		gMouse::setValid(true,(long)event->x,(long)event->y,event->button,event->state,0,0);
+		gMouse::validate();
+		gMouse::setMouse((int)event->x, (int)event->y, event->button, event->state);
 		data->onMouseRelease(data);
-		gMouse::setValid(false,0,0,0,0,0,0);
+		gMouse::invalidate();
 	}
 
 	return false;
 }
 
-gboolean tray_focus_In(GtkWidget *widget,GdkEventFocus *event,gTrayIcon *data)
+static gboolean cb_menu(GtkWidget *widget, gTrayIcon *data)
+{
+	if (!gApplication::userEvents()) return false;
+	if (data->onMenu)
+		data->onMenu(data);
+	
+	return false;
+}
+
+static gboolean tray_focus_In(GtkWidget *widget,GdkEventFocus *event,gTrayIcon *data)
 {
 	if (!gApplication::allEvents()) return false;
 
@@ -450,7 +472,7 @@ gboolean tray_focus_In(GtkWidget *widget,GdkEventFocus *event,gTrayIcon *data)
 	return false;
 }
 
-gboolean tray_focus_Out(GtkWidget *widget,GdkEventFocus *event,gTrayIcon *data)
+static gboolean tray_focus_Out(GtkWidget *widget,GdkEventFocus *event,gTrayIcon *data)
 {
 	if (!gApplication::allEvents()) return false;
 
@@ -458,34 +480,117 @@ gboolean tray_focus_Out(GtkWidget *widget,GdkEventFocus *event,gTrayIcon *data)
 	return false;
 }
 
-void gTrayIcon::setPicture(gPicture *pic)
+static gboolean cb_scroll(GtkWidget *widget, GdkEventScroll *event, gTrayIcon *data)
+{
+	int dt = 0;
+	int ort = 0;
+	
+	if (!gApplication::userEvents()) return false;
+
+	if (data->onMouseWheel)
+	{
+		switch (event->direction)
+		{
+			case GDK_SCROLL_UP: dt=1; ort=1; break;
+			case GDK_SCROLL_DOWN: dt=-1; ort=1; break;
+			case GDK_SCROLL_LEFT: dt=-1; ort=0; break;
+			case GDK_SCROLL_RIGHT:  dt=1; ort=0; break;
+		}
+		
+		gMouse::validate();
+		gMouse::setMouse((int)event->x, (int)event->y, 0, event->state);
+		gMouse::setWheel(dt, ort);
+		data->onMouseWheel(data);
+		gMouse::invalidate();
+	}
+	
+	return false;
+}
+
+
+GList *gTrayIcon::trayicons = NULL;
+
+#ifdef GOT_TRAYICON
+
+gTrayIcon::gTrayIcon()
+{
+	plug=NULL;
+	buftext=NULL;
+	_icon = NULL;
+
+	//onHide=false;
+	onMousePress=NULL;
+	onMouseRelease=NULL;
+	onMenu=NULL;
+	onFocusEnter=NULL;
+	onFocusLeave=NULL;
+	onDoubleClick=NULL;
+	onEnter=NULL;
+	onLeave=NULL;
+	
+	setPicture(0);
+	
+	trayicons = g_list_append(trayicons, (gpointer)this);
+}
+
+gTrayIcon::~gTrayIcon()
+{
+	gPicture::assign(&_icon);
+	
+	if (buftext) 
+	{
+		g_free(buftext);
+		buftext = NULL;
+	}
+	
+	if (plug) 
+		gtk_widget_destroy(plug);
+	
+	setVisible(false);
+	trayicons = g_list_remove(trayicons, (gpointer)this);
+
+	if (onDestroy) (*onDestroy)(this);
+}
+
+
+GdkPixbuf *gTrayIcon::getIcon()
+{
+	if (_icon)
+		return _icon->getPixbuf();
+	else
+		return gdk_pixbuf_new_from_xpm_data((const char**)_default_trayicon);
+}
+
+void gTrayIcon::updateMask()
+{
+	GdkPixbuf *icon = getIcon();
+	GdkPixmap *mask;
+	int w, h;
+	
+	if (!plug || !icon)
+		return;
+	
+	w = gdk_pixbuf_get_width(icon);
+	h = gdk_pixbuf_get_height(icon);
+		
+	mask = gdk_pixmap_new(NULL, w, h, 1);
+	gdk_pixbuf_render_threshold_alpha(icon, mask, 0, 0, 0, 0, w, h, 128);
+	
+	gtk_widget_shape_combine_mask(plug, mask, 0, 0);
+}
+
+void gTrayIcon::setPicture(gPicture *picture)
 {
 	GtkWidget *icon;
 
-	if (buficon) { g_object_unref(G_OBJECT(buficon)); buficon=NULL; }
-	if (pic)
-	{
-		if (pic->pic)
-		{
-			buficon=pic->getPixbuf();
-		}
-	}
+	gPicture::assign(&_icon, picture);
 
 	if (plug)
 	{
-		icon=gtk_bin_get_child(GTK_BIN(plug));
-		gtk_image_set_from_pixbuf(GTK_IMAGE(icon),buficon);
+		icon = gtk_bin_get_child(GTK_BIN(plug));
+		gtk_image_set_from_pixbuf(GTK_IMAGE(icon), getIcon());
+		updateMask();
 	}
-
-}
-
-gPicture* gTrayIcon::picture()
-{
-	gPicture *pic=NULL;
-
-	pic=gPicture::fromPixbuf(buficon);
-	return pic;
-
 }
 
 char* gTrayIcon::toolTip()
@@ -494,86 +599,75 @@ char* gTrayIcon::toolTip()
 
 }
 
+void gTrayIcon::updateTooltip()
+{
+	if (!plug)
+		return;
+		
+	gtk_tooltips_set_tip(gApplication::tipHandle(), plug, buftext, NULL);
+}
+
 void gTrayIcon::setToolTip(char* vl)
 {
-	GtkTooltips *tip;
-
-	if (!vl) vl="";
-
-	if (buftext) free(buftext);
-	buftext=(char*)malloc(sizeof(char)*(strlen(vl) + 1));
-	if (buftext) strcpy(buftext,vl);
-
-	if (plug && buftext)
-	{
-		tip=gtk_tooltips_new();
-		gtk_tooltips_set_tip(tip,plug,buftext,NULL);
-		if (strlen(buftext)) gtk_tooltips_enable(tip);
-		else gtk_tooltips_disable(tip);
-	}
-
+	if (buftext) 
+		g_free(buftext);
+		
+	buftext = vl && *vl ? g_strdup(vl) : NULL;
+	updateTooltip();
 }
 
-void gTrayIcon::show()
-{
-	GtkTooltips *tip;
-	GtkWidget *icon;
-	Window win;
-
-	if (!plug)
-	{
-		plug=gtk_plug_new(0);
-		icon=gtk_image_new_from_pixbuf(buficon);
-		gtk_container_add(GTK_CONTAINER(plug),icon);
-
-		if (buftext)
-		{
-			tip=gtk_tooltips_new();
-			gtk_tooltips_set_tip(tip,plug,buftext,NULL);
-			if (strlen(buftext)) gtk_tooltips_enable(tip);
-			else gtk_tooltips_disable(tip);
-		}
-
-		gtk_widget_show_all(plug);
-
-		//gtk_widget_add_events(plug,GDK_PROXIMITY_IN_MASK);
-		gtk_widget_add_events(plug,GDK_BUTTON_PRESS_MASK);
-		gtk_widget_add_events(plug,GDK_BUTTON_RELEASE_MASK);
-		gtk_widget_add_events(plug,GDK_ENTER_NOTIFY_MASK);
-		gtk_widget_add_events(plug,GDK_LEAVE_NOTIFY_MASK);
-		g_signal_connect(G_OBJECT(plug),"destroy",G_CALLBACK(tray_end),(gpointer)this);
-		g_signal_connect(G_OBJECT(plug),"event",G_CALLBACK(tray_event),(gpointer)this);
-		g_signal_connect(G_OBJECT(plug),"button-release-event",G_CALLBACK(tray_up),(gpointer)this);
-		g_signal_connect(G_OBJECT(plug),"button-press-event",G_CALLBACK(tray_down),(gpointer)this);
-		g_signal_connect(G_OBJECT(plug),"focus-in-event",G_CALLBACK(tray_focus_In),(gpointer)this);
-		g_signal_connect(G_OBJECT(plug),"focus-out-event",G_CALLBACK(tray_focus_Out),(gpointer)this);
-		g_signal_connect(G_OBJECT(plug),"enter-notify-event",G_CALLBACK(tray_enterleave),(gpointer)this);
-		g_signal_connect(G_OBJECT(plug),"leave-notify-event",G_CALLBACK(tray_enterleave),(gpointer)this);
-		win=gtk_plug_get_id(GTK_PLUG(plug));
-		XTray_RequestDock(gdk_display,win);
-	}
-}
-
-void gTrayIcon::hide()
-{
-	if (plug)
-	{
-		onHide=true;
-		gtk_widget_destroy(plug);
-		plug=NULL;
-	}
-}
-
-bool gTrayIcon::visible()
+bool gTrayIcon::isVisible()
 {
 	return (bool)plug;
 }
 
-
 void gTrayIcon::setVisible(bool vl)
 {
-	if (vl) show();
-	else    hide();
+	if (vl)
+	{
+		GtkWidget *icon;
+		Window win;
+	
+		if (!plug)
+		{
+			plug=gtk_plug_new(0);
+			//XSetWindowBackgroundPixmap(gtk_plug_get_id(plug), ParentRelative);
+			icon=gtk_image_new_from_pixbuf(getIcon());
+			gtk_container_add(GTK_CONTAINER(plug),icon);
+	
+			updateTooltip();
+			updateMask();
+			
+			gtk_widget_show_all(plug);
+	
+			//gtk_widget_add_events(plug,GDK_PROXIMITY_IN_MASK);
+			gtk_widget_add_events(plug,GDK_BUTTON_PRESS_MASK);
+			gtk_widget_add_events(plug,GDK_BUTTON_RELEASE_MASK);
+			gtk_widget_add_events(plug,GDK_ENTER_NOTIFY_MASK);
+			gtk_widget_add_events(plug,GDK_LEAVE_NOTIFY_MASK);
+			g_signal_connect(G_OBJECT(plug),"destroy",G_CALLBACK(tray_destroy),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"event",G_CALLBACK(tray_event),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"button-release-event",G_CALLBACK(tray_up),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"button-press-event",G_CALLBACK(tray_down),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"focus-in-event",G_CALLBACK(tray_focus_In),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"focus-out-event",G_CALLBACK(tray_focus_Out),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"enter-notify-event",G_CALLBACK(tray_enterleave),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"leave-notify-event",G_CALLBACK(tray_enterleave),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"popup-menu",G_CALLBACK(cb_menu),(gpointer)this);
+			g_signal_connect(G_OBJECT(plug),"scroll-event",G_CALLBACK(cb_scroll),(gpointer)this);
+			win=gtk_plug_get_id(GTK_PLUG(plug));
+			XTray_RequestDock(gdk_display,win);
+		}
+	}
+	else
+	{
+		if (plug)
+		{
+			//onHide=true;
+			gtk_widget_destroy(plug);
+			//plug=NULL;
+		}
+	}
 }
 
 long gTrayIcon::screenX()
@@ -612,49 +706,47 @@ long gTrayIcon::height()
 	return ret;
 }
 
-gTrayIcon::gTrayIcon()
+void gTrayIcon::exit()
 {
-	buficon=gdk_pixbuf_new_from_xpm_data((const char**)_default_trayicon);
-	plug=NULL;
-	buftext=NULL;
-
-	onHide=false;
-	onMousePress=NULL;
-	onMouseRelease=NULL;
-	onMenu=NULL;
-	onFocusEnter=NULL;
-	onFocusLeave=NULL;
-	onDoubleClick=NULL;
-	onEnter=NULL;
-	onLeave=NULL;
-}
-
-void gTrayIcon::destroy()
-{
-	if (buficon) g_object_unref(G_OBJECT(buficon));
-	if (plug) gtk_widget_destroy(plug);
-	if (buftext) free(buftext);
+	gTrayIcon *icon;
+	
+	while((icon = get(0)))
+		delete icon;
 }
 
 #else
 
-void gTrayIcon::setPicture(gPicture *pic)
-{
 
+
+gTrayIcon::gTrayIcon()
+{
 	stub("no-X11/gTrayIcon class");
 }
 
-gPicture* gTrayIcon::picture()
+gTrayIcon::~gTrayIcon()
 {
 	stub("no-X11/gTrayIcon class");
-	return NULL;
+}
 
+
+GdkPixbuf *gTrayIcon::getIcon()
+{
+	stub("no-X11/gTrayIcon class");
+}
+
+void gTrayIcon::updateMask()
+{
+	stub("no-X11/gTrayIcon class");
+}
+
+void gTrayIcon::setPicture(gPicture *picture)
+{
+	stub("no-X11/gTrayIcon class");
 }
 
 char* gTrayIcon::toolTip()
 {
 	stub("no-X11/gTrayIcon class");
-	return NULL;
 
 }
 
@@ -664,23 +756,11 @@ void gTrayIcon::setToolTip(char* vl)
 
 }
 
-void gTrayIcon::show()
+bool gTrayIcon::isVisible()
 {
 	stub("no-X11/gTrayIcon class");
 }
 
-void gTrayIcon::hide()
-{
-	stub("no-X11/gTrayIcon class");
-}
-
-bool gTrayIcon::visible()
-{
-	stub("no-X11/gTrayIcon class");
-	return false;
-}
-
-
 void gTrayIcon::setVisible(bool vl)
 {
 	stub("no-X11/gTrayIcon class");
@@ -689,19 +769,16 @@ void gTrayIcon::setVisible(bool vl)
 long gTrayIcon::screenX()
 {
 	stub("no-X11/gTrayIcon class");
-	return 0;
 }
 
 long gTrayIcon::screenY()
 {
 	stub("no-X11/gTrayIcon class");
-	return 0;
 }
 
 long gTrayIcon::width()
 {
 	stub("no-X11/gTrayIcon class");
-	return 0;
 }
 
 long gTrayIcon::height()
@@ -709,12 +786,7 @@ long gTrayIcon::height()
 	stub("no-X11/gTrayIcon class");
 }
 
-gTrayIcon::gTrayIcon()
-{
-	stub("no-X11/gTrayIcon class");
-}
-
-void gTrayIcon::destroy()
+void gTrayIcon::exit()
 {
 	stub("no-X11/gTrayIcon class");
 }
