@@ -2,7 +2,7 @@
 
   watcher.h
 
-  (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
+  (c) 2004-2007 - Daniel Campos Fernández <dcamposf@gmail.com>
   
   GTK+ component
   
@@ -26,21 +26,26 @@
 
 ***************************************************************************/
 
-typedef struct Watcher
-{
-	int fd;
-	int type;
-	void *callback;
-	long param;
+#ifndef __WATCHER_H
+#define __WATCHER_H
 
-} Watcher;
+#include <gtk/gtk.h>
+
+typedef struct
+{
+	void *channel;
+	void *func;
+	void *data;
+	guint id;
+} watchData;
 
 class CWatcher
 {
 public:
-	static void Init();
 	static void Add(int fd,int type,void *callback,long param);
 	static void Clear();
-	static void Perform();
 	static void Remove(int fd);
+	static int count();
 };
+
+#endif
