@@ -724,8 +724,8 @@ public:
      * @return The shaded image. The original is not changed.
      * @author Daniel M. Duley (mosfet)
      */
-    static QImage shade(QImage &src, bool color_shading=true, double azimuth=30.0,
-                        double elevation=30.0);
+    static QImage shade(QImage &src, bool color_shading, double azimuth,
+                        double elevation);
     /**
      * Swirls the image by a specified amount
      *
@@ -736,7 +736,7 @@ public:
      * @return The swirled image. The original is not changed.
      * @author Daniel M. Duley (mosfet)
      */
-     static QImage swirl(QImage &src, double degrees=50.0, unsigned int background =
+     static QImage swirl(QImage &src, double angle, unsigned int background =
                          0xFFFFFFFF);
 
      /**
