@@ -117,13 +117,13 @@ int CCURL_stream_read(GB_STREAM *stream, char *buffer, long len)
 	if (THIS->len_data == len)
 	{
 		THIS->len_data=0;
-		GB.Free((void**)&THIS->buf_data);
+		GB.Free(POINTER(&THIS->buf_data));
 		return 0;
 	}
 
 	THIS->len_data-=len;
 	memmove(THIS->buf_data,len+THIS->buf_data,THIS->len_data);
-	GB.Realloc((void**)&THIS->buf_data,THIS->len_data);
+	GB.Realloc(POINTER(&THIS->buf_data),THIS->len_data);
 
 	return 0;
 }
@@ -197,6 +197,17 @@ void CCURL_Manage_ErrCode(void *_object,long ErrCode)
  This CallBack is called each event loop by Gambas to test
  the status of curl descriptors
  ***************************************************************/
+
+static void stop_post()
+{
+	if (CCURL_pipe[0] < 0) return;
+	
+	GB.Watch (CCURL_pipe[0] ,GB_WATCH_NONE,CCURL_post_curl,0);
+	close(CCURL_pipe[0]);
+	close(CCURL_pipe[1]);
+	CCURL_pipe[0]=-1;
+}
+ 
 void CCURL_stop(void *_object)
 {
 	if (THIS_FILE)
@@ -211,7 +222,14 @@ void CCURL_stop(void *_object)
 		curl_easy_cleanup((CURL*)THIS_CURL);
 		THIS_CURL=(long)NULL;
 	}
+
 	THIS_STATUS=0;
+	stop_post();
+	
+/*	GB.Watch (CCURL_pipe[0] ,GB_WATCH_NONE,CCURL_post_curl,0);
+	close(CCURL_pipe[0]);
+	close(CCURL_pipe[1]);
+	CCURL_pipe[0]=-1;*/
 }
 
 void CCURL_init_post(void)
@@ -255,30 +273,15 @@ void CCURL_post_curl(long data)
 	while (nread);
 
 	if (!post)
-	{
+		stop_post();
+/*	{
 		GB.Watch (CCURL_pipe[0] ,GB_WATCH_NONE,CCURL_post_curl,0);
 		close(CCURL_pipe[0]);
 		close(CCURL_pipe[1]);
 		CCURL_pipe[0]=-1;
-	}
-	
-	
+	}*/	
 }
 
-////////////////////////////////////////////////////////////////////////
-//###################### PROXY related properties ######################
-////////////////////////////////////////////////////////////////////////
-/*********************************************
- Proxy name and port ( Host:Port format)
- *********************************************/
-BEGIN_PROPERTY (CCURL_Proxy)
-
-	GB.ReturnObject(THIS->proxy);
-
-END_PROPERTY
-////////////////////////////////////////////////////////////////////////
-//########################## Other properties ##########################
-////////////////////////////////////////////////////////////////////////
 /*********************************************
  FTP User ( User:Password format )
  *********************************************/
@@ -399,9 +402,9 @@ BEGIN_PROPERTY ( CCURL_URL )
 	if (THIS_URL)
 	{
 		tmp=(char*)THIS_URL;
-		GB.Free((void**)&tmp);
+		GB.Free(POINTER(&tmp));
 	}
-	GB.Alloc((void**)&tmp,(strlen(GB.ToZeroString(PROP(GB_STRING)))+1)*sizeof(char));
+	GB.Alloc(POINTER(&tmp),(strlen(GB.ToZeroString(PROP(GB_STRING)))+1)*sizeof(char));
 	strcpy(tmp,GB.ToZeroString(PROP(GB_STRING)));
 	Adv_correct_url(&tmp,(char*)THIS_PROTOCOL);
 	THIS_URL=(long)tmp;
@@ -427,10 +430,10 @@ BEGIN_METHOD_VOID(CCURL_new)
 	THIS_FILE=(long)NULL;
 	GB.StoreVariant(NULL, (void *)&THIS->tag);
 	Adv_user_NEW  (&THIS->user);
-	GB.New ((void**)&THIS->proxy,GB.FindClass(".Proxy"),NULL,NULL);
-	GB.Ref((void*)THIS->proxy);
-	Adv_proxy_NEW(&THIS->proxy->proxy);
-	THIS->proxy->parent_status=(int*)&THIS->stream._free[1];
+	//GB.New ((void**)&THIS->proxy,GB.FindClass(".Proxy"),NULL,NULL);
+	//GB.Ref((void*)THIS->proxy);
+	Adv_proxy_NEW(&THIS->proxy.proxy);
+	THIS->proxy.parent_status=(int*)&THIS->stream._free[1];
 
 END_METHOD
 
@@ -438,14 +441,13 @@ BEGIN_METHOD_VOID(CCURL_free)
 	
 	char *tmp=(char*)THIS_URL;
 	
-	if (tmp) GB.Free((void**)&tmp);
+	if (tmp) GB.Free(POINTER(&tmp));
 	if (THIS_FILE) fclose((FILE*)THIS_FILE);
 	if (THIS_CURL) curl_easy_cleanup((CURL*)THIS_CURL);
 	Adv_user_CLEAR  (&THIS->user);
-	Adv_proxy_CLEAR(&THIS->proxy->proxy);
-	GB.Unref((void**)&THIS->proxy);
+	Adv_proxy_CLEAR(&THIS->proxy.proxy);
 	tmp=(char*)THIS_PROTOCOL;
-	GB.Free((void**)&tmp);
+	GB.Free(POINTER(&tmp));
 	
 END_METHOD
 
@@ -500,7 +502,7 @@ GB_DESC CCurlDesc[] =
   GB_PROPERTY("Tag", "v", CCURL_tag),
   GB_PROPERTY("Async","b",CCURL_Mode),
   GB_PROPERTY("TimeOut","i",CCURL_TimeOut),
-  GB_PROPERTY_READ("Proxy",".Proxy",CCURL_Proxy),
+  GB_PROPERTY_SELF("Proxy",".CurlProxy"),
   GB_PROPERTY_READ("Status","i",CCURL_Status),
 
 
