@@ -79,9 +79,9 @@ int ftp_write_curl(void *buffer, size_t size, size_t nmemb, void *_object)
 	else
 	{
 		if (!THIS->len_data)
-			GB.Alloc((void**)&THIS->buf_data,nmemb);
+			GB.Alloc((void**)POINTER(&THIS->buf_data),nmemb);
 		else
-			GB.Realloc((void**)&THIS->buf_data,nmemb+THIS->len_data);
+			GB.Realloc((void**)POINTER(&THIS->buf_data),nmemb+THIS->len_data);
 		memcpy(THIS->buf_data+THIS->len_data,buffer,nmemb);
 		THIS->len_data+=nmemb;
 	}
@@ -100,7 +100,7 @@ void ftp_reset(CFTPCLIENT *mythis)
 
 	if (mythis->buf_data)
 	{
-		GB.Free((void**)&mythis->buf_data);
+		GB.Free((void**)POINTER(&mythis->buf_data));
 		mythis->buf_data=NULL;
 	}
 	
@@ -132,7 +132,7 @@ void ftp_initialize_curl_handle(void *_object)
 	curl_easy_setopt((CURL*)THIS_CURL, CURLOPT_VERBOSE,1);
 	curl_easy_setopt((CURL*)THIS_CURL, CURLOPT_PRIVATE,(char*)_object);
 
-	Adv_proxy_SET (&THIS->proxy->proxy,(CURL*)THIS_CURL);
+	Adv_proxy_SET (&THIS->proxy.proxy,(CURL*)THIS_CURL);
 	Adv_user_SET  (&THIS->user, (CURL*)THIS_CURL);
 	curl_easy_setopt((CURL*)THIS_CURL, CURLOPT_URL,THIS_URL);
 
@@ -254,11 +254,11 @@ BEGIN_METHOD_VOID(CFTPCLIENT_new)
 
 	char *tmp=NULL;	
 	
-	GB.Alloc((void**)&tmp,sizeof(char)*(1+strlen("ftp://127.0.0.1:21")));
+	GB.Alloc((void**)POINTER(&tmp),sizeof(char)*(1+strlen("ftp://127.0.0.1:21")));
 	THIS_URL=(long)tmp;
 	strcpy(tmp,"ftp://127.0.0.1:21");
 	tmp=NULL;
-	GB.Alloc((void**)&tmp,7);
+	GB.Alloc((void**)POINTER(&tmp),7);
 	strcpy(tmp,"ftp://");
 	THIS_PROTOCOL=(long)tmp;
 	Adv_user_SETAUTH (&THIS->user,CURLAUTH_BASIC);
