@@ -32,12 +32,29 @@ Thu Aug 16 2001 */
 
 #include "gb_common.h"
 
+#include <errno.h>
 #include <sys/socket.h>
 
 #include "libsmtp.h"
+#include "libsmtp_mime.h"
 
 //#define LIBSMTP_DEBUG
 
+/* Check if a string has only ASCII characters */
+
+static bool is_ascii_string(char *str)
+{
+	unsigned char c;
+	
+	while ((c = *str++))
+	{
+		if (c & 0x80)
+			return FALSE;
+	}
+	
+	return TRUE;
+}
+
 /* internal communication functions */
 
 /* Type is one of:
@@ -66,7 +83,7 @@ int libsmtp_int_read (GString *libsmtp_gstring_read, struct libsmtp_session_stru
   }
 
   #ifdef LIBSMTP_DEBUG
-    printf ("DEBUG in read: %s\n", libsmtp_int_rec_buffer);
+    printf ("DEBUG in read: %.*s", libsmtp_int_bytes, libsmtp_int_rec_buffer);
   #endif
 
   /* Update statistics */
@@ -95,7 +112,7 @@ int libsmtp_int_read (GString *libsmtp_gstring_read, struct libsmtp_session_stru
 
       /* Then fetch the rest of the string and save it */
       libsmtp_int_temp_buffer = strtok (NULL, "\0");
-      libsmtp_session->LastResponse = g_string_new (libsmtp_int_temp_buffer);
+      g_string_assign(libsmtp_session->LastResponse, libsmtp_int_temp_buffer);
       break;
   }
   return LIBSMTP_NOERR;
@@ -111,11 +128,11 @@ int libsmtp_int_send (GString *libsmtp_send_gstring, struct libsmtp_session_stru
   int libsmtp_int_bytes;
 
   #ifdef LIBSMTP_DEBUG
-    printf ("DEBUG in send: %s\n", libsmtp_send_gstring->str);
+    printf ("DEBUG in send: %s", libsmtp_send_gstring->str);
   #endif
 
   libsmtp_int_bytes=send (libsmtp_session->socket, libsmtp_send_gstring->str, libsmtp_send_gstring->len, 0);
-  if (libsmtp_int_bytes<0)
+  if (libsmtp_int_bytes != libsmtp_send_gstring->len)
   {
     libsmtp_session->ErrorCode=LIBSMTP_ERRORSENDFATAL;
     libsmtp_session->Stage=type;
@@ -148,7 +165,7 @@ int libsmtp_int_send_body (char *libsmtp_send_string, unsigned long int libsmtp_
   int libsmtp_int_bytes;
 
   #ifdef LIBSMTP_DEBUG
-    printf ("DEBUG in bsend : %s\n", libsmtp_send_string);
+    printf ("DEBUG in body send : %s\n", libsmtp_send_string);
   #endif
 
   libsmtp_int_bytes=send (libsmtp_session->socket, libsmtp_send_string, strlen(libsmtp_send_string), 0);
@@ -171,9 +188,7 @@ int libsmtp_dialogue (struct libsmtp_session_struct *libsmtp_session)
 {
   int libsmtp_temp;
   GString *libsmtp_temp_gstring;
-  GList *libsmtp_temp_glist, *libsmtp_swap_glist;
-    /* temp_glist is the index into the lists of recipients,
-       swap_glist is needed to adjust the beginning of of the list */
+  GList *libsmtp_temp_glist;
 
   libsmtp_temp_gstring = g_string_new(NULL);
 
@@ -240,7 +255,7 @@ int libsmtp_dialogue (struct libsmtp_session_struct *libsmtp_session)
   {
     libsmtp_temp_glist=g_list_nth (libsmtp_session->To, libsmtp_temp);
     #ifdef LIBSMTP_DEBUG
-      printf ("To: %s\n", libsmtp_temp_glist->data);
+      printf ("To: %s\n", (char *)libsmtp_temp_glist->data);
     #endif
     g_string_sprintf (libsmtp_temp_gstring, "RCPT TO: %s\r\n", \
         (char *)libsmtp_temp_glist->data);
@@ -451,7 +466,7 @@ int libsmtp_headers (struct libsmtp_session_struct *libsmtp_session)
 
   /* First the From: header */
 
-  g_string_sprintf (libsmtp_temp_gstring, "From: %s\n", \
+  g_string_sprintf (libsmtp_temp_gstring, "From: %s\r\n", \
                       libsmtp_session->From->str);
 
   if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 1))
@@ -459,11 +474,29 @@ int libsmtp_headers (struct libsmtp_session_struct *libsmtp_session)
 
   /* Then the Subject: header */
 
-  g_string_sprintf (libsmtp_temp_gstring, "Subject: %s\n", \
-                      libsmtp_session->Subject->str);
+	// BM: if the subject is not ASCII, then it is UTF-8
 
-  if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 1))
-    return LIBSMTP_ERRORSENDFATAL;
+	if (is_ascii_string(libsmtp_session->Subject->str))
+	{
+		g_string_sprintf (libsmtp_temp_gstring, "Subject: %s\r\n", \
+												libsmtp_session->Subject->str);
+	
+		if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 1))
+			return LIBSMTP_ERRORSENDFATAL;
+	}
+	else
+	{
+		/*g_string_assign(libsmtp_temp_gstring, "Subject: =?utf-8?q?");
+		if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 1))
+			return LIBSMTP_ERRORSENDFATAL;*/
+		
+		if (libsmtp_int_send_quoted_header("Subject:", libsmtp_session->Subject->str, libsmtp_session->Subject->len, libsmtp_session))
+			return LIBSMTP_ERRORSENDFATAL;
+	
+		//g_string_assign(libsmtp_temp_gstring, "?=\r\n");
+		//if (libsmtp_int_send(libsmtp_temp_gstring, libsmtp_session, 1))
+		//	return LIBSMTP_ERRORSENDFATAL;
+	}
 
   /* Then we send all the To: addresses */
 
@@ -605,12 +638,13 @@ int libsmtp_body_send_raw (char *libsmtp_body_data, unsigned long int libsmtp_in
   /* Headers should have been sent before body data goes out */
   if (libsmtp_session->Stage == LIBSMTP_HEADERS_STAGE)
   {
-    GString *libsmtp_temp_gstring = g_string_new (NULL);
     /* Now let there be a blank line */
-    libsmtp_temp_gstring = g_string_new ("\n");
+    GString *libsmtp_temp_gstring = g_string_new ("\r\n");
 
     if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 1))
       return LIBSMTP_ERRORSENDFATAL;
+      
+    g_string_free(libsmtp_temp_gstring, TRUE);
   }
 
   /* We now enter the body stage */
@@ -657,7 +691,7 @@ int libsmtp_body_end (struct libsmtp_session_struct *libsmtp_session)
     return LIBSMTP_ERRORREADFATAL;
 
   #ifdef LIBSMTP_DEBUG
-    printf ("DEBUG: %s\n", libsmtp_session->LastResponse);
+    printf ("DEBUG: %s\n", libsmtp_session->LastResponse->str);
   #endif
 
   if (libsmtp_session->LastResponseCode > 299)
@@ -690,7 +724,7 @@ int libsmtp_quit (struct libsmtp_session_struct *libsmtp_session)
     libsmtp_session->Stage = LIBSMTP_QUIT_STAGE;
 
     /* Lets tell him we are quitting. */
-    libsmtp_temp_gstring = g_string_new ("QUIT\n");
+    libsmtp_temp_gstring = g_string_new ("QUIT\r\n");
 
     if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 1))
       return LIBSMTP_ERRORSENDFATAL;
