@@ -240,7 +240,7 @@ int libsmtp_int_nextpart (struct libsmtp_session_struct *libsmtp_session)
     if (libsmtp_session->PartNow->Type == LIBSMTP_MIME_MULTIPART)
     {
       g_string_assign (libsmtp_temp_gstring, \
-         "This is a MIME multipart message. Your mail reader isn't MIME capable.\n\rYou might not be able to read parts or all of this message.\r\n");
+         "This is a MIME multipart message. Your mail reader isn't MIME capable.\r\nYou might not be able to read parts or all of this message.\r\n");
 
       if (libsmtp_int_send (libsmtp_temp_gstring, libsmtp_session, 2))
         return LIBSMTP_ERRORSENDFATAL;
