@@ -2,9 +2,9 @@
 
   GL.c
 
-  openGL component
+  The Gambas openGL component
 
-  (c) 2005-2006 Laurent Carlier <lordheavy@users.sourceforge.net>
+  (c) 2005-2007 Laurent Carlier <lordheavy@users.sourceforge.net>
                 Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
@@ -49,6 +49,12 @@
 
 /**************************************************************************/
 
+BEGIN_METHOD_VOID(GL_exit)
+
+	// free the memory reserved for glGet calls
+	freeAllocs();
+
+END_METHOD
 
 BEGIN_METHOD_VOID(GLCHECKERROR)
 
@@ -62,6 +68,9 @@ GB_DESC Cgl[] =
 {
 	GB_DECLARE("Gl",0), GB_NOT_CREATABLE(),
 	
+//	GB_STATIC_METHOD("_init", NULL, GL_init, NULL),
+	GB_STATIC_METHOD("_exit", NULL, GL_exit, NULL),
+
 	/* Check errors */
 	GB_STATIC_METHOD("GetError", "i", GLCHECKERROR, NULL),
 
@@ -71,7 +80,13 @@ GB_DESC Cgl[] =
 	GB_STATIC_METHOD("End", NULL, GLEND, NULL),
 	GB_STATIC_METHOD("Rectf", NULL, GLRECTF, "(X1)f(Y1)f(X2)f(Y2)f"),
 	GB_STATIC_METHOD("Recti", NULL, GLRECTI, "(X1)i(Y1)i(X2)i(Y2)i"),
+	GB_STATIC_METHOD("Vertex2f", NULL, GLVERTEXF, "(X)f(Y)f"),
+	GB_STATIC_METHOD("Vertex3f", NULL, GLVERTEXF, "(X)f(Y)f(Z)f"),
+	GB_STATIC_METHOD("Vertex4f", NULL, GLVERTEXF, "(X)f(Y)f(Z)f(W)f"),
 	GB_STATIC_METHOD("Vertexf", NULL, GLVERTEXF, "(X)f(Y)f[(Z)f(W)f]"),
+	GB_STATIC_METHOD("Vertex2i", NULL, GLVERTEXI, "(X)i(Y)i"),
+	GB_STATIC_METHOD("Vertex3i", NULL, GLVERTEXI, "(X)i(Y)i(Z)i"),
+	GB_STATIC_METHOD("Vertex4i", NULL, GLVERTEXI, "(X)i(Y)i(Z)i(W)i"),
 	GB_STATIC_METHOD("Vertexi", NULL, GLVERTEXI, "(X)i(Y)i[(Z)i(W)i]"),
 	GB_STATIC_METHOD("Vertexfv", NULL, GLVERTEXFV, "(Coords)Float[]"),
 	GB_STATIC_METHOD("Vertexiv", NULL, GLVERTEXIV, "(Coords)Integer[]"),
@@ -102,7 +117,11 @@ GB_DESC Cgl[] =
 	GB_STATIC_METHOD("Viewport", NULL, GLVIEWPORT, "(X)i(Y)i(Width)i(Height)i"),
 
 	/* Coloring and Lighting - see GLcolorLighting.h */
+	GB_STATIC_METHOD("Color3f", NULL, GLCOLORF, "(Red)f(Green)f(Blue)f"),
+	GB_STATIC_METHOD("Color4f", NULL, GLCOLORF, "(Red)f(Green)f(Blue)f(Alpha)f"),
 	GB_STATIC_METHOD("Colorf", NULL, GLCOLORF, "(Red)f(Green)f(Blue)f[(Alpha)f]"),
+	GB_STATIC_METHOD("Color3i", NULL, GLCOLORI, "(Red)i(Green)i(Blue)i"),
+	GB_STATIC_METHOD("Color4i", NULL, GLCOLORI, "(Red)i(Green)i(Blue)i(Alpha)i"),
 	GB_STATIC_METHOD("Colori", NULL, GLCOLORI, "(Red)i(Green)i(Blue)i[(Alpha)i]"),
 	GB_STATIC_METHOD("Colorfv", NULL, GLCOLORFV, "(Colors)Float[]"),
 	GB_STATIC_METHOD("Coloriv", NULL, GLCOLORIV, "(Colors)Integer[]"),
@@ -143,7 +162,13 @@ GB_DESC Cgl[] =
 	GB_STATIC_METHOD("PointSize", NULL, GLPOINTSIZE, "(Size)f"),
 	GB_STATIC_METHOD("PolygonMode", NULL, GLPOLYGONMODE, "(Face)i(Mode)i"),
 	//GB_STATIC_METHOD("PolygonStipple", NULL, GLPOLYGONSTIPPLE, "(Mask)i"), //TODO
+	GB_STATIC_METHOD("RasterPos2f", NULL, GLRASTERPOSF, "(X)f(Y)f"),
+	GB_STATIC_METHOD("RasterPos3f", NULL, GLRASTERPOSF, "(X)f(Y)f(Z)f"),
+	GB_STATIC_METHOD("RasterPos4f", NULL, GLRASTERPOSF, "(X)f(Y)f(Z)f(W)f"),
 	GB_STATIC_METHOD("RasterPosf", NULL, GLRASTERPOSF, "(X)f(Y)f[(Z)f(W)f]"),
+	GB_STATIC_METHOD("RasterPos2i", NULL, GLRASTERPOSI, "(X)i(Y)i"),
+	GB_STATIC_METHOD("RasterPos3i", NULL, GLRASTERPOSI, "(X)i(Y)i(Z)i"),
+	GB_STATIC_METHOD("RasterPos4i", NULL, GLRASTERPOSI, "(X)i(Y)i(Z)i(W)i"),
 	GB_STATIC_METHOD("RasterPosi", NULL, GLRASTERPOSI, "(X)i(Y)i[(Z)i(W)i]"),
 	GB_STATIC_METHOD("RasterPosfv", NULL, GLRASTERPOSFV, "(Coords)Float[]"),
 	GB_STATIC_METHOD("RasterPosiv", NULL, GLRASTERPOSIV, "(Coords)Integer[]"),
@@ -180,7 +205,15 @@ GB_DESC Cgl[] =
 	GB_STATIC_METHOD("DeleteTextures", NULL, GLDELETETEXTURES, "(Textures)Integer[]"),
 	GB_STATIC_METHOD("GenTextures", "Integer[]", GLGENTEXTURES, "(Count)i"),
 	GB_STATIC_METHOD("IsTexture", "b", GLISTEXTURE, "(Texture)i"),
+	GB_STATIC_METHOD("TexCoord1f", NULL, GLTEXCOORDF, "(S)f"),
+	GB_STATIC_METHOD("TexCoord2f", NULL, GLTEXCOORDF, "(S)f(T)f"),
+	GB_STATIC_METHOD("TexCoord3f", NULL, GLTEXCOORDF, "(S)f(T)f(R)f"),
+	GB_STATIC_METHOD("TexCoord4f", NULL, GLTEXCOORDF, "(S)f(T)f(R)f(Q)f"),
 	GB_STATIC_METHOD("TexCoordf", NULL, GLTEXCOORDF, "(S)f[(T)f(R)f(Q)f]"),
+	GB_STATIC_METHOD("TexCoord1i", NULL, GLTEXCOORDI, "(S)i"),
+	GB_STATIC_METHOD("TexCoord2i", NULL, GLTEXCOORDI, "(S)i(T)i"),
+	GB_STATIC_METHOD("TexCoord3i", NULL, GLTEXCOORDI, "(S)i(T)i(R)i"),
+	GB_STATIC_METHOD("TexCoord4i", NULL, GLTEXCOORDI, "(S)i(T)i(R)i(Q)i"),
 	GB_STATIC_METHOD("TexCoordi", NULL, GLTEXCOORDI, "(S)i[(T)i(R)i(Q)i]"),
 	GB_STATIC_METHOD("TexEnvf", NULL, GLTEXENVF, "(Target)i(Pname)i(Param)f"),
 	GB_STATIC_METHOD("TexEnvfv", NULL, GLTEXENVFV, "(Target)i(Pname)i(Params)Float[]"),
@@ -217,102 +250,10 @@ GB_DESC Cgl[] =
 	/* Selection and Feedback - see GLselectPixmap.h  */
 	GB_STATIC_METHOD("RenderMode", "i", GLRENDERMODE, "(Mode)i"),
 
-	/********************/
-	/* glGetxxxx calls  */
-	/* see GLinfo.h/c   */
-	/********************/
-
-	GB_STATIC_METHOD("GetAccumAlphaBits", "i", GLGETACCUMALPHABITS, NULL),
-	GB_STATIC_METHOD("GetAccumClearValue", "Float[]", GLGETACCUMCLEARVALUE, NULL),
-	GB_STATIC_METHOD("GetAccumBlueBits", "i", GLGETACCUMBLUEBITS, NULL),
-	GB_STATIC_METHOD("GetAccumGreenBits", "i", GLGETACCUMGREENBITS, NULL),
-	GB_STATIC_METHOD("GetAccumRedBits", "i", GLGETACCUMREDBITS, NULL),
-	GB_STATIC_METHOD("GetAlphaBias", "f", GLGETALPHABIAS, NULL),
-	GB_STATIC_METHOD("GetAlphaBits", "i", GLGETALPHABITS, NULL),
-	GB_STATIC_METHOD("GetAlphaScale", "f", GLGETALPHASCALE, NULL),
-	GB_STATIC_METHOD("GetAlphaTestFunc", "i", GLGETALPHATESTFUNC, NULL),
-	GB_STATIC_METHOD("GetAlphaTestRef", "f", GLGETALPHATESTREF, NULL),
-	GB_STATIC_METHOD("GetAttribStackDepth", "f", GLGETATTRIBSTACKDEPTH, NULL),
-	GB_STATIC_METHOD("GetAuxBuffers", "i", GLGETAUXBUFFERS, NULL),
-	GB_STATIC_METHOD("GetBlendDst", "i", GLGETBLENDDST, NULL),
-	GB_STATIC_METHOD("GetBlendSrc", "i", GLGETBLENDSRC, NULL),
-	GB_STATIC_METHOD("GetBlueBias", "f", GLGETBLUEBIAS, NULL),
-	GB_STATIC_METHOD("GetBlueBits", "i", GLGETBLUEBITS, NULL),
-	GB_STATIC_METHOD("GetBlueScale", "f", GLGETBLUESCALE, NULL),
-	GB_STATIC_METHOD("GetColorClearValue", "Float[];", GLGETCOLORCLEARVALUE, NULL),
-	GB_STATIC_METHOD("GetColorMaterialFace", "i", GLGETCOLORMATERIALFACE, NULL),
-	GB_STATIC_METHOD("GetColorMaterialParameter", "i", GLGETCOLORMATERIALPARAMETER, NULL),
-	GB_STATIC_METHOD("GetColorWritemask", "Boolean[];", GLGETCOLORWRITEMASK, NULL),
-	GB_STATIC_METHOD("GetCullFaceMode", "i", GLGETCULLFACEMODE, NULL),
-	GB_STATIC_METHOD("GetCurrentColor", "Float[]", GLGETCURRENTCOLOR, NULL),
-	GB_STATIC_METHOD("GetCurrentIndex", "f", GLGETCURRENTINDEX, NULL),
-	GB_STATIC_METHOD("GetCurrentNormal", "Float[]", GLGETCURRENTNORMAL, NULL),
-	GB_STATIC_METHOD("GetCurrentRasterColor", "Float[]", GLGETCURRENTRASTERCOLOR, NULL),
-	GB_STATIC_METHOD("GetDepthBits", "i", GLGETDEPTHBITS, NULL),
-	GB_STATIC_METHOD("GetDepthClearValue", "f", GLGETDEPTHCLEARVALUE, NULL),
-	GB_STATIC_METHOD("GetDepthFunc", "i", GLGETDEPTHFUNC, NULL),
-	GB_STATIC_METHOD("GetDepthRange", "Float[];", GLGETDEPTHRANGE, NULL),
-	GB_STATIC_METHOD("GetDepthWritemask", "b", GLGETDEPTHWRITEMASK, NULL),
-	GB_STATIC_METHOD("GetDoublebuffer", "b", GLGETDOUBLEBUFFER, NULL),
-	GB_STATIC_METHOD("GetDrawBuffer", "i", GLGETDRAWBUFFER, NULL),
-	GB_STATIC_METHOD("GetEdgeFlag", "b", GLGETEDGEFLAG, NULL),
-	GB_STATIC_METHOD("GetFrontFace", "i", GLGETFRONTFACE, NULL),
-	GB_STATIC_METHOD("GetFogColor", "Float[]", GLGETFOGCOLOR, NULL),
-	GB_STATIC_METHOD("GetFogDensity", "f", GLGETFOGDENSITY, NULL),
-	GB_STATIC_METHOD("GetFogEnd", "f", GLGETFOGEND, NULL),
-	GB_STATIC_METHOD("GetFogHint", "i", GLGETFOGHINT, NULL),
-	GB_STATIC_METHOD("GetFogIndex", "f", GLGETFOGINDEX, NULL),
-	GB_STATIC_METHOD("GetFogMode", "i", GLGETFOGMODE, NULL),
-	GB_STATIC_METHOD("GetFogStart", "f", GLGETFOGSTART, NULL),
-	GB_STATIC_METHOD("GetGreenBias", "f", GLGETGREENBIAS, NULL),
-	GB_STATIC_METHOD("GetGreenBits", "f", GLGETGREENBITS, NULL),
-	GB_STATIC_METHOD("GetGreenScale", "f", GLGETGREENSCALE, NULL),
-	GB_STATIC_METHOD("GetIndexBits", "i", GLGETINDEXBITS, NULL),
-	GB_STATIC_METHOD("GetIndexClearValue", "f", GLGETINDEXCLEARVALUE, NULL),
-	GB_STATIC_METHOD("GetIndexMode", "b", GLGETINDEXMODE, NULL),
-	GB_STATIC_METHOD("GetIndexOffset", "i", GLGETINDEXOFFSET, NULL),
-	GB_STATIC_METHOD("GetIndexShift", "i", GLGETINDEXSHIFT, NULL),
-	GB_STATIC_METHOD("GetIndexWritemask", "i", GLGETINDEXWRITEMASK, NULL),
-	GB_STATIC_METHOD("GetLightModelAmbient", "Float[]", GLGETLIGHTMODELAMBIENT, NULL),
-	GB_STATIC_METHOD("GetLightModelLocalViewer", "b", GLGETLIGHTMODELLOCALVIEWER, NULL),
-	GB_STATIC_METHOD("GetLightModelTwoSide", "f", GLGETLIGHTMODELTWOSIDE, NULL),
-	GB_STATIC_METHOD("GetLineSmoothHint", "i", GLGETLINESMOOTHHINT, NULL),
-	GB_STATIC_METHOD("GetLineStipplePattern", "i", GLGETLINESTIPPLEPATTERN, NULL),
-	GB_STATIC_METHOD("GetLineStippleRepeat", "i", GLGETLINESTIPPLEREPEAT, NULL),
-	GB_STATIC_METHOD("GetLineWidth", "f", GLGETLINEWIDTH, NULL),
-	GB_STATIC_METHOD("GetLineWidthGranularity", "f", GLGETLINEWIDTHGRANULARITY, NULL),
-	GB_STATIC_METHOD("GetLineWidthRange", "Float[]", GLGETLINEWIDTHRANGE, NULL),
-	GB_STATIC_METHOD("GetListBase", "i", GLGETLISTBASE, NULL),
-	GB_STATIC_METHOD("GetListIndex", "i", GLGETLISTINDEX, NULL),
-	GB_STATIC_METHOD("GetListMode", "i", GLGETLISTMODE, NULL),
-	GB_STATIC_METHOD("GetMatrixMode", "i", GLGETMATRIXMODE, NULL),
-	GB_STATIC_METHOD("GetPerspectiveCorrectionHint", "i", GLGETPERSPECTIVECORRECTIONHINT, NULL),
-	GB_STATIC_METHOD("GetPointSmoothHint", "i", GLGETPOINTSMOOTHHINT, NULL),
-	GB_STATIC_METHOD("GetPointSize", "f", GLGETPOINTSIZE, NULL),
-	GB_STATIC_METHOD("GetPointSizeGranularity", "f", GLGETPOINTSIZEGRANULARITY, NULL),
-	GB_STATIC_METHOD("GetPointSizeRange", "Float[]", GLGETPOINTSIZERANGE, NULL),
-	GB_STATIC_METHOD("GetPolygonMode", "Integer[]", GLGETPOLYGONMODE, NULL),
-	GB_STATIC_METHOD("GetPolygonSmoothHint", "i", GLGETPOLYGONSMOOTHHINT, NULL),
-	GB_STATIC_METHOD("GetRgbaMode", "b", GLGETRGBAMODE, NULL),
-	GB_STATIC_METHOD("GetScissorBox", "Integer[]", GLGETSCISSORBOX, NULL),
-	GB_STATIC_METHOD("GetShadeModel", "i", GLGETSHADEMODEL, NULL),
-	GB_STATIC_METHOD("GetStencilBits", "i", GLGETSTENCILBITS, NULL),
-	GB_STATIC_METHOD("GetStencilClearValue", "i", GLGETSTENCILCLEARVALUE, NULL),
-
-	/* Implementation limits */
-	GB_STATIC_METHOD("MaxListNesting", "i", GLMAXLISTNESTING, NULL),
-	GB_STATIC_METHOD("MaxAttribStackDepth", "i", GLMAXATTRIBSTACKDEPTH, NULL),
-	GB_STATIC_METHOD("MaxModelviewStackDepth", "i", GLMAXMODELVIEWSTACKDEPTH, NULL),
-	GB_STATIC_METHOD("MaxNameStackDepth", "i", GLMAXNAMESTACKDEPTH, NULL),
-	GB_STATIC_METHOD("MaxProjectionStackDepth", "i", GLMAXPROJECTIONSTACKDEPTH, NULL),
-	GB_STATIC_METHOD("MaxTextureStackDepth", "i", GLMAXTEXTURESTACKDEPTH, NULL),
-	GB_STATIC_METHOD("MaxEvalOrder", "i", GLMAXEVALORDER, NULL),
-	GB_STATIC_METHOD("MaxLights", "i", GLMAXLIGHTS, NULL),
-	GB_STATIC_METHOD("MaxClipPlanes", "i", GLMAXCLIPPLANES, NULL),
-	GB_STATIC_METHOD("MaxTextureSize", "i", GLMAXTEXTURESIZE, NULL),
-	//GB_STATIC_METHOD("MaxPixelMapTable", "i", GLMAXPIXELMAPTABLE, NULL), needed ? glPixelMap()
-	GB_STATIC_METHOD("MaxViewportDims", "i", GLMAXVIEWPORTDIMS, NULL),
-	GB_STATIC_METHOD("MaxClientAttribStackDepth", "i", GLMAXCLIENTATTRIBSTACKDEPTH, NULL),
+	/* glGetxxxx calls - see GLinfo.h/c   */
+	GB_STATIC_METHOD("GetBooleanv", "Boolean[];", GLGETBOOLEANV, "(Parameter)i[(Size)i]"),
+	GB_STATIC_METHOD("GetFloatv", "Float[];", GLGETFLOATV, "(Parameter)i[(Size)i]"),
+	GB_STATIC_METHOD("GetIntegerv", "Integer[];", GLGETINTEGERV, "(Parameter)i[(Size)i]"),
 
 	/********************/
 	/* opengl constants */
@@ -331,16 +272,31 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_POLYGON", "i", GL_POLYGON),
 
 	/* Matrix Mode */
+	GB_CONSTANT("GL_MATRIX_MODE", "i", GL_MATRIX_MODE),
 	GB_CONSTANT("GL_MODELVIEW", "i", GL_MODELVIEW),
 	GB_CONSTANT("GL_PROJECTION", "i", GL_PROJECTION),
 	GB_CONSTANT("GL_TEXTURE", "i", GL_TEXTURE),
 
+	/* Points */
+	GB_CONSTANT("GL_POINT_SMOOTH", "i", GL_POINT_SMOOTH),
+	GB_CONSTANT("GL_POINT_SIZE", "i", GL_POINT_SIZE),
+	GB_CONSTANT("GL_POINT_SIZE_GRANULARITY", "i", GL_POINT_SIZE_GRANULARITY),
+	GB_CONSTANT("GL_POINT_SIZE_RANGE", "i", GL_POINT_SIZE_RANGE),
+
 	/* Display Lists */
 	GB_CONSTANT("GL_COMPILE", "i", GL_COMPILE),
 	GB_CONSTANT("GL_COMPILE_AND_EXECUTE", "i", GL_COMPILE_AND_EXECUTE),
+	GB_CONSTANT("GL_LIST_BASE", "i", GL_LIST_BASE),
+	GB_CONSTANT("GL_LIST_INDEX", "i", GL_LIST_INDEX),
+	GB_CONSTANT("GL_LIST_MODE" ,"i", GL_LIST_MODE),
 
 	/* Accumulation buffer */
 	GB_CONSTANT("GL_ACCUM", "i", GL_ACCUM),
+	GB_CONSTANT("GL_ACCUM_ALPHA_BITS", "i", GL_ACCUM_ALPHA_BITS),
+	GB_CONSTANT("GL_ACCUM_CLEAR_VALUE", "i", GL_ACCUM_CLEAR_VALUE),
+	GB_CONSTANT("GL_ACCUM_BLUE_BITS", "i", GL_ACCUM_BLUE_BITS),
+	GB_CONSTANT("GL_ACCUM_GREEN_BITS", "i", GL_ACCUM_GREEN_BITS),
+	GB_CONSTANT("GL_ACCUM_RED_BITS", "i", GL_ACCUM_RED_BITS),
 	GB_CONSTANT("GL_ADD", "i", GL_ADD),
 	GB_CONSTANT("GL_LOAD", "i", GL_LOAD),
 	GB_CONSTANT("GL_MULT", "i", GL_MULT),
@@ -356,9 +312,8 @@ GB_DESC Cgl[] =
 
 	/* Alpha testing */
 	GB_CONSTANT("GL_ALPHA_TEST", "i", GL_ALPHA_TEST),
-
-	/* Evaluators */
-	GB_CONSTANT("GL_AUTO_NORMAL", "i", GL_AUTO_NORMAL),
+	GB_CONSTANT("GL_ALPHA_TEST_REF", "i", GL_ALPHA_TEST_REF),
+	GB_CONSTANT("GL_ALPHA_TEST_FUNC", "i", GL_ALPHA_TEST_FUNC),
 
 	/* glPush/PopAttrib bits */
 	GB_CONSTANT("GL_ACCUM_BUFFER_BIT", "i", GL_ACCUM_BUFFER_BIT),
@@ -391,15 +346,21 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_CCW", "i", GL_CCW),
 	GB_CONSTANT("GL_FRONT", "i", GL_FRONT),
 	GB_CONSTANT("GL_BACK", "i", GL_BACK),
+	GB_CONSTANT("GL_POLYGON_MODE", "i", GL_POLYGON_MODE),
 	GB_CONSTANT("GL_POLYGON_SMOOTH", "i", GL_POLYGON_SMOOTH),
 	GB_CONSTANT("GL_POLYGON_STIPPLE", "i", GL_POLYGON_STIPPLE),
+	GB_CONSTANT("GL_EDGE_FLAG", "i", GL_EDGE_FLAG),
 	GB_CONSTANT("GL_CULL_FACE", "i", GL_CULL_FACE),
+	GB_CONSTANT("GL_CULL_FACE_MODE", "i", GL_CULL_FACE_MODE),
+	GB_CONSTANT("GL_FRONT_FACE", "i", GL_FRONT_FACE),
+#if 0
 	// ogl 1.1 : void glPolygonOffset( GLfloat factor,GLfloat units )
-	//GB_CONSTANT("PolygonOffsetFactor", "i", GL_POLYGON_OFFSET_FACTOR),
-	//GB_CONSTANT("PolygonOffsetUnits", "i", GL_POLYGON_OFFSET_UNITS),
-	//GB_CONSTANT("PolygonOffsetPoint", "i", GL_POLYGON_OFFSET_POINT), // TODO glGet
-	//GB_CONSTANT("PolygonOffsetLine", "i", GL_POLYGON_OFFSET_LINE), // TODO glGet
-	//GB_CONSTANT("PolygonOffsetFill", "i", GL_POLYGON_OFFSET_FILL), // TODO glGet
+	GB_CONSTANT("GL_POLYGON_OFFSET_FACTOR", "i", GL_POLYGON_OFFSET_FACTOR),
+	GB_CONSTANT("GL_POLYGON_OFFSET_UNITS", "i", GL_POLYGON_OFFSET_UNITS),
+	GB_CONSTANT("GL_POLYGON_OFFSET_POINT", "i", GL_POLYGON_OFFSET_POINT),
+	GB_CONSTANT("GL_POLYGON_OFFSET_LINE", "i", GL_POLYGON_OFFSET_LINE),
+	GB_CONSTANT("GL_POLYGON_OFFSET_FILL", "i", GL_POLYGON_OFFSET_FILL),
+#endif
 
 	/* Depth buffer */
 	GB_CONSTANT("GL_NEVER", "i", GL_NEVER),
@@ -411,18 +372,48 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_GEQUAL", "i", GL_GEQUAL),
 	GB_CONSTANT("GL_ALWAYS", "i", GL_ALWAYS),
 	GB_CONSTANT("GL_DEPTH_TEST", "i", GL_DEPTH_TEST),
+	GB_CONSTANT("GL_DEPTH_BITS", "i", GL_DEPTH_BITS),
+	GB_CONSTANT("GL_DEPTH_CLEAR_VALUE", "i", GL_DEPTH_CLEAR_VALUE),
+	GB_CONSTANT("GL_DEPTH_FUNC", "i", GL_DEPTH_FUNC),
+	GB_CONSTANT("GL_DEPTH_RANGE", "i", GL_DEPTH_RANGE),
+	GB_CONSTANT("GL_DEPTH_WRITEMASK", "i", GL_DEPTH_WRITEMASK),
+	GB_CONSTANT("GL_DEPTH_COMPONENT", "i", GL_DEPTH_COMPONENT),
 
 	/* Lines */
 	GB_CONSTANT("GL_LINE_SMOOTH", "i", GL_LINE_SMOOTH),
 	GB_CONSTANT("GL_LINE_STIPPLE", "i", GL_LINE_STIPPLE),
+	GB_CONSTANT("GL_LINE_STIPPLE_PATTERN", "i", GL_LINE_STIPPLE_PATTERN),
+	GB_CONSTANT("GL_LINE_STIPPLE_REPEAT", "i", GL_LINE_STIPPLE_REPEAT),
+	GB_CONSTANT("GL_LINE_WIDTH", "i", GL_LINE_WIDTH),
+	GB_CONSTANT("GL_LINE_WIDTH_GRANULARITY", "i", GL_LINE_WIDTH_GRANULARITY),
+	GB_CONSTANT("GL_LINE_WIDTH_RANGE", "i", GL_LINE_WIDTH_RANGE),
 
 	/* Render Mode */
 	GB_CONSTANT("GL_FEEDBACK", "i", GL_FEEDBACK),
 	GB_CONSTANT("GL_RENDER", "i", GL_RENDER),
 	GB_CONSTANT("GL_SELECT", "i", GL_SELECT),
 
-	/* Points */
-	GB_CONSTANT("GL_POINT_SMOOTH", "i", GL_POINT_SMOOTH),
+	/* Feedback */
+	GB_CONSTANT("GL_2D", "i", GL_2D),
+	GB_CONSTANT("GL_3D", "i", GL_3D),
+	GB_CONSTANT("GL_3D_COLOR", "i", GL_3D_COLOR),
+	GB_CONSTANT("GL_3D_COLOR_TEXTURE", "i",  GL_3D_COLOR_TEXTURE),
+	GB_CONSTANT("GL_4D_COLOR_TEXTURE", "i",  GL_4D_COLOR_TEXTURE),
+	GB_CONSTANT("GL_POINT_TOKEN", "i", GL_POINT_TOKEN),
+	GB_CONSTANT("GL_LINE_TOKEN", "i", GL_LINE_TOKEN),
+	GB_CONSTANT("GL_LINE_RESET_TOKEN", "i", GL_LINE_RESET_TOKEN),
+	GB_CONSTANT("GL_POLYGON_TOKEN", "i", GL_POLYGON_TOKEN),
+	GB_CONSTANT("GL_BITMAP_TOKEN", "i", GL_BITMAP_TOKEN),
+	GB_CONSTANT("GL_DRAW_PIXEL_TOKEN", "i", GL_DRAW_PIXEL_TOKEN),
+	GB_CONSTANT("GL_COPY_PIXEL_TOKEN", "i", GL_COPY_PIXEL_TOKEN),
+	GB_CONSTANT("GL_PASS_THROUGH_TOKEN", "i", GL_PASS_THROUGH_TOKEN),
+	GB_CONSTANT("GL_FEEDBACK_BUFFER_POINTER", "i", GL_FEEDBACK_BUFFER_POINTER),
+	GB_CONSTANT("GL_FEEDBACK_BUFFER_SIZE", "i", GL_FEEDBACK_BUFFER_SIZE),
+	GB_CONSTANT("GL_FEEDBACK_BUFFER_TYPE", "i", GL_FEEDBACK_BUFFER_TYPE),
+
+	/* Selection */
+	GB_CONSTANT("GL_SELECTION_BUFFER_POINTER", "i", GL_SELECTION_BUFFER_POINTER),
+	GB_CONSTANT("GL_SELECTION_BUFFER_SIZE", "i", GL_SELECTION_BUFFER_SIZE),
 
 	/* Fog */
 	GB_CONSTANT("GL_FOG", "i", GL_FOG),
@@ -473,13 +464,18 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_LIGHT_MODEL_LOCAL_VIEWER", "i", GL_LIGHT_MODEL_LOCAL_VIEWER),
 	GB_CONSTANT("GL_LIGHT_MODEL_AMBIENT", "i", GL_LIGHT_MODEL_AMBIENT),
 	GB_CONSTANT("GL_FRONT_AND_BACK", "i", GL_FRONT_AND_BACK),
+	GB_CONSTANT("GL_SHADE_MODEL", "i", GL_SHADE_MODEL),
 	GB_CONSTANT("GL_FLAT", "i", GL_FLAT),
 	GB_CONSTANT("GL_SMOOTH", "i", GL_SMOOTH),
 	GB_CONSTANT("GL_COLOR_MATERIAL", "i", GL_COLOR_MATERIAL),
+	GB_CONSTANT("GL_COLOR_MATERIAL_FACE", "i", GL_COLOR_MATERIAL_FACE),
+	GB_CONSTANT("GL_COLOR_MATERIAL_PARAMETER", "i", GL_COLOR_MATERIAL_PARAMETER),
 	GB_CONSTANT("GL_NORMALIZE", "i", GL_NORMALIZE),
 
 	/* Blending */
 	GB_CONSTANT("GL_BLEND", "i", GL_BLEND),
+	GB_CONSTANT("GL_BLEND_SRC", "i", GL_BLEND_SRC),
+	GB_CONSTANT("GL_BLEND_DST", "i", GL_BLEND_DST),
 	GB_CONSTANT("GL_ZERO", "i", GL_ZERO),
 	GB_CONSTANT("GL_ONE", "i", GL_ONE),
 	GB_CONSTANT("GL_SRC_COLOR", "i", GL_SRC_COLOR),
@@ -492,15 +488,44 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_ONE_MINUS_DST_COLOR", "i", GL_ONE_MINUS_DST_COLOR),
 	GB_CONSTANT("GL_SRC_ALPHA_SATURATE", "i", GL_SRC_ALPHA_SATURATE),
 
-	/* Hints */
-	GB_CONSTANT("GL_FOG_HINT", "i", GL_FOG_HINT),
-	GB_CONSTANT("GL_LINE_SMOOTH_HINT", "i", GL_LINE_SMOOTH_HINT),
-	GB_CONSTANT("GL_PERSPECTIVE_CORRECTION_HINT", "i", GL_PERSPECTIVE_CORRECTION_HINT),
-	GB_CONSTANT("GL_POINT_SMOOTH_HINT", "i", GL_POINT_SMOOTH_HINT),
-	GB_CONSTANT("GL_POLYGON_SMOOTH_HINT", "i", GL_POLYGON_SMOOTH_HINT),
-	GB_CONSTANT("GL_DONT_CARE", "i", GL_DONT_CARE),
-	GB_CONSTANT("GL_FASTEST", "i", GL_FASTEST),
-	GB_CONSTANT("GL_NICEST", "i", GL_NICEST),
+	/* Logic Ops */
+	GB_CONSTANT("GL_LOGIC_OP", "i", GL_LOGIC_OP),
+	GB_CONSTANT("GL_INDEX_LOGIC_OP", "i", GL_INDEX_LOGIC_OP),
+	GB_CONSTANT("GL_COLOR_LOGIC_OP", "i", GL_COLOR_LOGIC_OP),
+	GB_CONSTANT("GL_LOGIC_OP_MODE", "i", GL_LOGIC_OP_MODE),
+	GB_CONSTANT("GL_CLEAR", "i", GL_CLEAR),
+	GB_CONSTANT("GL_SET", "i", GL_SET),
+	GB_CONSTANT("GL_COPY", "i", GL_COPY),
+	GB_CONSTANT("GL_COPY_INVERTED", "i", GL_COPY_INVERTED),
+	GB_CONSTANT("GL_NOOP", "i", GL_NOOP),
+	GB_CONSTANT("GL_INVERT", "i", GL_INVERT),
+	GB_CONSTANT("GL_AND", "i", GL_AND),
+	GB_CONSTANT("GL_NAND", "i", GL_NAND),
+	GB_CONSTANT("GL_OR", "i", GL_OR),
+	GB_CONSTANT("GL_NOR", "i", GL_NOR),
+	GB_CONSTANT("GL_XOR", "i", GL_XOR),
+	GB_CONSTANT("GL_EQUIV", "i", GL_EQUIV),
+	GB_CONSTANT("GL_AND_REVERSE", "i", GL_AND_REVERSE),
+	GB_CONSTANT("GL_AND_INVERTED", "i", GL_AND_INVERTED),
+	GB_CONSTANT("GL_OR_REVERSE", "i", GL_OR_REVERSE),
+	GB_CONSTANT("GL_OR_INVERTED", "i", GL_OR_INVERTED),
+
+	/* Stencil */
+	GB_CONSTANT("GL_STENCIL_BITS", "i", GL_STENCIL_BITS),
+	GB_CONSTANT("GL_STENCIL_TEST", "i", GL_STENCIL_TEST),
+	GB_CONSTANT("GL_STENCIL_CLEAR_VALUE", "i", GL_STENCIL_CLEAR_VALUE),
+	GB_CONSTANT("GL_STENCIL_FUNC", "i", GL_STENCIL_FUNC),
+	GB_CONSTANT("GL_STENCIL_VALUE_MASK", "i", GL_STENCIL_VALUE_MASK),
+	GB_CONSTANT("GL_STENCIL_FAIL", "i", GL_STENCIL_FAIL),
+	GB_CONSTANT("GL_STENCIL_PASS_DEPTH_FAIL", "i", GL_STENCIL_PASS_DEPTH_FAIL),
+	GB_CONSTANT("GL_STENCIL_PASS_DEPTH_PASS", "i", GL_STENCIL_PASS_DEPTH_PASS),
+	GB_CONSTANT("GL_STENCIL_REF", "i", GL_STENCIL_REF),
+	GB_CONSTANT("GL_STENCIL_WRITEMASK", "i", GL_STENCIL_WRITEMASK),
+	GB_CONSTANT("GL_STENCIL_INDEX", "i", GL_STENCIL_INDEX),
+	GB_CONSTANT("GL_KEEP", "i", GL_KEEP),
+	GB_CONSTANT("GL_REPLACE", "i", GL_REPLACE),
+	GB_CONSTANT("GL_INCR", "i", GL_INCR),
+	GB_CONSTANT("GL_DECR", "i", GL_DECR),
 
 	/* Buffers, Pixel Drawing/Reading */
 	GB_CONSTANT("GL_AUX0", "i", GL_AUX0),
@@ -514,10 +539,170 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_LEFT", "i", GL_LEFT),
 	GB_CONSTANT("GL_NONE", "i", GL_NONE),
 	GB_CONSTANT("GL_RIGHT", "i", GL_RIGHT),
+	GB_CONSTANT("GL_COLOR_INDEX", "i", GL_COLOR_INDEX),
+	GB_CONSTANT("GL_RED", "i", GL_RED),
+	GB_CONSTANT("GL_GREEN", "i", GL_GREEN),
+	GB_CONSTANT("GL_BLUE", "i", GL_BLUE),
+	GB_CONSTANT("GL_ALPHA", "i", GL_ALPHA),
+	GB_CONSTANT("GL_LUMINANCE", "i", GL_LUMINANCE),
+	GB_CONSTANT("GL_LUMINANCE_ALPHA", "i", GL_LUMINANCE_ALPHA),
+	GB_CONSTANT("GL_ALPHA_BITS", "i", GL_ALPHA_BITS),
+	GB_CONSTANT("GL_RED_BITS", "i", GL_RED_BITS),
+	GB_CONSTANT("GL_GREEN_BITS", "i", GL_GREEN_BITS),
+	GB_CONSTANT("GL_BLUE_BITS", "i", GL_BLUE_BITS),
+	GB_CONSTANT("GL_INDEX_BITS", "i", GL_INDEX_BITS),
+	GB_CONSTANT("GL_SUBPIXEL_BITS", "i", GL_SUBPIXEL_BITS),
+	GB_CONSTANT("GL_AUX_BUFFERS", "i", GL_AUX_BUFFERS),
+	GB_CONSTANT("GL_READ_BUFFER", "i", GL_READ_BUFFER),
+	GB_CONSTANT("GL_DRAW_BUFFER", "i", GL_DRAW_BUFFER),
+	GB_CONSTANT("GL_DOUBLEBUFFER", "i", GL_DOUBLEBUFFER),
+	GB_CONSTANT("GL_STEREO", "i", GL_STEREO),
+	GB_CONSTANT("GL_BITMAP", "i", GL_BITMAP),
+	GB_CONSTANT("GL_COLOR", "i", GL_COLOR),
+	GB_CONSTANT("GL_DEPTH", "i", GL_DEPTH),
+	GB_CONSTANT("GL_STENCIL", "i", GL_STENCIL),
+	GB_CONSTANT("GL_DITHER", "i", GL_DITHER),
+	GB_CONSTANT("GL_RGB", "i", GL_RGB),
+	GB_CONSTANT("GL_RGBA", "i", GL_RGBA),
+
+	/* Implementation limits */
+	GB_CONSTANT("GL_MAX_LIST_NESTING", "i", GL_MAX_LIST_NESTING),
+	GB_CONSTANT("GL_MAX_ATTRIB_STACK_DEPTH", "i", GL_MAX_ATTRIB_STACK_DEPTH),
+	GB_CONSTANT("GL_MAX_NAME_STACK_DEPTH", "i", GL_MAX_NAME_STACK_DEPTH),
+	GB_CONSTANT("GL_MAX_PROJECTION_STACK_DEPTH", "i", GL_MAX_PROJECTION_STACK_DEPTH),
+	GB_CONSTANT("GL_MAX_TEXTURE_STACK_DEPTH", "i", GL_MAX_TEXTURE_STACK_DEPTH),
+	GB_CONSTANT("GL_MAX_EVAL_ORDER", "i", GL_MAX_EVAL_ORDER),
+	GB_CONSTANT("GL_MAX_LIGHTS", "i", GL_MAX_LIGHTS),
+	GB_CONSTANT("GL_MAX_CLIP_PLANES", "i", GL_MAX_CLIP_PLANES),
+	GB_CONSTANT("GL_MAX_TEXTURE_SIZE", "i", GL_MAX_TEXTURE_SIZE),
+	GB_CONSTANT("GL_MAX_PIXEL_MAP_TABLE", "i", GL_MAX_PIXEL_MAP_TABLE),
+	GB_CONSTANT("GL_MAX_VIEWPORT_DIMS", "i", GL_MAX_VIEWPORT_DIMS),
+	GB_CONSTANT("GL_MAX_CLIENT_ATTRIB_STACK_DEPTH", "i", GL_MAX_CLIENT_ATTRIB_STACK_DEPTH),
+	GB_CONSTANT("GL_MAX_MODELVIEW_STACK_DEPTH", "i", GL_MAX_MODELVIEW_STACK_DEPTH),
+
+	/* Gets */
+	GB_CONSTANT("GL_ATTRIB_STACK_DEPTH", "i", GL_ATTRIB_STACK_DEPTH),
+	GB_CONSTANT("GL_CLIENT_ATTRIB_STACK_DEPTH", "i", GL_CLIENT_ATTRIB_STACK_DEPTH),
+	GB_CONSTANT("GL_COLOR_CLEAR_VALUE", "i", GL_COLOR_CLEAR_VALUE),
+	GB_CONSTANT("GL_COLOR_WRITEMASK", "i", GL_COLOR_WRITEMASK),
+	GB_CONSTANT("GL_CURRENT_INDEX", "i", GL_CURRENT_INDEX),
+	GB_CONSTANT("GL_CURRENT_COLOR", "i", GL_CURRENT_COLOR),
+	GB_CONSTANT("GL_CURRENT_NORMAL", "i", GL_CURRENT_NORMAL),
+	GB_CONSTANT("GL_CURRENT_RASTER_COLOR", "i", GL_CURRENT_RASTER_COLOR),
+	GB_CONSTANT("GL_CURRENT_RASTER_DISTANCE", "i", GL_CURRENT_RASTER_DISTANCE),
+	GB_CONSTANT("GL_CURRENT_RASTER_INDEX", "i", GL_CURRENT_RASTER_INDEX),
+	GB_CONSTANT("GL_CURRENT_RASTER_POSITION", "i", GL_CURRENT_RASTER_POSITION),
+	GB_CONSTANT("GL_CURRENT_RASTER_TEXTURE_COORDS", "i", GL_CURRENT_RASTER_TEXTURE_COORDS),
+	GB_CONSTANT("GL_CURRENT_RASTER_POSITION_VALID", "i", GL_CURRENT_RASTER_POSITION_VALID),
+	GB_CONSTANT("GL_CURRENT_TEXTURE_COORDS", "i", GL_CURRENT_TEXTURE_COORDS),
+	GB_CONSTANT("GL_INDEX_CLEAR_VALUE", "i", GL_INDEX_CLEAR_VALUE),
+	GB_CONSTANT("GL_INDEX_MODE", "i", GL_INDEX_MODE),
+	GB_CONSTANT("GL_INDEX_WRITEMASK", "i", GL_INDEX_WRITEMASK),
+	GB_CONSTANT("GL_MODELVIEW_MATRIX", "i", GL_MODELVIEW_MATRIX),
+	GB_CONSTANT("GL_MODELVIEW_STACK_DEPTH", "i", GL_MODELVIEW_STACK_DEPTH),
+	GB_CONSTANT("GL_NAME_STACK_DEPTH", "i", GL_NAME_STACK_DEPTH),
+	GB_CONSTANT("GL_PROJECTION_MATRIX", "i", GL_PROJECTION_MATRIX),
+	GB_CONSTANT("GL_PROJECTION_STACK_DEPTH", "i", GL_PROJECTION_STACK_DEPTH),
+	GB_CONSTANT("GL_RENDER_MODE", "i", GL_RENDER_MODE),
+	GB_CONSTANT("GL_RGBA_MODE", "i", GL_RGBA_MODE),
+	GB_CONSTANT("GL_TEXTURE_MATRIX", "i", GL_TEXTURE_MATRIX),
+	GB_CONSTANT("GL_TEXTURE_STACK_DEPTH", "i", GL_TEXTURE_STACK_DEPTH),
+	GB_CONSTANT("GL_VIEWPORT", "i", GL_VIEWPORT),
+
+	/* Evaluators */
+	GB_CONSTANT("GL_AUTO_NORMAL", "i", GL_AUTO_NORMAL),
+#if 0
+/* Evaluators */
+#define GL_MAP1_COLOR_4				0x0D90
+#define GL_MAP1_INDEX				0x0D91
+#define GL_MAP1_NORMAL				0x0D92
+#define GL_MAP1_TEXTURE_COORD_1			0x0D93
+#define GL_MAP1_TEXTURE_COORD_2			0x0D94
+#define GL_MAP1_TEXTURE_COORD_3			0x0D95
+#define GL_MAP1_TEXTURE_COORD_4			0x0D96
+#define GL_MAP1_VERTEX_3			0x0D97
+#define GL_MAP1_VERTEX_4			0x0D98
+#define GL_MAP2_COLOR_4				0x0DB0
+#define GL_MAP2_INDEX				0x0DB1
+#define GL_MAP2_NORMAL				0x0DB2
+#define GL_MAP2_TEXTURE_COORD_1			0x0DB3
+#define GL_MAP2_TEXTURE_COORD_2			0x0DB4
+#define GL_MAP2_TEXTURE_COORD_3			0x0DB5
+#define GL_MAP2_TEXTURE_COORD_4			0x0DB6
+#define GL_MAP2_VERTEX_3			0x0DB7
+#define GL_MAP2_VERTEX_4			0x0DB8
+#define GL_MAP1_GRID_DOMAIN			0x0DD0
+#define GL_MAP1_GRID_SEGMENTS			0x0DD1
+#define GL_MAP2_GRID_DOMAIN			0x0DD2
+#define GL_MAP2_GRID_SEGMENTS			0x0DD3
+#define GL_COEFF				0x0A00
+#define GL_ORDER				0x0A01
+#define GL_DOMAIN				0x0A02
+#endif
+	/* Hints */
+	GB_CONSTANT("GL_FOG_HINT", "i", GL_FOG_HINT),
+	GB_CONSTANT("GL_LINE_SMOOTH_HINT", "i", GL_LINE_SMOOTH_HINT),
+	GB_CONSTANT("GL_PERSPECTIVE_CORRECTION_HINT", "i", GL_PERSPECTIVE_CORRECTION_HINT),
+	GB_CONSTANT("GL_POINT_SMOOTH_HINT", "i", GL_POINT_SMOOTH_HINT),
+	GB_CONSTANT("GL_POLYGON_SMOOTH_HINT", "i", GL_POLYGON_SMOOTH_HINT),
+	GB_CONSTANT("GL_DONT_CARE", "i", GL_DONT_CARE),
+	GB_CONSTANT("GL_FASTEST", "i", GL_FASTEST),
+	GB_CONSTANT("GL_NICEST", "i", GL_NICEST),
 
 	/* Scissor box */
+	GB_CONSTANT("GL_SCISSOR_BOX", "i", GL_SCISSOR_BOX),
 	GB_CONSTANT("GL_SCISSOR_TEST", "i", GL_SCISSOR_TEST),
-
+#if 0
+/* Pixel Mode / Transfer */
+#define GL_MAP_COLOR				0x0D10
+#define GL_MAP_STENCIL				0x0D11
+#define GL_INDEX_SHIFT				0x0D12
+#define GL_INDEX_OFFSET				0x0D13
+#define GL_RED_SCALE				0x0D14
+#define GL_RED_BIAS				0x0D15
+#define GL_GREEN_SCALE				0x0D18
+#define GL_GREEN_BIAS				0x0D19
+#define GL_BLUE_SCALE				0x0D1A
+#define GL_BLUE_BIAS				0x0D1B
+#define GL_ALPHA_SCALE				0x0D1C
+#define GL_ALPHA_BIAS				0x0D1D
+#define GL_DEPTH_SCALE				0x0D1E
+#define GL_DEPTH_BIAS				0x0D1F
+#define GL_PIXEL_MAP_S_TO_S_SIZE		0x0CB1
+#define GL_PIXEL_MAP_I_TO_I_SIZE		0x0CB0
+#define GL_PIXEL_MAP_I_TO_R_SIZE		0x0CB2
+#define GL_PIXEL_MAP_I_TO_G_SIZE		0x0CB3
+#define GL_PIXEL_MAP_I_TO_B_SIZE		0x0CB4
+#define GL_PIXEL_MAP_I_TO_A_SIZE		0x0CB5
+#define GL_PIXEL_MAP_R_TO_R_SIZE		0x0CB6
+#define GL_PIXEL_MAP_G_TO_G_SIZE		0x0CB7
+#define GL_PIXEL_MAP_B_TO_B_SIZE		0x0CB8
+#define GL_PIXEL_MAP_A_TO_A_SIZE		0x0CB9
+#define GL_PIXEL_MAP_S_TO_S			0x0C71
+#define GL_PIXEL_MAP_I_TO_I			0x0C70
+#define GL_PIXEL_MAP_I_TO_R			0x0C72
+#define GL_PIXEL_MAP_I_TO_G			0x0C73
+#define GL_PIXEL_MAP_I_TO_B			0x0C74
+#define GL_PIXEL_MAP_I_TO_A			0x0C75
+#define GL_PIXEL_MAP_R_TO_R			0x0C76
+#define GL_PIXEL_MAP_G_TO_G			0x0C77
+#define GL_PIXEL_MAP_B_TO_B			0x0C78
+#define GL_PIXEL_MAP_A_TO_A			0x0C79
+#define GL_PACK_ALIGNMENT			0x0D05
+#define GL_PACK_LSB_FIRST			0x0D01
+#define GL_PACK_ROW_LENGTH			0x0D02
+#define GL_PACK_SKIP_PIXELS			0x0D04
+#define GL_PACK_SKIP_ROWS			0x0D03
+#define GL_PACK_SWAP_BYTES			0x0D00
+#define GL_UNPACK_ALIGNMENT			0x0CF5
+#define GL_UNPACK_LSB_FIRST			0x0CF1
+#define GL_UNPACK_ROW_LENGTH			0x0CF2
+#define GL_UNPACK_SKIP_PIXELS			0x0CF4
+#define GL_UNPACK_SKIP_ROWS			0x0CF3
+#define GL_UNPACK_SWAP_BYTES			0x0CF0
+#define GL_ZOOM_X				0x0D16
+#define GL_ZOOM_Y				0x0D17
+#endif
 	/* Texture mapping */
 	GB_CONSTANT("GL_TEXTURE_ENV", "i", GL_TEXTURE_ENV),
 	GB_CONSTANT("GL_TEXTURE_ENV_MODE", "i", GL_TEXTURE_ENV_MODE),
@@ -536,14 +721,12 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_TEXTURE_HEIGHT", "i", GL_TEXTURE_HEIGHT),
 	GB_CONSTANT("GL_TEXTURE_BORDER", "i", GL_TEXTURE_BORDER),
 	GB_CONSTANT("GL_TEXTURE_COMPONENTS", "i", GL_TEXTURE_COMPONENTS),
-#if 0	// needed ?
 	GB_CONSTANT("GL_TEXTURE_RED_SIZE", "i", GL_TEXTURE_RED_SIZE),
 	GB_CONSTANT("GL_TEXTURE_GREEN_SIZE", "i", GL_TEXTURE_GREEN_SIZE),
 	GB_CONSTANT("GL_TEXTURE_BLUE_SIZE", "i", GL_TEXTURE_BLUE_SIZE),
 	GB_CONSTANT("GL_TEXTURE_ALPHA_SIZE", "i", GL_TEXTURE_ALPHA_SIZE),
 	GB_CONSTANT("GL_TEXTURE_LUMINANCE_SIZE", "i", GL_TEXTURE_LUMINANCE_SIZE),
 	GB_CONSTANT("GL_TEXTURE_INTENSITY_SIZE", "i", GL_TEXTURE_INTENSITY_SIZE),
-#endif
 	GB_CONSTANT("GL_NEAREST_MIPMAP_NEAREST", "i", GL_NEAREST_MIPMAP_NEAREST),
 	GB_CONSTANT("GL_NEAREST_MIPMAP_LINEAR", "i", GL_NEAREST_MIPMAP_LINEAR),
 	GB_CONSTANT("GL_LINEAR_MIPMAP_NEAREST", "i", GL_LINEAR_MIPMAP_NEAREST),
@@ -565,6 +748,7 @@ GB_DESC Cgl[] =
 	GB_CONSTANT("GL_TEXTURE_GEN_R", "i", GL_TEXTURE_GEN_R),
 	GB_CONSTANT("GL_TEXTURE_GEN_Q", "i", GL_TEXTURE_GEN_Q),
 
+
 #if 0
 
 /* OpenGL 1.1 */
