@@ -2,10 +2,10 @@
 
   GLprimitives.h
 
-  openGL component
+  The Gambas openGL component
 
-  (c) 2005 Laurent Carlier <lordheavy@users.sourceforge.net>
-           Benoît Minisini <gambas@users.sourceforge.net>
+  (c) 2005-2007 Laurent Carlier <lordheavy@users.sourceforge.net>
+                Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
