@@ -4,7 +4,7 @@
 
   gb.pdf Component
 
-  (C) 2005 Daniel Campos Fernández <danielcampos@netcourrier.com>
+  (C) 2005-2007 Daniel Campos Fernández <dcamposf@gmail.com>
 
 
   This program is free software; you can redistribute it and/or modify
@@ -49,7 +49,6 @@ extern GB_DESC PdfIndexDesc[];
 extern GB_DESC PdfDocumentInfo[];
 extern GB_DESC PdfLayoutDesc[];
 extern GB_DESC PdfModeDesc[];
-extern GB_DESC PdfPermissionsDesc[];
 
 #else
 
@@ -58,24 +57,35 @@ extern GB_DESC PdfPermissionsDesc[];
 #endif
 
 
-typedef
-	struct
-	{
-		GB_BASE ob;
+typedef	struct
+{
+	GB_BASE ob;
 
-		char *buf;
-		long len;
+	char *buf;
+	long len;
 
-		PDFDoc *doc;
-		SplashOutputDev *dev;
-		Page *page;
-		GooList *index;
-		long indexpos;
+	PDFDoc *doc;
+	SplashOutputDev *dev;
+	Page *page;
+	unsigned long currpage;
+		
+	void **pindex;           // Parent of current index entries
+	GooList *index;          // Current entries
+	unsigned long currindex; // Current entry
+ 	unsigned long *oldindex; // Parent entry
 
-		double scale;
-		int rotation;
-	}
-	CPDFDOCUMENT;
+	Links *links;            // Page bookmarks
+	unsigned long lcurrent;  // Current bookmark
+
+	void *Found;             // Found text elements
+	unsigned long fcurrent;  // Current found text
+
+	LinkAction *action;      // Current link action 
+
+	double scale;
+	int rotation;
+
+} CPDFDOCUMENT;
 
 
 
