@@ -30,6 +30,8 @@
 #include <qpalette.h>
 
 #include "CConst.h"
+#include "CColor.h"
+#include "CDraw.h"
 #include "CGridView.h"
 
 DECLARE_EVENT(EVENT_Click);
@@ -49,6 +51,8 @@ void MyTableData::init()
 {
   alignment = ALIGN_NORMAL;
   bg = fg = -1; font = 0; text = 0; pict = 0;
+  padding = 0;
+  richText = 0;
 }
 
 void MyTableData::clear()
@@ -56,6 +60,7 @@ void MyTableData::clear()
   GB.Unref(POINTER(&pict));
   GB.Unref(POINTER(&font));
   GB.FreeString(&text);
+  GB.FreeString(&richText);
   init();
 }
 
@@ -157,6 +162,11 @@ QString MyTableItem::text()
 	return TO_QSTRING(data()->text);
 }
 
+QString MyTableItem::richText()
+{
+	return TO_QSTRING(data()->richText);
+}
+
 QPixmap MyTableItem::pixmap()
 {
 	CPICTURE *pict = data()->pict;
@@ -169,7 +179,7 @@ QPixmap MyTableItem::pixmap()
 
 int MyTableItem::alignment() const
 {
-  return ((MyTableItem *)this)->data()->alignment;
+  return CCONST_alignment(((MyTableItem *)this)->data()->alignment, ALIGN_NORMAL, false);
 }
 
 void MyTableItem::paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bool selected )
@@ -178,32 +188,58 @@ void MyTableItem::paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bo
   int w = cr.width();
   int h = cr.height();
   int x = 0;
+  int y = 0;
 
   QPixmap pix = pixmap();
-  QString txt = text();
+  QString txt;
+  bool rich;
   int _bg = d->bg;
   int _fg = d->fg;
-  int _alignment = d->alignment;
-
-  p->fillRect( 0, 0, w, h,
+  int _alignment = CCONST_alignment(d->alignment, ALIGN_NORMAL, true);
+  int _padding = QMAX(1, d->padding);
+  
+  /*if (((MyTable *)table())->noSelection())
+  	selected = false;
+  else*/ if (row() == table()->currentRow() && col() == table()->currentColumn())
+  	selected = ((MyTable *)table())->isRowReallySelected(row());
+  
+  p->fillRect(x, y, w, h,
     selected ?
       cg.brush(QColorGroup::Highlight)
-      : (_bg < 0) ? cg.brush(QColorGroup::Base) : QColor((uint)_bg));
+      : (_bg == COLOR_DEFAULT) ? cg.brush(QColorGroup::Base) : QColor((uint)_bg));
+
+  if (_padding)
+  {
+  	x += _padding;
+  	y += _padding;
+  	_padding *= 2;
+  	w -= _padding;
+  	h -= _padding;
+  	
+  	if (w < 1 || h < 1)
+  		return;
+  }
+
+	txt = richText();
+	rich = txt.length() > 0;
+	if (!rich)
+		txt = text();
 
   if (!pix.isNull())
   {
     if (txt.length() == 0)
-      p->drawPixmap((w - pix.width()) / 2, (h - pix.height() ) / 2, pix);
+      p->drawPixmap(x + (w - pix.width()) / 2, y + (h - pix.height() ) / 2, pix);
     else
     {
-      p->drawPixmap(2, (h - pix.height() ) / 2, pix);
-      x = pix.width() + 4;
+      p->drawPixmap(x + 2, y + (h - pix.height() ) / 2, pix);
+      x += pix.width() + 4;
+      w -= pix.width() + 4;
     }
   }
 
   if (selected)
     p->setPen(cg.highlightedText());
-  else if (_fg < 0)
+  else if (_fg == COLOR_DEFAULT)
     p->setPen(cg.text());
   else
     p->setPen(QColor((uint)_fg));
@@ -211,8 +247,10 @@ void MyTableItem::paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bo
 	if (d->font)
 		p->setFont(*(d->font->font));
 
-  p->drawText(x + 1, 0, w - x - 4, h,
-    wordWrap() ? (_alignment | WordBreak) : _alignment, txt );
+	if (rich)
+		DRAW_rich_text(p, QApplication::palette(table()).active(), x, y, w, h, wordWrap() ? (_alignment | WordBreak) : _alignment, txt );
+	else
+		p->drawText(x, y, w, h, wordWrap() ? (_alignment | WordBreak) : _alignment, txt );
     
 	if (d->font)
 		p->setFont(table()->font());
@@ -225,9 +263,12 @@ QSize MyTableItem::sizeHint() const
   QSize s;
   int x = 0;
   MyTableItem *item = (MyTableItem *)this;
+	MyTableData *d = item->data();
+	int padding;
   
   QPixmap pix = item->pixmap();
   QString t = item->text();
+  QFontMetrics fm = d->font ? QFontMetrics(*(d->font->font)) : table()->fontMetrics();
 
   if (!pix.isNull() )
   {
@@ -236,16 +277,27 @@ QSize MyTableItem::sizeHint() const
 	  x = pix.width() + 2;
 	}
 
+	padding = QMAX(1, d->padding) * 2;
+  
   if ( !wordWrap() && t.find( '\n' ) == -1 )
-		return QSize(s.width() + table()->fontMetrics().width(t) + 10,
-		             QMAX( s.height(), table()->fontMetrics().height() ) ).expandedTo( strutSize );
+  {
+  	s = QSize(s.width() + fm.width(t) + 10,
+		          QMAX( s.height(), fm.height() ) ).expandedTo( strutSize );
+	}
+	else
+	{
+		QRect r = fm.boundingRect(0, 0, table()->columnWidth( col() ) - padding, 0,
+							   wordWrap() ? (alignment() | WordBreak) : alignment(),
+							   t );
+		r.setWidth( QMAX( r.width() + 10, table()->columnWidth(col())));
 
-	QRect r = table()->fontMetrics().boundingRect( x + 2, 0, table()->columnWidth( col() ) - x - 4, 0,
-						   wordWrap() ? (alignment() | WordBreak) : alignment(),
-						   t );
-	r.setWidth( QMAX( r.width() + 10, table()->columnWidth(col())));
+  	s = QSize( r.width(), QMAX( s.height(), r.height() ) ).expandedTo( strutSize );
+  }
+  
+	s.setWidth(s.width() + padding);
+	s.setHeight(s.height() + padding);
 
-  return QSize( r.width(), QMAX( s.height(), r.height() ) ).expandedTo( strutSize );
+  return s;
 }
 
 /** MyTable ******************************************************************/
@@ -257,12 +309,15 @@ QTable(0, 0, parent)
   _header = 0;
   _rows = 0;
   _cols = 0;
-  _no_selection = 0;
-  
+  _last_col_width = 0;
+  _no_row = true;
+  _no_col = true;
+   
   setSelectionMode(NoSelection);
   setFocusStyle(FollowStyle);
   verticalHeader()->setMovingEnabled(false);
   horizontalHeader()->setMovingEnabled(false);
+  setDragEnabled(false);
   updateHeaders();
 }
 
@@ -392,6 +447,8 @@ void MyTable::setNumCols(int newCols)
 
   QTable::setNumCols(newCols);
 
+	_last_col_width = 0;
+
   if (newCols > col)
   {
     bool upd = horizontalHeader()->isUpdatesEnabled();
@@ -479,41 +536,182 @@ void MyTable::updateColumn(int col)
 
 void MyTable::setCurrentCell(int row, int col)
 {
-	clearSelection();
+	_no_row = row < 0 || row >= numRows();
+	_no_col = col < 0 || col >= numCols();
+	
+	if (_no_row || _no_col)
+	{
+		clearSelection();
+		return;
+	}
+	
 	QTable::setCurrentCell(row, col);
 }
 
-/*void MyTable::createCell(int row, int col)
+void MyTable::updateCurrentCell()
 {
-	int key = getKey(row, col);
-	
-	if (_dict[key])
-		return;
-		
-	_dict.insert(key, new MyTableItem);
+	setCurrentCell(currentRow(), currentColumn());
 }
 
-void MyTable::removeCell(int row, int col)
+void MyTable::getCurrentCell(int *row, int *col)
 {
-	int key = getKey(row, col);
-	MyTableItem *item = _dict[key];
+	if (row)
+	{
+		if (_no_row)
+			*row = -1;
+		else
+			*row = currentRow();
+	}
 	
-	if (!item)
+	if (col)
+	{
+		if (_no_col)
+			*col = -1;
+		else
+			*col = currentColumn();
+	}
+}
+
+int MyTable::findSelection(int row)
+{
+	int i;
+	QTableSelection s;
+	
+	for (i = 0; i < numSelections(); i++)
+	{
+		s = selection(i);	
+		if (row >= s.topRow() && row <= s.bottomRow())
+			return i;
+	}
+	
+	return -1;
+}
+
+bool MyTable::isRowReallySelected(int row)
+{
+	return findSelection(row) >= 0;
+}
+
+void MyTable::selectRow(int row, bool update)
+{
+	if (selectionMode() == NoSelection)
 		return;
 		
-	_dict.remove(key);
-	delete item;
-}*/
-
-
-/*
-void MyTable::swapColumns(int col1, int col2, bool swapHeader)
-{
-  QTable::swapColumns(col1, col2, swapHeader);
-  updateColumn(col1);
-  updateColumn(col2);
+	if (selectionMode() == SingleRow)
+	{
+		if (row == currentRow() && !isRowReallySelected(row))
+		{
+			setSelectionMode(NoSelection);
+			setSelectionMode(SingleRow);
+		}
+		else
+			setCurrentCell(row, currentColumn());
+		return;
+	}
+	
+	if (isRowReallySelected(row))
+		return;
+	
+	QTableSelection s(row, 0, row, numCols() - 1);
+	addSelection(s);
+	
+	if (update)
+		updateHeaderStates();
 }
-*/
+
+void MyTable::unselectRow(int row)
+{
+	int i;
+	QTableSelection s;
+	bool u = false;
+	
+	if (selectionMode() != MultiRow)
+		return;
+	
+	for(;;)
+	{
+		i = findSelection(row);
+		if (i < 0)
+			break;
+		
+		s = selection(i);
+		removeSelection(i);
+		u = true;
+		
+		for (i = s.topRow(); i <= s.bottomRow(); i++)
+			if (i != row)
+				selectRow(i);
+	}
+				
+	if (u) 
+	{
+		updateHeaderStates();
+	}
+}
+
+
+void MyTable::selectRows(int start, int length)
+{
+	if (selectionMode() == NoSelection)
+		return;
+
+	length += start;
+
+	if (selectionMode() == SingleRow)
+	{
+		selectRow(length - 1);
+		return;
+	}
+
+	for (int i = start; i < length; i++)
+		selectRow(i, false);
+		
+	updateHeaderStates();
+}
+
+void MyTable::updateLastColumn()
+{
+	int n = numCols() - 1;
+	
+	if (n < 0)
+		return;
+	
+	if (!_last_col_width)
+		_last_col_width = columnWidth(n);
+	
+	if (((columnPos(n) + _last_col_width) < visibleWidth()) && (columnWidth(n) != visibleWidth() - columnPos(n)))
+		setColumnWidth(n, visibleWidth() - columnPos(n));
+}
+
+void MyTable::resizeEvent(QResizeEvent *e)
+{
+	QTable::resizeEvent(e);
+	updateLastColumn();
+}
+
+void MyTable::columnWidthChanged(int col)
+{
+	QTable::columnWidthChanged(col);
+	//if (col != (numCols() - 1))
+		updateLastColumn();
+}
+
+void MyTableItem::move(int srow, int scol, int drow, int dcol)
+{
+  MyTableData *data;
+  int skey = getKey(srow, scol);
+  int dkey = getKey(drow, dcol);
+  
+  _dict.remove(dkey);
+  
+  data = _dict.take(skey);
+  if (data)
+  	_dict.insert(dkey, data);
+}
+
+
+
+/** GridView *****************************************************************/
 
 static MyTableItem *get_item(void *_object, bool create)
 {
@@ -534,6 +732,20 @@ static void text_property(void *_object, void *_param, bool create)
 	}
 }
 
+static void rich_text_property(void *_object, void *_param, bool create)
+{
+	MyTableItem *item = get_item(_object, create);
+	
+  if (READ_PROPERTY)
+    GB.ReturnString(item->data()->richText);
+  else
+  {
+    GB.StoreString(PROP(GB_STRING), &item->data(create)->richText);
+    if (create)
+			WIDGET->updateCell(THIS->row, THIS->col);
+	}
+}
+
 static void picture_property(void *_object, void *_param, bool create)
 {
 	MyTableItem *item = get_item(_object, create);	
@@ -626,6 +838,20 @@ static void foreground_property(void *_object, void *_param, bool create)
 }
 
 
+static void padding_property(void *_object, void *_param, bool create)
+{
+	MyTableItem *item = get_item(_object, create);
+		
+  if (READ_PROPERTY)
+    GB.ReturnInteger(item->data()->padding);
+  else
+  {
+    item->data(create)->padding = QMAX(0, VPROP(GB_INTEGER));
+    if (create)
+			WIDGET->updateCell(THIS->row, THIS->col);
+	}
+}
+
 void MyTable::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cliph)
 {
 	if (isVisible())
@@ -645,6 +871,13 @@ BEGIN_PROPERTY(CGRIDVIEW_data_text)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(CGRIDVIEW_data_rich_text)
+
+	rich_text_property(_object, _param, false);
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(CGRIDVIEW_data_picture)
 
 	picture_property(_object, _param, false);
@@ -680,6 +913,15 @@ BEGIN_PROPERTY(CGRIDVIEW_data_foreground)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(CGRIDVIEW_data_padding)
+
+	padding_property(_object, _param, false);
+
+END_PROPERTY
+
+
+
+
 /***************************************************************************
 
   GridViewItem
@@ -733,6 +975,13 @@ BEGIN_PROPERTY(CGRIDITEM_text)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(CGRIDITEM_rich_text)
+
+	rich_text_property(_object, _param, true);
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(CGRIDITEM_picture)
 
 	picture_property(_object, _param, true);
@@ -768,6 +1017,12 @@ BEGIN_PROPERTY(CGRIDITEM_foreground)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(CGRIDITEM_padding)
+
+	padding_property(_object, _param, true);
+
+END_PROPERTY
+
 
 BEGIN_METHOD_VOID(CGRIDITEM_ensure_visible)
 
@@ -785,6 +1040,17 @@ BEGIN_METHOD_VOID(CGRIDITEM_refresh)
 END_METHOD
 
 
+BEGIN_METHOD_VOID(CGRIDITEM_clear)
+
+	MyTableItem *item = get_item(THIS, false);
+  item->clear();
+  
+  WIDGET->item()->setRow(-1);
+  WIDGET->item()->setCol(-1);
+  WIDGET->updateCell(THIS->row, THIS->col);
+
+END_METHOD
+
 /***************************************************************************
 
   GridViewRows
@@ -852,31 +1118,25 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDROWS_resizable)
 
+	int row = THIS->row;
+
   if (READ_PROPERTY)
-    GB.ReturnBoolean(WIDGET->verticalHeader()->isResizeEnabled());
+    GB.ReturnBoolean(WIDGET->verticalHeader()->isResizeEnabled(row));
   else
-    WIDGET->verticalHeader()->setResizeEnabled(VPROP(GB_BOOLEAN));
+    WIDGET->verticalHeader()->setResizeEnabled(VPROP(GB_BOOLEAN), row);
 
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CGRIDROWS_moveable)
+// BEGIN_PROPERTY(CGRIDROWS_moveable)
+// 
+//   if (READ_PROPERTY)
+//     GB.ReturnBoolean(WIDGET->verticalHeader()->isMovingEnabled());
+//   else
+//     WIDGET->verticalHeader()->setMovingEnabled(VPROP(GB_BOOLEAN));
+// 
+// END_PROPERTY
 
-  if (READ_PROPERTY)
-    GB.ReturnBoolean(WIDGET->verticalHeader()->isMovingEnabled());
-  else
-    WIDGET->verticalHeader()->setMovingEnabled(VPROP(GB_BOOLEAN));
-
-END_PROPERTY
-
-
-static void select_rows(CGRIDVIEW *_object, int start, int length)
-{
-	length += start;
-
-	for (int i = start; i < length; i++)
-		WIDGET->selectRow(i);
-}
 
 BEGIN_METHOD(CGRIDROWS_select, GB_INTEGER start; GB_INTEGER length)
 
@@ -891,18 +1151,16 @@ BEGIN_METHOD(CGRIDROWS_select, GB_INTEGER start; GB_INTEGER length)
 		length = 0;
 
 	WIDGET->clearSelection();
-
-	select_rows(THIS, start, length);
+	WIDGET->selectRows(start, length);
 
 END_PROPERTY
 
 
 BEGIN_METHOD(CGRIDROWS_select_all, GB_BOOLEAN sel)
 
+	WIDGET->clearSelection();
 	if (VARGOPT(sel, TRUE))
-		select_rows(THIS, 0, WIDGET->numRows());
-	else
-		WIDGET->clearSelection();
+		WIDGET->selectRows(0, WIDGET->numRows());
 
 END_PROPERTY
 
@@ -934,10 +1192,67 @@ END_METHOD
 BEGIN_METHOD_VOID(CGRIDROWS_selected)
 
 	if (READ_PROPERTY)
-  	GB.ReturnBoolean(WIDGET->isRowSelected(THIS->row, true));
+  	GB.ReturnBoolean(WIDGET->isRowReallySelected(THIS->row));
 	else
-		WIDGET->selectRow(THIS->row);
+	{
+		if (VPROP(GB_BOOLEAN))
+		{
+			if (!WIDGET->isRowReallySelected(THIS->row))
+				WIDGET->selectRow(THIS->row);
+		}
+		else
+			WIDGET->unselectRow(THIS->row);
+	}
+	
+END_METHOD
 
+
+BEGIN_METHOD(CGRIDROWS_remove, GB_INTEGER start; GB_INTEGER length)
+
+	int start = VARG(start);
+	int length = VARGOPT(length, 1);
+	int i, j, dst;
+
+	if (start < 0 || start >= WIDGET->numRows() || length <= 0 || (start + length) > WIDGET->numRows())
+	{
+		GB.Error(GB_ERR_ARG);
+		return;
+	}
+	
+	dst = start;
+	
+	for (i = start + length; i < WIDGET->numRows(); i++)
+	{
+		for (j = 0; j < WIDGET->numCols(); j++)
+			WIDGET->moveItem(i, j, dst, j);
+		dst++;
+	}
+		
+	WIDGET->setNumRows(WIDGET->numRows() - length);
+
+END_METHOD
+
+
+BEGIN_METHOD(CGRIDROWS_insert, GB_INTEGER start; GB_INTEGER length)
+
+	int start = VARG(start);
+	int length = VARGOPT(length, 1);
+	int i, j;
+
+	if (start < 0 || length <= 0 || start > WIDGET->numRows())
+	{
+		GB.Error(GB_ERR_ARG);
+		return;
+	}
+	
+	for (i = WIDGET->numRows() - 1; i >= start; i--)
+	{
+		for (j = 0; j < WIDGET->numCols(); j++)
+			WIDGET->moveItem(i, j, i + length, j);
+	}
+
+	WIDGET->setNumRows(WIDGET->numRows() + length);
+	
 END_METHOD
 
 
@@ -955,8 +1270,8 @@ static void update_autoresize(CGRIDVIEW *_object)
   //WIDGET->horizontalHeader()->setResizeEnabled(!THIS->autoresize);
 
   //WIDGET->horizontalHeader()->setStretchEnabled(true);
-  if (WIDGET->numCols())
-  	WIDGET->setColumnStretchable(WIDGET->numCols() - 1, THIS->autoresize);
+  //if (WIDGET->numCols())
+  //	WIDGET->setColumnStretchable(WIDGET->numCols() - 1, THIS->autoresize);
 }
 
 BEGIN_METHOD(CGRIDCOLS_get, GB_INTEGER col)
@@ -1023,11 +1338,13 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CGRIDCOLS_resizable)
 
+	int col = THIS->col;
+
   if (READ_PROPERTY)
-    GB.ReturnBoolean(WIDGET->horizontalHeader()->isResizeEnabled());
+    GB.ReturnBoolean(WIDGET->horizontalHeader()->isResizeEnabled(col));
   else
   {
-	  WIDGET->horizontalHeader()->setResizeEnabled(VPROP(GB_BOOLEAN));
+	  WIDGET->horizontalHeader()->setResizeEnabled(VPROP(GB_BOOLEAN), col);
   	//THIS->autoresize = !VPROP(GB_BOOLEAN);
   	//update_autoresize(THIS);
 	}
@@ -1035,14 +1352,14 @@ BEGIN_PROPERTY(CGRIDCOLS_resizable)
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CGRIDCOLS_moveable)
-
-  if (READ_PROPERTY)
-    GB.ReturnBoolean(WIDGET->horizontalHeader()->isMovingEnabled());
-  else
-    WIDGET->horizontalHeader()->setMovingEnabled(VPROP(GB_BOOLEAN));
-
-END_PROPERTY
+// BEGIN_PROPERTY(CGRIDCOLS_moveable)
+// 
+//   if (READ_PROPERTY)
+//     GB.ReturnBoolean(WIDGET->horizontalHeader()->isMovingEnabled());
+//   else
+//     WIDGET->horizontalHeader()->setMovingEnabled(VPROP(GB_BOOLEAN));
+// 
+// END_PROPERTY
 
 
 BEGIN_PROPERTY(CGRIDCOLS_text)
@@ -1096,6 +1413,7 @@ BEGIN_METHOD(CGRIDVIEW_new, GB_OBJECT parent)
 
   THIS->row = -1;
   THIS->col = -1;
+  THIS->autoresize = true;
 
   wid->show();
 
@@ -1111,41 +1429,34 @@ END_METHOD
 
 static void set_current_cell(CGRIDVIEW *_object, int row, int col)
 {
-	if (row < 0 || col < 0 || row >= WIDGET->numRows() || col >= WIDGET->numCols())
-	{
-		//QTable::SelectionMode mode = WIDGET->selectionMode();
-		//WIDGET->setSelectionMode(QTable::NoSelection);
-		//WIDGET->setSelectionMode(mode);
-		WIDGET->clearSelection();
-	}
-	else
-	{
-		/*if (row == WIDGET->currentRow() && WIDGET->selectionMode() == QTable::SingleRow)
-		{
-			WIDGET->blockSignals(true);
-	    WIDGET->setCurrentCell(row == 0 ? 1 : row - 1, col);
-	    WIDGET->blockSignals(false);
-		}*/
-    WIDGET->setCurrentCell(row, col);
-	}
+	WIDGET->clearSelection();
+  WIDGET->setCurrentCell(row, col);
 }
 
 BEGIN_PROPERTY(CGRIDVIEW_row)
 
+	int row, col;
+	
+	WIDGET->getCurrentCell(&row, &col);
+
   if (READ_PROPERTY)
-    GB.ReturnInteger(WIDGET->currentRow());
+    GB.ReturnInteger(row);
   else
-    set_current_cell(THIS, VPROP(GB_INTEGER), WIDGET->currentColumn());
+    set_current_cell(THIS, VPROP(GB_INTEGER), col);
 
 END_PROPERTY
 
 
 BEGIN_PROPERTY(CGRIDVIEW_column)
 
+	int row, col;
+	
+	WIDGET->getCurrentCell(&row, &col);
+
   if (READ_PROPERTY)
-    GB.ReturnInteger(WIDGET->currentColumn());
+    GB.ReturnInteger(col);
   else
-    set_current_cell(THIS, WIDGET->currentRow(), VPROP(GB_INTEGER));
+    set_current_cell(THIS, row, VPROP(GB_INTEGER));
 
 END_PROPERTY
 
@@ -1193,8 +1504,7 @@ END_METHOD
 
 BEGIN_PROPERTY(CGRIDVIEW_current)
 
-  THIS->row = WIDGET->currentRow();
-  THIS->col = WIDGET->currentColumn();
+	WIDGET->getCurrentCell(&THIS->row, &THIS->col);
 
   if (CGridView::check(WIDGET, THIS->row, THIS->col))
     return;
@@ -1272,19 +1582,40 @@ BEGIN_PROPERTY(CGRIDVIEW_mode)
   }
   else
   {
-    switch(VPROP(GB_INTEGER))
+  	int mode = VPROP(GB_INTEGER);
+  	
+  	if (mode == WIDGET->selectionMode())
+  		return;
+  	
+    switch(mode)
     {
       case 0: WIDGET->setSelectionMode(QTable::NoSelection); break;
       case 1: WIDGET->setSelectionMode(QTable::SingleRow); break;
       case 2: WIDGET->setSelectionMode(QTable::MultiRow); break;
-      case 3: WIDGET->setSelectionMode(QTable::Single); break;
-      case 4: WIDGET->setSelectionMode(QTable::Multi); break;
+      //case 3: WIDGET->setSelectionMode(QTable::Single); break;
+      //case 4: WIDGET->setSelectionMode(QTable::Multi); break;
     }
   }
 
 END_PROPERTY
 
 
+BEGIN_PROPERTY(CGRIDVIEW_client_x)
+
+  WIDGET->updateScrollBars();
+  GB.ReturnInteger(WIDGET->contentsRect().x());
+
+END_PROPERTY
+
+
+BEGIN_PROPERTY(CGRIDVIEW_client_y)
+
+  WIDGET->updateScrollBars();
+  GB.ReturnInteger(WIDGET->contentsRect().y());
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(CGRIDVIEW_client_width)
 
   WIDGET->updateScrollBars();
@@ -1339,33 +1670,60 @@ BEGIN_PROPERTY(CGRIDVIEW_scroll_y)
 
 END_PROPERTY
 
-BEGIN_METHOD(CGRIDVIEW_row_at, GB_INTEGER ypos)
-
-  int ypos = VARG(ypos);
-
-  ypos = ypos + WIDGET->contentsY();
+static int row_at(CGRIDVIEW *_object, int y)
+{
+  y += WIDGET->contentsY();
 
   if (!WIDGET->horizontalHeader()->isHidden())
-    ypos = ypos - WIDGET->horizontalHeader()->height();
+    y -= WIDGET->horizontalHeader()->height();
 
-  GB.ReturnInteger(WIDGET->rowAt(ypos));
+	return WIDGET->rowAt(y);
+}
+
+static int column_at(CGRIDVIEW *_object, int x)
+{
+  x += WIDGET->contentsX();
+
+  if (!WIDGET->verticalHeader()->isHidden())
+    x -= WIDGET->verticalHeader()->width();
+
+	return WIDGET->columnAt(x);
+}
+
+BEGIN_METHOD(CGRIDVIEW_row_at, GB_INTEGER ypos)
+
+  GB.ReturnInteger(row_at(THIS, VARG(ypos)));
 
 END_PROPERTY
 
 BEGIN_METHOD(CGRIDVIEW_column_at, GB_INTEGER xpos)
 
-  int xpos = VARG(xpos);
-
-  xpos = xpos + WIDGET->contentsX();
-
-  if (!WIDGET->verticalHeader()->isHidden())
-    xpos = xpos - WIDGET->verticalHeader()->width();
-
-  GB.ReturnInteger(WIDGET->columnAt(xpos));
+  GB.ReturnInteger(column_at(THIS, VARG(xpos)));
 
 END_PROPERTY
 
+BEGIN_METHOD(CGRIDVIEW_find, GB_INTEGER x; GB_INTEGER y)
 
+	int px, py;
+
+	px = row_at(THIS, VARG(y));
+	py = column_at(THIS, VARG(x));
+	
+	if (px < 0 || py < 0)
+	{
+		GB.ReturnBoolean(TRUE);
+		return;
+	}
+	
+	THIS->row = px;
+	THIS->col = py;
+
+	GB.ReturnBoolean(FALSE);
+
+END_METHOD
+
+
+#if 0
 BEGIN_PROPERTY(CGRIDVIEW_autoresize)
 
   if (READ_PROPERTY)
@@ -1377,7 +1735,7 @@ BEGIN_PROPERTY(CGRIDVIEW_autoresize)
 	}
 
 END_PROPERTY
-
+#endif
 #if 0
 BEGIN_PROPERTY(CGRIDVIEW_table)
 
@@ -1408,6 +1766,7 @@ GB_DESC CGridViewDataDesc[] =
   //GB_PROPERTY("Alignment", "i", CGRIDITEM_alignment),
 
   GB_PROPERTY("Text", "s", CGRIDVIEW_data_text),
+  GB_PROPERTY("RichText", "s", CGRIDVIEW_data_rich_text),
   GB_PROPERTY("Picture", "Picture", CGRIDVIEW_data_picture),
   GB_PROPERTY("Font", "Font", CGRIDVIEW_data_font),
   GB_PROPERTY("Alignment", "i", CGRIDVIEW_data_alignment),
@@ -1415,6 +1774,7 @@ GB_DESC CGridViewDataDesc[] =
   GB_PROPERTY("BackColor", "i", CGRIDVIEW_data_background),
   GB_PROPERTY("Foreground", "i", CGRIDVIEW_data_foreground),
   GB_PROPERTY("ForeColor", "i", CGRIDVIEW_data_foreground),
+  GB_PROPERTY("Padding", "i", CGRIDVIEW_data_padding),
 
   GB_END_DECLARE
 };
@@ -1429,11 +1789,13 @@ GB_DESC CGridItemDesc[] =
   GB_PROPERTY("Picture", "Picture", CGRIDITEM_picture),
   GB_PROPERTY("Font", "Font", CGRIDITEM_font),
   GB_PROPERTY("Text", "s", CGRIDITEM_text),
+  GB_PROPERTY("RichText", "s", CGRIDITEM_rich_text),
   GB_PROPERTY("Alignment", "i", CGRIDITEM_alignment),
   GB_PROPERTY("Background", "i", CGRIDITEM_background),
   GB_PROPERTY("BackColor", "i", CGRIDITEM_background),
   GB_PROPERTY("Foreground", "i", CGRIDITEM_foreground),
   GB_PROPERTY("ForeColor", "i", CGRIDITEM_foreground),
+  GB_PROPERTY("Padding", "i", CGRIDITEM_padding),
 
   GB_PROPERTY_READ("X", "i", CGRIDITEM_x),
   GB_PROPERTY_READ("Y", "i", CGRIDITEM_y),
@@ -1444,7 +1806,7 @@ GB_DESC CGridItemDesc[] =
   GB_PROPERTY_READ("W", "i", CGRIDITEM_width),
   GB_PROPERTY_READ("H", "i", CGRIDITEM_height),
 
-  //GB_METHOD("Clear", NULL, CGRIDITEM_clear, NULL),
+  GB_METHOD("Clear", NULL, CGRIDITEM_clear, NULL),
   GB_METHOD("EnsureVisible", NULL, CGRIDITEM_ensure_visible, NULL),
   GB_METHOD("Refresh", NULL, CGRIDITEM_refresh, NULL),
 
@@ -1456,13 +1818,15 @@ GB_DESC CGridRowDesc[] =
 {
   GB_DECLARE(".GridViewRow", 0), GB_VIRTUAL_CLASS(),
 
-  GB_PROPERTY_READ("X", "i", CGRIDITEM_x),
+  GB_PROPERTY_READ("Y", "i", CGRIDITEM_y),
+  GB_PROPERTY_READ("Top", "i", CGRIDITEM_y),
   GB_PROPERTY("Height", "i", CGRIDROWS_height),
   GB_PROPERTY("H", "i", CGRIDROWS_height),
   GB_PROPERTY("Text", "s", CGRIDROWS_text),
   GB_PROPERTY("Title", "s", CGRIDROWS_text),
   GB_PROPERTY("Selected", "b", CGRIDROWS_selected),
   GB_METHOD("Refresh", NULL, CGRIDROWS_refresh, NULL),
+  GB_PROPERTY("Resizable", "b", CGRIDROWS_resizable),
 
   GB_END_DECLARE
 };
@@ -1472,12 +1836,14 @@ GB_DESC CGridColumnDesc[] =
 {
   GB_DECLARE(".GridViewColumn", 0), GB_VIRTUAL_CLASS(),
 
-  GB_PROPERTY_READ("Y", "i", CGRIDITEM_y),
+  GB_PROPERTY_READ("X", "i", CGRIDITEM_x),
+  GB_PROPERTY_READ("Left", "i", CGRIDITEM_x),
   GB_PROPERTY("Width", "i", CGRIDCOLS_width),
   GB_PROPERTY("W", "i", CGRIDCOLS_width),
   GB_PROPERTY("Text", "s", CGRIDCOLS_text),
   GB_PROPERTY("Title", "s", CGRIDCOLS_text),
   GB_METHOD("Refresh", NULL, CGRIDCOLS_refresh, NULL),
+  GB_PROPERTY("Resizable", "b", CGRIDCOLS_resizable),
 
   GB_END_DECLARE
 };
@@ -1494,10 +1860,12 @@ GB_DESC CGridRowsDesc[] =
   GB_PROPERTY_READ("Width", "i", CGRIDROWS_width),
   GB_PROPERTY_READ("W", "i", CGRIDROWS_width),
   GB_PROPERTY("Resizable", "b", CGRIDROWS_resizable),
-  GB_PROPERTY("Moveable", "b", CGRIDROWS_moveable),
+  //GB_PROPERTY("Moveable", "b", CGRIDROWS_moveable),
   GB_METHOD("Select", NULL, CGRIDROWS_select, "[(Start)i(Length)i]"),
   GB_METHOD("SelectAll", NULL, CGRIDROWS_select_all, "[(Selected)b]"),
-  GB_METHOD("UnSelect", NULL, CGRIDROWS_unselect, NULL),
+  GB_METHOD("Unselect", NULL, CGRIDROWS_unselect, NULL),
+  GB_METHOD("Remove", NULL, CGRIDROWS_remove, "(Start)i[(Length)i]"),
+  GB_METHOD("Insert", NULL, CGRIDROWS_insert, "(Start)i[(Length)i]"),
   GB_END_DECLARE
 };
 
@@ -1513,7 +1881,9 @@ GB_DESC CGridColumnsDesc[] =
   GB_PROPERTY_READ("Height", "i", CGRIDCOLS_height),
   GB_PROPERTY_READ("H", "i", CGRIDCOLS_height),
   GB_PROPERTY("Resizable", "b", CGRIDCOLS_resizable),
-  GB_PROPERTY("Moveable", "b", CGRIDCOLS_moveable),
+  //GB_PROPERTY("Moveable", "b", CGRIDCOLS_moveable),
+  //GB_METHOD("Remove", NULL, CGRIDCOLS_remove, "(Start)i[(Length)i]"),
+  //GB_METHOD("Insert", NULL, CGRIDCOLS_insert, "(Start)i[(Length)i]"),
 
   GB_END_DECLARE
 };
@@ -1547,14 +1917,17 @@ GB_DESC CGridViewDesc[] =
   GB_PROPERTY("ScrollBar", "i", CGRIDVIEW_scrollbars),
   GB_PROPERTY("Header", "i", CGRIDVIEW_header),
   GB_PROPERTY("Mode", "i", CGRIDVIEW_mode),
-  GB_PROPERTY("AutoResize", "b", CGRIDVIEW_autoresize),
+  //GB_PROPERTY("AutoResize", "b", CGRIDVIEW_autoresize),
   GB_PROPERTY("Resizable", "b", CGRIDCOLS_resizable),
 
   GB_METHOD("RowAt", "i", CGRIDVIEW_row_at, "(Y)i"),
   GB_METHOD("ColumnAt", "i", CGRIDVIEW_column_at,"(X)i"),
+  GB_METHOD("Find", "b", CGRIDVIEW_find, "(X)i(Y)i"),
 
   GB_PROPERTY("ScrollX", "i", CGRIDVIEW_scroll_x),
   GB_PROPERTY("ScrollY", "i", CGRIDVIEW_scroll_y),
+  GB_PROPERTY_READ("ClientX", "i", CGRIDVIEW_client_x),
+  GB_PROPERTY_READ("ClientY", "i", CGRIDVIEW_client_y),
   GB_PROPERTY_READ("ClientWidth", "i", CGRIDVIEW_client_width),
   GB_PROPERTY_READ("ClientW", "i", CGRIDVIEW_client_width),
   GB_PROPERTY_READ("ClientHeight", "i", CGRIDVIEW_client_height),
@@ -1564,10 +1937,7 @@ GB_DESC CGridViewDesc[] =
 
   GB_METHOD("Refresh", NULL, CGRIDVIEW_refresh, "[(X)i(Y)i(Width)i(Height)i]"),
 
-  GB_CONSTANT("_Properties", "s", 
-  	"*,Mode{Select.None;Single;Multiple}=None,Grid=True,Header{GridView.None;Vertical;Horizontal;Both}=None,"
-  	"Scrollbar{Scroll.*}=Both,Border=True,Resizable=True,AutoResize"),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
+	GRIDVIEW_DESCRIPTION,
 
   GB_EVENT("Change", NULL, NULL, &EVENT_Change),
   GB_EVENT("Select", NULL, NULL, &EVENT_Select),
@@ -1626,10 +1996,15 @@ bool CGridView::check(QTable *table, long row, long col)
 
 void CGridView::changed(void)
 {
+	int row, col;
+	
 	GET_SENDER(_object);
   MyTable *w = (MyTable *)sender();
 
-  if (w->currentRow() < 0 || w->currentColumn() < 0)
+	w->updateCurrentCell();
+	w->getCurrentCell(&row, &col);
+
+  if (row < 0 || col < 0)
     return;
 
   GB.Raise(THIS, EVENT_Change, 0);
