@@ -51,9 +51,17 @@ DECLARE_EVENT(EVENT_Cancel);
 DECLARE_EVENT(EVENT_Compare);
 DECLARE_EVENT(EVENT_Expand);
 DECLARE_EVENT(EVENT_Collapse);
-DECLARE_EVENT(EVENT_ColumnClick);
+//DECLARE_EVENT(EVENT_ColumnClick);
 
 
+static int _selection_mode[] = 
+{
+	SELECT_NONE, QListView::NoSelection, 
+	SELECT_SINGLE, QListView::Single, 
+	SELECT_MULTIPLE, QListView::Extended, 
+	CONST_MAGIC
+};
+
 /***************************************************************************
 
   class MyListViewItem
@@ -121,6 +129,7 @@ MyListViewItem::~MyListViewItem()
 {
   MyListView *lw = (MyListView *)container->widget.widget;
   MyListViewItem *par = (MyListViewItem *)parent();
+  MyListViewItem *next = (MyListViewItem *)nextSibling();
 
   //qDebug("< delete item %s %p", key, this);
 
@@ -128,21 +137,24 @@ MyListViewItem::~MyListViewItem()
   {
     if (par->last == this)
       par->last = prev;
+    // Why that ?
+    //par->takeItem(this);
   }
   else
   {
     if (lw->last == this)
       lw->last = prev;
+    // Why that ?
+    //lw->takeItem(this);
   }
 
-  par = ((MyListViewItem *)nextSibling());
-  if (par)
-    par->prev = prev;
+  if (next)
+    next->prev = prev;
 
   //qDebug("container = %p ->item = %p", container, container->item);
 
   if (container->item == this)
-    container->item = par;
+    container->item = 0;
   if (container->save == this)
     container->save = 0;
 
@@ -171,7 +183,7 @@ int MyListViewItem::compare(QListViewItem *i, int col, bool ascending) const
 
   container->compare = 0;
   GB.Raise(container, EVENT_Compare, 2, GB_T_STRING, key, 0, GB_T_STRING, ((MyListViewItem *)i)->key, 0);
-  return ascending ? container->compare : - container->compare;
+  return container->compare;
 }
 
 #if 0
@@ -244,6 +256,8 @@ MyListView::MyListView(QWidget *parent)
 : QListView(parent)
 {
   last = 0;
+  _auto_resize = true;
+  _column = -1;
 }
 
 void MyListView::contentsDragEnterEvent(QDragEnterEvent *e)
@@ -269,6 +283,49 @@ void MyListView::contentsDragLeaveEvent(QDragLeaveEvent *e)
 	//QListView::contentsDragLeaveEvent(e);
 }
 
+void MyListView::setAutoResize(bool a)
+{
+	_auto_resize = a;
+	
+	for (int i = 0; i < columns(); i++)
+		setAutoResize(i, a);
+}
+
+void MyListView::setAutoResize(int col, bool a)
+{
+	setColumnWidthMode(col, QListView::Manual);
+	if (a)
+	{
+		QFontMetrics fm(font());
+		QString s = columnText(col);
+		setColumnWidth(col, fm.width(s) + 16);
+		setColumnWidthMode(col, QListView::Maximum);
+	}
+}
+
+int MyListView::minimumWidth(int col)
+{
+	QFontMetrics fm(font());
+	QString s = columnText(col);
+	return fm.width(s) + 16;
+}
+
+void MyListView::setColumnText(int col, const QString &s)
+{
+	QListView::setColumnText(col, s);
+	if (_auto_resize)
+	{
+		int w = minimumWidth(col);
+		if (columnWidth(col) < w)
+			setColumnWidth(col, w);
+	}
+}
+
+void MyListView::clear()
+{
+	QListView::clear();
+	last = 0;
+}
 
 /***************************************************************************
 
@@ -297,7 +354,7 @@ static MyListView *listview_init(void *_object, void *parent)
   QObject::connect(wid, SIGNAL(pressed(QListViewItem *)), &CTreeView::manager, SLOT(clicked(QListViewItem *)));
   QObject::connect(wid, SIGNAL(itemRenamed(QListViewItem *, int)), &CTreeView::manager, SLOT(renamed(QListViewItem *, int)));
 
-  CWIDGET_new(wid, (void *)_object, "ListView");
+  CWIDGET_new(wid, (void *)_object);
 
   THIS->dict = new QAsciiDict<MyListViewItem>;
   THIS->sorted = -1;
@@ -307,6 +364,7 @@ static MyListView *listview_init(void *_object, void *parent)
   THIS->rename = false;
 
   wid->setSorting(-1);
+  wid->setSelectionMode(QListView::Single);
 
   return wid;
 }
@@ -321,7 +379,7 @@ BEGIN_METHOD(CTREEVIEW_new, GB_OBJECT parent)
 
   wid->addColumn("-");
   wid->header()->hide();
-  //wid->setRootIsDecorated(true);
+  wid->setRootIsDecorated(true);
   wid->show();
 
 END_METHOD
@@ -346,7 +404,7 @@ BEGIN_METHOD(CCOLUMNVIEW_new, GB_OBJECT parent)
 
   MyListView *wid = listview_init(_object, VARG(parent));
 
-  QObject::connect(wid, SIGNAL(pressed(QListViewItem *, const QPoint &, int)), &CTreeView::manager, SLOT(columnClicked(QListViewItem *, const QPoint &, int)));
+  //QObject::connect(wid, SIGNAL(pressed(QListViewItem *, const QPoint &, int)), &CTreeView::manager, SLOT(columnClicked(QListViewItem *, const QPoint &, int)));
   QObject::connect(wid->header(), SIGNAL(clicked(int)), &CTreeView::manager, SLOT(headerClicked(int)));
   QObject::connect(wid->header(), SIGNAL(sizeChange(int, int, int)), &CTreeView::manager, SLOT(headerSizeChange(int, int, int)));
   QObject::connect(wid, SIGNAL(expanded(QListViewItem *)), &CTreeView::manager, SLOT(expanded(QListViewItem *)));
@@ -356,7 +414,11 @@ BEGIN_METHOD(CCOLUMNVIEW_new, GB_OBJECT parent)
   //wid->setColumnAlignment(0, Qt::AlignLeft);
   //wid->setColumnWidthMode(0, QListView::Manual);
   wid->header()->setMovingEnabled(false);
+  wid->header()->setResizeEnabled(false);
   wid->setAllColumnsShowFocus(true);
+  
+  wid->setResizeMode(QListView::LastColumn);
+
   wid->show();
 
 END_METHOD
@@ -392,6 +454,30 @@ BEGIN_METHOD_VOID(CTREEVIEW_clear)
 
 END_METHOD
 
+BEGIN_METHOD_VOID(CCOLUMNVIEW_clear)
+
+  WIDGET->clear();
+  THIS->dict->clear();
+  
+  if (WIDGET->isAutoResize())
+  {
+		//QListView::ResizeMode mode = WIDGET->resizeMode();
+		WIDGET->setResizeMode(QListView::NoColumn);
+		
+		for (int i = 0; i < WIDGET->columns(); i++)
+		{
+			WIDGET->setColumnWidthMode(i, QListView::Manual);
+			WIDGET->setColumnWidth(i, 16);
+			WIDGET->setColumnText(i, WIDGET->columnText(i));
+			WIDGET->setColumnWidthMode(i, QListView::Maximum);  	
+		}
+		
+		WIDGET->setAutoResize(WIDGET->isAutoResize());
+		WIDGET->setResizeMode(QListView::LastColumn);
+	}
+
+END_METHOD
+
 
 BEGIN_METHOD(CTREEVIEW_add, GB_STRING key; GB_STRING text; GB_OBJECT picture; GB_STRING parent; GB_STRING after)
 
@@ -572,6 +658,9 @@ BEGIN_METHOD(CTREEVIEW_find, GB_INTEGER x; GB_INTEGER y)
   QPoint p(VARG(x), VARG(y));
   MyListViewItem *lvi;
 
+	if (!WIDGET->header()->isHidden())
+		p.setY(p.y() - WIDGET->header()->height());
+
   //qDebug("before: %d %d", p.x(), p.y());
   WIDGET->viewport()->mapFrom(WIDGET, p);
   //qDebug("after: %d %d", p.x(), p.y());
@@ -600,19 +689,9 @@ END_METHOD
 BEGIN_PROPERTY(CTREEVIEW_mode)
 
   if (READ_PROPERTY)
-    GB.ReturnInteger(
-    	CCONST_convert(WIDGET->selectionMode(), 3, 
-    		QListView::NoSelection, SELECT_NONE, 
-    		QListView::Single, SELECT_SINGLE, 
-    		QListView::Extended, SELECT_MULTIPLE, 
-    		SELECT_NONE));
+    GB.ReturnInteger(CCONST_convert(_selection_mode, WIDGET->selectionMode(), SELECT_NONE, false));
   else
-    WIDGET->setSelectionMode((QListView::SelectionMode)
-    	CCONST_convert(VPROP(GB_INTEGER), -3, 
-    		QListView::NoSelection, SELECT_NONE, 
-    		QListView::Single, SELECT_SINGLE, 
-    		QListView::Extended, SELECT_MULTIPLE, 
-    		QListView::NoSelection));
+    WIDGET->setSelectionMode((QListView::SelectionMode)CCONST_convert(_selection_mode, VPROP(GB_INTEGER), SELECT_NONE, true));
 
 END_PROPERTY
 
@@ -638,7 +717,7 @@ BEGIN_PROPERTY(CTREEVIEW_key)
 
   MyListViewItem *item = (MyListViewItem *)(WIDGET->currentItem());
 
-  THIS->item = item;
+  //THIS->item = item;
 
   if (item == 0)
     GB.ReturnNull();
@@ -733,6 +812,7 @@ END_PROPERTY
 
 BEGIN_METHOD_VOID(CTREEVIEW_first)
 
+	//qDebug("CTREEVIEW_first: %d", WIDGET->childCount());
   return_item(THIS, (MyListViewItem *)(WIDGET->firstChild()));
 
 END_METHOD
@@ -859,6 +939,13 @@ BEGIN_PROPERTY(CTREEVIEWITEM_expanded)
 
   MyListViewItem *it = THIS->item;
 
+  if (THIS->item->childCount() == 0)
+  {
+    if (READ_PROPERTY)
+      GB.ReturnBoolean(false);
+    return;
+  }
+
   if (READ_PROPERTY)
     GB.ReturnBoolean(it->isOpen());
   else
@@ -1098,17 +1185,13 @@ END_PROPERTY
 
 BEGIN_PROPERTY(CTREEVIEWITEM_w)
 
-	int w, m;
-  QRect r = WIDGET->itemRect(THIS->item);
-	
-	if (r.isValid())
-	{
-		m = WIDGET->treeStepSize() * THIS->item->depth();
-		w = r.width() - m;
-	}
-	else
-		w = 0;
+	int w;
+  //QRect r = WIDGET->itemRect(THIS->item);
 	
+	w = WIDGET->header()->cellPos(WIDGET->header()->count() - 1) 
+	    + WIDGET->header()->cellSize(WIDGET->header()->count() - 1)
+	    - WIDGET->treeStepSize() * THIS->item->depth();
+
 	GB.ReturnInteger(w);
 
 END_PROPERTY
@@ -1128,13 +1211,6 @@ END_PROPERTY
 
 ***************************************************************************/
 
-BEGIN_PROPERTY(CLISTVIEW_columns)
-
-  RETURN_SELF();
-
-END_PROPERTY
-
-
 BEGIN_PROPERTY(CLISTVIEW_resizable)
 
   if (READ_PROPERTY)
@@ -1148,29 +1224,14 @@ BEGIN_PROPERTY(CLISTVIEW_resizable)
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CLISTVIEW_auto_resize)
-
-  if (READ_PROPERTY)
-    GB.ReturnBoolean(WIDGET->resizeMode() == QListView::LastColumn);
-  else
-  {
-    WIDGET->setResizeMode(VPROP(GB_BOOLEAN) ? QListView::LastColumn : QListView::NoColumn);
-    for (int i = 0; i < WIDGET->columns(); i++)
-	   	WIDGET->setColumnWidthMode(i, VPROP(GB_BOOLEAN) ? QListView::Manual : QListView::Maximum);
-
-	}
-
-END_PROPERTY
-
-
-BEGIN_PROPERTY(CLISTVIEW_moveable)
+/*BEGIN_PROPERTY(CLISTVIEW_moveable)
 
   if (READ_PROPERTY)
     GB.ReturnBoolean(WIDGET->header()->isMovingEnabled());
   else
     WIDGET->header()->setMovingEnabled(VPROP(GB_BOOLEAN));
 
-END_PROPERTY
+END_PROPERTY*/
 
 
 BEGIN_PROPERTY(CLISTVIEW_columns_count)
@@ -1194,24 +1255,6 @@ BEGIN_PROPERTY(CLISTVIEW_columns_count)
     return;
   }
 
-  //mode = WIDGET->resizeMode();
-  //WIDGET->setResizeMode(
-
-  /*
-  numeric = new bool[col];
-  memset(numeric, 0, col * sizeof(*numeric));
-
-  if (THIS->numeric)
-  {
-    n = QMIN(WIDGET->columns(), col);
-    for (i = 0; i < n; i++)
-      numeric[i] = THIS->numeric[i];
-    delete[] THIS->numeric;
-  }
-
-  THIS->numeric = numeric;
-  */
-
   mode = WIDGET->resizeMode();
   WIDGET->setResizeMode(QListView::NoColumn);
 
@@ -1224,9 +1267,9 @@ BEGIN_PROPERTY(CLISTVIEW_columns_count)
   {
     while (col > WIDGET->columns())
     {
-      WIDGET->addColumn("");
-      if (mode == QListView::NoColumn)
-      	WIDGET->setColumnWidthMode(WIDGET->columns() - 1, QListView::Maximum);
+      WIDGET->addColumn("", 16);
+      //if (mode == QListView::NoColumn)
+      WIDGET->setAutoResize(WIDGET->columns() - 1, WIDGET->isAutoResize());
     }
   }
 
@@ -1284,6 +1327,7 @@ END_PROPERTY
 BEGIN_PROPERTY(CLISTVIEW_column_width)
 
   int w;
+  QListView::ResizeMode mode;
 
   if (READ_PROPERTY)
     GB.ReturnInteger(WIDGET->columnWidth(WIDGET->_column));
@@ -1294,7 +1338,22 @@ BEGIN_PROPERTY(CLISTVIEW_column_width)
     //  updateLastColumn(WIDGET);
     //else
     //WIDGET->setColumnWidthMode(_column, QListView::Manual);
-    WIDGET->setColumnWidth(WIDGET->_column, w);
+		mode = WIDGET->resizeMode();
+		WIDGET->setResizeMode(QListView::NoColumn);
+		    
+    if (w < 0)
+    {
+			//WIDGET->setColumnWidthMode(WIDGET->_column, QListView::Manual);
+    	//WIDGET->setColumnWidth(WIDGET->_column, WIDGET->minimumWidth(WIDGET->_column));
+    	WIDGET->setColumnWidthMode(WIDGET->_column, QListView::Maximum);
+    }
+    else
+    {
+			WIDGET->setColumnWidthMode(WIDGET->_column, QListView::Manual);
+    	WIDGET->setColumnWidth(WIDGET->_column, w);
+    }
+    
+    WIDGET->setResizeMode(mode);
   }
 
 END_PROPERTY
@@ -1303,13 +1362,14 @@ END_PROPERTY
 BEGIN_PROPERTY(CLISTVIEW_column_alignment)
 
   if (READ_PROPERTY)
-    GB.ReturnInteger(WIDGET->columnAlignment(WIDGET->_column) | Qt::AlignVCenter);
+    GB.ReturnInteger(CCONST_alignment((WIDGET->columnAlignment(WIDGET->_column) | Qt::AlignVCenter) & ALIGN_MASK, ALIGN_NORMAL, false));
   else
-    WIDGET->setColumnAlignment(WIDGET->_column, VPROP(GB_INTEGER) & Qt::AlignHorizontal_Mask);
+    WIDGET->setColumnAlignment(WIDGET->_column, CCONST_alignment(VPROP(GB_INTEGER), ALIGN_NORMAL, true)  & Qt::AlignHorizontal_Mask);
 
 END_PROPERTY
 
 
+
 BEGIN_PROPERTY(CLISTVIEW_columns_sort)
 
   int sort;
@@ -1353,7 +1413,7 @@ END_PROPERTY
 */
 
 
-BEGIN_PROPERTY(CLISTVIEW_column_auto_resize)
+BEGIN_PROPERTY(CLISTVIEW_auto_resize)
 
   if (READ_PROPERTY)
     GB.ReturnBoolean(WIDGET->isAutoResize());
@@ -1443,8 +1503,8 @@ static void init_settings()
   if (init)
   	return;
   	
-	GB.GetFunction(&_get_settings_func, GB.FindClass("Qt"), "_GetColumnViewSettings", "ColumnView;", "s");
-	GB.GetFunction(&_set_settings_func, GB.FindClass("Qt"), "_SetColumnViewSettings", "ColumnView;s", "");
+	GB.GetFunction(&_get_settings_func, GB.FindClass("_Qt"), "_GetColumnViewSettings", "ColumnView;", "s");
+	GB.GetFunction(&_set_settings_func, GB.FindClass("_Qt"), "_SetColumnViewSettings", "ColumnView;s", "");
 	
 	init = true;
 }
@@ -1655,9 +1715,6 @@ GB_DESC CListViewDesc[] =
   GB_PROPERTY_READ("Key", "s", CTREEVIEW_key),
   GB_PROPERTY_READ("Item", ".ListViewItem", CTREEVIEW_item),
 
-  GB_CONSTANT("_Properties", "s", CLISTVIEW_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
-
   GB_PROPERTY_READ("ClientWidth", "i", CTREEVIEW_client_width),
   GB_PROPERTY_READ("ClientW", "i",  CTREEVIEW_client_width),
   GB_PROPERTY_READ("ClientHeight", "i", CTREEVIEW_client_height),
@@ -1670,6 +1727,8 @@ GB_DESC CListViewDesc[] =
   GB_EVENT("Cancel", NULL, NULL, &EVENT_Cancel),
   GB_EVENT("Compare", NULL, "(Key)s(OtherKey)s", &EVENT_Compare),
 
+	LISTVIEW_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
@@ -1693,7 +1752,7 @@ GB_DESC CTreeViewDesc[] =
   //GB_PROPERTY("Root", "b", CTREEVIEW_show_root),
   GB_PROPERTY("Sorted", "b", CTREEVIEW_sorted),
   GB_PROPERTY("Editable", "b", CTREEVIEW_editable),
-  GB_PROPERTY("ScrollBar", "i<Scroll>", CWIDGET_scrollbar),
+  GB_PROPERTY("ScrollBar", "i", CWIDGET_scrollbar),
   GB_PROPERTY("Border", "b", CWIDGET_border_simple),
   GB_PROPERTY("Compare", "i", CLISTVIEW_compare),
 
@@ -1738,9 +1797,6 @@ GB_DESC CTreeViewDesc[] =
   GB_PROPERTY_READ("Key", "s", CTREEVIEW_key),
   GB_PROPERTY_READ("Item", ".TreeViewItem", CTREEVIEW_item),
 
-  GB_CONSTANT("_Properties", "s", CTREEVIEW_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
-
   GB_PROPERTY_READ("ClientWidth", "i", CTREEVIEW_client_width),
   GB_PROPERTY_READ("ClientW", "i",  CTREEVIEW_client_width),
   GB_PROPERTY_READ("ClientHeight", "i", CTREEVIEW_client_height),
@@ -1755,6 +1811,8 @@ GB_DESC CTreeViewDesc[] =
   GB_EVENT("Expand", NULL, NULL, &EVENT_Expand),
   GB_EVENT("Collapse", NULL, NULL, &EVENT_Collapse),
 
+	TREEVIEW_DESCRIPTION,
+
   GB_END_DECLARE
 };
 
@@ -1770,11 +1828,9 @@ GB_DESC CColumnViewColumnDesc[] =
   GB_DECLARE(".ColumnViewColumn", 0), GB_VIRTUAL_CLASS(),
 
   GB_PROPERTY("Text", "s", CLISTVIEW_column_text),
-  GB_PROPERTY("Alignment", "i<Align,Left,Center,Right>", CLISTVIEW_column_alignment),
+  GB_PROPERTY("Alignment", "i", CLISTVIEW_column_alignment),
   GB_PROPERTY("W", "i", CLISTVIEW_column_width),
   GB_PROPERTY("Width", "i", CLISTVIEW_column_width),
-  GB_PROPERTY("AutoResize", "b", CLISTVIEW_column_auto_resize),
-  //GB_PROPERTY("Numeric", "b", CLISTVIEW_column_numeric),
 
   GB_END_DECLARE
 };
@@ -1789,8 +1845,6 @@ GB_DESC CColumnViewColumnsDesc[] =
   GB_PROPERTY("Sort", "i", CLISTVIEW_columns_sort),
   GB_PROPERTY("Ascending", "b", CLISTVIEW_columns_ascending),
 
-  //GB_METHOD("Adjust", NULL, CLISTVIEW_columns_adjust, NULL),
-
   GB_END_DECLARE
 };
 
@@ -1801,9 +1855,7 @@ GB_DESC CColumnViewDesc[] =
   GB_METHOD("_new", NULL, CCOLUMNVIEW_new, "(Parent)Container;"),
   GB_METHOD("_free", NULL, CTREEVIEW_free, NULL),
 
-  //GB_PROPERTY("Drop", "b", CTREEVIEW_drop),
   GB_PROPERTY("Mode", "i", CTREEVIEW_mode),
-  //GB_PROPERTY("Root", "b", CTREEVIEW_show_root),
   GB_PROPERTY("Sorted", "b", CTREEVIEW_sorted),
   GB_PROPERTY("Editable", "b", CTREEVIEW_editable),
   GB_PROPERTY("ScrollBar", "i", CWIDGET_scrollbar),
@@ -1827,7 +1879,7 @@ GB_DESC CColumnViewDesc[] =
 
   GB_METHOD("_get", ".ColumnViewItem", CTREEVIEW_get, "(Key)s"),
 
-  GB_METHOD("Clear", NULL, CTREEVIEW_clear, NULL),
+  GB_METHOD("Clear", NULL, CCOLUMNVIEW_clear, NULL),
   GB_METHOD("Add", ".ColumnViewItem", CTREEVIEW_add, "(Key)s(Text)s[(Picture)Picture;(Parent)s(After)s]"),
   GB_METHOD("Remove", NULL, CTREEVIEW_remove, "(Key)s"),
   GB_METHOD("Exist", "b", CTREEVIEW_exist, "(Key)s"),
@@ -1846,13 +1898,13 @@ GB_DESC CColumnViewDesc[] =
   GB_EVENT("Compare", NULL, "(Key)s(OtherKey)s", &EVENT_Compare),
   GB_EVENT("Expand", NULL, NULL, &EVENT_Expand),
   GB_EVENT("Collapse", NULL, NULL, &EVENT_Collapse),
-  GB_EVENT("ColumnClick", NULL, "(Column)i", &EVENT_ColumnClick),
+  //GB_EVENT("ColumnClick", NULL, "(Column)i", &EVENT_ColumnClick),
 
-  GB_PROPERTY_READ("Columns", ".ColumnViewColumns", CLISTVIEW_columns),
+  GB_PROPERTY_SELF("Columns", ".ColumnViewColumns"),
 
   GB_PROPERTY("Resizable", "b", CLISTVIEW_resizable),
-  GB_PROPERTY("AutoResize", "b", CLISTVIEW_auto_resize),
   GB_PROPERTY("Header", "b", CTREEVIEW_header),
+  GB_PROPERTY("AutoResize", "b", CLISTVIEW_auto_resize),
 
   GB_PROPERTY_READ("ClientWidth", "i", CTREEVIEW_client_width),
   GB_PROPERTY_READ("ClientW", "i",  CTREEVIEW_client_width),
@@ -1861,8 +1913,7 @@ GB_DESC CColumnViewDesc[] =
 
   GB_PROPERTY("Settings", "s", CCOLUMNVIEW_settings),
 
-  GB_CONSTANT("_Properties", "s", CCOLUMNVIEW_PROPERTIES),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
+	COLUMNVIEW_DESCRIPTION,
 
   GB_END_DECLARE
 };
@@ -1898,6 +1949,7 @@ static void raise_event(void *_object, int ev, QListViewItem *it)
     it = WIDGET->currentItem();
   THIS->item = (MyListViewItem *)it;
   GB.Raise(THIS, ev, 0);
+  // TODO: what happens if old is destroyed during the event?
   THIS->item = old;
 }
 
@@ -1958,17 +2010,17 @@ void CTreeView::renamed(QListViewItem *it, int col)
   GB.Post((void (*)())post_rename_event, (long)THIS);
 }
 
-void CTreeView::columnClicked(QListViewItem *it, const QPoint &p, int c)
-{
-  GET_SENDER(_object);
-
-  if (it == 0)
-    return;
-
-  THIS->item = (MyListViewItem *)it;
-
-  GB.Raise(_object, EVENT_ColumnClick, 1, GB_T_INTEGER, c);
-}
+// void CTreeView::columnClicked(QListViewItem *it, const QPoint &p, int c)
+// {
+//   GET_SENDER(_object);
+// 
+//   if (it == 0)
+//     return;
+// 
+//   THIS->item = (MyListViewItem *)it;
+// 
+//   GB.Raise(_object, EVENT_ColumnClick, 1, GB_T_INTEGER, c);
+// }
 
 void CTreeView::expanded(QListViewItem *it)
 {
