@@ -50,8 +50,9 @@
 
 #include "CWidget.h"
 #include "CMenu.h"
-#include "CWindow.h"
 #include "CKey.h"
+#include "CDraw.h"
+#include "CWindow.h"
 
 #ifndef NO_X_WINDOW
 #include "x11.h"
@@ -68,11 +69,12 @@ DECLARE_EVENT(EVENT_Move);
 DECLARE_EVENT(EVENT_Resize);
 DECLARE_EVENT(EVENT_Show);
 DECLARE_EVENT(EVENT_Hide);
-DECLARE_EVENT(EVENT_Embed);
+DECLARE_EVENT(EVENT_Title);
+DECLARE_EVENT(EVENT_Icon);
 
 CWINDOW *CWINDOW_Main = 0;
 CWINDOW *CWINDOW_Current = 0;
-static CWINDOW *CWINDOW_LastActive = 0;
+CWINDOW *CWINDOW_LastActive = 0;
 CWINDOW *CWINDOW_Active = 0;
 
 long CWINDOW_Embedder = 0;
@@ -103,6 +105,8 @@ static void clear_mask(CWINDOW *_object)
 {
 	//qDebug("clear_mask: %p", _object);
 
+	THIS->reallyMasked = false;
+
   if (!THIS->toplevel)
   {
     WIDGET->clearMask();
@@ -134,6 +138,8 @@ static void define_mask(CWINDOW *_object, CPICTURE *new_pict, bool new_mask)
 	//if (THIS->embedded)
 	//	return;
 
+  THIS->reallyMasked = true;
+  
   if (!new_pict)
   {
     root->clearMask();
@@ -219,7 +225,7 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent)
   #endif
   const char *name = GB.GetClassName(THIS);
 
-  if (MISSING(parent))
+  if (MISSING(parent) || !VARG(parent))
   {
 	  #ifndef NO_X_WINDOW
     if (CWINDOW_Embedder && !CWINDOW_Embedded)
@@ -284,6 +290,7 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent)
 
   if (win)
   {
+		win->_object = THIS;
     //win->setCentralWidget(container);
     //win->setOpaqueMoving(true);
     win->installEventFilter(&CWindow::manager);
@@ -488,9 +495,9 @@ static bool do_close(CWINDOW *_object, long ret, bool destroyed = false)
     if (destroyed || closed)
     {
       CWIDGET_set_flag(THIS, WF_CLOSED);
-    	CWIDGET_set_flag(THIS, WF_IN_CLOSE);
+    	/*CWIDGET_set_flag(THIS, WF_IN_CLOSE);
 			qApp->sendEvent(WIDGET, new QEvent(EVENT_CLOSE));
-    	CWIDGET_clear_flag(THIS, WF_IN_CLOSE);
+    	CWIDGET_clear_flag(THIS, WF_IN_CLOSE);*/
     }
 
     if (closed)
@@ -571,22 +578,15 @@ BEGIN_METHOD_VOID(CWINDOW_show)
 
   if (!THIS->toplevel)
   {
-    /*if (!THIS->shown)
-    {
-      if (THIS->xembed)
-        WIDGET->move(0, 0);
-    }*/
-
-    //WIDGET->raise();
 	  CWIDGET_clear_flag(THIS, WF_CLOSED);
     WIDGET->show();
   }
   else
   {
-    /*if (CWINDOW_Current)
-      WINDOW->showActivate(); //GB.Error("A modal window is already displayed");
-    else*/
-      WINDOW->showActivate();
+    //if (CWINDOW_Current)
+    //  WINDOW->showModal(); //GB.Error("A modal window is already displayed");
+    //else
+    WINDOW->showActivate();
   }
 
 END_METHOD
@@ -681,7 +681,10 @@ BEGIN_PROPERTY(CWINDOW_text)
 		GB.ReturnNewZeroString(TO_UTF8(WIDGET->caption()));
 	else
 	{
-		WIDGET->setCaption(QSTRING_PROP());
+		QString s = QSTRING_PROP();
+		THIS->title = s.length() > 0;
+		WIDGET->setCaption(s);
+		GB.Raise(THIS, EVENT_Title, 0);
     qApp->sendEvent(WIDGET, new QEvent(EVENT_TITLE));
 	}
 
@@ -725,6 +728,22 @@ BEGIN_METHOD_VOID(CWINDOW_menu_next)
 END_PROPERTY
 
 
+BEGIN_METHOD(CWINDOW_menu_get, GB_INTEGER index)
+
+  CWINDOW *window = OBJECT(CWINDOW);
+  int index = VARG(index);
+
+  if (window->menu == NULL || index < 0 || index >= (int)window->menu->count())
+  {
+    GB.Error(GB_ERR_BOUND);
+    return;
+  }
+
+  GB.ReturnObject(window->menu->at(index));
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(CWINDOW_border)
 
   if (!THIS->toplevel)
@@ -754,6 +773,7 @@ BEGIN_PROPERTY(CWINDOW_icon)
 		//else
 			SET_PIXMAP(WIDGET->setIcon, &(THIS->icon), PROP(GB_OBJECT));
 		//WIDGET->setIcon(PICTURE_set(&(THIS->icon), PROPERTY(CPICTURE *)));
+    GB.Raise(THIS, EVENT_Icon, 0);
     qApp->sendEvent(WIDGET, new QEvent(EVENT_ICON));
 	}
 
@@ -1109,16 +1129,6 @@ BEGIN_METHOD_VOID(CWINDOW_control_next)
 END_PROPERTY
 
 
-BEGIN_PROPERTY(CWINDOW_scale)
-
-	if (READ_PROPERTY)
-		GB.ReturnBoolean(THIS->scale);
-	else
-		THIS->scale = VPROP(GB_BOOLEAN);
-
-END_PROPERTY
-
-
 BEGIN_METHOD(CWINDOW_reparent, GB_OBJECT container; GB_INTEGER x; GB_INTEGER y)
 
 	QPoint p;
@@ -1160,6 +1170,12 @@ BEGIN_METHOD(CWINDOW_get, GB_STRING name)
 END_METHOD
 
 
+BEGIN_PROPERTY(CWINDOW_closed)
+
+	GB.ReturnBoolean(CWIDGET_test_flag(THIS, WF_CLOSED));
+
+END_PROPERTY
+
 
 /***************************************************************************/
 
@@ -1168,6 +1184,7 @@ GB_DESC CWindowMenusDesc[] =
   GB_DECLARE(".WindowMenus", 0), GB_VIRTUAL_CLASS(),
 
   GB_METHOD("_next", "Menu", CWINDOW_menu_next, NULL),
+  GB_METHOD("_get", "Menu", CWINDOW_menu_get, "(Index)i"),
   GB_PROPERTY_READ("Count", "i", CWINDOW_menu_count),
 
   GB_END_DECLARE
@@ -1222,6 +1239,7 @@ GB_DESC CWindowDesc[] =
   GB_METHOD("Center", NULL, CWINDOW_center, NULL),
   GB_PROPERTY_READ("Modal", "b", CWINDOW_modal),
   GB_PROPERTY_READ("TopLevel", "b", CWINDOW_top_level),
+  GB_PROPERTY_READ("Closed", "b", CWINDOW_closed),
 
   GB_METHOD("Delete", NULL, CWINDOW_delete, NULL),
 
@@ -1244,17 +1262,15 @@ GB_DESC CWindowDesc[] =
   GB_PROPERTY("SkipTaskbar", "b", CWINDOW_skip_taskbar),
   GB_PROPERTY("ToolBox", "b", CWINDOW_tool),
   GB_PROPERTY("Visible", "b", CWINDOW_visible),
-  GB_PROPERTY("Arrangement", "i<Arrange>", CCONTAINER_arrangement),
+  GB_PROPERTY("Arrangement", "i", CCONTAINER_arrangement),
   GB_PROPERTY("Padding", "i", CCONTAINER_padding),
   GB_PROPERTY("Spacing", "i", CCONTAINER_spacing),
   GB_PROPERTY("AutoResize", "b", CCONTAINER_auto_resize),
-  GB_PROPERTY("Scale", "b", CWINDOW_scale),
-  GB_PROPERTY("Action", "s", CCONTROL_action),
 
   GB_PROPERTY_SELF("Menus", ".WindowMenus"),
   GB_PROPERTY_SELF("Controls", ".WindowControls"),
 
-  GB_CONSTANT("_Properties", "s", "*,Action,Text,Icon,Picture,Mask,Persistent,Border{Window.None;Fixed;Resizable}=Resizable,ToolBox,Stacking{Window.Normal;Above;Below}=Normal,Minimized,Maximized,FullScreen,Sticky,SkipTaskbar," CARRANGEMENT_PROPERTIES),
+  GB_CONSTANT("_Properties", "s", CWINDOW_PROPERTIES),
   GB_CONSTANT("_DefaultEvent", "s", "Open"),
   GB_CONSTANT("_Arrangement", "i", ARRANGE_FILL),
 
@@ -1266,7 +1282,10 @@ GB_DESC CWindowDesc[] =
   GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),
   GB_EVENT("Show", NULL, NULL, &EVENT_Show),
   GB_EVENT("Hide", NULL, NULL, &EVENT_Hide),
-  GB_EVENT("Embed", NULL, NULL, &EVENT_Embed),
+  GB_EVENT("Title", NULL, NULL, &EVENT_Title),
+  GB_EVENT("Icon", NULL, NULL, &EVENT_Icon),
+  
+  GB_INTERFACE("Draw", &DRAW_Interface),
 
   GB_END_DECLARE
 };
@@ -1381,9 +1400,9 @@ void MyMainWindow::showEvent(QShowEvent *e)
   {
 	  //CWINDOW *ob = (CWINDOW *)CWidget::get(this);
   	//qDebug("_activate: %s %p", GB.GetClassName(ob), ob);
+    setActiveWindow();
     raise();
     setFocus();
-    setActiveWindow();
     _activate = false;
   }
 }
@@ -1403,6 +1422,15 @@ void MyMainWindow::initProperties()
   #endif
 }
 
+void MyMainWindow::afterShow()
+{
+	if (!CWIDGET_test_flag(THIS, WF_CLOSED))
+	{
+		define_mask(THIS, THIS->picture, THIS->masked);	
+		THIS->loopLevel = CWINDOW_Current ? CWINDOW_Current->loopLevel : 0;
+	}
+}
+
 void MyMainWindow::showActivate()
 {
   CWIDGET *_object = CWidget::get(this);
@@ -1440,6 +1468,9 @@ void MyMainWindow::showActivate()
 
   CWIDGET_set_flag(THIS, WF_IN_SHOW);
 
+	if (!THIS->title && border != BorderNone)
+		setCaption(GB.Application.Title());
+
   initProperties();
 
   if (!THIS->shown)
@@ -1474,10 +1505,11 @@ void MyMainWindow::showActivate()
   }
   else
   {
+    _activate = true;
+    
     if (windowState() & WindowMinimized)
     {
       setWindowState(windowState() & ~WindowMinimized);
-      _activate = true;
 			//qDebug("_activate set #2");
     }
     
@@ -1499,11 +1531,13 @@ void MyMainWindow::showActivate()
 
   }
 
-  define_mask(THIS, THIS->picture, THIS->masked);
+	afterShow();
 
   CWIDGET_clear_flag(THIS, WF_IN_SHOW);
 }
 
+static void post_show_event(void *_object);
+
 void MyMainWindow::showModal(void)
 {
   WFlags flags = getWFlags();
@@ -1514,7 +1548,10 @@ void MyMainWindow::showModal(void)
   CWINDOW *save = CWINDOW_Current;
   QPoint p = pos();
 
-  if (testWFlags(WShowModal))
+  if (CWIDGET_test_flag(THIS, WF_IN_CLOSE) || CWIDGET_test_flag(THIS, WF_IN_SHOW))
+    return;
+  
+  if (isModal())
     return;
 
   mustCenter = true;
@@ -1543,17 +1580,28 @@ void MyMainWindow::showModal(void)
     setSizeGrip(true);
 	}
 
-  //XSetTransientForHint(x11Display(), winId(), None );
-
-  showActivate();
+	THIS->enterLoop = false; // Do not call exitLoop() if we do not entered the loop yet!
+	
+  CWIDGET_clear_flag(THIS, WF_CLOSED); // Normaly done in showActivate()
+  
+  CWIDGET_set_flag(THIS, WF_IN_SHOW);
+	post_show_event(THIS);
+  CWIDGET_clear_flag(THIS, WF_IN_SHOW);
 
 	if (!CWIDGET_test_flag(THIS, WF_CLOSED))
 	{
+		//qDebug("showModal: %p", THIS);
+  	show();
+		afterShow();
+  	
+	  THIS->loopLevel++;
 		CWINDOW_Current = THIS;
+		THIS->enterLoop = true;
+		//qDebug("enterLoop: %p\n", THIS);
 		qApp->eventLoop()->enterLoop();
 		CWINDOW_Current = save;
 	}
-
+	
   if (persistent)
   {
     setSizeGrip(false);
@@ -1701,14 +1749,9 @@ void MyMainWindow::moveEvent(QMoveEvent *e)
 
 	if (THIS->toplevel)
 	{
-		if (!testWFlags(Qt::WStyle_NoBorderEx))
-		{
+		if (getBorder() && !THIS->reallyMasked)
 			if (geometry().x() == frameGeometry().x() && geometry().y() == frameGeometry().y())
-			{
-				//qDebug("...Ignore");
 				return;
-			}
-		}
 
 		if (!isHidden())
 		{
@@ -1905,17 +1948,15 @@ void MyMainWindow::closeEvent(QCloseEvent *e)
   //if (_object == CWINDOW_Main && qApp->loopLevel() > 1)
   //  return;
 
-  //qDebug("closeEvent: level = %d", qApp->loopLevel());
+  //qDebug("closeEvent");
 
   if (MAIN_in_wait)
     goto IGNORE;
 
-  if (CWINDOW_Current && (THIS != CWINDOW_Current))
-    goto IGNORE;
-
-  if (!THIS)
+  //if (CWINDOW_Current && (THIS != CWINDOW_Current))
+  if (CWINDOW_Current && (THIS->loopLevel != CWINDOW_Current->loopLevel))
   {
-    qWarning("closeEvent: THIS == NULL");
+  	//qDebug("ignore close event");
     goto IGNORE;
   }
 
@@ -1933,16 +1974,16 @@ void MyMainWindow::closeEvent(QCloseEvent *e)
       cancel = true;
   }
 
-	modal = testWFlags(Qt::WShowModal) && THIS->shown;
+	if (cancel)
+		goto IGNORE;
+
+	modal = testWFlags(Qt::WShowModal); // && THIS->shown;
 
   CWIDGET_set_flag(THIS, WF_CLOSED);
- 	qApp->sendEvent(WIDGET, new QEvent(EVENT_CLOSE));
+ 	//qApp->sendEvent(WIDGET, new QEvent(EVENT_CLOSE));
 
   if (!CWIDGET_test_flag(_object, WF_PERSISTENT))
   {
-    if (cancel)
-      goto IGNORE;
-
     if (CWINDOW_Main == THIS)
     {
       deleteAll();
@@ -1964,11 +2005,12 @@ void MyMainWindow::closeEvent(QCloseEvent *e)
   }
   THIS->shown = FALSE;
   
-  if (modal)
-  {
-    //qDebug("exit_loop");
-    qApp->eventLoop()->exitLoop();
-  }
+  //qDebug("THIS->enterLoop = %d", THIS->enterLoop);
+	if (modal && THIS->enterLoop)
+	{
+		//qDebug("exitLoop: %p", THIS);
+		qApp->eventLoop()->exitLoop();
+	}
 
   return;
 
@@ -2280,6 +2322,7 @@ void CWINDOW_set_cancel_button(CWINDOW *win, QPushButton *button, bool on)
 
 static void post_show_event(void *_object)
 {
+	//qDebug("post_show_event");
   //qDebug("> post_show_event %s %p", GB.GetClassName(THIS), THIS);
   if (THIS->shown)
     return;
