@@ -37,9 +37,6 @@ DECLARE_EVENT(EVENT_Highlight);
 DECLARE_EVENT(EVENT_Margin);
 
 static int _x1, _y1, _x2, _y2;
-static void *analyze_symbol = 0;
-static void *analyze_type = 0;
-static void *analyze_pos = 0;
 static int _style;
 
 static int _highlight_state = 0;
@@ -50,127 +47,6 @@ static GHighlightArray *_highlight_data = NULL;
 
 static QT_PICTURE _breakpoint_picture = 0;
 
-static QString purge(const QString &s, bool comment, bool string)
-{
-  QString r;
-  QChar c;
-  uint i;
-  bool in_comment = false;
-  char wait = 0;
-
-  for (i = 0; i < s.length(); i++)
-  {
-    c = s[i];
-
-    switch(wait)
-    {
-      case 0:
-
-        if (in_comment)
-        {
-        	if (!comment)
-          	c = ' ';
-				}
-        else if (c == '"')
-          wait = '"';
-        else if (c == '\'')
-          in_comment = true;
-
-        break;
-
-      case '"':
-        if (c == '"')
-          wait = 0;
-				else if (c == '\\')
-				{
-					if (string)
-					{
-						if (i < s.length())
-							r += c;
-						i++;
-						c = s[i];
-					}
-					else
-					{
-						i++;
-						if (i < s.length())
-							r += ' ';
-						c = ' ';
-					}
-				}
-        else
-        {
-        	if (!string)
-          	c = ' ';
-				}
-        break;
-    }
-
-    r += c;
-  }
-
-  return r;
-}
-
-static void analyze(QString &s)
-{
-  GHighlightArray data;
-  GB_ARRAY garray, tarray, parray;
-  int i, n, pos, len, p;
-  char *str;
-  bool proc;
-  uint state = 0;
-  int tag = 0;
-
-  GString gs = s;
-  GDocument::highlightGambas(0, state, tag, gs, &data, proc);
-  s = gs.getString();
-
-  n = 0;
-  for (i = 0; i < GB.Count(data); i++)
-  {
-    if (data[i].state != GLine::Normal)
-      n++;
-  }
-
-  //n = data.count() >> 1;
-  GB.Array.New(&garray, GB_T_STRING, n);
-  GB.Array.New(&tarray, GB_T_INTEGER, n);
-  GB.Array.New(&parray, GB_T_INTEGER, n);
-
-  pos = 0;
-  i = 0;
-  for (p = 0; p < GB.Count(data); p++)
-  {
-    len = data[p].len;
-
-    if (data[p].state != GLine::Normal)
-    {
-      GB.NewString(&str, TO_UTF8(s.mid(pos, len)), 0);
-      *((char **)GB.Array.Get(garray, i)) = str;
-      *((int *)GB.Array.Get(tarray, i)) = data[p].state;
-      *((int *)GB.Array.Get(parray, i)) = pos;
-      i++;
-    }
-
-    pos += len;
-  }
-
-	GB.FreeArray(&data);
-
-  GB.Unref(&analyze_symbol);
-  analyze_symbol = garray;
-  GB.Ref(garray);
-
-  GB.Unref(&analyze_type);
-  analyze_type = tarray;
-  GB.Ref(tarray);
-
-  GB.Unref(&analyze_pos);
-  analyze_pos = parray;
-  GB.Ref(parray);
-}
-
 static void highlightCustom(GEditor *master, uint &state, int &tag, GString &s, GHighlightArray *data, bool &proc)
 {
   void *object = QT.GetObject((QWidget *)master);
@@ -193,15 +69,6 @@ static void highlightCustom(GEditor *master, uint &state, int &tag, GString &s,
 }
 
 
-BEGIN_METHOD_VOID(CHIGHLIGHT_exit)
-
-  GB.Unref(&analyze_symbol);
-  GB.Unref(&analyze_type);
-  GB.Unref(&analyze_pos);
-
-END_METHOD
-
-
 BEGIN_PROPERTY(CHIGHLIGHT_state)
 
 	if (READ_PROPERTY)
@@ -329,8 +196,14 @@ static void print_text(void *_object, const QString &s)
 		DOC->remove(line, 0, line, DOC->lineLength(line));
 		WIDGET->cursorGoto(line, 0, false);
 	}
+// 	if (col < DOC->lineLength(line))
+// 	{
+// 		end = col + s.length();
+// 		if (end > DOC->lineLength(line))
+// 			end = DOC->lineLength(line);
+// 		DOC->remove(line, col, line, end);
+// 	}
 	WIDGET->insert(s);
-	
 }
 
 BEGIN_METHOD(CEDITORDOC_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
@@ -348,15 +221,15 @@ BEGIN_METHOD(CEDITORDOC_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 		//if (s[i].latin1() < 32)
 		//	qDebug("%d", s[i].latin1());
 		code = s[i].unicode();
-		if (code < 32 && code != 10)
+		if (code < 32)
 		{
 			if (i > j)
 				print_text(THIS, s.mid(j, i - j));
 				
 			j = i + 1;
 			
- 			WIDGET->getCursor(&line, &col);
- 			
+ 			WIDGET->getCursor(&line, &col);		
+			
 			if (code == '\t')
 			{
 				QString spc = "        ";
@@ -367,6 +240,11 @@ BEGIN_METHOD(CEDITORDOC_print, GB_STRING str; GB_INTEGER y; GB_INTEGER x)
 			{
 				WIDGET->cursorGoto(line, 0, false);
 			}
+			else if (code == '\n')
+			{
+				WIDGET->cursorGoto(line, DOC->lineLength(line), false);
+				WIDGET->insert("\n");
+			}
 			else
 			{
 				QString tmp;
@@ -556,46 +434,46 @@ BEGIN_METHOD(CEDITORDOC_find_next_word, GB_STRING word; GB_INTEGER line)
 
 END_METHOD
 
-BEGIN_METHOD(CEDITORDOC_purge_line, GB_INTEGER line; GB_BOOLEAN comment; GB_BOOLEAN string)
-
-  bool comment = VARGOPT(comment, false);
-  bool string = VARGOPT(string, false);
-  
-  if (comment && string)	
-  	GB.ReturnNewZeroString(DOC->getLine(VARG(line)).utf8());
-	else
-	{
-		QString s = DOC->getLine(VARG(line)).getString();
-  	GB.ReturnNewZeroString(TO_UTF8(purge(s, VARGOPT(comment, false), VARGOPT(string, false))));
-	}
-
-END_METHOD
-
-BEGIN_METHOD(CEDITORDOC_analyze, GB_STRING text)
-
-  QString s = QSTRING_ARG(text);
-  analyze(s);
-  GB.ReturnObject(analyze_symbol);
-
-END_METHOD
-
-BEGIN_PROPERTY(CEDITORDOC_analyze_symbols)
-
-  GB.ReturnObject(analyze_symbol);
-
-END_PROPERTY
-
-BEGIN_PROPERTY(CEDITORDOC_analyze_types)
-
-  GB.ReturnObject(analyze_type);
-
-END_PROPERTY
-
-BEGIN_PROPERTY(CEDITORDOC_analyze_positions)
-
-  GB.ReturnObject(analyze_pos);
-
-END_PROPERTY
+// BEGIN_METHOD(CEDITORDOC_purge_line, GB_INTEGER line; GB_BOOLEAN comment; GB_BOOLEAN string)
+// 
+//   bool comment = VARGOPT(comment, false);
+//   bool string = VARGOPT(string, false);
+//   
+//   if (comment && string)	
+//   	GB.ReturnNewZeroString(DOC->getLine(VARG(line)).utf8());
+// 	else
+// 	{
+// 		QString s = DOC->getLine(VARG(line)).getString();
+//   	GB.ReturnNewZeroString(TO_UTF8(purge(s, VARGOPT(comment, false), VARGOPT(string, false))));
+// 	}
+// 
+// END_METHOD
+// 
+// BEGIN_METHOD(CEDITORDOC_analyze, GB_STRING text)
+// 
+//   QString s = QSTRING_ARG(text);
+//   analyze(s);
+//   GB.ReturnObject(analyze_symbol);
+// 
+// END_METHOD
+// 
+// BEGIN_PROPERTY(CEDITORDOC_analyze_symbols)
+// 
+//   GB.ReturnObject(analyze_symbol);
+// 
+// END_PROPERTY
+// 
+// BEGIN_PROPERTY(CEDITORDOC_analyze_types)
+// 
+//   GB.ReturnObject(analyze_type);
+// 
+// END_PROPERTY
+// 
+// BEGIN_PROPERTY(CEDITORDOC_analyze_positions)
+// 
+//   GB.ReturnObject(analyze_pos);
+// 
+// END_PROPERTY
 
 BEGIN_PROPERTY(CEDITORDOC_highlight)
 
@@ -905,28 +783,28 @@ GB_DESC CHighlightDesc[] =
   GB_CONSTANT("Gambas", "i", GDocument::Gambas),
   GB_CONSTANT("Custom", "i", GDocument::Custom),
 
-  GB_CONSTANT("Background", "i", GLine::Background),
-  GB_CONSTANT("Normal", "i", GLine::Normal),
-  GB_CONSTANT("Keyword", "i", GLine::Keyword),
-  GB_CONSTANT("Function", "i", GLine::Subr),
-  GB_CONSTANT("Operator", "i", GLine::Operator),
-  GB_CONSTANT("Symbol", "i", GLine::Symbol),
-  GB_CONSTANT("Number", "i", GLine::Number),
-  GB_CONSTANT("String", "i", GLine::String),
-  GB_CONSTANT("Comment", "i", GLine::Comment),
-  GB_CONSTANT("Breakpoint", "i", GLine::Breakpoint),
-  GB_CONSTANT("Current", "i", GLine::Current),
-  GB_CONSTANT("DataType", "i", GLine::Datatype),
-  GB_CONSTANT("Selection", "i", GLine::Selection),
-  GB_CONSTANT("Highlight", "i", GLine::Highlight),
-  GB_CONSTANT("CurrentLine", "i", GLine::Line),
-  GB_CONSTANT("Error", "i", GLine::Error),
+  GB_CONSTANT("Background", "i", HIGHLIGHT_BACKGROUND),
+  GB_CONSTANT("Normal", "i", HIGHLIGHT_NORMAL),
+  GB_CONSTANT("Keyword", "i", HIGHLIGHT_KEYWORD),
+  GB_CONSTANT("Function", "i", HIGHLIGHT_SUBR),
+  GB_CONSTANT("Operator", "i", HIGHLIGHT_OPERATOR),
+  GB_CONSTANT("Symbol", "i", HIGHLIGHT_SYMBOL),
+  GB_CONSTANT("Number", "i", HIGHLIGHT_NUMBER),
+  GB_CONSTANT("String", "i", HIGHLIGHT_STRING),
+  GB_CONSTANT("Comment", "i", HIGHLIGHT_COMMENT),
+  GB_CONSTANT("Breakpoint", "i", HIGHLIGHT_BREAKPOINT),
+  GB_CONSTANT("Current", "i", HIGHLIGHT_CURRENT),
+  GB_CONSTANT("DataType", "i", HIGHLIGHT_DATATYPE),
+  GB_CONSTANT("Selection", "i", HIGHLIGHT_SELECTION),
+  GB_CONSTANT("Highlight", "i", HIGHLIGHT_HIGHLIGHT),
+  GB_CONSTANT("CurrentLine", "i", HIGHLIGHT_LINE),
+  GB_CONSTANT("Error", "i", HIGHLIGHT_ERROR),
 
-  GB_STATIC_METHOD("_exit", NULL, CHIGHLIGHT_exit, NULL),
-  GB_STATIC_METHOD("Analyze", "String[]", CEDITORDOC_analyze, "(Text)s"),
+  //GB_STATIC_METHOD("_exit", NULL, CHIGHLIGHT_exit, NULL),
+  /*GB_STATIC_METHOD("Analyze", "String[]", CEDITORDOC_analyze, "(Text)s"),
   GB_STATIC_PROPERTY_READ("Symbols", "String[]", CEDITORDOC_analyze_symbols),
   GB_STATIC_PROPERTY_READ("Types", "Integer[]", CEDITORDOC_analyze_types),
-  GB_STATIC_PROPERTY_READ("Positions", "Integer[]", CEDITORDOC_analyze_positions),
+  GB_STATIC_PROPERTY_READ("Positions", "Integer[]", CEDITORDOC_analyze_positions),*/
 
   GB_STATIC_PROPERTY("State", "i", CHIGHLIGHT_state),
   GB_STATIC_PROPERTY("Tag", "i", CHIGHLIGHT_tag),
@@ -1052,7 +930,7 @@ GB_DESC CEditorDesc[] =
   GB_PROPERTY("Text", "s", CEDITORDOC_text),
   GB_PROPERTY_READ("Length", "i", CEDITORDOC_length),
 
-  GB_PROPERTY("Highlight", "i<Highlight,None,Gambas,Custom>", CEDITORDOC_highlight),
+  GB_PROPERTY("Highlight", "i", CEDITORDOC_highlight),
 
   GB_PROPERTY("TabSize", "i", CEDITORDOC_tab_length),
   GB_METHOD("Reset", NULL, CEDITORDOC_reset, NULL),
@@ -1072,7 +950,7 @@ GB_DESC CEditorDesc[] =
   GB_PROPERTY_READ("Selected", "b", CEDITORDOC_selected),
   GB_PROPERTY_READ("Selection", ".Editor.Selection", CEDITORDOC_sel),
   GB_PROPERTY_SELF("Lines", ".Editor.Lines"),
-  GB_METHOD("GetPurgedLine", "s", CEDITORDOC_purge_line, "(Line)i[(Comment)b(String)b]"),
+  //GB_METHOD("GetPurgedLine", "s", CEDITORDOC_purge_line, "(Line)i[(Comment)b(String)b]"),
 
   GB_METHOD("FindNextBreakpoint", "i", CEDITORDOC_find_next_breakpoint, "(Line)i"),
   GB_METHOD("FindNextWord", "i", CEDITORDOC_find_next_word, "(Word)s(Line)i"),
@@ -1084,7 +962,7 @@ GB_DESC CEditorDesc[] =
   GB_EVENT("Margin", NULL, "(LineNumber)i", &EVENT_Margin),
   
   GB_CONSTANT("_DefaultEvent", "s", "KeyPress"),
-  GB_CONSTANT("_Properties", "s", "*,Font{Font:Fixed},Highlight=0,ReadOnly=False,TabSize{Range:1;16}=2"),
+  GB_CONSTANT("_Properties", "s", "*,Font{Font:Fixed},Highlight{Highlight.None;Custom;Gambas}=None,ReadOnly=False,TabSize{Range:1;16}=2"),
 
   GB_END_DECLARE
 };
