@@ -302,6 +302,7 @@ void GDocument::insert(int y, int x, const GString & text)
   GLine *l;
   int n = 1;
   GString rest;
+  int yy;
 
   if (readOnly)
   {
@@ -316,14 +317,19 @@ void GDocument::insert(int y, int x, const GString & text)
 		v->ny = v->y;
 	}
 
+	while (y >= (int)lines.count())
+	{
+		yy = (int)lines.count();
+    lines.insert(yy, new GLine());
+    lines.at(yy)->modified = lines.at(yy)->changed = true;
+	}
+
   for(;;)
   {
     pos2 = text.find('\n', pos);
     if (pos2 < 0)
       pos2 = text.length();
 
-    //qDebug("insert '%s' at %d:%d", text.mid(pos, pos2 - pos).latin1(), y, x);
-
     l = lines.at(y);
 
     if (pos2 > pos)
@@ -763,33 +769,74 @@ bool GDocument::redo()
 
 int GDocument::wordLeft(int y, int x, bool word)
 {
-  int xw;
+  int xw = x;
   GString s = lines.at(y)->s;
 
-  xw = x;
-  for(;;)
-  {
-    if (xw == 0 || s.isWordChar(xw - 1) ^ word)
-      break;
-    xw--;
-  }
+	if (!word)
+	{
+		while (xw > 0 && s.isSpace(xw - 1))
+			xw--;
+	}
+	
+	if (xw > 0)
+	{
+		if (s.isWordChar(xw - 1))
+		{
+			for(;;)
+			{
+				xw--;
+				if (xw <= 0 || !s.isWordChar(xw - 1))
+					break;
+			}
+		}
+		else if (!word)
+		{
+			for(;;)
+			{
+				xw--;
+				if (xw <= 0 || s.isWordChar(xw - 1) || s.isSpace(xw - 1))
+					break;
+			}
+		}
+	}
 
   return xw;
 }
 
 int GDocument::wordRight(int y, int x, bool word)
 {
-  int xw;
+  int xw = x;
   GString s = lines.at(y)->s;
+  int len = s.length();
 
-  xw = x;
-  for(;;)
-  {
-    if (xw == (int)s.length() || s.isWordChar(xw) ^ word)
-      break;
-    xw++;
-  }
+	if (xw < len)
+	{
+		if (s.isWordChar(xw))
+		{
+			for(;;)
+			{
+				xw++;
+				if (xw >= len || !s.isWordChar(xw))
+					break;
+			}
+		}
+		else if (!word)
+		{
+			for(;;)
+			{
+				xw++;
+				if (xw >= len || s.isWordChar(xw) || s.isSpace(xw))
+					break;
+			}
+		}
+	}
 
+	if (!word)
+	{
+		while (xw < len && s.isSpace(xw))
+			xw++;
+	}
+	
   return xw;
 }
 
