@@ -50,6 +50,7 @@
 #include <qtooltip.h>
 #include <qpushbutton.h>
 #include <qscrollview.h>
+#include <qmap.h>
 
 CWIDGET *CWIDGET_destroy_list = 0;
 CWIDGET *CWIDGET_destroy_last = 0;
@@ -65,8 +66,9 @@ GB_CLASS CLASS_Drawing;
 GB_CLASS CLASS_DrawingArea;
 GB_CLASS CLASS_Printer;
 
-
-
+#ifndef NO_X_WINDOW
+static QMap<int, int> _x11_to_qt_keycode;
+#endif
 
 /* Control class */
 
@@ -233,6 +235,8 @@ void CWIDGET_new(QWidget *w, void *_object, char *klass /* = NULL */, bool no_fi
 
 	THIS->flag.default_bg = true;
 	THIS->flag.default_fg = true;
+
+	//WIDGET->setName(THIS->name);
 }
 
 
@@ -918,7 +922,7 @@ BEGIN_PROPERTY(CWIDGET_foreground)
 END_PROPERTY
 #endif
 
-static void set_color(CWIDGET *_object, int bg, int fg)
+void CWIDGET_set_color(CWIDGET *_object, int bg, int fg)
 {
 	//qDebug("set_color: (%s %p) bg = %06X fg = %06X", GB.GetClassName(THIS), THIS, bg, fg);
 	//if (bg == COLOR_DEFAULT && fg == COLOR_DEFAULT)
@@ -934,7 +938,7 @@ static void set_color(CWIDGET *_object, int bg, int fg)
   THIS->flag.default_fg = fg == COLOR_DEFAULT;
 }
 
-static int get_background(CWIDGET *_object)
+int CWIDGET_get_background(CWIDGET *_object)
 {
 	if (THIS->flag.default_bg)
 		return COLOR_DEFAULT;
@@ -942,7 +946,7 @@ static int get_background(CWIDGET *_object)
 		return WIDGET->paletteBackgroundColor().rgb() & 0xFFFFFF;
 }
 	
-static int get_foreground(CWIDGET *_object)
+int CWIDGET_get_foreground(CWIDGET *_object)
 {
 	if (THIS->flag.default_fg)
 		return COLOR_DEFAULT;
@@ -953,20 +957,20 @@ static int get_foreground(CWIDGET *_object)
 BEGIN_PROPERTY(CCONTROL_background)
 
   if (READ_PROPERTY)
-    GB.ReturnInteger(get_background(THIS));
+    GB.ReturnInteger(CWIDGET_get_background(THIS));
   else
   {
   	int col = VPROP(GB_INTEGER);
-  	if (col != get_background(THIS))
+  	if (col != CWIDGET_get_background(THIS))
   	{
 			if (WIDGET->paletteBackgroundPixmap())
 			{
 				QPixmap p(*WIDGET->paletteBackgroundPixmap());
-				set_color(THIS, col, get_foreground(THIS));
+				CWIDGET_set_color(THIS, col, CWIDGET_get_foreground(THIS));
 				WIDGET->setPaletteBackgroundPixmap(p);
 			}
 			else
-				set_color(THIS, col, get_foreground(THIS));
+				CWIDGET_set_color(THIS, col, CWIDGET_get_foreground(THIS));
 		}
   }
 
@@ -976,13 +980,13 @@ END_PROPERTY
 BEGIN_PROPERTY(CCONTROL_foreground)
 
   if (READ_PROPERTY)
-    GB.ReturnInteger(get_foreground(THIS));
+    GB.ReturnInteger(CWIDGET_get_foreground(THIS));
   else
   {
   	int col = VPROP(GB_INTEGER);
-  	if (col != get_foreground(THIS))
+  	if (col != CWIDGET_get_foreground(THIS))
   	{
-			set_color(THIS, get_background(THIS), col);
+			CWIDGET_set_color(THIS, CWIDGET_get_background(THIS), col);
     }
 	}
 
@@ -1567,10 +1571,11 @@ static void post_dblclick_event(void *control)
   GB.Unref(&control);
 }
 
-static void post_gotfocus_event(void *control)
+static void post_gotfocus_event(void *_object)
 {
-  GB.Raise(control, EVENT_GotFocus, 0);
-  GB.Unref(&control);
+	if (WIDGET && WIDGET->hasFocus())
+  	GB.Raise(THIS, EVENT_GotFocus, 0);
+  GB.Unref(&_object);
 }
 
 static void post_lostfocus_event(void *control)
@@ -1706,6 +1711,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 
 		/* GB.Raise() can free the control, so we must reference it as we may raise two successive events now */
 		GB.Ref(control);
+		cancel = false;
 
     if (GB.CanRaise(control, event_id))
     {
@@ -1724,7 +1730,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
       CMOUSE_info.y = p.y();
       CMOUSE_info.state = state;
 
-      GB.Raise(control, event_id, 0); //, GB_T_INTEGER, p.x(), GB_T_INTEGER, p.y(), GB_T_INTEGER, state);
+      cancel = GB.Raise(control, event_id, 0); //, GB_T_INTEGER, p.x(), GB_T_INTEGER, p.y(), GB_T_INTEGER, state);
 
       CMOUSE_clear(false);
       
@@ -1732,7 +1738,7 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
       	return true;*/
     }
     
-    if (event_id == EVENT_MouseMove && (state & QMouseEvent::MouseButtonMask)  && GB.CanRaise(control, EVENT_MouseDrag)
+    if (!cancel && event_id == EVENT_MouseMove && (state & QMouseEvent::MouseButtonMask)  && GB.CanRaise(control, EVENT_MouseDrag)
     	  && ((abs(p.x() - CMOUSE_info.sx) + abs(p.y() - CMOUSE_info.sy)) > 8)) // QApplication::startDragDistance()))
 		{		
     	/*if (!design && CWIDGET_test_flag(control, WF_SCROLLVIEW))
@@ -1749,12 +1755,15 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
       CMOUSE_info.y = p.y();
       CMOUSE_info.state = state;
 		
-			GB.Raise(control, EVENT_MouseDrag, 0);
+			cancel = GB.Raise(control, EVENT_MouseDrag, 0);
       
       CMOUSE_clear(false);
 		}
   
 		GB.Unref(POINTER(&control));
+		
+		if (cancel)
+			return true;
   }
   else if (type == QEvent::MouseButtonDblClick)
   {
@@ -1773,16 +1782,17 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
   {
     QKeyEvent *kevent = (QKeyEvent *)event;
 
-    /*if (type == QEvent::KeyPress)
-      qDebug("QKeyEvent: %s %p (%d): %p (R%d S%d): %d %s",
-        widget->className(), widget, ((QWidget *)widget)->isTopLevel(),
-        control, real, original, kevent->key(), (char *)kevent->text().latin1());*/
-
     #if QT_VERSION <= 0x030005
     if (!real || !original)
       goto _DESIGN;
     #endif
 
+// 		qDebug("QKeyEvent: %s (%s %p) (%s %p) TL:%d -> %d %s",
+// 			type == QEvent::KeyPress ? "KeyPress" : "KeyRelease",
+// 			widget->className(), widget, GB.GetClassName(control), control,
+// 			((QWidget *)widget)->isTopLevel(),  
+// 			kevent->key(), (char *)kevent->text().latin1());
+
     event_id = (type == QEvent::KeyRelease) ? EVENT_KeyRelease : EVENT_KeyPress;
 
     #if QT_VERSION > 0x030005
@@ -1803,7 +1813,21 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
 			GB.NewString(&CKEY_info.text, TO_UTF8(kevent->text()), 0);
 			CKEY_info.state = kevent->state();
 			CKEY_info.code = kevent->key();
-
+			CKEY_info.release = type == QEvent::KeyRelease;
+			
+			#ifndef NO_X_WINDOW
+			if (type == QEvent::KeyPress && CKEY_info.code)
+				_x11_to_qt_keycode.insert(MAIN_x11_last_key_code, CKEY_info.code);
+			else if (type == QEvent::KeyRelease && CKEY_info.code == 0)
+			{
+				if (_x11_to_qt_keycode.contains(MAIN_x11_last_key_code))
+				{
+					CKEY_info.code = _x11_to_qt_keycode[MAIN_x11_last_key_code];
+					_x11_to_qt_keycode.remove(MAIN_x11_last_key_code);
+				}
+			}
+			#endif
+			
       cancel = GB.Raise(control, event_id, 0);
 
       CKEY_clear(false);
@@ -1839,6 +1863,10 @@ _ACCEL:
       goto _DESIGN;
     #endif
 
+// 		qDebug("QIMEvent: IMEnd (%s %p) (%s %p) TL:%d",
+// 			widget->className(), widget, GB.GetClassName(control), control,
+// 			((QWidget *)widget)->isTopLevel());
+
     event_id = EVENT_KeyPress;
 
     if (GB.CanRaise(control, event_id))
@@ -1971,69 +1999,14 @@ _STANDARD:
 
 /** Action *****************************************************************/
 
-static GB_FUNCTION _action_register_func;
-static GB_FUNCTION _action_raise_func;
-static GB_FUNCTION _action_get_func;
-
-static void init_action()
-{
-	static bool init = false;
-
-	if (init)
-		return;
-
-  GB.GetFunction(&_action_register_func, GB.FindClass("Action"), "Register", "os", "");
-  GB.GetFunction(&_action_raise_func, GB.FindClass("Action"), "Raise", "o", "");
-  GB.GetFunction(&_action_get_func, GB.FindClass("Action"), "Get", "o", "s");
-
-	init = true;
-}
-
-void CACTION_register(CWIDGET *control, const char *key)
-{
-	//qDebug("CACTION_register: (%s %p) %s", GB.GetClassName(control), control, key);
-
-	if ((!key || !*key) && !CWIDGET_test_flag(control, WF_ACTION)) 
-		//&& !GB.Is(control, CLASS_UserControl) && !GB.Is(control, CLASS_UserContainer))
-		return;
-
-	init_action();
-
-	GB.Push(2,
-		GB_T_OBJECT, control,
-		GB_T_STRING, key, 0);
-
-	GB.Call(&_action_register_func, 2, true);
-
-	if (key && *key)
-		CWIDGET_set_flag(control, WF_ACTION);
-	else
-		CWIDGET_clear_flag(control, WF_ACTION);
-}
-
-void CACTION_raise(void *_object)
-{
-	init_action();
-
-	//qDebug("CACTION_raise: (%s %p)", GB.GetClassName(THIS), THIS);
-
-	if (!CWIDGET_test_flag(THIS, WF_ACTION))
-		return;
-
-	GB.Push(1, GB_T_OBJECT, THIS);
-	GB.Call(&_action_raise_func, 1, true);
-}
-
-void CACTION_get(void *_object)
-{
-	init_action();
-
-	GB.Push(1, GB_T_OBJECT, THIS);
-	GB.Call(&_action_get_func, 1, false);
-
-	// The result is in the good place, as if we did a GB.ReturnString()
-}
+#define HAS_ACTION(_control) CWIDGET_test_flag((CWIDGET *)(_control), WF_ACTION)
+#define SET_ACTION(_control, _flag) \
+	if (_flag) \
+		CWIDGET_set_flag((CWIDGET *)(_control), WF_ACTION); \
+	else \
+		CWIDGET_clear_flag((CWIDGET *)(_control), WF_ACTION);
 
+#include "gb.form.action.h"
 
 static void gray_image(QImage &img)
 {
@@ -2053,14 +2026,26 @@ static void gray_image(QImage &img)
 	}
 }
 
-void CWIDGET_iconset(QIconSet &icon, QPixmap &pixmap)
+void CWIDGET_iconset(QIconSet &icon, QPixmap &pixmap, int size)
 {
 	QImage img;
 	QPixmap disabled;
-
-	icon = QIconSet(pixmap, QIconSet::Small);
+	QPixmap normal;
 
 	img = pixmap.convertToImage().convertDepth(32);
+	if (size > 0)
+	{
+		size = ((size + 1) & ~3);
+		img = img.smoothScale(size, size, QImage::ScaleMax);
+	}
+	
+	if (size <= 0)
+		normal = pixmap;
+	else
+		normal.convertFromImage(img);
+		
+	icon = QIconSet(normal, QIconSet::Small);
+	
 	gray_image(img);
 	
 	disabled.convertFromImage(img);
