@@ -30,6 +30,7 @@
 #include "main.h"
 
 #include "SDL.h"
+#include "SDLcursor.h"
 
 typedef
   struct {
@@ -44,8 +45,22 @@ typedef
 
 #ifndef __CMOUSE_CPP
 extern GB_DESC CMouse[];
+extern GB_DESC CCursor[];
 extern CMOUSE_INFO CMOUSE_info;
+#else
+
+#define THIS ((CCURSOR *)_object)
+
 #endif /* __CMOUSE_CPP */
 
+typedef
+  struct _CCURSOR {
+    GB_BASE ob;
+    int x;
+    int y;
+    SDLcursor *cursor;
+    }
+  CCURSOR;
+
 #endif /* __CMOUSE_H */
 
