@@ -4,7 +4,7 @@
 
   Webcam capture component
 
-  (C) 2005 Daniel Campos Fernández <danielcampos@netcourrier.com>
+  (C) 2005-2007 Daniel Campos Fernández <dcamposf@gmail.com>
 
   Based on the GPL code from "video-capture":
 
@@ -837,7 +837,7 @@ BEGIN_PROPERTY(CWEBCAM_picture)
 
 END_PROPERTY
 
-BEGIN_METHOD(CWEBCAM_save,GB_STRING File; GB_INTEGER Format; GB_INTEGER Quality;)
+BEGIN_METHOD(CWEBCAM_save,GB_STRING File; GB_INTEGER Quality;)
 
 	char *File;
 	char *ext=NULL;
@@ -863,7 +863,7 @@ BEGIN_METHOD(CWEBCAM_save,GB_STRING File; GB_INTEGER Format; GB_INTEGER Quality;
 		if (quality>100) quality=100;
 	}
 
-	if (!MISSING(Format))
+	/*if (!MISSING(Format))
 	{
 		switch(VARG(Format))
 		{
@@ -874,19 +874,27 @@ BEGIN_METHOD(CWEBCAM_save,GB_STRING File; GB_INTEGER Format; GB_INTEGER Quality;
 		}
 	}
 	else
-	{
-		for (b=strlen(File)-1;b>=0;b--)
-			if (File[b]=='.') { ext=File+b+1; break; }
+	{*/
+	
+	format = 0;
+	
+	for (b=strlen(File)-1;b>=0;b--)
+		if (File[b]=='.') { ext=File+b+1; break; }
 
-		if (ext)
-		{
-			if (!strcasecmp(ext,"jpeg")) format=3;
-			else if (!strcasecmp(ext,"jpg")) format=3;
-			else if (!strcasecmp(ext,"png")) format=2;
-			else if (!strcasecmp(ext,"ppm")) format=1;
-			else { GB.Error("Unknown format"); return; }
-		}
+	if (ext)
+	{
+		if (!strcasecmp(ext,"jpeg")) format=3;
+		else if (!strcasecmp(ext,"jpg")) format=3;
+		else if (!strcasecmp(ext,"png")) format=2;
+		else if (!strcasecmp(ext,"ppm")) format=1;
 	}
+	
+	if (!format)
+	{ 
+		GB.Error("Unknown format"); 
+		return; 
+	}
+
 
 	fd=fopen(File, "w");
 	if (!fd)
@@ -1129,7 +1137,7 @@ END_PROPERTY
 
 GB_DESC CFeaturesDesc[] =
 {
-	GB_DECLARE(".VideoDevFeatures", 0),
+	GB_DECLARE(".VideoDeviceFeatures", 0),
 	GB_VIRTUAL_CLASS(),
 
 	GB_PROPERTY_READ("Name","s",CFEATURES_name),
@@ -1147,7 +1155,7 @@ GB_DESC CFeaturesDesc[] =
 
 GB_DESC CTunerDesc[] =
 {
-	GB_DECLARE(".VideoDevTuner", 0),
+	GB_DECLARE(".VideoDeviceTuner", 0),
 	GB_VIRTUAL_CLASS(),
 
 	GB_PROPERTY_READ("Name","s",CTUNER_name),
@@ -1167,28 +1175,28 @@ GB_DESC CWebcamDesc[] =
 
   GB_HOOK_CHECK(CWEBCAM_check),
 
-  GB_CONSTANT("JPEG","i",FMT_JPEG),
-  GB_CONSTANT("PPM","i",FMT_PPM),
-  GB_CONSTANT("PNG","i",FMT_PNG),
+  //GB_CONSTANT("Jpeg","i",FMT_JPEG),
+  //GB_CONSTANT("Ppm","i",FMT_PPM),
+  //GB_CONSTANT("Png","i",FMT_PNG),
 
   GB_CONSTANT("Hz","i",1),
   GB_CONSTANT("Khz","i",0),
 
-  GB_CONSTANT("PAL","i",0),//NORM_PAL),
-  GB_CONSTANT("NTSC","i",4),//NORM_NTSC),
-  GB_CONSTANT("SECAM","i",8),//NORM_SECAM),
-  GB_CONSTANT("AUTO","i",12),//NORM_AUTO),
+  GB_CONSTANT("Pal","i",0),//NORM_PAL),
+  GB_CONSTANT("Ntsc","i",4),//NORM_NTSC),
+  GB_CONSTANT("Secam","i",8),//NORM_SECAM),
+  GB_CONSTANT("Auto","i",12),//NORM_AUTO),
 
-  GB_CONSTANT("TV","i",0), //IN_TV),
-  GB_CONSTANT("COMPOSITE1","i",1), //IN_COMPOSITE1),
-  GB_CONSTANT("COMPOSITE2","i",2), //IN_COMPOSITE2),
-  GB_CONSTANT("SVIDEO","i",3), //IN_SVIDEO),
+  GB_CONSTANT("Tv","i",0), //IN_TV),
+  GB_CONSTANT("Composite1","i",1), //IN_COMPOSITE1),
+  GB_CONSTANT("Composite2","i",2), //IN_COMPOSITE2),
+  GB_CONSTANT("SVideo","i",3), //IN_SVIDEO),
 
   GB_METHOD("_new",NULL,CWEBCAM_new,"(Device)s"),
   GB_METHOD("_free",NULL,CWEBCAM_free,NULL),
 
-  GB_PROPERTY_SELF("Tuner",".VideoDevTuner"),
-  GB_PROPERTY_SELF("Features",".VideoDevFeatures"),
+  GB_PROPERTY_SELF("Tuner",".VideoDeviceTuner"),
+  GB_PROPERTY_SELF("Features",".VideoDeviceFeatures"),
 
   GB_PROPERTY_READ("Width","i",CWEBCAM_width),
   GB_PROPERTY_READ("Height","i",CWEBCAM_height),
@@ -1204,7 +1212,7 @@ GB_DESC CWebcamDesc[] =
   GB_PROPERTY("Picture","Picture",CWEBCAM_picture),
 
   GB_METHOD("Resize",NULL,CWEBCAM_size,"(Width)i(Height)i"),
-  GB_METHOD("Save",NULL,CWEBCAM_save,"(File)s[(Format)i(Quality)i]"),
+  GB_METHOD("Save",NULL,CWEBCAM_save,"(File)s[(Quality)i]"),
 
   GB_END_DECLARE
 };
