@@ -68,7 +68,8 @@ static struct option Long_options[] =
   { "help", 0, NULL, 'h' },
   { "verbose", 0, NULL, 'v' },
   { "trans", 0, NULL, 't' },
-  { "public", 0, NULL, 'p' },
+  { "public-control", 0, NULL, 'p' },
+  { "public-module", 0, NULL, 'm' },
   { "swap", 0, NULL, 's' },
   { "class", 1, NULL, 'c' },
   /*{ "dump", 0, NULL, 'd' },*/
@@ -78,17 +79,17 @@ static struct option Long_options[] =
 };
 #endif
 
-static char **path_list;
-static int path_current;
-
 static bool main_debug = FALSE;
 static bool main_verbose = FALSE;
 static bool main_compile_all = FALSE;
 static bool main_trans = FALSE;
 static bool main_public = FALSE;
+static bool main_public_module = FALSE;
 static bool main_swap = FALSE;
 //static char *main_class_file = NULL;
 
+static char **_files = NULL;
+
 static void get_arguments(int argc, char **argv)
 {
   const char *dir;
@@ -100,9 +101,9 @@ static void get_arguments(int argc, char **argv)
   for(;;)
   {
     #ifdef __GNU_LIBRARY__
-      opt = getopt_long(argc, argv, "gvaVhtpsr:", Long_options, &index);
+      opt = getopt_long(argc, argv, "gvaVhtpmsr:", Long_options, &index);
     #else
-      opt = getopt(argc, argv, "gvaVhtpsr:");
+      opt = getopt(argc, argv, "gvaVhtpmsr:");
     #endif
     if (opt < 0) break;
 
@@ -132,6 +133,10 @@ static void get_arguments(int argc, char **argv)
         main_public = TRUE;
         break;
 
+      case 'm':
+        main_public_module = TRUE;
+        break;
+
       case 's':
         main_swap = TRUE;
         break;
@@ -157,7 +162,8 @@ static void get_arguments(int argc, char **argv)
           "  -v  --verbose              verbose output\n"
           "  -a  --all                  compile all\n"
           "  -t  --trans                output translation files\n"
-          "  -p  --public               form controls are public\n"
+          "  -p  --public-control       form controls are public\n"
+          "  -m  --public-module        module symbols are public by default\n"
           "  -s  --swap                 swap endianness\n"
           "  -r  --root <directory>     gives the gambas installation directory\n"
           "  -V  --version              display version\n"
@@ -169,6 +175,7 @@ static void get_arguments(int argc, char **argv)
           "  -a                         compile all\n"
           "  -t                         output translation files\n"
           "  -p                         form controls are public\n"
+          "  -m                         module symbols are public by default\n"
           "  -s                         swap endianness\n"
           "  -r <directory>             gives the gambas installation directory\n"
           "  -V                         display version\n"
@@ -244,6 +251,7 @@ static void compile_file(const char *file)
   JOB->debug = main_debug;
   JOB->verbose = main_verbose;
   JOB->swap = main_swap;
+  JOB->public_module = main_public_module;
   //JOB->class_file = main_class_file;
 
   if (JOB->verbose)
@@ -273,38 +281,12 @@ _FIN:
 }
 
 
-static void path_add(const char *path)
+static int compare_path(char **a, char **b)
 {
-  *((char **)ARRAY_add(&path_list)) = STR_copy(path);
+	return strcmp(*a, *b);
 }
 
-
-static void path_init(const char *first)
-{
-  ARRAY_create(&path_list);
-
-  if (*first)
-    chdir(first);
-
-  path_add(FILE_get_current_dir());
-
-  path_current = 0;
-}
-
-
-static void path_exit(void)
-{
-  ARRAY_delete(&path_list);
-}
-
-
-static long path_count(void)
-{
-  return ARRAY_count(path_list);
-}
-
-
-int main(int argc, char **argv)
+static void init_files(const char *first)
 {
   DIR *dir;
   char *path;
@@ -313,7 +295,73 @@ int main(int argc, char **argv)
   const char *file;
   struct stat info;
   const char *ext;
+	
+  ARRAY_create(&_files);
 
+  if (*first)
+  {
+    if (chdir(first))
+    {
+			fprintf(stderr, "gbc: cannot switch to directory: %s\n", first);
+			exit(1);
+		}
+  }
+
+	path = STR_copy(FILE_get_current_dir());
+  dir = opendir(path);
+	if (!dir)
+	{
+		fprintf(stderr, "gbc: cannot browse directory: %s\n", path);
+		exit(1);
+	}
+  
+	while ((dirent = readdir(dir)) != NULL)
+	{
+		file_name = dirent->d_name;
+		if (*file_name == '.')
+			continue;
+
+		file = FILE_cat(path, file_name, NULL);
+
+		if (stat(file_name, &info))
+		{
+			fprintf(stderr, "gbc: warning: cannot stat file: %s\n", file);
+			continue;
+		}
+
+		if (!S_ISDIR(info.st_mode))
+		{
+			ext = FILE_get_ext(file);
+
+			if ((strcmp(FILE_get_ext(file), "module") == 0)
+					|| (strcmp(FILE_get_ext(file), "class") == 0))
+			  *((char **)ARRAY_add(&_files)) = STR_copy(file);
+		}
+	}
+
+	closedir(dir);
+	STR_free(path);
+
+	// Sort paths
+	qsort(_files, ARRAY_count(_files), sizeof(*_files), (int (*)(const void *, const void *))compare_path);
+}
+
+
+static void exit_files(void)
+{
+	int i;
+	
+	for (i = 0; i < ARRAY_count(_files); i++)
+		STR_free(_files[i]);
+	
+  ARRAY_delete(&_files);
+}
+
+
+int main(int argc, char **argv)
+{
+	int i;
+	
   MEMORY_init();
   COMMON_init();
 
@@ -323,64 +371,15 @@ int main(int argc, char **argv)
   {
     COMPILE_init();
 
-    path_init(FILE_get_dir(COMP_project));
+    init_files(FILE_get_dir(COMP_project));
 
-    for(;;)
-    {
-      if (path_current >= path_count())
-        break;
-
-      path = path_list[path_current++];
-      dir = opendir(path);
-
-      if (dir == NULL)
-      {
-        fprintf(stderr, "gbc: Warning: Cannot open dir: %s\n", path);
-        goto _NEXT_PATH;
-      }
-
-      if (chdir(path) != 0)
-      {
-        fprintf(stderr, "gbc: Warning: Cannot change dir: %s\n", path);
-        goto _NEXT_PATH;
-      }
-
-      while ((dirent = readdir(dir)) != NULL)
-      {
-        file_name = dirent->d_name;
-        if (*file_name == '.')
-          continue;
-
-        file = FILE_cat(path, file_name, NULL);
-
-        if (stat(file_name, &info))
-        {
-          fprintf(stderr, "gbc: Warning: Cannot stat file: %s\n", file);
-          continue;
-        }
-
-        // No recursion anymore!
-        //if (S_ISDIR(info.st_mode))
-        //  path_add(file);
-        //else
-        if (!S_ISDIR(info.st_mode))
-        {
-          ext = FILE_get_ext(file);
-
-          if ((strcasecmp(FILE_get_ext(file), "module") == 0)
-              || (strcasecmp(FILE_get_ext(file), "class") == 0))
-            compile_file(file);
-        }
-      }
-
-_NEXT_PATH:
-      if (dir != NULL) closedir(dir);
-      FREE(&path, "main");
-    }
-
-    path_exit();
+		for (i = 0; i < ARRAY_count(_files); i++)
+			compile_file(_files[i]);
 
+    exit_files();
+    
     COMPILE_exit();
+    
     printf("OK\n");
   }
   CATCH
