@@ -59,13 +59,13 @@ static TABLE *arch_table;
 static FILE *arch_file = NULL;
 static char *arch_buffer;
 
-static long pos_start;
+static int pos_start;
 
-static void write_long(ulong val)
+static void write_long(uint val)
 {
  	if (ARCH_swap)
- 		SWAP_long((long *)&val);
-  if (fwrite(&val, sizeof(ulong), 1, arch_file) < 1)
+ 		SWAP_long((int *)&val);
+  if (fwrite(&val, sizeof(uint), 1, arch_file) < 1)
     THROW("Write error");
 }
 
@@ -79,15 +79,15 @@ static void write_short(ushort val)
 }
 
 
-static long get_pos(void)
+static int get_pos(void)
 {
   return ftell(arch_file);
 }
 
 
-static void write_long_at(long pos, ulong val)
+static void write_long_at(int pos, uint val)
 {
-  long old_pos = get_pos();
+  int old_pos = get_pos();
 
   fseek(arch_file, pos, SEEK_SET);
   write_long(val);
@@ -181,13 +181,13 @@ PUBLIC void ARCH_init(void)
 
 
 #if ARCH_VERSION == 2
-static void compress_file_name(const char *src, long lsrc, char **dst, long *ldst)
+static void compress_file_name(const char *src, int lsrc, char **dst, int *ldst)
 {
   char *p;
   char tpath[MAX_PATH];
   char tpath2[MAX_PATH];
-  long len;
-  long ind;
+  int len;
+  int ind;
   ARCH_SYMBOL *sym;
 
 	strncpy(tpath, src, lsrc);
@@ -209,7 +209,7 @@ static void compress_file_name(const char *src, long lsrc, char **dst, long *lds
 	  	THROW("&1: not in archive!\n", tpath);
 		}
 	
-		len = sprintf(tpath2, "/%ld:%s", ind, p + 1);
+		len = sprintf(tpath2, "/%d:%s", ind, p + 1);
 		strcpy(tpath, tpath2);
 	}
 
@@ -225,11 +225,11 @@ PUBLIC void ARCH_exit(void)
 {
   int i;
   ARCH_SYMBOL *sym;
-  long pos_str;
+  int pos_str;
 
   /* Write strings */
 
-  write_long_at(pos_start + sizeof(long), get_pos());
+  write_long_at(pos_start + sizeof(int), get_pos());
 
   pos_str = 0;
 
@@ -241,9 +241,9 @@ PUBLIC void ARCH_exit(void)
 
   /* Write file names */
 
-  write_long_at(pos_start + sizeof(long) * 2, get_pos());
+  write_long_at(pos_start + sizeof(int) * 2, get_pos());
 
-  write_long_at(pos_start + sizeof(long) * 3, TABLE_count(arch_table));
+  write_long_at(pos_start + sizeof(int) * 3, TABLE_count(arch_table));
 
   for (i = 0; i < TABLE_count(arch_table); i++)
   {
@@ -279,15 +279,15 @@ PUBLIC void ARCH_exit(void)
 }
 
 
-PUBLIC long ARCH_add_file(const char *path)
+PUBLIC int ARCH_add_file(const char *path)
 {
   char *rel_path;
   ARCH_SYMBOL *sym;
   FILE *file;
   struct stat info;
-  long len, len_read;
+  int len, len_read;
   
-  long ind;
+  int ind;
   
   #if ARCH_VERSION == 2
   compress_file_name(&path[arch_dir_pos], strlen(&path[arch_dir_pos]), &rel_path, &len);
@@ -337,11 +337,11 @@ PUBLIC long ARCH_add_file(const char *path)
 		fclose(file);
 
 		if (ARCH_verbose)
-			printf("Adding file %s (%ld bytes)", rel_path, sym->len);
+			printf("Adding file %s (%d bytes)", rel_path, sym->len);
 	}
 	
 	if (ARCH_verbose)
-		printf(" -> %ld\n", ind);
+		printf(" -> %d\n", ind);
 	
 	return ind;
 }
