@@ -42,19 +42,19 @@ static FILE *_flist = NULL;
 static FILE *_finfo = NULL;
 
 
-static const char *get_name(long index)
+static const char *get_name(int index)
 {
   return TABLE_get_symbol_name(JOB->class->table, index);
 }
 
 
-static const char *get_string(long index)
+static const char *get_string(int index)
 {
   return TABLE_get_symbol_name(JOB->class->string, index);
 }
 
 
-static void dump_name(long index)
+static void dump_name(int index)
 {
   printf("%s", get_name(index));
 }
@@ -62,7 +62,7 @@ static void dump_name(long index)
 
 static void dump_type(TYPE type, boolean as)
 {
-  long value;
+  int value;
   TYPE_ID id;
   CLASS_ARRAY *array;
   int i;
@@ -79,7 +79,7 @@ static void dump_type(TYPE type, boolean as)
     {
       if (i > 0)
         printf(",");
-      printf("%ld", array->dim[i]);
+      printf("%d", array->dim[i]);
     }
     printf("] AS ");
     dump_type(array->type, FALSE);
@@ -164,8 +164,8 @@ PUBLIC void CLASS_dump(void)
 
   printf("\n");
 
-  printf("Static size : %ld octets\n", class->size_stat);
-  printf("Dynamic size : %ld octets\n\n", class->size_dyn);
+  printf("Static size : %d octets\n", class->size_stat);
+  printf("Dynamic size : %d octets\n\n", class->size_dyn);
 
   for (i = 0; i < TABLE_count(class->table); i++)
   {
@@ -255,7 +255,7 @@ static void export_newline(void)
 
 static void export_type(TYPE type, bool scomma)
 {
-  long value;
+  int value;
   TYPE_ID id;
 
   id = TYPE_get_id(type);
@@ -303,7 +303,7 @@ PUBLIC void CLASS_export(void)
   TYPE type;
   CLASS_SYMBOL *sym;
   char kind;
-  long val;
+  int val;
   FUNCTION *func;
   EVENT *event;
   EXTFUNC *extfunc;
@@ -406,7 +406,7 @@ PUBLIC void CLASS_export(void)
           case T_BYTE:
           case T_SHORT:
           case T_INTEGER:
-            fprintf(_finfo, "%ld\n", val);
+            fprintf(_finfo, "%d\n", val);
             break;
 
           case T_LONG:
