@@ -46,7 +46,18 @@
 #include <dirent.h>
 
 #include <dlfcn.h>
-#include <ltdl.h>
+
+#ifdef OS_LINUX
+	#define lt_dlinit() (0)
+	#define lt_dlhandle void *
+	#define lt_dlopenext(_path) dlopen(_path, RTLD_LAZY)
+	#define lt_dlsym(_handle, _symbol) dlsym(_handle, _symbol)
+	#define lt_dlclose(_handle) dlclose(_handle)
+	#define lt_dlerror() dlerror()
+#else
+	#include <ltdl.h>
+#endif
+
 #include <ctype.h>
 
 #include "gb_component.h"
@@ -70,6 +81,8 @@ static bool _verbose = FALSE;
 static bool _format = FALSE;
 static bool _nopreload = FALSE;
 
+static void analyze(const char *comp, bool include);
+
 #ifdef __GNU_LIBRARY__
 static struct option LongOptions[] =
 {
@@ -90,16 +103,51 @@ static void print(const char *fmt, ...)
   va_start(args, fmt);
 
   vfprintf(out_info, fmt, args);
+  
+  va_end(args);
 }
 
-
-static void error2(const char *msg, const char *msg2, bool must_exit)
+static void warning(const char *fmt, ...)
 {
-  fprintf(stderr, "gbi: %s: %s%s%s\n", msg, msg2, (errno != 0) ? ": " : "", (errno != 0) ? strerror(errno) : "");
+  va_list args;
+
+	fprintf(stderr, "gbi" GAMBAS_VERSION_STRING ": warning: ");
+		
+  va_start(args, fmt);
+
+  vfprintf(stderr, fmt, args);
+  
+  putc('\n', stderr);
+  
+  va_end(args);
+}
+
+static void error(bool must_exit, const char *fmt, ...)
+{
+  va_list args;
+
+	fprintf(stderr, "gbi" GAMBAS_VERSION_STRING ": ERROR: ");
+		
+  va_start(args, fmt);
+
+  vfprintf(stderr, fmt, args);
+  
+  putc('\n', stderr);
+  
+  va_end(args);
+
   if (must_exit)
   	exit(1);
 }
 
+// static void error2(const char *msg, const char *msg2, bool must_exit)
+// {
+//   fprintf(stderr, "gbi" GAMBAS_VERSION_STRING ": ERROR: %s: %s%s%s\n", msg, msg2, (errno != 0) ? ": " : "", (errno != 0) ? strerror(errno) : "");
+//   if (must_exit)
+//   	exit(1);
+// }
+
+
 static void init(void)
 {
   const char *path;
@@ -116,7 +164,7 @@ static void init(void)
   strcpy(_info_path, FILE_cat(_root, "share/gambas" GAMBAS_VERSION_STRING "/info", NULL));
 
   if (lt_dlinit())
-    error2("Cannot initialize plug-in management", lt_dlerror(), TRUE);
+    error(TRUE, "Cannot initialize plug-in management: %s", lt_dlerror());
 
   /*if (putenv("LD_BIND_NOW=true"))
     error2("Cannot set LD_BIND_NOW", strerror(errno));
@@ -459,6 +507,7 @@ static bool analyze_native_component(const char *path)
 {
   lt_dlhandle lib;
   GB_DESC **desc;
+  char **include;
   GB_DESC **p;
   bool ret = FALSE;
   int nclass;
@@ -473,10 +522,18 @@ static bool analyze_native_component(const char *path)
   lib = lt_dlopenext(path);
   if (!lib)
   {
-    error2(path, lt_dlerror(), FALSE);
+    error(FALSE, "Cannot load shared library: %s", lt_dlerror());
     return TRUE;
 	}
 
+	include = lt_dlsym(lib, LIB_INCLUDE);
+	if (include)
+	{
+		if (_verbose)
+			fprintf(stderr, "Including %s\n", *include);
+		analyze(*include, TRUE);
+	}
+
   desc = lt_dlsym(lib, LIB_CLASS);
 
   if (desc)
@@ -503,11 +560,12 @@ static bool analyze_native_component(const char *path)
   }
   else
   {
-  	fprintf(stderr, "warning: cannot find '" LIB_CLASS "' symbol in shared library.\n");
+  	warning("cannot find '" LIB_CLASS "' symbol in shared library.");
     ret = TRUE;
 	}
 
-  lt_dlclose(lib);
+	// Do not close shared libraries
+  // lt_dlclose(lib);
 
   return ret;
 }
@@ -525,7 +583,7 @@ static bool analyze_gambas_component(const char *path)
 
   if (ARCH_find(arch, ".info", 0, &find))
   {
-  	fprintf(stderr, "warning: .info file not found in component archive.\n");
+  	warning(".info file not found in component archive.");
     return TRUE;
 	}
 
@@ -533,7 +591,7 @@ static bool analyze_gambas_component(const char *path)
 
   if (ARCH_find(arch, ".list", 0, &find))
   {
-  	fprintf(stderr, "warning: .list file not found in component archive.\n");
+  	warning(".list file not found in component archive.");
     return TRUE;
 	}
 
@@ -549,24 +607,24 @@ static void preload(char **argv, char *lib)
 #if DO_PRELOADING
   char buf[256];
 
-  if (_nopreload || getenv("GAMBAS_PRELOAD"))
+  if (_nopreload || getenv("GB_PRELOAD") || !lib || !*lib)
     return;
 
   sprintf(buf, "LD_PRELOAD=%s", lib);
   putenv(buf);
-  putenv("GAMBAS_PRELOAD=1");
+  putenv("GB_PRELOAD=1");
 
   execv(argv[0], argv);
 #endif
 }
 
 
-static void analyze(const char *comp)
+static void analyze(const char *comp, bool include)
 {
   bool native, gambas;
   char *name;
-  char *path_list;
-  char *path_info;
+  char *path_list = NULL;
+  char *path_info = NULL;
   bool ok;
 
   name = STR_copy(comp);
@@ -578,26 +636,25 @@ static void analyze(const char *comp)
 
   if (!native && !gambas)
   {
-    fprintf(stderr, "gbi2: warning: component %s not found\n", name);
+    warning("component %s not found", name);
     STR_free(name);
     return;
   }
 
-  path_info = STR_cat(FILE_cat(_info_path, name, NULL), ".info", NULL);
-  path_list = STR_cat(FILE_cat(_info_path, name, NULL), ".list", NULL);
-
-	puts(name);
-
-  //puts(path_info);
-  out_info = fopen(path_info, "w");
-  if (!out_info)
-    error2("Cannot write file", path_info, TRUE);
-
-  //puts(path_list);
-  out_list = fopen(path_list, "w");
-  if (!out_list)
-    error2("Cannot write file", path_list, TRUE);
-
+  if (!include)
+  {
+		path_info = STR_cat(FILE_cat(_info_path, name, NULL), ".info", NULL);
+		path_list = STR_cat(FILE_cat(_info_path, name, NULL), ".list", NULL);
+	
+		out_info = fopen(path_info, "w");
+		if (!out_info)
+			error(TRUE, "Cannot write file: %s", path_info);
+	
+		out_list = fopen(path_list, "w");
+		if (!out_list)
+			error(TRUE, "Cannot write file: %s", path_list);
+	}
+	
   fflush(stdout);
   ok = TRUE;
 
@@ -618,18 +675,21 @@ static void analyze(const char *comp)
       	ok = FALSE;
   }
 
-  fclose(out_info);
-  fclose(out_list);
-
-  if (!ok)
-  {
-    unlink(path_info);
-    unlink(path_list);
-  }
-
-  STR_free(path_info);
-  STR_free(path_list);
+	if (!include)
+	{
+		fclose(out_info);
+		fclose(out_list);
+	
+		if (!ok)
+		{
+			unlink(path_info);
+			unlink(path_list);
+		}
 
+		STR_free(path_info);
+		STR_free(path_list);
+	}
+	
   STR_free(name);
 }
 
@@ -638,7 +698,7 @@ int main(int argc, char **argv)
 {
   DIR *dir;
   struct dirent *dirent;
-  char *name;
+  const char *name;
   int opt;
   int save_fd;
   #ifdef __GNU_LIBRARY__
@@ -683,7 +743,7 @@ int main(int argc, char **argv)
           "\n"
           "GAMBAS Component Informer version " VERSION " " __DATE__ " " __TIME__ "\n"
           COPYRIGHT
-          "Usage: gbi" GAMBAS_VERSION_STRING " [options] [component]\n"
+          "Usage: gbi" GAMBAS_VERSION_STRING " [options] [components]\n"
           "Options:"
           #ifdef __GNU_LIBRARY__
           "\n"
@@ -715,9 +775,7 @@ int main(int argc, char **argv)
   {
     preload(argv,
       "libqt-mt.so.3 "
-      #if HAVE_KDE_COMPONENT
       "libkdecore.so.4 "
-      #endif
       );
 
     if (_verbose)
@@ -728,7 +786,7 @@ int main(int argc, char **argv)
 
     dir = opendir(_lib_path);
     if (dir == NULL)
-      error2("Cannot read directory", _lib_path, TRUE);
+      error(TRUE, "Cannot read directory: %s", _lib_path);
 
     save_fd = dup(STDOUT_FILENO);
 
@@ -737,20 +795,34 @@ int main(int argc, char **argv)
       name = dirent->d_name;
       if (strcmp(FILE_get_ext(name), "component"))
         continue;
-      analyze(FILE_get_basename(name));
+      name = FILE_get_basename(name);
+      puts(name);
+      analyze(name, FALSE);
     }
 
     closedir(dir);
   }
   else
   {
-    if (strncmp(argv[optind], "gb.qt.kde", 9) == 0)
-      preload(argv, "libqt-mt.so.3 libkdecore.so.4");
-    else if (strncmp(argv[optind], "gb.qt", 5) == 0)
-      preload(argv, "libqt-mt.so.3");
-
-    analyze(argv[optind]);
-  }
+  	if (!getenv("GB_PRELOAD"))
+  	{
+			for (ind = optind; ind < argc; ind++)
+			{
+				name = argv[ind];			
+				if (strncmp(name, "gb.qt.kde", 9) == 0)
+					preload(argv, "libqt-mt.so.3 libkdecore.so.4");
+				else if (strncmp(name, "gb.qt", 5) == 0)
+					preload(argv, "libqt-mt.so.3");
+			}
+		}
+  
+  	for (ind = optind; ind < argc; ind++)
+  	{
+  		name = argv[ind];
+			puts(name);
+			analyze(name, FALSE);
+		}
+	}
 
   exit(0);
 }
