@@ -1,8 +1,6 @@
 /***************************************************************************
 
-  common_check.h
-
-  Swapping routines for endianess management
+  gb_common_check.h
 
   Copyright (c) 2000-2007 Benoit Minisini <gambas@freesurf.fr>
 
@@ -35,7 +33,7 @@ EXTERN sigjmp_buf CHECK_jump;
 PUBLIC void CHECK_enter(void);
 PUBLIC void CHECK_leave(void);
 PUBLIC bool CHECK_got_error(void);
-PUBLIC bool CHECK_address(void *ptr, long len);
-PUBLIC bool CHECK_strlen(char *ptr, long *len);
+PUBLIC bool CHECK_address(void *ptr, int len);
+PUBLIC bool CHECK_strlen(char *ptr, int *len);
 
 #endif
