@@ -31,13 +31,12 @@
 #include "gb_buffer.h"
 #include "gb_error.h"
 #include "gbx_debug.h"
-
 #include "gbx_exec.h"
 #include "gbx_api.h"
 
 
-PUBLIC ERROR_INFO ERROR_info;
-
+PUBLIC ERROR_INFO ERROR_info = { 0 };
+static int _lock = 0;
 
 static const char *_message[64] =
 {
@@ -109,6 +108,31 @@ static const char *_message[64] =
 static ERROR_CONTEXT *_current = NULL;
 
 
+PUBLIC void ERROR_lock()
+{
+	_lock++;
+}
+
+
+PUBLIC void ERROR_unlock()
+{
+	_lock--;
+}
+
+static void ERROR_reset()
+{
+	ERROR_info.code = 0;
+	DEBUG_free_backtrace(&ERROR_info.backtrace);
+}
+
+PUBLIC void ERROR_clear()
+{
+	if (_lock)
+		return;
+	ERROR_reset();
+}
+
+
 PUBLIC void ERROR_enter(ERROR_CONTEXT *err)
 {
   CLEAR(err);
@@ -168,6 +192,8 @@ PUBLIC void ERROR_define(const char *pattern, char *arg[])
     }
   }
 
+	ERROR_clear();
+
   if ((long)pattern >= 0 && (long)pattern < 256)
   {
     ERROR_info.code = (long)pattern;
@@ -223,6 +249,12 @@ PUBLIC void ERROR_define(const char *pattern, char *arg[])
   ERROR_info.cp = CP;
   ERROR_info.fp = FP;
   ERROR_info.pc = PC;
+  
+  if (EXEC_debug || (CP && CP->debug))
+  {
+  	DEBUG_free_backtrace(&ERROR_info.backtrace);
+  	ERROR_info.backtrace = DEBUG_backtrace();
+  }
 }
 
 PUBLIC void PROPAGATE()
@@ -402,8 +434,18 @@ PUBLIC void ERROR_print_at(FILE *where, bool msgonly, bool newline)
 PUBLIC void ERROR_print(void)
 {
   static bool lock = FALSE;
+  
+  DEBUG_BACKTRACE *bt = ERROR_info.backtrace;
 
   ERROR_print_at(stderr, FALSE, TRUE);
+  
+  if (bt)
+  {
+  	int i;
+  	
+		for(i = 0; i < ARRAY_count(bt); i++)
+			fprintf(stderr, "%d: %s\n", i, DEBUG_get_position(bt[i].cp, bt[i].fp, bt[i].pc));
+	}
 
   if (EXEC_main_hook_done && !EXEC_debug && EXEC_Hook.error && !lock)
   {
@@ -420,14 +462,22 @@ PUBLIC void ERROR_save(ERROR_INFO *save)
   save->cp = ERROR_info.cp;
   save->fp = ERROR_info.fp;
   save->pc = ERROR_info.pc;
+  save->backtrace = ERROR_info.backtrace;
+  ERROR_info.backtrace = NULL;
   strcpy(save->msg, ERROR_info.msg);
 }
 
 PUBLIC void ERROR_restore(ERROR_INFO *save)
 {
+	ERROR_reset();
+	
   ERROR_info.code = save->code;
   ERROR_info.cp = save->cp;
   ERROR_info.cp = save->fp;
   ERROR_info.pc = save->pc;
+  ERROR_info.backtrace = save->backtrace;
+  save->backtrace = NULL;
   strcpy(ERROR_info.msg, save->msg);
 }
+
+
