@@ -104,6 +104,7 @@ typedef
     void *fp;
     void *pc;
     char msg[MAX_ERROR_MSG + 1];
+    void *backtrace;
     }
   ERROR_INFO;
 
@@ -147,8 +148,6 @@ EXTERN ERROR_INFO ERROR_info;
   	} \
 	}
 
-#define ERROR_clear() (ERROR_info.code = 0)
-
 //PUBLIC void ERROR_clear(void);
 PUBLIC const char *ERROR_get(void);
 
@@ -169,4 +168,8 @@ PUBLIC void ERROR_print_at(FILE *where, bool msgonly, bool newline);
 PUBLIC void ERROR_save(ERROR_INFO *save);
 PUBLIC void ERROR_restore(ERROR_INFO *save);
 
+PUBLIC void ERROR_clear();
+PUBLIC void ERROR_lock();
+PUBLIC void ERROR_unlock();
+
 #endif
