@@ -81,7 +81,11 @@ static void init(const char *file)
 	}
 	_welcome = TRUE;
 
-	PROJECT_load(); // Call STACK_init()
+	if (file)
+		PROJECT_load(); // Call STACK_init()
+	else
+		STACK_init();
+		
  	LOCAL_init();
 }
 
@@ -109,6 +113,7 @@ static void main_exit()
   EVENT_exit();
   FILE_exit();
   STRING_exit();
+	ERROR_clear();
   STACK_exit();
 }
 
@@ -119,7 +124,7 @@ int main(int argc, char **argv)
   CLASS_DESC_METHOD *startup = NULL;
 
   int i, n;
-  char *file = NULL;
+  char *file = ".";
   bool nopreload = FALSE;
 
   MEMORY_init();
@@ -145,7 +150,7 @@ int main(int argc, char **argv)
     if (n == 1)
       LIBRARY_preload(file, argv);
 
-    for (i = 1; i < (argc - n); i++)
+    for (i = 0; i < (argc - n); i++)
       argv[i] = argv[i + n];
 
     argc -= n;
@@ -176,35 +181,61 @@ int main(int argc, char **argv)
   }*/
   else
   {
-    for (i = 1; i < argc; i++)
-    {
-      if (strcmp(argv[i], "-h") == 0)
-      {
+  	if (argc == 2)
+  	{
+  		if (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help"))
+  		{
         printf(
           "\n"
           "GAMBAS Interpreter version " VERSION " " __DATE__ " " __TIME__ "\n"
           COPYRIGHT
           "Usage: gbx" GAMBAS_VERSION_STRING " [options] [<project file>] -- ...\n"
-          "       gbx" GAMBAS_VERSION_STRING " -x <archive file> ...\n\n"
+          "       gbx" GAMBAS_VERSION_STRING " -e <expression>\n\n"
           "Options:\n"
-          "  -g   enter debugging mode\n"
-          "  -V   display version\n"
-          "  -h   display this help\n"
+          "  -V --version   display version\n"
+          "  -h --help      display this help\n"
+          "  -e             evaluate an expression\n"
+          "  -g             enter debugging mode\n"
 #if DO_PRELOADING          
-          "  -p   disable preloading\n"
+          "  -p             disable preloading\n"
 #endif
-          "  -x   execute an archive\n"
+          "  -x             execute an archive\n"
           "\n"
           );
 
         my_exit(0);
-      }
-      else if (strcmp(argv[i], "-V") == 0)
+  		}
+      else if (!strcmp(argv[1], "-V") || !strcmp(argv[1], "--version"))
       {
         printf(VERSION "\n");
         my_exit(0);
       }
-      else if (strcmp(argv[i], "-g") == 0)
+  	}
+  	else
+  	
+  	if (argc == 3 && !strcmp(argv[1], "-e"))
+  	{
+  		TRY
+  		{
+  			init(NULL);
+  			EVAL_string(argv[2]);
+  		}
+  		CATCH
+  		{
+				if (ERROR_info.code && ERROR_info.code != E_ABORT)
+					ERROR_print_at(stderr, TRUE, TRUE);
+				main_exit();
+				_exit(1);
+  		}
+  		END_TRY
+  		
+  		main_exit();
+  		_exit(0);	
+  	}
+  	
+    for (i = 1; i < argc; i++)
+    {
+      if (strcmp(argv[i], "-g") == 0)
       {
         EXEC_debug = TRUE;
       }
@@ -242,10 +273,6 @@ int main(int argc, char **argv)
 
     if (!nopreload)
       LIBRARY_preload(file, argv);
-    /*printf("argc = %d\n", argc);
-    for (i = 0; i < argc; i++)
-      printf("argv[%d] = '%s'\n", i, argv[i]);
-    printf("\n");*/
 
     for (i = 1; i <= (argc - n); i++)
       argv[i] = argv[i + n - 1];
@@ -263,7 +290,9 @@ int main(int argc, char **argv)
   TRY
   {
     init(file);
-    argv[0] = PROJECT_name;
+    
+		if (!EXEC_arch)
+    	argv[0] = PROJECT_name;
 
     HOOK(main)(&argc, argv);
     EXEC_main_hook_done = TRUE;
