@@ -27,7 +27,7 @@
 
 #include "gambas.h"
 
-PUBLIC int GB_GetInterface(const char *library, long version, void *iface);
+PUBLIC int GB_GetInterface(const char *library, int version, void *iface);
 PUBLIC void *GB_Hook(int type, void *hook);
 
 PUBLIC int GB_LoadComponent(const char *name);
@@ -36,7 +36,7 @@ PUBLIC const char *GB_CurrentComponent(void);
 PUBLIC void GB_Push(int nval, ...);
 PUBLIC int GB_CanRaise(void *object, int event_id);
 PUBLIC int GB_Raise(void *object, int event_id, int nparam, ...);
-PUBLIC long GB_GetEvent(void *class, char *name);
+PUBLIC int GB_GetEvent(void *class, char *name);
 PUBLIC char *GB_GetLastEventName();
 PUBLIC int GB_Stopped(void);
 
@@ -56,34 +56,35 @@ PUBLIC void GB_ListEnum(void *);
 PUBLIC int GB_NextEnum(void);
 PUBLIC void GB_StopAllEnum(void *);
 
-PUBLIC void GB_Return(unsigned long type, ...);
-PUBLIC void GB_ReturnInteger(long val);
+PUBLIC void GB_Return(unsigned int type, ...);
+PUBLIC void GB_ReturnInteger(int val);
 #define GB_ReturnInt GB_ReturnInteger
 PUBLIC void GB_ReturnLong(long long val);
 PUBLIC void GB_ReturnBoolean(int val);
 PUBLIC void GB_ReturnObject(void *val);
 PUBLIC void GB_ReturnNull(void);
 PUBLIC void GB_ReturnFloat(double val);
-PUBLIC void GB_ReturnPtr(unsigned long type, void *value);
+PUBLIC void GB_ReturnPtr(unsigned int type, void *value);
 PUBLIC void GB_ReturnDate(GB_DATE *date);
 
 PUBLIC void GB_ReturnString(char *str);
-PUBLIC void GB_ReturnConstString(const char *str, long len);
+PUBLIC void GB_ReturnConstString(const char *str, int len);
 PUBLIC void GB_ReturnConstZeroString(const char *str);
-PUBLIC void GB_ReturnNewString(const char *src, long len);
+PUBLIC void GB_ReturnNewString(const char *src, int len);
 PUBLIC void GB_ReturnNewZeroString(const char *src);
 
 PUBLIC void *GB_GetClass(void *object);
 PUBLIC char *GB_GetClassName(void *object);
 PUBLIC void *GB_FindClass(const char *name);
 PUBLIC int GB_ExistClass(const char *name);
+PUBLIC int GB_ExistClassLocal(const char *name);
 
 PUBLIC char *GB_ToZeroString(GB_STRING *src);
 
-PUBLIC int GB_LoadFile(const char *path, long lenp, char **addr, long *len);
-PUBLIC void GB_ReleaseFile(char **addr, long len);
+PUBLIC int GB_LoadFile(const char *path, int lenp, char **addr, int *len);
+PUBLIC void GB_ReleaseFile(char **addr, int len);
 PUBLIC char *GB_GetTempDir(void);
-PUBLIC char *GB_RealFileName(const char *path, long len);
+PUBLIC char *GB_RealFileName(const char *path, int len);
 
 PUBLIC int GB_IsMissing(int param);
 
@@ -95,14 +96,15 @@ PUBLIC void GB_StoreString(GB_STRING *src, char **dst);
 PUBLIC void GB_StoreObject(GB_OBJECT *object, void **dst);
 PUBLIC void GB_StoreVariant(GB_VARIANT *src, void *dst);
 
-PUBLIC void GB_Watch(int fd, int flag, void *callback, long param);
+PUBLIC void GB_Watch(int fd, int flag, void *callback, int param);
 
 PUBLIC int GB_New(void **object, void *class_name, const char *name, void *parent);
 PUBLIC int GB_CheckObject(void *object);
 PUBLIC int GB_Is(void *object, void *class);
 
 PUBLIC int GB_GetFunction(GB_FUNCTION *func, void *object, const char *name, const char *sign, const char *type);
-PUBLIC GB_VALUE *GB_Call(GB_FUNCTION *_func, int nparam, int release);
+PUBLIC GB_VALUE *GB_Call(GB_FUNCTION *func, int nparam, int release);
+PUBLIC void *GB_GetClassInterface(void *class, const char *name);
 
 PUBLIC const char *GB_AppName(void);
 PUBLIC const char *GB_AppTitle(void);
@@ -114,37 +116,37 @@ PUBLIC char *GB_SystemCharset(void);
 
 PUBLIC void *GB_Eval(void *, void *);
 
-PUBLIC void GB_ArrayNew(GB_ARRAY *array, long type, long size);
-PUBLIC long GB_ArrayCount(GB_ARRAY array);
+PUBLIC void GB_ArrayNew(GB_ARRAY *array, unsigned int type, int size);
+PUBLIC int GB_ArrayCount(GB_ARRAY array);
 PUBLIC void *GB_ArrayAdd(GB_ARRAY array);
-PUBLIC void *GB_ArrayGet(GB_ARRAY array, long index);
+PUBLIC void *GB_ArrayGet(GB_ARRAY array, int index);
 
 PUBLIC void GB_CollectionNew(GB_COLLECTION *col, int mode);
-PUBLIC long GB_CollectionCount(GB_COLLECTION col);
-PUBLIC void GB_CollectionSet(GB_COLLECTION col, const char *key, long len, GB_VARIANT *value);
-PUBLIC int GB_CollectionGet(GB_COLLECTION col, const char *key, long len, GB_VARIANT *value);
-PUBLIC int GB_CollectionEnum(GB_COLLECTION col, GB_VARIANT *value, char **key, long *len);
+PUBLIC int GB_CollectionCount(GB_COLLECTION col);
+PUBLIC void GB_CollectionSet(GB_COLLECTION col, const char *key, int len, GB_VARIANT *value);
+PUBLIC int GB_CollectionGet(GB_COLLECTION col, const char *key, int len, GB_VARIANT *value);
+PUBLIC int GB_CollectionEnum(GB_COLLECTION col, GB_VARIANT *value, char **key, int *len);
 
-PUBLIC void GB_Alloc(void **addr, long len);
+PUBLIC void GB_Alloc(void **addr, int len);
 PUBLIC void GB_Free(void **addr);
-PUBLIC void GB_Realloc(void **addr, long len);
+PUBLIC void GB_Realloc(void **addr, int len);
 
 PUBLIC void GB_FreeString(char **str);
-PUBLIC long GB_StringLength(const char *str);
-PUBLIC bool GB_ConvString(char **result, const char *str, long len, const char *src, const char *dst);
+PUBLIC int GB_StringLength(const char *str);
+PUBLIC bool GB_ConvString(char **result, const char *str, int len, const char *src, const char *dst);
 
 PUBLIC int GB_Conv(GB_VALUE *, GB_TYPE);
 
-PUBLIC int GB_NumberToString(int flag, double value, const char *format, char **str, long *len);
+PUBLIC int GB_NumberToString(int flag, double value, const char *format, char **str, int *len);
 
 PUBLIC void GB_HashTableNew(GB_HASHTABLE *hash, int mode);
-PUBLIC void GB_HashTableAdd(GB_HASHTABLE hash, const char *key, long len, void *data);
-PUBLIC void GB_HashTableRemove(GB_HASHTABLE hash, const char *key, long len);
-PUBLIC int GB_HashTableGet(GB_HASHTABLE hash, const char *key, long len, void **data);
+PUBLIC void GB_HashTableAdd(GB_HASHTABLE hash, const char *key, int len, void *data);
+PUBLIC void GB_HashTableRemove(GB_HASHTABLE hash, const char *key, int len);
+PUBLIC int GB_HashTableGet(GB_HASHTABLE hash, const char *key, int len, void **data);
 PUBLIC void GB_HashTableEnum(GB_HASHTABLE hash, GB_HASHTABLE_ENUM_FUNC func);
 
-PUBLIC void GB_NewArray(void *pdata, long size, long count);
-PUBLIC long GB_CountArray(void *data);
+PUBLIC void GB_NewArray(void *pdata, int size, int count);
+PUBLIC int GB_CountArray(void *data);
 PUBLIC void *GB_Add(void *pdata);
 
 PUBLIC void GB_StreamInit(GB_STREAM *, int fd);
@@ -153,10 +155,10 @@ PUBLIC int GB_tolower(int c);
 PUBLIC int GB_toupper(int c);
 
 PUBLIC void GB_SubCollectionNew(GB_SUBCOLLECTION *subcollection, GB_SUBCOLLECTION_DESC *desc, void *container);
-PUBLIC void GB_SubCollectionAdd(void *_object, const char *key, long len, void *value);
-PUBLIC void GB_SubCollectionRemove(void *_object, const char *key, long len);
+PUBLIC void GB_SubCollectionAdd(void *_object, const char *key, int len, void *value);
+PUBLIC void GB_SubCollectionRemove(void *_object, const char *key, int len);
 PUBLIC void *GB_SubCollectionContainer(void *_object);
-PUBLIC void *GB_SubCollectionGet(void *_object, const char *key, long len);
+PUBLIC void *GB_SubCollectionGet(void *_object, const char *key, int len);
 
 PUBLIC void GB_PrintData(GB_TYPE type, void *addr);
 
@@ -174,8 +176,8 @@ PUBLIC void *GB_DebugGetExec(void);
 #ifndef __GBX_API_C
 EXTERN void *GAMBAS_Api[];
 EXTERN void *GAMBAS_DebugApi[];
-EXTERN unsigned long GAMBAS_ReturnType;
-EXTERN unsigned long GAMBAS_MissingParam;
+EXTERN unsigned int GAMBAS_ReturnType;
+EXTERN unsigned int GAMBAS_MissingParam;
 EXTERN bool GAMBAS_Error;
 EXTERN bool GAMBAS_DoNotRaiseEvent;
 EXTERN bool GAMBAS_StopEvent;
