@@ -34,7 +34,6 @@
 #include "gb_error.h"
 #include "gb_alloc.h"
 
-#include "gbx_list.h"
 #include "gbx_string.h"
 #include "gbx_stream.h"
 #include "gbx_component.h"
@@ -54,7 +53,7 @@ static ARCHIVE *_archive_list = NULL;
 static char *arch_pattern = NULL;
 static char *arch_path = NULL;
 static int arch_len_path;
-static long arch_index = 0;
+static int arch_index = 0;
 static ARCH *arch_dir = NULL;
 
 
@@ -84,7 +83,7 @@ static void load_exported_class(ARCHIVE *arch)
 {
   /*STREAM stream;*/
   char *buffer;
-  long len;
+  int len;
   char *name;
 
   /* COMPONENT_current is set => it will look in the archive */
@@ -301,16 +300,16 @@ PUBLIC void ARCHIVE_stat(ARCHIVE *arch, const char *path, FILE_STAT *info)
   info->mode = 0400;
 
   info->size = find.len;
-  info->atime = (long)buf.st_mtime;
-  info->mtime = (long)buf.st_mtime;
-  info->ctime = (long)buf.st_mtime;
+  info->atime = (int)buf.st_mtime;
+  info->mtime = (int)buf.st_mtime;
+  info->ctime = (int)buf.st_mtime;
   info->hidden = (*FILE_get_name(path) == '.');
   info->uid = buf.st_uid;
   info->gid = buf.st_gid;
 }
 
 
-PUBLIC bool ARCHIVE_read(ARCHIVE *arch, long pos, void *buffer, long len)
+PUBLIC bool ARCHIVE_read(ARCHIVE *arch, int pos, void *buffer, int len)
 {
   return ARCH_read(arch->arch, pos, buffer, len);
 }
@@ -349,7 +348,7 @@ PUBLIC void ARCHIVE_dir_first(ARCHIVE *arch, const char *path, const char *patte
 }
 
 
-PUBLIC bool ARCHIVE_dir_next(char **name, long *len, int attr)
+PUBLIC bool ARCHIVE_dir_next(char **name, int *len, int attr)
 {
   SYMBOL *sym;
   char *s = NULL;
