@@ -38,6 +38,7 @@
 #include <unistd.h>
 
 #include "gb_common.h"
+#include "gb_error.h"
 #include "gbx_api.h"
 #include "gbx_class.h"
 #include "gbx_project.h"
@@ -52,6 +53,7 @@
 
 static GB_FUNCTION signal_func;
 static bool has_signal_func;
+static bool _daemon = FALSE;
 
 extern char **environ;
 
@@ -113,6 +115,13 @@ BEGIN_PROPERTY(CUSER_id)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(CUSER_group)
+
+  GB_ReturnInteger(getgid());
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(CAPPLICATION_name)
 
   GB_ReturnConstZeroString(PROJECT_name);
@@ -222,6 +231,18 @@ BEGIN_METHOD_VOID(CAPPLICATION_args_next)
 END_METHOD
 
 
+BEGIN_PROPERTY(CAPPLICATION_env_count)
+
+	int n = 0;
+	
+	while(environ[n])
+		n++;
+
+  GB_ReturnInt(n);
+
+END_PROPERTY
+
+
 BEGIN_METHOD(CAPPLICATION_env_get, GB_STRING key)
 
   GB_ReturnConstZeroString(getenv(GB_ToZeroString(ARG(key))));
@@ -266,6 +287,35 @@ BEGIN_PROPERTY(CAPPLICATION_return)
 
 END_PROPERTY
 
+static void init_again(int old_pid)
+{
+	char old[MAX_PATH];
+	
+	FILE_remove_temp_file();
+  sprintf(old, FILE_TEMP_DIR, getuid(), old_pid);
+  rename(old, FILE_make_temp(NULL, NULL));
+}
+
+BEGIN_PROPERTY(CAPPLICATION_daemon)
+
+	int old_pid;
+
+	if (READ_PROPERTY)
+		GB_ReturnBoolean(_daemon);
+	else
+	{
+		if (!_daemon && VPROP(GB_BOOLEAN))
+		{
+			old_pid = getpid();
+			if (daemon(FALSE, FALSE))
+				THROW_SYSTEM(errno, NULL);
+			// Argh! daemon() changes the current process id...
+			_daemon = TRUE;
+			init_again(old_pid);
+		}
+	}
+
+END_PROPERTY
 
 
 BEGIN_PROPERTY(CSYSTEM_path)
@@ -307,6 +357,15 @@ BEGIN_PROPERTY(CSYSTEM_byte_order)
 
 END_PROPERTY
 
+BEGIN_PROPERTY(CSYSTEM_backtrace)
+
+	DEBUG_BACKTRACE *bt = DEBUG_backtrace();
+	
+	GB_ReturnObject(DEBUG_get_string_array_from_backtrace(bt));
+	DEBUG_free_backtrace(&bt);
+	
+END_PROPERTY
+
 #endif
 
 PUBLIC GB_DESC NATIVE_AppArgs[] =
@@ -325,6 +384,7 @@ PUBLIC GB_DESC NATIVE_AppEnv[] =
 {
   GB_DECLARE(".ApplicationEnv", 0), GB_VIRTUAL_CLASS(),
 
+  GB_STATIC_PROPERTY_READ("Count", "i", CAPPLICATION_env_count),
   GB_STATIC_METHOD("_get", "s", CAPPLICATION_env_get, "(Key)s"),
   GB_STATIC_METHOD("_put", NULL, CAPPLICATION_env_put, "(Value)s(Key)s"),
   GB_STATIC_METHOD("_next", "s", CAPPLICATION_env_next, NULL),
@@ -349,6 +409,7 @@ PUBLIC GB_DESC NATIVE_App[] =
   GB_STATIC_PROPERTY_READ("Version", "s", CAPPLICATION_version),
   GB_STATIC_PROPERTY_READ("Dir", "i", CAPPLICATION_dir),
   GB_STATIC_PROPERTY("Return", "i", CAPPLICATION_return),
+  GB_STATIC_PROPERTY("Daemon", "b", CAPPLICATION_daemon),
 
   GB_END_DECLARE
 };
@@ -366,6 +427,7 @@ PUBLIC GB_DESC NATIVE_System[] =
   GB_STATIC_PROPERTY_READ("Domain", "s", CSYSTEM_domain),
   GB_STATIC_PROPERTY_SELF("User", "User"),
   GB_STATIC_PROPERTY_READ("ByteOrder", "i", CSYSTEM_byte_order),
+  GB_STATIC_PROPERTY_READ("Backtrace", "String[]", CSYSTEM_backtrace),
 
   GB_END_DECLARE
 };
@@ -376,6 +438,7 @@ PUBLIC GB_DESC NATIVE_User[] =
 
   GB_STATIC_PROPERTY_READ("Name", "s", CUSER_name),
   GB_STATIC_PROPERTY_READ("Id", "i", CUSER_id),
+  GB_STATIC_PROPERTY_READ("Group", "i", CUSER_group),
   GB_STATIC_PROPERTY_READ("Home", "s", CUSER_home),
 
   GB_END_DECLARE
