@@ -687,6 +687,18 @@ BEGIN_METHOD(object_class, GB_OBJECT object)
 
   void *object = VARG(object);
 
+  if (GB_CheckObject(object))
+    return;
+
+  GB_ReturnObject(OBJECT_class(object));
+
+END_METHOD
+
+
+BEGIN_METHOD(object_type, GB_OBJECT object)
+
+  void *object = VARG(object);
+
   if (GB_CheckObject(object))
     return;
 
@@ -810,6 +822,40 @@ BEGIN_METHOD(object_count, GB_OBJECT object)
 END_METHOD
 
 
+BEGIN_METHOD(object_new, GB_STRING class; GB_OBJECT params)
+
+  CLASS *class = CLASS_find(GB_ToZeroString(ARG(class)));
+  GB_ARRAY params = VARGOPT(params, NULL);
+  int i, np = 0;
+  void *object;
+
+  if (!class)
+  {
+    GB_Error("Unknown class");
+    return;
+  }
+
+	if (params)
+		np = GB_ArrayCount(params);
+
+	if (np)
+	{
+  	STACK_check(np);
+
+		for (i = 0; i < np; i++)
+		{
+			CARRAY_get_value(params, i, SP);
+			PUSH();
+		}
+	}
+
+	object = EXEC_create_object(class, np, NULL);
+	OBJECT_UNREF_KEEP(&object, "object_new");
+	GB_ReturnObject(object);
+
+END_METHOD
+
+
 #endif
 
 
@@ -918,11 +964,12 @@ PUBLIC GB_DESC NATIVE_Object[] =
   GB_STATIC_METHOD("SetProperty", NULL, object_set_property, "(Object)o(Property)s(Value)v"),
   GB_STATIC_METHOD("Attach", NULL, object_attach, "(Object)o(Parent)o(Name)s"),
   GB_STATIC_METHOD("Detach", NULL, object_detach, "(Object)o"),
-  GB_STATIC_METHOD("Class", "s", object_class, "(Object)o"),
-  GB_STATIC_METHOD("Type", "s", object_class, "(Object)o"),
+  GB_STATIC_METHOD("Class", "Class", object_class, "(Object)o"),
+  GB_STATIC_METHOD("Type", "s", object_type, "(Object)o"),
   GB_STATIC_METHOD("Is", "b", object_is, "(Object)o(Class)s"),
   GB_STATIC_METHOD("Parent", "o", object_get_parent, "(Object)o"),
   GB_STATIC_METHOD("Call", "v", object_call, "(Object)o(Method)s[(Arguments)Array;]"),
+  GB_STATIC_METHOD("New", "o", object_new, "(Class)s[(Arguments)Array;]"),
   GB_STATIC_METHOD("IsValid", "b", object_is_valid, "(Object)o"),
   GB_STATIC_METHOD("Lock", NULL, object_lock, "(Object)o"),
   GB_STATIC_METHOD("Unlock", NULL, object_unlock, "(Object)o"),
