@@ -60,7 +60,7 @@ static void collection_free(CCOLLECTION *col)
 }
 
 
-static void *collection_get_key(CCOLLECTION *col, const char *key, long len)
+static void *collection_get_key(CCOLLECTION *col, const char *key, int len)
 {
   if (!key || !*key)
   {
@@ -75,7 +75,7 @@ static void *collection_get_key(CCOLLECTION *col, const char *key, long len)
 }
 
 
-static void *collection_add_key(CCOLLECTION *col, const char *key, long len)
+static void *collection_add_key(CCOLLECTION *col, const char *key, int len)
 {
   if (!key || !*key)
   {
@@ -90,7 +90,7 @@ static void *collection_add_key(CCOLLECTION *col, const char *key, long len)
 }
 
 
-static void collection_remove_key(CCOLLECTION *col, const char *key, long len)
+static void collection_remove_key(CCOLLECTION *col, const char *key, int len)
 {
   void *value;
   HASH_NODE *last;
@@ -164,7 +164,7 @@ END_PROPERTY
 BEGIN_PROPERTY(collection_key)
 
   char *key;
-  long len;
+  int len;
 
   if (HASH_TABLE_get_last_key(THIS->hash_table, &key, &len))
     GB_ReturnNull();
@@ -339,12 +339,12 @@ PUBLIC void GB_CollectionNew(GB_COLLECTION *col, int mode)
   OBJECT_create_native((void **)col, CLASS_Collection, (VALUE *)&param);
 }
 
-PUBLIC long GB_CollectionCount(GB_COLLECTION col)
+PUBLIC int GB_CollectionCount(GB_COLLECTION col)
 {
   return HASH_TABLE_size(((CCOLLECTION *)col)->hash_table);
 }
 
-PUBLIC void GB_CollectionSet(GB_COLLECTION col, const char *key, long len, GB_VARIANT *value)
+PUBLIC void GB_CollectionSet(GB_COLLECTION col, const char *key, int len, GB_VARIANT *value)
 {
   VARIANT *data;
 
@@ -358,7 +358,7 @@ PUBLIC void GB_CollectionSet(GB_COLLECTION col, const char *key, long len, GB_VA
   }
 }
 
-PUBLIC int GB_CollectionGet(GB_COLLECTION col, const char *key, long len, GB_VARIANT *value)
+PUBLIC int GB_CollectionGet(GB_COLLECTION col, const char *key, int len, GB_VARIANT *value)
 {
   void *val;
 
@@ -376,7 +376,7 @@ PUBLIC int GB_CollectionGet(GB_COLLECTION col, const char *key, long len, GB_VAR
   }
 }
 
-PUBLIC int GB_CollectionEnum(GB_COLLECTION col, GB_VARIANT *value, char **key, long *len)
+PUBLIC int GB_CollectionEnum(GB_COLLECTION col, GB_VARIANT *value, char **key, int *len)
 {
 	static HASH_ENUM iter;
   void *val;
