@@ -37,6 +37,8 @@
 
 #include "gbx_object.h"
 #include "gbx_string.h"
+#include "gbx_stack.h"
+#include "gbx_exec.h"
 
 #include "gbx_c_error.h"
 
@@ -100,6 +102,18 @@ BEGIN_METHOD_VOID(CERROR_propagate)
 
 END_METHOD
 
+
+BEGIN_PROPERTY(CERROR_backtrace)
+
+	DEBUG_BACKTRACE *bt = (DEBUG_BACKTRACE *)ERROR_info.backtrace;
+	
+	if (!bt)
+		GB_ReturnNull();
+	else
+		GB_ReturnObject(DEBUG_get_string_array_from_backtrace(bt));
+	
+END_PROPERTY
+
 #endif
 
 PUBLIC GB_DESC NATIVE_Error[] =
@@ -110,6 +124,7 @@ PUBLIC GB_DESC NATIVE_Error[] =
   GB_STATIC_PROPERTY_READ("Text", "s", CERROR_text),
   GB_STATIC_PROPERTY_READ("Class", "Class", CERROR_class),
   GB_STATIC_PROPERTY_READ("Where", "s", CERROR_where),
+  GB_STATIC_PROPERTY_READ("Backtrace", "String[]", CERROR_backtrace),
 
   GB_STATIC_METHOD("Clear", NULL, CERROR_clear, NULL),
   GB_STATIC_METHOD("Raise", NULL, CERROR_raise, "(Message)s"),
