@@ -34,16 +34,17 @@
 #include <sys/stat.h>
 
 #include "gb_common.h"
+#include "gb_list.h"
+#include "gb_file.h"
+
 #include "gbx_api.h"
 #include "gambas.h"
 #include "gbx_class.h"
-#include "gb_file.h"
 #include "gbx_stream.h"
 #include "gbx_exec.h"
 #include "gbx_project.h"
 #include "gbx_string.h"
 #include "gbx_date.h"
-#include "gbx_list.h"
 
 #include "gbx_c_file.h"
 
