@@ -25,7 +25,7 @@
 
 #include "gambas.h"
 #include "gbx_object.h"
-#include "gbx_list.h"
+#include "gb_list.h"
 
 #ifndef __GBX_C_GAMBAS_C
 extern GB_DESC NATIVE_Gambas[];
@@ -38,6 +38,7 @@ typedef
 		OBJECT ob;
 		LIST list;
 		ushort *event;
+		bool after;
 	}
 	COBSERVER;
 
