@@ -25,7 +25,7 @@
 
 #include "gambas.h"
 #include "gbx_variant.h"
-#include "gbx_list.h"
+#include "gb_list.h"
 
 typedef
 	GB_TIMER CTIMER;
