@@ -84,18 +84,18 @@ static CLASS *Class;
 static int NSection;
 #endif
 
-#define RELOCATE(_ptr) (_ptr = (char *)&class->string[(long)(_ptr)])
+#define RELOCATE(_ptr) (_ptr = (char *)&class->string[(int)(_ptr)])
 
 static void SWAP_type(CTYPE *p)
 {
 	SWAP_short(&p->value);
 }
 
-static char *get_section(char *sec_name, char **section, long one, short *psize, char *swapcode)
+static char *get_section(char *sec_name, char **section, int one, short *psize, char *swapcode)
 {
-  char *current = *section + sizeof(long);
-  long section_size = *((long *)(*section));
-  long i, n;
+  char *current = *section + sizeof(int);
+  int section_size = *((int *)(*section));
+  int i, n;
   char *p;
   char *ps;
   size_t ss;
@@ -107,7 +107,7 @@ static char *get_section(char *sec_name, char **section, long one, short *psize,
 
   #ifdef DEBUG
   NSection++;
-  printf("Section #%d %s %08lX %ld %ld\n", NSection + 1, sec_name, (long)(current - (char *)Class->data), one, section_size);
+  printf("Section #%d %s %08lX %d %d\n", NSection + 1, sec_name, (int)(current - (char *)Class->data), one, section_size);
   #endif
 
   /*
@@ -115,7 +115,7 @@ static char *get_section(char *sec_name, char **section, long one, short *psize,
     THROW(E_FORMAT);
   */
 
-  *section += section_size + sizeof(long);
+  *section += section_size + sizeof(int);
 
   if (section_size % one)
     THROW(E_CLASS, ClassName, "Bad format in ", sec_name);
@@ -161,12 +161,12 @@ static char *get_section(char *sec_name, char **section, long one, short *psize,
           else if (*ps == 'l')
           {
             func = SWAP_long;
-            ss = sizeof(long);
+            ss = sizeof(int);
           }
           else if (*ps == 't')
           {
             func = SWAP_type;
-            ss = sizeof(long);
+            ss = sizeof(int);
           }
           else
           {
@@ -216,7 +216,7 @@ static void conv_type(CLASS *class, void *ptype)
 
 	if (_swap)
 	{
-		SWAP_long((long *)&ctype);
+		SWAP_long((int *)&ctype);
 		SWAP_type(&ctype);
 	}
 
@@ -231,20 +231,40 @@ static void conv_type(CLASS *class, void *ptype)
 }
 
 
-static void check_version(long loaded)
+static void check_version(int loaded)
 {
-  long current = GAMBAS_PCODE_VERSION;
+	static bool warning = FALSE;
+	
+  int current = GAMBAS_PCODE_VERSION;
 
-  if ((GAMBAS_FULL_VERSION & 0xFFFF0000) == 0x109)
+  if (GAMBAS_FULL_VERSION > 0x01090000 && GAMBAS_FULL_VERSION <= 0x0200FFFF)
   {
-    current &= ~0xFFFF;
-    loaded &= ~0xFFFF;
+    if (loaded == current)
+      return;
+    
+    if ((COMPONENT_current && COMPONENT_current->warning) || warning)
+    	return;
+    
+    fprintf(stderr, "WARNING: ");
+    if (COMPONENT_current)
+    	fprintf(stderr, "%s: ", COMPONENT_current->name);
+    fprintf(stderr, "current bytecode version is ");
+    fprintf(stderr, "%X.%X.%X ", current >> 24, (current >> 16) & 0xFF, current & 0xFFFF);
+    fprintf(stderr, "and project bytecode version is ");
+    fprintf(stderr, "%X.%X.%X. You should recompile your project.\n", loaded >> 24, (loaded >> 16) & 0xFF, loaded & 0xFFFF);
+    
+    if (COMPONENT_current)
+    	COMPONENT_current->warning = TRUE;
+    else
+    	warning = TRUE;
+    	
+    return;
   }
-
+  
   if (loaded > current)
-    THROW(E_CLASS, ClassName, "Interpreter too old. Please upgrade.", "");
+    THROW(E_CLASS, ClassName, "Version too recent. Please upgrade Gambas.", "");
   if (loaded < current)
-    THROW(E_CLASS, ClassName, "Version too old. Please recompile.", "");
+    THROW(E_CLASS, ClassName, "Version too old. Please recompile the project.", "");
 }
 
 
@@ -267,10 +287,10 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
   CLASS_EXTERN *ext;
   short n_array;
   VALUE value;
-  long len_data;
-  long first;
+  int len_data;
+  int first;
   short n_desc;
-  long offset;
+  int offset;
   int first_event;
   FUNC_DEBUG *debug;
   COMPONENT *save;
@@ -359,16 +379,16 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
 
   if (_swap)
   {
-    SWAP_long((long *)&header->magic);
-    SWAP_long((long *)&header->version);
-    SWAP_long((long *)&header->flag);
+    SWAP_long((int *)&header->magic);
+    SWAP_long((int *)&header->version);
+    SWAP_long((int *)&header->flag);
   }
 
   if (header->magic != OUTPUT_MAGIC)
   {
   	int fd;
   	
-  	fprintf(stderr, "Dumping class file at /tmp/gambas-bad-header.out: %ld bytes\n", len_data);
+  	fprintf(stderr, "Dumping class file at /tmp/gambas-bad-header.out: %d bytes\n", len_data);
   	fd = open("/tmp/gambas-bad-header.out", O_CREAT | O_WRONLY, 0666);
   	write(fd, class->data, len_data);
   	close(fd);
@@ -388,15 +408,15 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
   info = (CLASS_INFO *)get_section("info section", &section, sizeof(CLASS_INFO), NULL, "s2l2");
   start = (CLASS_DESC *)get_section("description section", &section, sizeof(CLASS_DESC), &n_desc, "l*");
   class->load->cst = (CLASS_CONST *)get_section("constant section", &section, sizeof(CLASS_CONST), &class->load->n_cst, "l*");
-  class->load->class_ref = (CLASS **)get_section("reference section", &section, sizeof(long), &n_class_ref, "l");
-  class->load->unknown = (char **)get_section("unknown section", &section, sizeof(long), &n_unknown, "l");
+  class->load->class_ref = (CLASS **)get_section("reference section", &section, sizeof(int), &n_class_ref, "l");
+  class->load->unknown = (char **)get_section("unknown section", &section, sizeof(int), &n_unknown, "l");
   class->load->stat = (CLASS_VAR *)get_section("static section", &section, sizeof(CLASS_VAR), &class->load->n_stat, "tl");
   class->load->dyn = (CLASS_VAR *)get_section("dynamic section", &section, sizeof(CLASS_VAR), &class->load->n_dyn, "tl");
   class->load->event = (CLASS_EVENT *)get_section("event section", &section, sizeof(CLASS_EVENT), &class->n_event, "ls2l2");
   class->load->ext = (CLASS_EXTERN *)get_section("extern section", &section, sizeof(CLASS_EXTERN), &class->load->n_ext, "ls2l3");
   class->load->func = (FUNCTION *)get_section("function section", &section, sizeof(FUNCTION), &class->load->n_func, "lb4s4l4");
   local = (CLASS_LOCAL *)get_section("local section", &section, sizeof(CLASS_LOCAL), NULL, "l");
-  class->load->array = (CLASS_ARRAY **)get_section("array section", &section, sizeof(long), &n_array, "l");
+  class->load->array = (CLASS_ARRAY **)get_section("array section", &section, sizeof(int), &n_array, "l");
 
   /* Loading code */
 
@@ -454,7 +474,7 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
 
   for (i = 0; i < n_class_ref; i++)
   {
-  	offset = (long)class->load->class_ref[i];
+  	offset = (int)class->load->class_ref[i];
   	if (offset >= 0)
     	class->load->class_ref[i] = CLASS_find(&class->string[offset]);
 		else
@@ -489,7 +509,7 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
 			{
 	      for (j = 0; j < func->n_local; j++)
 	      {
-	      	SWAP_long((long *)&func->local[j].type);
+	      	SWAP_long((int *)&func->local[j].type);
 	      	SWAP_type(&func->local[j].type);
 				}
       }
@@ -549,11 +569,11 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
 
   if (n_array > 0)
   {
-    n_array = *((long *)class->load->array) / sizeof(long);
+    n_array = *((int *)class->load->array) / sizeof(int);
 
     for (i = 0; i < n_array; i++)
     {
-      class->load->array[i] = (CLASS_ARRAY *)((char *)class->load->array + ((long *)class->load->array)[i]);
+      class->load->array[i] = (CLASS_ARRAY *)((char *)class->load->array + ((int *)class->load->array)[i]);
       conv_type(class, &(class->load->array[i]->type));
     }
   }
@@ -637,7 +657,7 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
     cc = &class->load->cst[i];
     conv_type(class, &(cc->type));
     if (!TYPE_is_integer_long(cc->type))
-      cc->_string.addr += (long)class->string;
+      cc->_string.addr += (int)class->string;
 
     if (cc->type == T_FLOAT || cc->type == T_SINGLE)
     {
@@ -655,7 +675,7 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
       */
       if (_swap)
       {
-        long val;
+        int val;
 
         val = cc->_swap.val[0];
         cc->_swap.val[0] = cc->_swap.val[1];
@@ -705,7 +725,7 @@ PUBLIC void CLASS_load_without_init(CLASS *class)
 
         desc->property.read = (void (*)())desc->gambas.val1;
         desc->property.write = (void (*)())desc->gambas.val2;
-        if ((long)desc->property.write == -1)
+        if ((int)desc->property.write == -1)
           desc->gambas.name = *desc->gambas.name == 'p' ? "r" : "R";
         desc->property.native = FALSE;
 
