@@ -33,7 +33,6 @@
 #include "gb_common_case.h"
 #include "gb_error.h"
 #include "gb_alloc.h"
-#include "gbx_list.h"
 #include "gb_replace.h"
 #include "gb_file.h"
 #include "gbx_debug.h"
@@ -101,8 +100,8 @@ PUBLIC void COMPONENT_load_all(void)
 
   if (EXEC_debug)
   {
-  	COMPONENT_create("gb.debug");
     COMPONENT_create("gb.eval");
+  	COMPONENT_create("gb.debug");
 	}
 
   LIST_for_each(comp, _component_list)
@@ -297,3 +296,17 @@ PUBLIC void COMPONENT_signal(int signal, void *param)
       (*comp->library->signal)(signal, param);
   }
 }
+
+PUBLIC bool COMPONENT_get_info(const char *key, void **value)
+{
+  COMPONENT *comp;
+  
+  LIST_for_each(comp, _component_list)
+  {
+    if (comp->library && comp->library->info)
+      if ((*comp->library->info)(key, value))
+      	return FALSE;
+  }
+  
+  return TRUE;
+}
