@@ -26,7 +26,7 @@
 #define __GBX_COMPONENT_H
 
 #include "gambas.h"
-#include "gbx_list.h"
+#include "gb_list.h"
 #include "gb_component.h"
 
 #include "gbx_library.h"
@@ -43,7 +43,8 @@ typedef
     unsigned preload : 1;
     unsigned loaded : 1;
     unsigned user : 1;
-    unsigned _reserved : 29;
+    unsigned warning : 1;    // Set when the bytecode warning was displayed by the class loader for this component
+    unsigned _reserved : 28;
     }
   PACKED
   COMPONENT;
@@ -76,4 +77,6 @@ PUBLIC void COMPONENT_signal(int signal, void *param);
 
 #define COMPONENT_is_library(comp) ((comp)->library != NULL)
 
+PUBLIC bool COMPONENT_get_info(const char *key, void **value);
+
 #endif
