@@ -62,7 +62,7 @@ static const short days_in_year[2][14] =
   {  0, 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 }  /* leap year */
 };
 
-static long date_timezone;
+static int date_timezone;
 static int date_daylight;
 
 static double _start_time;
@@ -122,7 +122,7 @@ PUBLIC void DATE_init(void)
 
   tm = localtime(&t);
 
-  #if defined(OS_FREEBSD) || defined(OS_MACOSX)
+  #if defined(OS_FREEBSD) || defined(OS_MACOSX) || defined(OS_OPENBSD)
   date_timezone = tm->tm_gmtoff;
   date_daylight = tm->tm_isdst;
   #else
@@ -140,11 +140,11 @@ PUBLIC void DATE_init(void)
 
 PUBLIC DATE_SERIAL *DATE_split(VALUE *value)
 {
-  static long last_nday, last_nmsec;
+  static int last_nday, last_nmsec;
   static DATE_SERIAL last_date = { 0 };
 
-  long nday, nmsec;
-  long A, B, C, D, E, M;
+  int nday, nmsec;
+  int A, B, C, D, E, M;
 
   nday = value->_date.date;
   nmsec = value->_date.time;
@@ -216,7 +216,7 @@ PUBLIC DATE_SERIAL *DATE_split(VALUE *value)
 PUBLIC bool DATE_make(DATE_SERIAL *date, VALUE *val)
 {
   short year;
-  long nday;
+  int nday;
   bool timezone;
 
   if (date->year == 0)
@@ -280,7 +280,7 @@ static int get_current_year(void)
 }
 
 
-PUBLIC void DATE_from_time(time_t time, long usec, VALUE *val)
+PUBLIC void DATE_from_time(time_t time, int usec, VALUE *val)
 {
   struct tm *tm;
   DATE_SERIAL date;
@@ -451,7 +451,7 @@ static void set_time(DATE_SERIAL *date, int which, int value)
 }
 
 
-PUBLIC boolean DATE_from_string(const char *str, long len, VALUE *val, boolean local)
+PUBLIC boolean DATE_from_string(const char *str, int len, VALUE *val, boolean local)
 {
   DATE_SERIAL date;
   LOCAL_INFO *info = LOCAL_get(local);
@@ -627,7 +627,7 @@ PUBLIC void DATE_void_value(VALUE *value)
 }
 
 
-PUBLIC void DATE_add(VALUE *date, int period, long val)
+PUBLIC void DATE_add(VALUE *date, int period, int val)
 {
   long long add_time = 0;
   long long add_date;
@@ -724,8 +724,8 @@ __ADD_DATE_TIME:
   if (new_time < INT32_MIN || new_time > INT32_MAX || new_date <= 0 || new_date > INT32_MAX)
     THROW(E_OVERFLOW);
 
-  date->_date.date = (long)new_date;
-  date->_date.time = (long)new_time;
+  date->_date.date = (int)new_date;
+  date->_date.time = (int)new_time;
   return;
 
 __MAKE_DATE:
@@ -736,7 +736,7 @@ __MAKE_DATE:
 }
 
 
-PUBLIC long DATE_diff(VALUE *date1, VALUE *date2, int period)
+PUBLIC int DATE_diff(VALUE *date1, VALUE *date2, int period)
 {
   long long diff = 0;
   DATE_SERIAL ds1 = {0};
