@@ -30,9 +30,12 @@
 #include <stdarg.h>
 
 #include "gb_buffer.h"
+#include "gb_array.h"
 #include "gb_error.h"
 #include "gbx_exec.h"
 #include "gbx_api.h"
+#include "gbx_class.h"
+#include "gbx_c_array.h"
 
 #include "gbx_debug.h"
 
@@ -71,6 +74,9 @@ PUBLIC const char *DEBUG_get_position(CLASS *cp, FUNCTION *fp, PCODE *pc)
 #endif
   ushort line = 0;
 
+	if (!cp || !pc)
+		return "?";
+		
   if (fp != NULL && fp->debug)
     calc_line_from_position(cp, fp, pc, &line);
 
@@ -128,7 +134,7 @@ PUBLIC void DEBUG_where(void)
 }
 
 
-PUBLIC bool DEBUG_get_value(const char *sym, long len, GB_VARIANT *ret)
+PUBLIC bool DEBUG_get_value(const char *sym, int len, GB_VARIANT *ret)
 {
   int i;
   VALUE value;
@@ -183,7 +189,8 @@ PUBLIC bool DEBUG_get_value(const char *sym, long len, GB_VARIANT *ret)
     }
   }
 
-	class = CLASS_look_global(sym, len);
+	//class = CLASS_look_global(sym, len);
+	class = CLASS_look(sym, len);
 	if (class)
 	{
 		if (class->auto_create && class->instance)
@@ -218,40 +225,131 @@ __FOUND:
   return FALSE;
 }
 
-#if 0
-PUBLIC char *DEBUG_get_backtrace(void)
+int DEBUG_can_be_used_like_an_array(void *object, CLASS *class)
 {
-	char *result = NULL;
-  int i;
-  STACK_CONTEXT *context;
-  ushort line;
+	CLASS_DESC *desc;
+  char type;
+  int index;
 
-  if (CP)
-    sprintf(COMMON_buffer, "%s\n", DEBUG_get_current_position());
+	//fprintf(stderr, "DEBUG_can_be_used_like_an_array: %p %s ?\n", object, class->name);
+
+	if (!object)
+		return 0;
+	
+	if (class == CLASS_Class)
+	{
+		class = (CLASS *)object;
+		object = NULL;
+		CLASS_load(class);
+	}
+
+  index = class->special[SPEC_GET];
+  if (index == NO_SYMBOL)
+  {
+  	//fprintf(stderr, "No _get method\n");
+    return 0;
+  }
+	desc = class->table[index].desc;
+	if (desc->method.npmin != 1 || desc->method.npmax != 1 || *desc->method.signature != T_INTEGER)
+	{
+  	//fprintf(stderr, "No _get(Arg AS Integer) method\n");
+		return 0;
+	}
+	
+	index = CLASS_find_symbol(class, "Count");
+	if (index == NO_SYMBOL)
+	{
+  	//fprintf(stderr, "No Count symbol\n");		
+		return 0;
+	}
+		
+	desc = class->table[index].desc;
+	type = CLASS_DESC_get_type(desc);
+	if (type != 'r' && type != 'R')
+	{
+  	//fprintf(stderr, "Count is not a read-only property\n");		
+		return 0;
+	}
+	
+	
+  if (desc->property.native)
+  {
+    if (EXEC_call_native(desc->property.read, object, desc->property.type, 0))
+    {
+	  	//fprintf(stderr, "Count has failed\n");
+    	return 0;
+    }
+  }
   else
-    sprintf(COMMON_buffer, "?\n");
-    
-	STRING_add(&result, COMMON_buffer, 0);
+  {
+    EXEC.class = desc->property.class;
+    EXEC.object = object;
+    EXEC.drop = FALSE;
+    EXEC.nparam = 0;
+    EXEC.native = FALSE;
+    EXEC.index = (int)desc->property.read;
+    //EXEC.func = &class->load->func[(int)desc->property.read];
 
+    EXEC_function_keep();
+
+    TEMP = *RP;
+    UNBORROW(RP);
+    RP->type = T_VOID;
+  }
+
+	VALUE_conv(&TEMP, GB_T_INTEGER);
+	return TEMP._integer.value;
+}
+
+
+
+PUBLIC DEBUG_BACKTRACE *DEBUG_backtrace()
+{
+	DEBUG_BACKTRACE *result;
+  int i, n;
+  STACK_CONTEXT *context;
+
+	n = 1;
   for (i = 0; i < (STACK_frame_count - 1); i++)
   {
     context = &STACK_frame[i];
-
-    if (context->pc)
-    {
-      line = 0;
-      if (calc_line_from_position(context->cp, context->fp, context->pc, &line))
-        sprintf(COMMON_buffer, "%s.?.?\n", context->cp->name);
-      else
-        sprintf(COMMON_buffer, "%s.%s.%d\n", context->cp->name, context->fp->debug->name, line);
-    }
-    else if (context->cp)
-      sprintf(COMMON_buffer, "?\n");
+    if (!context->pc)
+    	continue;
+    n++;
+  }
 	
-	  STRING_add(&result, COMMON_buffer, 0);
+	ARRAY_create(&result);
+	ARRAY_add_many(&result, n);
+
+	result[0].pc = PC;
+	result[0].cp = CP;
+	result[0].fp = FP;
+
+	n = 0;
+  for (i = 0; i < (STACK_frame_count - 1); i++)
+  {
+    context = &STACK_frame[i];
+    if (!context->pc)
+    	continue;
+    n++;
+  	result[n].pc = context->pc;
+  	result[n].cp = context->cp;
+  	result[n].fp = context->fp;
   }
   
-  STRING_extend_end(&result);
   return result;
 }
-#endif
+
+
+PUBLIC GB_ARRAY DEBUG_get_string_array_from_backtrace(DEBUG_BACKTRACE *bt)
+{
+	GB_ARRAY array;
+	int i, n;
+	
+	n = ARRAY_count(bt);
+	GB_ArrayNew(&array, GB_T_STRING, n);
+	for (i = 0; i < n; i++)
+		STRING_new((char **)GB_ArrayGet(array, i), DEBUG_get_position(bt[i].cp, bt[i].fp, bt[i].pc), 0);
+
+	return array;
+}
