@@ -46,6 +46,14 @@
 /* Component management debugging */
 #define DEBUG_COMP     0
 
+typedef
+	struct {
+		void *pc;
+		void *cp;
+		void *fp;
+	}
+	DEBUG_BACKTRACE;
+
 #ifndef __GBX_DEBUG_C
 
 EXTERN DEBUG_INTERFACE DEBUG;
@@ -53,12 +61,16 @@ EXTERN DEBUG_INFO *DEBUG_info;
 
 #endif
 
-PUBLIC void DEBUG_init(void);
-PUBLIC void DEBUG_exit(void);
-PUBLIC const char *DEBUG_get_position(CLASS *cp, FUNCTION *fp, PCODE *pc);
-PUBLIC const char *DEBUG_get_current_position(void);
-PUBLIC void DEBUG_where(void);
-PUBLIC bool DEBUG_get_value(const char *sym, long len, GB_VARIANT *ret);
-//PUBLIC char *DEBUG_get_backtrace(void);
+void DEBUG_init(void);
+void DEBUG_exit(void);
+const char *DEBUG_get_position(CLASS *cp, FUNCTION *fp, PCODE *pc);
+const char *DEBUG_get_current_position(void);
+void DEBUG_where(void);
+bool DEBUG_get_value(const char *sym, int len, GB_VARIANT *ret);
+int DEBUG_can_be_used_like_an_array(void *object, CLASS *class);
+DEBUG_BACKTRACE *DEBUG_backtrace();
+GB_ARRAY DEBUG_get_string_array_from_backtrace(DEBUG_BACKTRACE *bt);
+
+#define DEBUG_free_backtrace(_bt) ARRAY_delete(_bt)
 
 #endif
