@@ -22,7 +22,6 @@
 
 ***************************************************************************/
 
-
 #ifndef __GBX_EXPRESSION_H
 #define __GBX_EXPRESSION_H
 
@@ -32,9 +31,9 @@
 typedef
   struct {
     char *source;
-    long len;
+    int len;
     PATTERN *pattern;
-    long pattern_count;
+    int pattern_count;
     PATTERN *current;
     PATTERN *tree;
     CLASS exec_class;
@@ -48,10 +47,10 @@ typedef
     /*TABLE *variable;*/
     CLASS **class;
     char **unknown;
-    long *var;
+    int *var;
     short nvar;
     short last_code;
-    long stack_usage;
+    int stack_usage;
     void *op;
     unsigned analyze : 1;
     unsigned _reserved : 31;
@@ -61,7 +60,7 @@ typedef
 typedef
   struct {
     SYMBOL sym;
-    long local;
+    int local;
     }
   EVAL_SYMBOL;
   
