@@ -73,7 +73,7 @@ PUBLIC STREAM_CLASS STREAM_null =
 };
 #endif
 
-PUBLIC long STREAM_eff_read;
+PUBLIC int STREAM_eff_read;
 
 PUBLIC bool STREAM_in_archive(const char *path)
 {
@@ -89,7 +89,7 @@ PUBLIC bool STREAM_in_archive(const char *path)
   return FALSE;
 }
 
-static int STREAM_get_readable(int fd, long *len)
+static int STREAM_get_readable(int fd, int *len)
 {
 	int ret;
   off_t off;
@@ -196,7 +196,7 @@ PUBLIC void STREAM_flush(STREAM *stream)
 }
 
 
-PUBLIC void STREAM_read(STREAM *stream, void *addr, long len)
+PUBLIC void STREAM_read(STREAM *stream, void *addr, int len)
 {
   STREAM_eff_read = 0;
 
@@ -249,7 +249,7 @@ PUBLIC char STREAM_getchar(STREAM *stream)
 }
 
 
-PUBLIC void STREAM_read_max(STREAM *stream, void *addr, long len)
+PUBLIC void STREAM_read_max(STREAM *stream, void *addr, int len)
 {
   STREAM_eff_read = 0;
 
@@ -272,7 +272,7 @@ PUBLIC void STREAM_read_max(STREAM *stream, void *addr, long len)
 }
 
 
-PUBLIC void STREAM_write(STREAM *stream, void *addr, long len)
+PUBLIC void STREAM_write(STREAM *stream, void *addr, int len)
 {
   if (!stream->type)
     THROW(E_CLOSED);
@@ -336,7 +336,7 @@ PUBLIC void STREAM_seek(STREAM *stream, long long pos, int whence)
 }
 
 
-static void add_string(char **addr, long *len_str, const char *src, int len)
+static void add_string(char **addr, int *len_str, const char *src, int len)
 {
   STRING_extend(addr, *len_str + len);
   memcpy(&((*addr)[*len_str]), src, len);
@@ -348,7 +348,7 @@ static void add_string(char **addr, long *len_str, const char *src, int len)
 static void input(STREAM *stream, char **addr, boolean line)
 {
   int len = 0;
-  long len_str = 0;
+  int len_str = 0;
   unsigned char c, lc = 0;
   void *test;
   bool eol;
@@ -439,7 +439,7 @@ PUBLIC void STREAM_input(STREAM *stream, char **addr)
 }
 
 
-PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len)
+PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, int len)
 {
   unsigned char data[4];
 
@@ -470,12 +470,12 @@ PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len)
       STREAM_read(stream, data, sizeof(short));
       if (stream->common.swap)
         SWAP_short((short *)data);
-      value->_integer.value = (long)*((short *)data);
+      value->_integer.value = (int)*((short *)data);
       break;
 
     case T_INTEGER:
 
-      STREAM_read(stream, &value->_integer.value, sizeof(long));
+      STREAM_read(stream, &value->_integer.value, sizeof(int));
       if (stream->common.swap)
         SWAP_long(&value->_integer.value);
       break;
@@ -491,7 +491,7 @@ PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len)
 
       STREAM_read(stream, data, sizeof(float));
       if (stream->common.swap)
-        SWAP_long((long *)data);
+        SWAP_long((int *)data);
       value->_float.value = (double)*((float *)data);
       break;
 
@@ -504,8 +504,8 @@ PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len)
 
     case T_DATE:
 
-      STREAM_read(stream, &value->_date.date, sizeof(long));
-      STREAM_read(stream, &value->_date.time, sizeof(long));
+      STREAM_read(stream, &value->_date.date, sizeof(int));
+      STREAM_read(stream, &value->_date.time, sizeof(int));
       if (stream->common.swap)
       {
         SWAP_long(&value->_date.date);
@@ -552,9 +552,9 @@ PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len)
               *data &= 0x3F;
 
               if (!EXEC_big_endian)
-                SWAP_long((long *)data);
+                SWAP_long((int *)data);
 
-              len = *((long *)data);
+              len = *((int *)data);
               break;
           }
         }
@@ -588,7 +588,7 @@ PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len)
 }
 
 
-PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len)
+PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, int len)
 {
   unsigned char data[8];
   bool write_zero;
@@ -627,10 +627,10 @@ PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len)
 
     case T_INTEGER:
 
-      *((long *)data) = value->_integer.value;
+      *((int *)data) = value->_integer.value;
       if (stream->common.swap)
-        SWAP_long((long *)data);
-      STREAM_write(stream, data, sizeof(long));
+        SWAP_long((int *)data);
+      STREAM_write(stream, data, sizeof(int));
       break;
 
     case T_LONG:
@@ -645,7 +645,7 @@ PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len)
 
       *((float *)data) = (float)value->_float.value;
       if (stream->common.swap)
-        SWAP_long((long *)data);
+        SWAP_long((int *)data);
       STREAM_write(stream, data, sizeof(float));
       break;
 
@@ -659,15 +659,15 @@ PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len)
 
     case T_DATE:
 
-      *((long *)data) = value->_date.date;
+      *((int *)data) = value->_date.date;
       if (stream->common.swap)
-        SWAP_long((long *)data);
-      STREAM_write(stream, data, sizeof(long));
+        SWAP_long((int *)data);
+      STREAM_write(stream, data, sizeof(int));
 
-      *((long *)data) = value->_date.time;
+      *((int *)data) = value->_date.time;
       if (stream->common.swap)
-        SWAP_long((long *)data);
-      STREAM_write(stream, data, sizeof(long));
+        SWAP_long((int *)data);
+      STREAM_write(stream, data, sizeof(int));
 
       break;
 
@@ -696,9 +696,9 @@ PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len)
           }
           else
           {
-            *((long *)data) = len | 0xC0000000;
+            *((int *)data) = len | 0xC0000000;
             if (!EXEC_big_endian)
-              SWAP_long((long *)data);
+              SWAP_long((int *)data);
             STREAM_write(stream, data, 4);
           }
         }
@@ -746,7 +746,7 @@ PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len)
 }
 
 
-PUBLIC void STREAM_load(const char *path, char **buffer, long *rlen)
+PUBLIC void STREAM_load(const char *path, char **buffer, int *rlen)
 {
   STREAM stream;
   long long len;
@@ -766,7 +766,7 @@ PUBLIC void STREAM_load(const char *path, char **buffer, long *rlen)
 }
 
 
-PUBLIC bool STREAM_map(const char *path, char **buffer, long *rlen)
+PUBLIC bool STREAM_map(const char *path, char **buffer, int *rlen)
 {
   STREAM stream;
 
@@ -839,7 +839,7 @@ __ERROR:
 PUBLIC void STREAM_lof(STREAM *stream, long long *len)
 {
   int fd;
-  long ilen;
+  int ilen;
 
   if (!stream->type)
     THROW(E_CLOSED);
@@ -858,7 +858,7 @@ PUBLIC void STREAM_lof(STREAM *stream, long long *len)
 PUBLIC bool STREAM_eof(STREAM *stream)
 {
   int fd;
-  long ilen;
+  int ilen;
 
   if (!stream->type)
     THROW(E_CLOSED);
@@ -874,7 +874,7 @@ PUBLIC bool STREAM_eof(STREAM *stream)
 }
 
 
-PUBLIC int STREAM_read_direct(int fd, char *buffer, long len)
+PUBLIC int STREAM_read_direct(int fd, char *buffer, int len)
 {
   ssize_t eff_read;
   ssize_t len_read;
@@ -904,7 +904,7 @@ PUBLIC int STREAM_read_direct(int fd, char *buffer, long len)
   return FALSE;
 }
 
-PUBLIC int STREAM_write_direct(int fd, char *buffer, long len)
+PUBLIC int STREAM_write_direct(int fd, char *buffer, int len)
 {
   ssize_t eff_write;
   ssize_t len_write;
