@@ -34,9 +34,9 @@ typedef
   struct STREAM_CLASS {
     int (*open)(union STREAM *stream, const char *path, int mode, void *data);
     int (*close)(union STREAM *stream);
-    int (*read)(union STREAM *stream, char *buffer, long len);
+    int (*read)(union STREAM *stream, char *buffer, int len);
     int (*getchar)(union STREAM *stream, char *buffer);
-    int (*write)(union STREAM *stream, char *buffer, long len);
+    int (*write)(union STREAM *stream, char *buffer, int len);
     int (*seek)(union STREAM *stream, long long pos, int whence);
     int (*tell)(union STREAM *stream, long long *pos);
     int (*flush)(union STREAM *stream);
@@ -62,7 +62,7 @@ typedef
 typedef
   struct {
     STREAM_COMMON common;
-    long _reserved[6];
+    int _reserved[6];
     }
   PACKED
   STREAM_RESERVED;
@@ -71,7 +71,7 @@ typedef
   struct {
     STREAM_COMMON common;
     long long size;
-    long fd;
+    int fd;
     unsigned is_term : 1;
     unsigned _reserved : 31;
     }
@@ -94,7 +94,7 @@ typedef
     STREAM_COMMON common;
     void *addr;
     //long size;
-    long pos;
+    int pos;
     }
   PACKED
   STREAM_MEMORY;
@@ -103,9 +103,9 @@ typedef
   struct {
     STREAM_COMMON common;
     ARCHIVE *arch;
-    long size;
-    long start;
-    long pos;
+    int size;
+    int start;
+    int pos;
     }
   PACKED
   STREAM_ARCH;
@@ -113,8 +113,8 @@ typedef
 typedef
   struct {
     STREAM_COMMON common;
-    long fdr;
-    long fdw;
+    int fdr;
+    int fdw;
     unsigned read_something : 1;
     unsigned _reserved : 31;
     }
@@ -156,7 +156,7 @@ enum {
 	ST_EOL_MAC = 2
 	};
 
-EXTERN long STREAM_eff_read;
+EXTERN int STREAM_eff_read;
 
 #ifndef __STREAM_IMPL_C
 
@@ -197,27 +197,27 @@ PUBLIC bool STREAM_in_archive(const char *path);
 PUBLIC void STREAM_open(STREAM *stream, const char *path, int mode);
 
 PUBLIC void STREAM_close(STREAM *stream);
-PUBLIC void STREAM_write(STREAM *stream, void *addr, long len);
+PUBLIC void STREAM_write(STREAM *stream, void *addr, int len);
 PUBLIC void STREAM_line_input(STREAM *stream, char **addr);
 PUBLIC void STREAM_input(STREAM *stream, char **addr);
 PUBLIC long long STREAM_tell(STREAM *stream);
 PUBLIC void STREAM_seek(STREAM *stream, long long pos, int whence);
-PUBLIC void STREAM_read(STREAM *stream, void *addr, long len);
+PUBLIC void STREAM_read(STREAM *stream, void *addr, int len);
 PUBLIC char STREAM_getchar(STREAM *stream);
-PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len);
-PUBLIC void STREAM_write(STREAM *stream, void *addr, long len);
-PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len);
+PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, int len);
+PUBLIC void STREAM_write(STREAM *stream, void *addr, int len);
+PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, int len);
 PUBLIC void STREAM_write_eol(STREAM *stream);
 PUBLIC void STREAM_flush(STREAM *stream);
 PUBLIC int STREAM_handle(STREAM *stream);
 PUBLIC void STREAM_lof(STREAM *stream, long long *len);
 PUBLIC bool STREAM_eof(STREAM *stream);
 
-PUBLIC void STREAM_load(const char *path, char **buffer, long *len);
-PUBLIC bool STREAM_map(const char *path, char **buffer, long *len);
+PUBLIC void STREAM_load(const char *path, char **buffer, int *len);
+PUBLIC bool STREAM_map(const char *path, char **buffer, int *len);
 
-PUBLIC int STREAM_read_direct(int fd, char *buffer, long len);
-PUBLIC int STREAM_write_direct(int fd, char *buffer, long len);
+PUBLIC int STREAM_read_direct(int fd, char *buffer, int len);
+PUBLIC int STREAM_write_direct(int fd, char *buffer, int len);
 //PUBLIC int STREAM_read_buffered(FILE *file, char *buffer, long len);
 //PUBLIC int STREAM_write_buffered(FILE *file, char *buffer, long len);
 
