@@ -78,12 +78,12 @@ PUBLIC boolean SUBR_check_string(VALUE *param);
 PUBLIC void SUBR_check_integer(VALUE *param);
 PUBLIC void SUBR_check_float(VALUE *param);
 
-PUBLIC long SUBR_get_integer(VALUE *param);
+PUBLIC int SUBR_get_integer(VALUE *param);
 PUBLIC double SUBR_get_float(VALUE *param);
 
 PUBLIC char *SUBR_get_string(VALUE *param);
 PUBLIC char *SUBR_copy_string(VALUE *param);
-PUBLIC void SUBR_get_string_len(VALUE *param, char **str, long *len);
+PUBLIC void SUBR_get_string_len(VALUE *param, char **str, int *len);
 
 /* subr_math.c */
 
@@ -105,7 +105,7 @@ PUBLIC void SUBR_randomize(void);
 PUBLIC void SUBR_rnd(void);
 PUBLIC void SUBR_round(void);
 
-PUBLIC void SUBR_add_quick(long value);
+PUBLIC void SUBR_add_quick(int value);
 
 /* subr_string.c */
 
@@ -217,4 +217,6 @@ PUBLIC void SUBR_eval(void);
 PUBLIC void SUBR_array(void);
 PUBLIC void SUBR_debug(void);
 
+PUBLIC void EVAL_string(char *expr);
+
 #endif
