@@ -44,6 +44,17 @@
 static EVAL_INTERFACE EVAL;
 static CCOLLECTION *eval_env;
 
+static void init_eval()
+{
+	static bool init = FALSE;
+	
+	if (init)
+		return;
+		
+	COMPONENT_load(COMPONENT_create("gb.eval"));
+	LIBRARY_get_interface_by_name("gb.eval", EVAL_INTERFACE_VERSION, &EVAL);
+	init = TRUE;
+}
 
 PUBLIC void SUBR_error(void)
 {
@@ -72,12 +83,13 @@ PUBLIC void SUBR_wait(void)
 
     for(;;)
     {
-      HOOK_DEFAULT(wait, WATCH_wait)((long)(wait * 1000 + 0.5));
+      HOOK_DEFAULT(wait, WATCH_wait)((int)(wait * 1000 + 0.5));
 
       if (DATE_timer(&time, FALSE))
         break;
 
-      if (time >= stop)
+			wait = stop - time;
+      if (wait <= 0.0)
         break;
     }
   }
@@ -95,8 +107,8 @@ PUBLIC void SUBR_sleep(void)
 
   wait = SUBR_get_float(PARAM);
 
-	rem.tv_sec = (time_t)(long)wait;
-	rem.tv_nsec = (long)(frac(wait) * 1E9);
+	rem.tv_sec = (time_t)(int)wait;
+	rem.tv_nsec = (int)(frac(wait) * 1E9);
 
 	while (nanosleep(&rem, &rem) < 0);
 
@@ -126,6 +138,9 @@ PUBLIC void SUBR_exec(void)
     cmd = (void *)(PARAM->_object.object);
   }
 
+	if (!cmd)
+		THROW(E_ARG);
+
   VALUE_conv(&PARAM[1], T_BOOLEAN);
   wait = PARAM[1]._boolean.value;
 
@@ -189,7 +204,7 @@ PUBLIC void SUBR_exec(void)
 }
 
 
-static bool get_value(const char *sym, long len, GB_VARIANT *value)
+static bool get_value(const char *sym, int len, GB_VARIANT *value)
 {
   if (eval_env)
     if (!GB_CollectionGet(eval_env, sym, len, value))
@@ -199,22 +214,47 @@ static bool get_value(const char *sym, long len, GB_VARIANT *value)
   return TRUE;
 }
 
+PUBLIC void EVAL_string(char *expr)
+{
+	int len;
+  EXPRESSION *eval;
+
+	init_eval();
+	len = strlen(expr);
+	eval_env = NULL;
+	
+  EVAL.New((void **)(void *)&eval, expr, len);
+
+  if (EVAL.Compile(eval))
+    goto _ERROR;
+
+  if (!EVAL.Run(eval, get_value))
+    goto _ERROR;
+
+  goto _FREE;
+
+_ERROR:
+  EVAL.Free((void **)(void *)&eval);
+  PROPAGATE();
+
+_FREE:
+  EVAL.Free((void **)(void *)&eval);
+
+  VALUE_to_string(RETURN, &expr, &len);
+	STREAM_write(CSTREAM_stream(&CFILE_out), expr, len);
+	STREAM_write_eol(CSTREAM_stream(&CFILE_out));
+	STREAM_flush(CSTREAM_stream(&CFILE_out));
+}
+
 PUBLIC void SUBR_eval(void)
 {
-  static bool init = FALSE;
   char *expr;
-  long len;
+  int len;
   EXPRESSION *eval;
 
   SUBR_ENTER();
 
-  if (!init)
-  {
-    COMPONENT_load(COMPONENT_create("gb.eval"));
-    LIBRARY_get_interface_by_name("gb.eval", EVAL_INTERFACE_VERSION, &EVAL);
-    init = TRUE;
-  }
-
+	init_eval();
   SUBR_get_string_len(PARAM, &expr, &len);
 
   if (NPARAM == 2)
