@@ -42,7 +42,7 @@
 PUBLIC void SUBR_cat(void)
 {
   int i;
-  long len, len_cat;
+  int len, len_cat;
   char *str, *ptr;
 
   SUBR_ENTER();
@@ -85,9 +85,9 @@ PUBLIC void SUBR_cat(void)
 PUBLIC void SUBR_file(void)
 {
   int i;
-  long length;
+  int length;
   char *addr;
-  long len;
+  int len;
   char *str, *ptr;
   boolean slash;
 
@@ -143,7 +143,7 @@ PUBLIC void SUBR_file(void)
 
 PUBLIC void SUBR_left(void)
 {
-  long val;
+  int val;
 
   SUBR_ENTER();
 
@@ -173,8 +173,8 @@ _FIN:
 
 PUBLIC void SUBR_right(void)
 {
-  long val;
-  long new_len;
+  int val;
+  int new_len;
 
   SUBR_ENTER();
 
@@ -208,8 +208,8 @@ _FIN:
 
 PUBLIC void SUBR_mid(void)
 {
-  long start;
-  long len;
+  int start;
+  int len;
 
   SUBR_ENTER();
 
@@ -263,7 +263,7 @@ FIN:
 
 PUBLIC void SUBR_len(void)
 {
-  long len;
+  int len;
 
   SUBR_GET_PARAM(1);
 
@@ -311,7 +311,7 @@ PUBLIC void SUBR_string(void)
   int i;
   char *d;
   char *s;
-  long ld, ls;
+  int ld, ls;
 
   SUBR_ENTER_PARAM(2);
 
@@ -478,9 +478,9 @@ PUBLIC void SUBR_asc(void)
 PUBLIC void SUBR_instr(void)
 {
   boolean right, nocase = FALSE;
-  long is, pos;
+  int is, pos;
   char *ps, *pp;
-  long ls, lp;
+  int ls, lp;
 
   SUBR_ENTER();
 
@@ -527,7 +527,7 @@ PUBLIC void SUBR_like(void)
 {
   char *pattern;
   char *string;
-  long len_pattern, len_string;
+  int len_pattern, len_string;
   boolean ret;
 
   SUBR_ENTER_PARAM(2);
@@ -547,7 +547,7 @@ PUBLIC void SUBR_like(void)
 static int subst_nparam;
 static VALUE *subst_param;
 
-static void get_subst(int np, char **str, long *len)
+static void get_subst(int np, char **str, int *len)
 {
   if (np > 0 && np < subst_nparam)
     VALUE_get_string(&subst_param[np], str, len);
@@ -562,7 +562,7 @@ static void get_subst(int np, char **str, long *len)
 PUBLIC void SUBR_subst(void)
 {
   char *string;
-  long len;
+  int len;
   int np;
 
   SUBR_ENTER();
@@ -595,8 +595,8 @@ PUBLIC void SUBR_replace(void)
   char *ps;
   char *pp;
   char *pr;
-  long ls, lp, lr;
-  long is, pos;
+  int ls, lp, lr;
+  int is, pos;
   bool nocase = FALSE;
 
   SUBR_ENTER();
@@ -657,7 +657,7 @@ PUBLIC void SUBR_split(void)
 {
   CARRAY *array;
   char *str;
-  long lstr;
+  int lstr;
   char *sep = "";
   char *esc = "";
   bool no_void = FALSE;
@@ -704,9 +704,9 @@ PUBLIC void SUBR_scan(void)
 {
   CARRAY *array;
   char *str;
-  long len_str;
+  int len_str;
   char *pat;
-  long len_pat;
+  int len_pat;
 
   SUBR_ENTER_PARAM(2);
 
@@ -731,7 +731,7 @@ PUBLIC void SUBR_iconv(void)
   const char *src;
   const char *dst;
   char *result;
-  long len;
+  int len;
 
   SUBR_ENTER_PARAM(3);
 
@@ -763,7 +763,7 @@ PUBLIC void SUBR_sconv(void)
   const char *src;
   const char *dst;
   char *result;
-  long len;
+  int len;
 
   SUBR_ENTER_PARAM(1);
 
@@ -853,7 +853,7 @@ PUBLIC void SUBR_is_chr(void)
   };
 
   char *addr;
-  long len;
+  int len;
   int i;
   int (*func)(int);
 
