@@ -49,7 +49,7 @@ PUBLIC void SUBR_bit(void)
 
   long long val;
   unsigned long long uval;
-  long bit;
+  int bit;
   TYPE type;
   int n;
 
@@ -104,7 +104,7 @@ __ASL:
 	if (type == T_LONG)
 		val <<= bit;
 	else
-		val = ((long)val << bit);
+		val = ((int)val << bit);
   goto __END;
 
 __ASR:
@@ -112,7 +112,7 @@ __ASR:
 	if (type == T_LONG)
 		val >>= bit;
 	else
-		val = ((long)val >> bit);
+		val = ((int)val >> bit);
 	goto __END;
 
 __ROL:
@@ -135,9 +135,9 @@ __LSL:
 	}
 	else
 	{
-		uval = (unsigned long)val;
+		uval = (unsigned int)val;
 		uval <<= bit;
-		val = (long)uval;
+		val = (int)uval;
 	}
 	goto __END;
 
@@ -151,9 +151,9 @@ __LSR:
 	}
 	else
 	{
-		uval = (unsigned long)val;
+		uval = (unsigned int)val;
 		uval >>= bit;
-		val = (long)uval;
+		val = (int)uval;
 	}
 	goto __END;
 
@@ -518,7 +518,7 @@ PUBLIC void SUBR_strcomp(void)
 {
   int mode = GB_COMP_BINARY;
   char *s1, *s2;
-  long l1, l2;
+  int l1, l2;
   int ret;
 
   SUBR_ENTER();
