@@ -26,13 +26,12 @@
 #define __GBX_SUBST_H
 
 typedef
-  void (*SUBST_FUNC)(int, char **, long *);
+  void (*SUBST_FUNC)(int, char **, int *);
 
 PUBLIC void SUBST_init(void);
-PUBLIC void SUBST_add(const char *src, long len);
+PUBLIC void SUBST_add(const char *src, int len);
 PUBLIC void SUBST_add_char(unsigned char c);
 PUBLIC void SUBST_exit(void);
 PUBLIC char *SUBST_buffer(void);
 
-
 #endif
