@@ -35,28 +35,28 @@
 typedef
   struct {
     TYPE type;
-    long value;
+    int value;
     }
   VALUE_BOOLEAN;
 
 typedef
   struct {
     TYPE type;
-    long value;
+    int value;
     }
   VALUE_BYTE;
 
 typedef
   struct {
     TYPE type;
-    long value;
+    int value;
     }
   VALUE_SHORT;
 
 typedef
   struct {
     TYPE type;
-    long value;
+    int value;
     }
   VALUE_INTEGER;
 
@@ -87,8 +87,8 @@ typedef
 typedef
   struct {
     TYPE type;
-    long date;  /* number of days */
-    long time;  /* number of milliseconds */
+    int date;  /* number of days */
+    int time;  /* number of milliseconds */
     }
   VALUE_DATE;
 
@@ -96,8 +96,8 @@ typedef
   struct {
     TYPE type;
     char *addr;
-    long start;
-    long len;
+    int start;
+    int len;
     }
   VALUE_STRING;
 
@@ -130,7 +130,7 @@ typedef
   struct {
     TYPE type;
     TYPE ptype;
-    long value[2];
+    int value[2];
     }
   VALUE_VOID;
 
@@ -217,8 +217,8 @@ PUBLIC void VALUE_put_forget(void);
 */
 
 PUBLIC void VALUE_free(void *addr, TYPE type);
-PUBLIC void VALUE_to_string(VALUE *value, char **addr, long *len);
-PUBLIC void VALUE_from_string(VALUE *value, const char *addr, long len);
+PUBLIC void VALUE_to_string(VALUE *value, char **addr, int *len);
+PUBLIC void VALUE_from_string(VALUE *value, const char *addr, int len);
 
 PUBLIC void VALUE_class_read(CLASS *class, VALUE *value, char *addr, CTYPE ctype);
 PUBLIC void VALUE_class_write(CLASS *class, VALUE *value, char *addr, CTYPE ctype);
@@ -227,7 +227,7 @@ PUBLIC void VALUE_class_constant(CLASS *class, VALUE *value, int ind);
 
 PUBLIC bool VALUE_is_null(VALUE *val);
 
-PUBLIC void VALUE_get_string(VALUE *val, char **text, long *length);
+PUBLIC void VALUE_get_string(VALUE *val, char **text, int *length);
 
 #define VALUE_conv(_value, _type) \
 { \
