@@ -41,7 +41,8 @@ static GB_SUBCOLLECTION_DESC _fields_desc =
   ".ResultFields",
   (void *)CRESULTFIELD_get,
   (void *)CRESULTFIELD_exist,
-  (void *)NULL
+  (void *)NULL,
+  (void *)CRESULTFIELD_release
 };
 
 
@@ -154,7 +155,7 @@ static bool load_buffer(CRESULT *_object, int vpos)
 
       void_buffer(THIS);
 
-      THIS->driver->Result.Fill(THIS->handle, pos, THIS->buffer,
+      THIS->driver->Result.Fill(&THIS->conn->db, THIS->handle, pos, THIS->buffer,
         (pos > 0) && (pos == (DELETE_MAP_virtual_to_real(THIS->dmap, THIS->pos) + 1)));
 
       if (THIS->mode == RESULT_EDIT)
@@ -371,6 +372,7 @@ BEGIN_METHOD_VOID(CRESULT_free)
   DELETE_MAP_free(&THIS->dmap);
 
   GB.Unref(POINTER(&THIS->conn));
+  GB.Unref(POINTER(&THIS->fields));
 
 END_METHOD
 
@@ -588,7 +590,6 @@ BEGIN_METHOD_VOID(CRESULT_next)
 
 END_METHOD
 
-
 BEGIN_METHOD_VOID(CRESULT_update)
 
   int i;
@@ -609,34 +610,46 @@ BEGIN_METHOD_VOID(CRESULT_update)
       q_add(info->table);
       q_add(THIS->driver->GetQuote());
       q_add(" ( ");
-
-      comma = FALSE;
-      for (i = 0; i < info->nfield; i++)
-      {
-        if (THIS->buffer[i].type == GB_T_NULL)
-          continue;
+			
+			comma = FALSE;
+			for (i = 0; i < info->nfield; i++)
+			{
+				if (THIS->buffer[i].type == GB_T_NULL)
+					continue;
 				if (!BARRAY_test(THIS->changed, i))
 					continue;
-        if (comma) q_add(", ");
-	      q_add(THIS->driver->GetQuote());
-        q_add(info->field[i].name);
-  	    q_add(THIS->driver->GetQuote());
-        comma = TRUE;
-      }
+				if (comma) q_add(", ");
+				q_add(THIS->driver->GetQuote());
+				q_add(info->field[i].name);
+				q_add(THIS->driver->GetQuote());
+				comma = TRUE;
+			}
+			
+			if (!comma)
+			{
+				q_add(THIS->driver->GetQuote());
+				q_add(info->field[0].name);
+				q_add(THIS->driver->GetQuote());
+			}
 
       q_add(" ) VALUES ( ");
 
-      comma = FALSE;
-      for (i = 0; i < info->nfield; i++)
-      {
-        if (THIS->buffer[i].type == GB_T_NULL)
-          continue;
+			comma = FALSE;
+			for (i = 0; i < info->nfield; i++)
+			{
+				if (THIS->buffer[i].type == GB_T_NULL)
+					continue;
 				if (!BARRAY_test(THIS->changed, i))
 					continue;
-        if (comma) q_add(", ");
-        DB_FormatVariant(THIS->driver, &THIS->buffer[i], q_add_length);
-        comma = TRUE;
-      }
+				if (comma) q_add(", ");
+				DB_FormatVariant(THIS->driver, &THIS->buffer[i], q_add_length);
+				comma = TRUE;
+			}
+			
+			if (!comma)
+			{
+				DB_FormatVariant(THIS->driver, &THIS->buffer[0], q_add_length);
+			}
 
       q_add(" )");
 
@@ -680,6 +693,8 @@ BEGIN_METHOD_VOID(CRESULT_update)
       break;
   }
 
+	BARRAY_clear_all(THIS->changed, THIS->info.nfield);
+
 END_METHOD
 
 
