@@ -38,7 +38,8 @@ static GB_SUBCOLLECTION_DESC _fields_desc =
   ".TableFields",
   (void *)CFIELD_get,
   (void *)CFIELD_exist,
-  (void *)CFIELD_list
+  (void *)CFIELD_list,
+  (void *)CFIELD_release
 };
 
 static GB_SUBCOLLECTION_DESC _indexes_desc =
@@ -46,13 +47,14 @@ static GB_SUBCOLLECTION_DESC _indexes_desc =
   ".TableIndexes",
   (void *)CINDEX_get,
   (void *)CINDEX_exist,
-  (void *)CINDEX_list
+  (void *)CINDEX_list,
+  (void *)CINDEX_release
 };
 
 
 static int valid_table(CTABLE *_object)
 {
-  return (THIS->conn->db.handle == NULL);
+  return !THIS->conn || !THIS->conn->db.handle;
 }
 
 
@@ -88,12 +90,16 @@ static CTABLE *make_table(CCONNECTION *conn, const char *name, bool must_exist)
   if (check_table(conn, (char *)name, must_exist))
     return NULL;
 
+	//fprintf(stderr, "make_table: '%s'\n", name);
+  
   GB.New(POINTER(&_object), GB.FindClass("Table"), NULL, NULL);
   THIS->conn = conn;
-  GB.Ref(conn);
+  //GB.Ref(conn);
   THIS->driver = conn->driver;
   GB.NewString(&THIS->name, name, 0);
 
+	//fprintf(stderr, "make_table: -> %p '%s'\n", THIS, THIS->name);
+  
   return _object;
 }
 
@@ -115,6 +121,11 @@ void CTABLE_list(CCONNECTION *conn, char ***list)
   conn->driver->Table.List(&conn->db, list);
 }
 
+void CTABLE_release(CCONNECTION *conn, void *_object)
+{
+	THIS->conn = NULL;
+}
+
 
 /***************************************************************************
 
@@ -300,14 +311,19 @@ END_METHOD
 
 BEGIN_METHOD_VOID(CTABLE_free)
 
+	//fprintf(stderr, "CTABLE_free: %p '%s'\n", THIS, THIS->name);
+
   if (!valid_table(THIS))
     GB.SubCollection.Remove(THIS->conn->tables, THIS->name, 0);
-  GB.Unref(POINTER(&THIS->conn));
+  //GB.Unref(POINTER(&THIS->conn));
 
   GB.FreeString(&THIS->name);
   GB.FreeString(&THIS->type);
   DB_FreeStringArray(&THIS->primary);
 
+  GB.Unref(POINTER(&THIS->fields));
+  GB.Unref(POINTER(&THIS->indexes));
+  
   free_new_fields(THIS);
 
 END_METHOD
@@ -385,7 +401,6 @@ BEGIN_METHOD(CTABLE_add, GB_STRING name; GB_STRING type)
     return;
 
   GB.SubCollection.Add(THIS, STRING(name), LENGTH(name), table);
-  //GB.Ref(table);
 
   if (!MISSING(type))
     GB.StoreString(ARG(type), &table->type);
@@ -409,7 +424,6 @@ BEGIN_METHOD(CTABLE_remove, GB_STRING name)
 
 END_METHOD
 
-
 GB_DESC CConnectionTablesDesc[] =
 {
   GB_DECLARE(".ConnectionTables", 0), GB_INHERITS(".SubCollection"),
