@@ -38,7 +38,7 @@ extern GB_INTERFACE GB;
 #define MAX_DRIVER 8
 
 bool DB_Open(DB_DESC *desc, DB_DRIVER **driver, DB_DATABASE *db);
-char *DB_MakeQuery(DB_DRIVER *driver, char *pattern, long len, int narg, GB_VALUE *arg);
+char *DB_MakeQuery(DB_DRIVER *driver, char *pattern, int len, int narg, GB_VALUE *arg);
 void DB_Format(DB_DRIVER *driver, GB_VALUE *arg, DB_FORMAT_CALLBACK func);
 void DB_FormatVariant(DB_DRIVER *driver, GB_VARIANT_VALUE *arg, DB_FORMAT_CALLBACK func);
 
@@ -54,10 +54,9 @@ void DB_TryAnother(char *);
 
 void q_init(void);
 void q_add(char *str);
-void q_add_length(char *str, long len);
+void q_add_length(char *str, int len);
 char *q_get(void);
 char *q_steal(void);
-long q_length(void);
-
+int q_length(void);
 
 #endif /* __MAIN_H */
