@@ -74,6 +74,7 @@ static bool _fifo;
 PUBLIC void DEBUG_break_on_next_line(void)
 {
   DEBUG_info.stop = TRUE;
+  DEBUG_info.leave = FALSE;
   DEBUG_info.fp = NULL;
   DEBUG_info.bp = NULL;
 }
@@ -200,6 +201,8 @@ PUBLIC DEBUG_INFO *DEBUG_init(GB_DEBUG_INTERFACE *debug, bool fifo)
   signal(SIGUSR1, signal_user);
   signal(SIGPIPE, SIG_IGN);
 
+  setlinebuf(_out);
+
   return &DEBUG_info;
 }
 
@@ -379,6 +382,7 @@ static void command_go(const char *cmd)
   GB.Signal(GB_SIGNAL_DEBUG_CONTINUE, 0);
 
   DEBUG_info.stop = FALSE;
+  DEBUG_info.leave = FALSE;
   DEBUG_info.fp = NULL;
   DEBUG_info.bp = NULL;
 }
@@ -393,6 +397,7 @@ static void command_next(const char *cmd)
 {
   GB.Signal(GB_SIGNAL_DEBUG_FORWARD, 0);
   DEBUG_info.stop = TRUE;
+  DEBUG_info.leave = FALSE;
   DEBUG_info.fp = FP;
   DEBUG_info.bp = BP;
 }
@@ -401,15 +406,22 @@ static void command_from(const char *cmd)
 {
   STACK_CONTEXT *sc = GB_DEBUG.GetStack(0); //STACK_get_current();
 
-  if (sc)
+  if (sc && sc->pc)
   {
 	  GB.Signal(GB_SIGNAL_DEBUG_FORWARD, 0);
     DEBUG_info.stop = TRUE;
+    DEBUG_info.leave = FALSE;
     DEBUG_info.fp = sc->fp;
     DEBUG_info.bp = sc->bp;
   }
   else
-    command_go(cmd);
+  {
+	  GB.Signal(GB_SIGNAL_DEBUG_FORWARD, 0);
+    DEBUG_info.stop = TRUE;
+    DEBUG_info.leave = TRUE;
+    DEBUG_info.fp = FP;
+    DEBUG_info.bp = BP;
+  }
 }
 
 
@@ -417,13 +429,13 @@ static void command_set_breakpoint(const char *cmd)
 {
   char class_name[64];
   ushort line;
-  CLASS *class;
+  //CLASS *class;
 
   if (sscanf(cmd, "+%64[^.].%hu", class_name, &line) != 2)
     WARNING("Cannot set breakpoint: syntax error");
   else
   {
-    class = (CLASS *)GB.FindClass(class_name);
+    //class = (CLASS *)GB.FindClassLocal(class_name);
     //CLASS_load_without_init(class);
 		//fprintf(stderr, "command_set_breakpoint: %s %s\n", class->name, class->component ? class->component->name : "?");
     set_breakpoint(GB.FindClass(class_name), line);
@@ -666,6 +678,8 @@ static void command_eval(const char *cmd)
 
 __ERROR:
   //fprintf(_out, "%s\n", ERROR_info.msg);
+  if (*cmd != '!')
+  	fprintf(out, "!");
   GB_DEBUG.PrintError(out, TRUE, FALSE);
 
 __FREE:
@@ -785,7 +799,7 @@ PUBLIC void DEBUG_main(boolean error)
 
     if (!_fifo)
     {
-      fprintf(_out, "(gbx) ");
+      fprintf(_out, "> ");
       fflush(NULL);
     }
 
