@@ -33,7 +33,8 @@
 
 typedef
   struct {
-    boolean stop;
+    unsigned stop : 1;
+    unsigned leave : 1;
     FUNCTION *fp;
     VALUE *bp;
     void *op;
@@ -50,19 +51,20 @@ typedef
 		void (*PrintError)(FILE *where, bool msgonly, bool newline);
 		void (*SaveError)(void *);
 		void (*RestoreError)(void *);
-		void (*ToString)(GB_VALUE *value, char **addr, long *len);
-		int (*FormatDate)(GB_DATE_SERIAL *date, int fmt_type, const char *fmt, long len_fmt, char **str, long *len_str);
-		int (*FormatNumber)(double number, int fmt_type, const char *fmt, long len_fmt, char **str, long *len_str, bool local);
-		bool (*GetValue)(const char *sym, long len, GB_VARIANT *ret);
+		void (*ToString)(GB_VALUE *value, char **addr, int *len);
+		int (*FormatDate)(GB_DATE_SERIAL *date, int fmt_type, const char *fmt, int len_fmt, char **str, int *len_str);
+		int (*FormatNumber)(double number, int fmt_type, const char *fmt, int len_fmt, char **str, int *len_str, bool local);
+		bool (*GetValue)(const char *sym, int len, GB_VARIANT *ret);
 		void (*GetArrayValue)(GB_ARRAY array, int index, GB_VALUE *value);
-		int (*EnumCollection)(GB_COLLECTION col, GB_VARIANT *value, char **key, long *len);
+		int (*EnumCollection)(GB_COLLECTION col, GB_VARIANT *value, char **key, int *len);
 		void *(*GetNextSortedSymbol)(void *klass, int *index);
+		int (*CanBeUsedLikeAnArray)(void *object, CLASS *klass);
 		}
 	GB_DEBUG_INTERFACE;
 
 typedef
 	struct {
-		long version;
+		int version;
 		DEBUG_INFO *(*Init)(GB_DEBUG_INTERFACE *debug, int fifo);
 		void (*Exit)(void);
 		void (*Welcome)(void);
