@@ -49,7 +49,7 @@ static int _level;
 
 static void print_value(VALUE *value);
 
-static void print_string(const char *s, long len)
+static void print_string(const char *s, int len)
 {
 	int i;
 	uchar c;
@@ -166,7 +166,7 @@ static void print_value(VALUE *value)
 
   VALUE conv;
   char *addr;
-  long len;
+  int len;
 
   //*more = FALSE;
 
@@ -199,7 +199,7 @@ __BYTE:
 __SHORT:
 __INTEGER:
 
-  fprintf(_where, "%ld", value->_integer.value);
+  fprintf(_where, "%d", value->_integer.value);
   goto __RETURN;
 
 __LONG:
@@ -254,7 +254,7 @@ __CLASS:
     CLASS *class = value->_class.class;
     //*more = (!CLASS_is_native(class) && class->load->n_stat > 0);
 
-    fprintf(_where, "CLASS %s", class->name);
+    fprintf(_where, "(%s)", class->name);
     goto __RETURN;
   }
 
@@ -276,7 +276,7 @@ __RETURN:
 PUBLIC void PRINT_value(FILE *where, VALUE *value, bool format)
 {
   char *pval;
-  long lpval;
+  int lpval;
 
   if (format)
   {
@@ -312,11 +312,12 @@ PUBLIC void PRINT_object(FILE *where, VALUE *value)
 	VALUE conv;
 	void *object;
 	int i, index;
-	long count;
-	CLASS *class;
+	int count;
+	CLASS *class, *real_class;
 	CLASS_DESC_SYMBOL *cd;
 	char *key;
-	long len;
+	int len;
+	bool static_class;
 	
 	_where = where;
 	
@@ -327,28 +328,57 @@ PUBLIC void PRINT_object(FILE *where, VALUE *value)
   	GB.Conv((GB_VALUE *)value, (GB_TYPE)value->_variant.vtype);
 	}
 	
-	if (value->type < T_OBJECT)
+	if (value->type < T_OBJECT && value->type != T_CLASS)
 	{
 		//fprintf(_where, "\n");
 		return;
 	}
 		
-	object = value->_object.object;
+	real_class = NULL;
+	
+	if (value->type == T_CLASS)
+	{
+		object = value->_class.class;
+		class = (CLASS *)object;
+		static_class = TRUE;
+	}
+	else
+	{
+		object = value->_object.object;
+		real_class = (CLASS *)GB.GetClass(object);
+		if (value->type == T_OBJECT)
+			class = real_class;
+		else
+			class = value->_object.class;
+			
+		if (real_class == class)
+			real_class = NULL;
+			
+		/*if (value->type != T_OBJECT && value->_object.class->is_virtual)
+		{
+		}
+		else
+			class = (CLASS *)GB.GetClass(object);*/
+		//fprintf(stderr, "PRINT_object: %s %s\n", class->name, value->_object.class->name);
+		static_class = FALSE;
+	}
+		
 	if (!object)
 		return;
 		
-	fprintf(_where, "%s", OBJECT_class(object)->name);
+	fprintf(_where, "%s ", class->name);
+	if (real_class) fprintf(_where, "%s", real_class->name);
 	
 	if (GB.Is(object, GB.FindClass("Array")))
 	{
-		fprintf(_where, " %ld", GB.Array.Count(object));
+		fprintf(_where, " %d", GB.Array.Count(object));
 		return;
 	}
 	
 	if (GB.Is(object, GB.FindClass("Collection")))
 	{
 		count = GB.Collection.Count(object);
-		fprintf(_where, " %ld", count);
+		fprintf(_where, " %d", count);
 		
 		GB_DEBUG.EnumCollection(object, NULL, NULL, NULL);
 		
@@ -364,17 +394,23 @@ PUBLIC void PRINT_object(FILE *where, VALUE *value)
 		return;
 	}
 	
-	class = (CLASS *)GB.GetClass(object);
+	if (!class->is_virtual && real_class)
+		class = real_class;
 	
-	index = 0;
+	count = GB_DEBUG.CanBeUsedLikeAnArray(object, class);
 	
 	fprintf(_where, " S:");
 	
+	index = 0;
+	
 	for(;;)
 	{
 		cd = (CLASS_DESC_SYMBOL *)GB_DEBUG.GetNextSortedSymbol(class, &index);
 		if (!cd)
 			break;
+		key = cd->name;
+		if (cd->len == 0 || (cd->len == 1 && key[0] == '.'))
+			continue;
 	
 	  switch(CLASS_DESC_get_type(cd->desc))
 		{
@@ -382,31 +418,40 @@ PUBLIC void PRINT_object(FILE *where, VALUE *value)
 			case CD_STATIC_PROPERTY:
 			case CD_STATIC_PROPERTY_READ:
 			case CD_CONSTANT:
-				fprintf(_where, " %.*s", cd->len, cd->name);
+				fprintf(_where, " %.*s", cd->len, key);
 				break;
 		}
 	}
 	
-	index = 0;
+	if (count > 0 && static_class)
+		fprintf(_where, " [%d]", count);
 	
 	fprintf(_where, " D:");
 	
+	index = 0;
+	
 	for(;;)
 	{
 		cd = (CLASS_DESC_SYMBOL *)GB_DEBUG.GetNextSortedSymbol(class, &index);
 		if (!cd)
 			break;
+		key = cd->name;
+		if (cd->len == 0 || (cd->len == 1 && key[0] == '.'))
+			continue;
 	
 	  switch(CLASS_DESC_get_type(cd->desc))
 		{
 			case CD_VARIABLE:
 			case CD_PROPERTY:
 			case CD_PROPERTY_READ:
-				fprintf(_where, " %.*s", cd->len, cd->name);
+				fprintf(_where, " %.*s", cd->len, key);
 				break;
 		}
 	}
 	
+	if (count > 0 && !static_class)
+		fprintf(_where, " [%d]", count);
+	
 	//fprintf(_where, "\n");
 }
 
