@@ -34,7 +34,7 @@
 #include "eval_read.h"
 
 
-//#define DEBUG
+//#define DEBUG 1
 
 PUBLIC const char *READ_source_ptr;
 #define source_ptr READ_source_ptr
@@ -96,7 +96,7 @@ static char _buffer[BUF_MAX + 1];
 PUBLIC char *READ_get_pattern(PATTERN *pattern)
 {
   int type = PATTERN_type(*pattern);
-  long index = PATTERN_index(*pattern);
+  int index = PATTERN_index(*pattern);
   const char *str;
   const char *before = _no_quote ? "" : "'";
   const char *after = _no_quote ? "" : "'";
@@ -144,10 +144,10 @@ PUBLIC char *READ_get_pattern(PATTERN *pattern)
 PUBLIC void READ_dump_pattern(PATTERN *pattern)
 {
   int type = PATTERN_type(*pattern);
-  long index = PATTERN_index(*pattern);
-  long pos;
+  int index = PATTERN_index(*pattern);
+  int pos;
 
-  pos = (long)(pattern - EVAL->pattern);
+  pos = (int)(pattern - EVAL->pattern);
   if (pos < 0 || pos >= EVAL->pattern_count)
     return;
   
@@ -210,7 +210,7 @@ static unsigned char next_char(void)
 
 #ifdef DEBUG
 
-static void add_pattern(int type, long index)
+static void add_pattern(int type, int index)
 {
   EVAL->pattern[EVAL->pattern_count++] = PATTERN_make(type, index);
   READ_dump_pattern(&EVAL->pattern[EVAL->pattern_count - 1]);
@@ -304,7 +304,7 @@ static void add_number()
 {
   unsigned char car;
   const char *start;
-  long index;
+  int index;
   char sign;
   PATTERN last_pattern;
 
@@ -415,7 +415,7 @@ static void add_identifier(bool no_res)
   unsigned char car;
   const char *start;
   int len;
-  long index;
+  int index;
   int type;
   int flag;
   PATTERN last_pattern;  
@@ -546,7 +546,14 @@ static void add_identifier(bool no_res)
 
 IDENTIFIER:
 
-  TABLE_add_symbol(EVAL->table, start, len, NULL, &index);
+	if (!EVAL->analyze && PATTERN_is(last_pattern, RS_EXCL))
+	{
+  	TABLE_add_symbol(EVAL->string, start, len, NULL, &index);
+  	type = RT_STRING;
+  }
+	else
+	  TABLE_add_symbol(EVAL->table, start, len, NULL, &index);
+  
   add_pattern(type, index);
 }
 
@@ -557,8 +564,8 @@ static void add_operator()
   const char *start;
   const char *end;
   int len;
-  long op = NO_SYMBOL;
-  long index;
+  int op = NO_SYMBOL;
+  int index;
 
   start = source_ptr;
   end = start;
@@ -595,22 +602,23 @@ static void add_operator()
 
 static int xdigit_val(unsigned char c)
 {
-  c = tolower(c);
-
   if (c >= '0' && c <= '9')
     return (c - '0');
   else if (c >= 'a' && c <= 'f')
     return (c - 'a' + 10);
+   else if (c >= 'A' && c <= 'F')
+   	return (c - 'A' + 10);
   else
     return (-1);
 }
 
+
 static void add_string()
 {
   unsigned char car;
   const char *start;
   int len;
-  long index;
+  int index;
   int newline;
   bool jump;
   char *p;
@@ -622,10 +630,14 @@ static void add_string()
   jump = FALSE;
   p = (char *)source_ptr;
 
+  //fprintf(stderr, "EVAL_read: add_string: [%d] %s\n", source_ptr - EVAL->source, source_ptr);
+
   for(;;)
   {
     source_ptr++;
     car = get_char();
+    
+    //fprintf(stderr, "EVAL_read: car = %d jump = %d\n", car, jump);
 
     if (jump)
     {
@@ -692,12 +704,16 @@ static void add_string()
 
   p[1] = 0;
 
+  //fprintf(stderr, "EVAL_read: add_string (end): [%d] %s\n", source_ptr - EVAL->source, source_ptr);
+
   TABLE_add_symbol(EVAL->string, start + 1, len, NULL, &index);
 
   add_pattern(RT_STRING, index);
 
   for (i = 0; i < newline; i++)
     add_newline();
+    
+  source_ptr -= newline;
 }
 
 
@@ -706,7 +722,7 @@ static void add_comment()
   unsigned char car;
   const char *start;
   int len;
-  long index;
+  int index;
   int type;
   bool space = FALSE;
 
@@ -759,7 +775,7 @@ static void add_string_for_analyze()
   unsigned char car;
   const char *start;
   int len;
-  long index;
+  int index;
   int type;
 
   start = source_ptr;
@@ -814,7 +830,6 @@ PUBLIC void EVAL_read(void)
   source_ptr = EVAL->source;
   begin_line = TRUE;
 
-  //while (source_ptr < source_length)
   for(;;)
   {
     car = get_char();
