@@ -53,7 +53,7 @@ static void arch_error(const char *msg)
     THROW(E_ARCH, name, msg);
 }
 
-static void read_at(ARCH *arch, long pos, void *buf, long len)
+static void read_at(ARCH *arch, int pos, void *buf, int len)
 {
   memcpy(buf, &arch->addr[pos], len);
 }
@@ -61,9 +61,9 @@ static void read_at(ARCH *arch, long pos, void *buf, long len)
 
 static void load_arch(ARCH *arch, const char *path)
 {
-  long len;
+  int len;
   int i;
-  long pos;
+  int pos;
   struct stat info;
 
 	_path = path;
@@ -90,7 +90,7 @@ static void load_arch(ARCH *arch, const char *path)
   if (_swap)
   {
   	fprintf(stderr, "gbx: Swapping archive\n");
-  	SWAP_longs((long *)&arch->header, 6);
+  	SWAP_longs((int *)&arch->header, 6);
 	}
 
   //if (arch->header.version != ARCH_VERSION)
@@ -224,7 +224,7 @@ PUBLIC void ARCH_get_absolute_path(const char *path, int len_path, char *abs_pat
 
 PUBLIC bool ARCH_find(ARCH *arch, const char *path, int len_path, ARCH_FIND *find)
 {
-  long ind;
+  int ind;
   ARCH_SYMBOL *sym;
   char tpath[MAX_PATH];
   int len_tpath;
@@ -258,13 +258,15 @@ PUBLIC bool ARCH_find(ARCH *arch, const char *path, int len_path, ARCH_FIND *fin
 				break;
 		
 		  sym = &arch->symbol[ind];
-			len_tpath = sprintf(tpath2, "/%ld:%s", ind, p + 1);
+			len_tpath = sprintf(tpath2, "/%d:%s", ind, p + 1);
 			strcpy(tpath, tpath2);
 		}
+		
+  	SYMBOL_find(arch->symbol, arch->header.n_symbol, sizeof(ARCH_SYMBOL), TF_NORMAL, tpath, len_tpath, 0, &ind);
 	}
+	else
+  	SYMBOL_find_old(arch->symbol, arch->header.n_symbol, sizeof(ARCH_SYMBOL), TF_NORMAL, tpath, len_tpath, 0, &ind);
 	
-	SYMBOL_find(arch->symbol, arch->header.n_symbol, sizeof(ARCH_SYMBOL), TF_NORMAL, tpath, len_tpath, 0, &ind);
-
   if (ind == NO_SYMBOL)
     return TRUE;
 
@@ -278,7 +280,7 @@ PUBLIC bool ARCH_find(ARCH *arch, const char *path, int len_path, ARCH_FIND *fin
 }
 
 
-PUBLIC bool ARCH_read(ARCH *arch, long pos, void *buffer, long len)
+PUBLIC bool ARCH_read(ARCH *arch, int pos, void *buffer, int len)
 {
   /*if (lseek(arch->fd, pos, SEEK_SET) < 0)
     return TRUE;
