@@ -25,8 +25,8 @@
 #ifndef __GB_COMMON_SWAP_H
 #define __GB_COMMON_SWAP_H
 
-PUBLIC void SWAP_long(long *val);
-PUBLIC void SWAP_longs(long *val, int n);
+PUBLIC void SWAP_long(int *val);
+PUBLIC void SWAP_longs(int *val, int n);
 
 PUBLIC void SWAP_short(short *val);
 PUBLIC void SWAP_double(double *val);
