@@ -41,8 +41,8 @@
 
 PUBLIC short PCODE_dump(FILE *out, short addr, PCODE *code)
 {
-  static const char *op_comp[] = { "=", "<>", ">", "<=", "<", ">=", "IS", "CASE" };
-  static const char *op_arith[] = { "+", "-" , "*", "/", "NEG", "\\", "MOD", "POW", "AND", "OR", "NOT", "XOR", "&", "LIKE", "&/" };
+  static const char *op_comp[] = { "=", "<>", ">", "<=", "<", ">=", "==", "CASE" };
+  static const char *op_arith[] = { "+", "-" , "*", "/", "NEG", "\\", "MOD", "POW", "AND", "OR", "XOR", "NOT", "&", "LIKE", "&/" };
 
   int j;
   unsigned short op;
