@@ -28,9 +28,9 @@
 typedef
   struct {
     TYPE type;
-    long index;
+    int index;
     PATTERN *optional;
-    long value;
+    int value;
     }
   TRANS_PARAM;  /* must be the same as PARAM in class.h */
 
@@ -39,8 +39,8 @@ typedef
 
 typedef
   struct {
-    long type;
-    long ival;
+    int type;
+    int ival;
     long long lval;
     double dval;
     }
@@ -49,16 +49,16 @@ typedef
 typedef
   struct {
     TYPE type;
-    long ndim;
-    long dim[MAX_ARRAY_DIM];
+    int ndim;
+    int dim[MAX_ARRAY_DIM];
     }
   TRANS_ARRAY;
 
 typedef
   struct {
-    long index;               		/* index in symbol table */
+    int index;               		/* index in symbol table */
     TYPE type;                		/* data type */
-    long value;               		/* value */
+    int value;               		/* value */
     TRANS_ARRAY array;        		/* array dimensions */
     PATTERN *init;            		/* initialization code */
     long long lvalue;         		/* The value of a LONG constant */
@@ -70,12 +70,12 @@ typedef
 
 typedef
   struct {
-    long index;
+    int index;
     TYPE type;
-    long nparam;
+    int nparam;
     TRANS_PARAM param[MAX_PARAM_FUNC];
     PATTERN *start;
-    long line;
+    int line;
     unsigned vararg : 1;
     }
   PACKED
@@ -83,9 +83,9 @@ typedef
 
 typedef
   struct {
-    long index;
+    int index;
     TYPE type;
-    long nparam;
+    int nparam;
     TRANS_PARAM param[MAX_PARAM_FUNC];
     }
   PACKED
@@ -93,10 +93,10 @@ typedef
 
 typedef
   struct {
-    long index;
+    int index;
     TYPE type;
-    long line;
-    long comment;
+    int line;
+    int comment;
     bool read;
     bool _reserved[3];
     }
@@ -105,21 +105,21 @@ typedef
 
 typedef
   struct {
-    long index;
+    int index;
     TYPE type;
-    long nparam;
+    int nparam;
     TRANS_PARAM param[MAX_PARAM_FUNC];
-    long library;
-    long alias;
+    int library;
+    int alias;
     }
   PACKED
   TRANS_EXTERN;
 
 typedef
   struct {
-    long type;
-    long value;
-    long state;
+    int type;
+    int value;
+    int state;
     short *pos;
     short *pos_break;
     short *pos_continue;
@@ -132,8 +132,8 @@ typedef
 
 typedef
   struct {
-    long index;
-    long line;
+    int index;
+    int line;
     ushort pos;
     short ctrl_id;
     }
@@ -142,7 +142,7 @@ typedef
 
 typedef
   struct {
-    long index;
+    int index;
     ushort pos;
     short ctrl_id;
     }
